/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator;

import java.util.Vector;
import java.util.Locale;
import java.util.List;
import java.util.ResourceBundle;
import java.io.IOException;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.UIManager;
import javax.swing.JOptionPane;
import javax.swing.ToolTipManager;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Document;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.compare.ElementAssociation;
import org.xmloperator.framework.document.compare.DocumentAssociation;
import org.xmloperator.framework.document.compare.DocumentAssociationAccess;
import org.xmloperator.framework.document.util.PresseFragment;
import org.xmloperator.framework.document.manage.DocumentUtil;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.observer.ViewObserver;
import org.xmloperator.framework.view.global.DocumentTypeInfoView;
import org.xmloperator.framework.view.global.TransformerInfoView;
import org.xmloperator.framework.view.global.GlobalViewAccess;
import org.xmloperator.framework.view.instance.ToolViewAccess;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ViewAccess;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.DialogSelection;
import org.xmloperator.framework.view.dialog.DialogNewDocument;
import org.xmloperator.framework.control.task.TaskControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.framework.control.instance.OptionsControl;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.framework.control.instance.ViewControl;
import org.xmloperator.framework.control.instance.ExternalControl;
import org.xmloperator.framework.control.instance.ToolControl;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.global.GlobalFileControl;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.FragmentUtils;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.ViewObserverImpl;
import org.xmloperator.util.view.FileDialogs;
import org.xmloperator.option.impl.MenuItemDataImpl;
import org.xmloperator.option.OptionsImpl;
import org.xmloperator.document.FullyFledgedDocumentImpl;
import org.xmloperator.view.main.ToolViewActivator;
import org.xmloperator.operation.OperationPrinter;
import org.xmloperator.operation.OperationStack;
import org.xmloperator.operation.OperationStackImpl;
import org.xmloperator.operation.OperationSaver;
import org.xmloperator.operation.CompactOperationSaver;
import org.xmloperator.control.task.DocumentsMergeTask;
import org.xmloperator.control.task.FlushingTask;
import org.xmloperator.control.task.DocumentCompareTask;
import org.xmloperator.control.instance.FileControlImpl;
import org.xmloperator.control.instance.EditControlImpl;
import org.xmloperator.control.instance.DocumentViewControlImpl;
import org.xmloperator.control.instance.OptionsControlImpl;
import org.xmloperator.control.instance.ViewControlImpl;
import org.xmloperator.Global; // Reciprocal import

public final class Tool implements ToolControl {
  private static final Global global = new Global();
  // Instance's resources
  private int instanceIdentifier;
  private ActionListener instanceDeletedNotificationListener = null;
  private Options options;
  private FullyFledgedDocumentImpl fullyFledgedDocument;
  private OperationSaver operationSaver;
  private OperationStack operationStack;
  private List additionalMenuItemDataList;
  // Instance's control
  private ViewControlImpl viewControl;
  private FileControlImpl fileControl;
  private EditControlImpl editControl;
  private DocumentViewControlImpl documentViewControl;
  private OptionsControlImpl optionsControl;
  private FlushingTask flushingTask;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  public Tool() {
    global.lastInstanceIdentifier++;
    this.instanceIdentifier = global.lastInstanceIdentifier;
    global.instanceList.add(this);
    // Resources initialisation
    this.options = global.toolOptions;
    this.fullyFledgedDocument = new FullyFledgedDocumentImpl(options);
    this.operationSaver = new CompactOperationSaver();
    this.operationStack = new OperationStackImpl();
    this.operationStack.setOperationPrinter(operationSaver);
    this.additionalMenuItemDataList = new Vector();
    // Control initialisation
    this.flushingTask = new FlushingTask(operationSaver);
    global.taskRunner.addRefreshingTask(flushingTask);
    this.viewControl = new ViewControlImpl(instanceIdentifier,
      options, fullyFledgedDocument, operationStack,
      global.documentAssociationAccess, additionalMenuItemDataList);
    this.documentViewControl = new DocumentViewControlImpl(
        this.global.getTaskControl(), (GlobalControl)global,
        fullyFledgedDocument, (ViewAccess)viewControl, options);
    this.optionsControl =
      new OptionsControlImpl(fullyFledgedDocument, options);
    this.editControl =
      new EditControlImpl(
        this.fullyFledgedDocument, this.options, this.operationStack,
      (ViewAccess)this.viewControl,
      (DocumentViewControl)this.documentViewControl,
      (GlobalViewAccess)this.global.globalViewControl);
    this.fileControl =
      new FileControlImpl(global.getTaskControl(),
      (GlobalOptionsControl)global.globalOptionsControl,
      (GlobalViewControl)global.globalViewControl, (ToolControl)this,
      (ViewControl)viewControl, (OptionsControl)optionsControl,
      (EditControl)editControl, (DocumentViewControl)documentViewControl,
      fullyFledgedDocument, options, operationStack, operationSaver);
    // Control connections
    this.fullyFledgedDocument.setTreeViewDocumentListener(
        new ToolViewActivator((ToolViewAccess)this.viewControl));
    this.viewControl.setControleurs(
      (GlobalControl)global,
      (GlobalViewControl)global.globalViewControl,
      (GlobalOptionsControl)global.globalOptionsControl,
      (ToolControl)this,
      (FileControl)fileControl,
      (EditControl)editControl,
      (DocumentViewControl)documentViewControl,
      (OptionsControl)optionsControl);
    this.optionsControl.setControleurs(
      (GlobalControl)global,
      (GlobalViewControl)global.globalViewControl,
      (ViewControl)viewControl);
    // Open View
    int localeIndex = options.getIntOptionValue(Options.LOCALEINDEX);
    Locale.setDefault(Constants.locales[localeIndex >= 0 ? localeIndex : 0]);
    String lookAndFeelClassName;
    try {
      lookAndFeelClassName = Constants.lookAndFeelClassNames[
        global.toolOptions.getIntOptionValue(Options.LOOKANDFEELINDEX)];
    }
    catch (Exception exception) {
      lookAndFeelClassName = UIManager.getSystemLookAndFeelClassName();
    }
    try {
      UIManager.setLookAndFeel(lookAndFeelClassName);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    this.viewControl.buildToolFrame();
  }

  public static void main(String[] args) {
    if (global.instanceList.size() != 0)
      return; // Already running
    ExceptionControler.editor_email = "info@xmloperator.net";
    ExceptionControler.software = Constants.TOOL + " " + Constants.VERSION;
    try {
      int argumentCount = 0;
      String documentSystemId = null;
      String schemaSystemId = null;
      for (int k = 0; k < args.length; k++) {
        String arg = args[k];
        if (arg.length() == 0)
          continue;
        if (arg.charAt(0) == '.') {
          if (arg.equals(".log"))
            try {
              System.setOut(new PrintStream(new FileOutputStream("out.log")));
              System.setErr(new PrintStream(new FileOutputStream("err.log")));
            }
            catch (IOException exception) {
            }
          else
            System.err.println("Unknown arg. : " + arg);
        }
        else if (arg.endsWith("." + Constants.TOOL_OPTIONS_FILE_EXTENSION))
          global.toolOptionsPathName = arg;
        else {
          argumentCount++;
          switch (argumentCount) {
            case 1:
              documentSystemId = arg;
              break;
            case 2:
              schemaSystemId = arg;
              break;
            default:
              System.err.println("Too much arg.");
          }
        }
      }

      // Verify Xerces version
      final String xercesExpectedVersion = "Xerces-J 2.";
      final String xercesFoundVersion =
        org.apache.xerces.impl.Version.getVersion();
      if (!xercesFoundVersion.startsWith(xercesExpectedVersion)) {
        System.err.println(xercesExpectedVersion + "x.x is expected. "
          + xercesFoundVersion + " is found.");
        System.exit(1);
      }

      // User interface tuning
      ToolTipManager toolTipManager = ToolTipManager.sharedInstance();
      toolTipManager.setInitialDelay(300);
      toolTipManager.setDismissDelay(10000);
      toolTipManager.setReshowDelay(100);
      // Launch a tool instance
      global.hasToolToExit = true;
      global.createToolInstance(documentSystemId, schemaSystemId);
    }
    catch(Exception exception) {
      exception.printStackTrace();
      System.exit(1);
    }
  }

  public ViewControl getViewControl() {
    return viewControl;
  }

  public FileControl getFileControl() {
    return fileControl;
  }

  public DocumentViewControl getDocumentViewControl() {
    return documentViewControl;
  }

  private void displayResourceMessage(String key) {
    viewControl.getToolView().getToolStatusBar().
      displayMessage(getString(key));
  }

  private int getAnotherInstanceIdentifier() {
    int instanceCount = global.instanceList.size();
    if (instanceCount == 1) {
      displayResourceMessage(Label.THERE_IS_NO_OTHER_OPENED_INSTANCE);
      return -1;
    }
    Vector instanceIdentifierList = new Vector();
    for (int k = 0; k < instanceCount; k++) {
      int instanceIdentifier =
        ((Tool)global.instanceList.get(k)).instanceIdentifier;
      if (instanceIdentifier != this.instanceIdentifier)
        instanceIdentifierList.add(new Integer(instanceIdentifier));
    }
    if (instanceIdentifierList.size() == 1)
      return ((Integer)instanceIdentifierList.get(0)).intValue();
    DialogSelection dialogSelectionInstance =
        this.global.globalViewControl.getDialogSelectionInstance();
    dialogSelectionInstance.setItemList(instanceIdentifierList);
    dialogSelectionInstance.show();
    return dialogSelectionInstance.isOK() ?
      ((Integer)dialogSelectionInstance.getSelectedItem()).intValue() : -1;
  }

  /////////////////////////////////
  // ExternalControl implementation
  /////////////////////////////////

  public int getInstanceCount() {
    return global.getInstanceCount();
  }

  public void setInstanceDeletedNotificationListener(
    ActionListener instanceDeletedNotificationListener) {
    this.instanceDeletedNotificationListener =
      instanceDeletedNotificationListener;
  }

  public void deleteInstance() {
    boolean isLastInstance = global.getInstanceCount() == 1;
    try {
      options.setThreeStateOptionValue(Options.AUTOMATICLOADING,
        fullyFledgedDocument.getDocument() != null ?
        ThreeStateOption.TRUE : ThreeStateOption.FALSE);
      global.taskRunner.removeRefreshingTask(flushingTask);
      if (!fileControl.closeDocument())
        return;
      if (isLastInstance)
        optionsControl.saveToolOptions();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    try {
      viewControl.deleteInstanceView();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    global.instanceList.remove(this);
    try {
      Tool associatedInstance = global.getToolInstance(
        global.documentAssociationAccess.getAssociatedInstance(
        instanceIdentifier));
      if (associatedInstance != null) {
        global.documentAssociationAccess.clearDocumentAssociation();
        associatedInstance.viewControl.rebuildDocumentTreeView();
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    if (isLastInstance) {
      try {
        global.globalViewControl.deleteGlobalView();
      }
      catch (Exception exception) {
        ExceptionControler.printStackTrace(exception);
      }
      if (global.hasToolToExit)
        System.exit(0);
    }
    if (instanceDeletedNotificationListener != null)
      instanceDeletedNotificationListener.actionPerformed(
        new ActionEvent(this, 0, "EXIT"));
  }

  public FullyFledgedDocument getFullyFledgedDocument() {
    return (FullyFledgedDocument)fullyFledgedDocument;
  }

  public void addMenuItem(String label, ActionListener actionListener,
    String command) {
    additionalMenuItemDataList.add(new MenuItemDataImpl(label, actionListener,
      command));
    viewControl.getMenuHelp().addMenuItem(label, actionListener, command);
  }

  public void removeMenuItem() {
    additionalMenuItemDataList.remove(additionalMenuItemDataList.size() - 1);
    viewControl.getMenuHelp().removeMenuItem();
  }

  /////////////////////////////
  // ToolControl implementation
  /////////////////////////////

  public int getInstanceIdentifier() {
    return instanceIdentifier;
  }

  public void importFragment() {
    Tool sourceInstance =
        global.getToolInstance(getAnotherInstanceIdentifier());
    if (sourceInstance == null)
      return;
    Document document = fullyFledgedDocument.getDocument();
    PresseFragment presseFragment = fullyFledgedDocument.getPresseFragment();
    if (document == null || presseFragment == null)
      return;
    Node[] sourceNodes = sourceInstance.viewControl.getToolView().
                         getDocumentTreeView().getSelectedNodes();
    if (sourceNodes == null || !areNodesMovable(sourceNodes))
      return;
    DocumentFragment importedFragment = document.createDocumentFragment();
    FragmentUtils.importNodesToFragment(importedFragment, sourceNodes);
    presseFragment.setFragment(importedFragment);
  }

  private static final boolean areNodesMovable(Node[] nodes) {
    for (int k = 0; k < nodes.length; k++) {
      int nodeType = nodes[k].getNodeType();
      if (nodeType == Node.DOCUMENT_NODE
        || nodeType == Node.DOCUMENT_TYPE_NODE
        || nodeType == Node.ATTRIBUTE_NODE)
        return false;
    }
    return true;
  }

  public void compareDocument() {
    deleteDocumentAssociation();
    Tool comparedInstance =
        global.getToolInstance(getAnotherInstanceIdentifier());
    if (comparedInstance == null)
      return;
    Document document1 = fullyFledgedDocument.getDocument();
    Document document2 = comparedInstance.fullyFledgedDocument.getDocument();
    if (document1 == null || document2 == null)
      return;
    global.taskRunner.appendTask(new DocumentCompareTask(document1, document2,
        instanceIdentifier, comparedInstance.instanceIdentifier,
        global.documentAssociationAccess,
        (ViewAccess)viewControl, (ViewAccess)comparedInstance.viewControl));
    global.taskRunner.appendTask(viewControl.treeViewRebuildTask);
    global.taskRunner.appendTask(
        comparedInstance.viewControl.treeViewRebuildTask);
  }

  public void mergeDocuments() {
    Tool associatedInstance = global.getToolInstance(
        global.documentAssociationAccess.getAssociatedInstance(
        instanceIdentifier));
    if (associatedInstance == null) {
      displayResourceMessage(Label.THERE_IS_NO_ASSOCIATED_DOCUMENT);
      return;
    }
    Document document1 = fullyFledgedDocument.getDocument();
    Document document2 =
        associatedInstance.fullyFledgedDocument.getDocument();
    DocumentAssociation documentAssociation =
        global.documentAssociationAccess.getDocumentAssociation();
    if (document1 == null || document2 == null || documentAssociation == null)
      return;
    DocumentTypeInfo documentTypeInfo =
        fullyFledgedDocument.getDocumentTypeInfo();
    if (documentTypeInfo
        != associatedInstance.fullyFledgedDocument.getDocumentTypeInfo()) {
      displayResourceMessage(Label.THE_DOCUMENTS_HAVE_NOT_THE_SAME_TYPE);
      return;
    }
    Element rootElement1 = document1.getDocumentElement();
    ElementAssociation rootElementAssociation =
        documentAssociation.getElementAssociation(rootElement1);
    if (rootElementAssociation == null
        || rootElementAssociation.getAssociatedElement(rootElement1)
        != document2.getDocumentElement()) {
      displayResourceMessage(Label.ROOT_ELEMENTS_ARE_NOT_ASSOCIATED);
      return;
    }
    DialogNewDocument dialogNewDocument =
        this.global.globalViewControl.getDialogNewDocument();
    dialogNewDocument.setTitle(getString(Label.MERGE_AS));
    dialogNewDocument.show();
    if (!dialogNewDocument.isOK())
      return;
    String documentPathName = dialogNewDocument.getDocumentPathName();
    if (documentPathName.lastIndexOf('.') < 0)
      documentPathName += ".xml";
    if (!FileDialogs.isFileWritable(documentPathName))
      return;
    global.taskRunner.appendTask(new DocumentsMergeTask(documentPathName,
        fullyFledgedDocument.getDTD(), documentTypeInfo, documentAssociation,
        options, (ViewAccess)viewControl,
        (ViewAccess)associatedInstance.viewControl));
  }

  public void gotoAssociatedElement() {
    Node node =
        viewControl.getToolView().getDocumentTreeView().getSelectedNode();
    if (node == null)
      return;
    if (node.getNodeType() != Node.ELEMENT_NODE) {
      node = node.getParentNode();
      if (node.getNodeType() != Node.ELEMENT_NODE)
        return;
    }
    DocumentAssociation documentAssociation =
        global.documentAssociationAccess.getDocumentAssociation();
    Tool associatedInstance = global.getToolInstance(
        global.documentAssociationAccess.getAssociatedInstance(
        instanceIdentifier));
    if (documentAssociation == null || associatedInstance == null)
      return;
    ElementAssociation elementAssociation =
        documentAssociation.getElementAssociation((Element)node);
    if (elementAssociation != null)
      associatedInstance.documentViewControl.gotoNode(
          elementAssociation.getAssociatedElement((Element)node));
  }

  public void deleteCurrentDocumentAssociation() {
    if (global.documentAssociationAccess.getAssociatedInstance(
        instanceIdentifier) >= 1)
      deleteDocumentAssociation();
  }

  private static void deleteDocumentAssociation() {
    Tool instance1 = global.getToolInstance(
      global.documentAssociationAccess.getInstanceIdentifier1());
    Tool instance2 = global.getToolInstance(
      global.documentAssociationAccess.getInstanceIdentifier2());
    if (instance1 == null || instance2 == null)
      return;
    global.documentAssociationAccess.clearDocumentAssociation();
    instance1.viewControl.rebuildDocumentTreeView();
    instance2.viewControl.rebuildDocumentTreeView();
  }

  public void execCommand(String command, boolean hasInstanceToBeClosed)
      throws IOException {
    int documentPathPosition = command.indexOf("%D");
    boolean isAbsolutePathName = true;
    if (documentPathPosition < 0) {
      documentPathPosition = command.indexOf("%d");
      isAbsolutePathName = false;
    }
    if (documentPathPosition >= 0) {
      String systemId = fullyFledgedDocument.getDocumentSystemId();
      if (isAbsolutePathName)
        systemId = FileUtils.expandedSystemId(systemId);
      else
        systemId = FileUtils.reducedSystemId(systemId);
      String pathName = FileUtils.fromSystemIdToFilePathName(systemId);
      if (pathName == null) {
        displayResourceMessage(Label.THERE_IS_NO_OPENED_DOCUMENT_FILE);
        return;
      }
      command = command.substring(0, documentPathPosition)
              + FileUtils.unnormalisePathName(pathName)
              + command.substring(documentPathPosition + 2);
    }
    viewControl.getToolView().getToolStatusBar().displayMessage(command);
    Runtime.getRuntime().exec(command);
    if (hasInstanceToBeClosed)
      deleteInstance();
  }
}
