/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.compare.impl;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.compare.ElementCharacterization;
import org.xmloperator.framework.document.compare.ElementCharacterizationAccess;
import org.xmloperator.framework.document.compare.ElementAssociation;
import org.xmloperator.framework.document.compare.DocumentAssociation;
import org.xmloperator.framework.document.compare.MutableDocumentAssociation;
import org.xmloperator.framework.document.compare.CancelException;
import org.xmloperator.framework.document.compare.BuildingDocumentAssociation;
import org.xmloperator.compare.impl.ElementCharacterizationImpl;
import org.xmloperator.compare.impl.ElementAssociationImpl;

public final class DocumentAssociationImpl
    implements BuildingDocumentAssociation, ElementCharacterizationAccess {
  private static final boolean DEBUG = false;
  private Document document1;
  private Document document2;
  private boolean isCanceled = false;
  private final Map fromElementToElementCharacterization = new HashMap();
  private final Map fromElement1ToElementAssociation = new HashMap();
  private final Map fromElement2ToElementAssociation = new HashMap();

  public DocumentAssociationImpl(Document document1, Document document2) {
    this.document1 = document1;
    this.document2 = document2;
  }

  public void cancel() {
    this.isCanceled = true;
  }

  public void clear() {
    this.fromElementToElementCharacterization.clear();
    this.fromElement1ToElementAssociation.clear();
    this.fromElement2ToElementAssociation.clear();
  }

  private ElementAssociation getElement1Association(Element element) {
    return (ElementAssociation)fromElement1ToElementAssociation.get(element);
  }

  private ElementAssociation getElement2Association(Element element) {
    return (ElementAssociation)fromElement2ToElementAssociation.get(element);
  }

  private static final String debugElement(Element element) {
    String localName = element.getLocalName();
    String idValue = element.getAttribute("id");
    return idValue != null ? localName + "(" + idValue + ")" : localName;
  }

  ///////////////////////////////////////////////
  // ElementCharacterizationAccess implementation
  ///////////////////////////////////////////////

  public ElementCharacterization getElementCharacterization(Element element) {
    ElementCharacterization elementCharacterization = (ElementCharacterization)
        this.fromElementToElementCharacterization.get(element);
    if (elementCharacterization == null) {
      elementCharacterization = new ElementCharacterizationImpl(
          element, (ElementCharacterizationAccess)this);
      this.fromElementToElementCharacterization.put(
          element, elementCharacterization);
    }
    return elementCharacterization;
  }

  /////////////////////////////////////
  // DocumentAssociation implementation
  /////////////////////////////////////

  public ElementCharacterizationAccess getElementCharacterizationAccess() {
    return (ElementCharacterizationAccess)this;
  }

  public Document getDocument1() {
    return document1;
  }

  public Document getDocument2() {
    return document2;
  }

  public Document getAssociatedDocument(Document document) {
    if (document == document1)
      return document2;
    else if (document == document2)
      return document1;
    else
      return null;
  }

  public Collection elementAssociations() {
    return Collections.unmodifiableCollection(
        fromElement1ToElementAssociation.values());
  }

  public ElementAssociation getElementAssociation(Element element) {
    Document document = element.getOwnerDocument();
    if (document == document1)
      return getElement1Association(element);
    if (document == document2)
      return getElement2Association(element);
    return null;
  }

  ////////////////////////////////////////////
  // MutableDocumentAssociation implementation
  ////////////////////////////////////////////

  public ElementAssociation addElementAssociation(
      Element element1, Element element2) {
    if (element1.getOwnerDocument() != document1
      || element2.getOwnerDocument() != document2)
      throw new IllegalArgumentException();
    {
      ElementAssociation elementAssociation1 =
          (ElementAssociation)fromElement1ToElementAssociation.get(element1);
      ElementAssociation elementAssociation2 =
          (ElementAssociation)fromElement2ToElementAssociation.get(element2);
      if (elementAssociation1 != null)
        if (elementAssociation1.getElement2() == element2)
          return null; // Association already build.
        else
          throw new IllegalArgumentException("Element1 already associated");
      if (elementAssociation2 != null)
        throw new IllegalArgumentException("Element2 already associated");
    }
    if (!this.isAssociationCoherentWithAncestor(element1, element2))
      return null;
    if (DEBUG)
      System.out.println(
          "addElementAssociation(" + debugElement(element1) + ")");
    this.removeUncoherentChildAssociations(element1, element2);
    this.removeUncoherentChildAssociations(element2, element1);
    ElementAssociation elementAssociation =
        new ElementAssociationImpl(element1, element2);
    fromElement1ToElementAssociation.put(element1, elementAssociation);
    fromElement2ToElementAssociation.put(element2, elementAssociation);
    elementAssociation.addDescendantCount(
        this.computeDescendantCount(element1));
    this.adjustAncestors(elementAssociation, +1);
    return elementAssociation;
  }

  private boolean isAssociationCoherentWithAncestor(
    Element element1, Element element2) {
    ElementAssociation ancestorElementAssociation = null;
    Node ancestor1 = element1.getParentNode();
    while (ancestor1.getNodeType() == Node.ELEMENT_NODE) {
      ancestorElementAssociation =
        this.getElement1Association((Element)ancestor1);
      if (ancestorElementAssociation != null)
        break;
      ancestor1 = ancestor1.getParentNode();
    }
    if (ancestorElementAssociation == null)
      return true;
    Element ancestorElement2 = ancestorElementAssociation.getElement2();
    Node ancestor2 = element2.getParentNode();
    while (ancestor2.getNodeType() == Node.ELEMENT_NODE) {
      if (ancestor2 == ancestorElement2)
        return true;
      ancestor2 = ancestor2.getParentNode();
    }
    return false;
  }

  private void removeUncoherentChildAssociations(Element parentElement,
      Element associatedParent) {
    Node childNode = parentElement.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE) {
        this.removeUncoherentChildAssociations(
            (Element)childNode, associatedParent);
        ElementAssociation childElementAssociation =
            this.getElementAssociation((Element)childNode);
        if (childElementAssociation != null
            && !isDescendant(
            childElementAssociation.getAssociatedElement((Element)childNode),
            associatedParent))
          this.removeElementAssociation(childElementAssociation);
      }
      childNode = childNode.getNextSibling();
    }
  }

  private int computeDescendantCount(Element element) {
    int descendantCount = 0;
    Node childNode = element.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE) {
        ElementAssociation elementAssociation =
            this.getElementAssociation((Element)childNode);
        descendantCount += elementAssociation != null ?
                           elementAssociation.getDescendantCount() :
                           this.computeDescendantCount((Element)childNode);
      }
      childNode = childNode.getNextSibling();
    }
    return descendantCount;
  }

  public void removeElementAssociation(
      ElementAssociation elementAssociation) {
    if (DEBUG)
      System.out.println("removeElementAssociation("
                         + debugElement(elementAssociation.getElement1())
                         + ")");
    if (this.fromElement1ToElementAssociation.remove(
        elementAssociation.getElement1()) == null)
      throw new IllegalArgumentException();
    if (this.fromElement2ToElementAssociation.remove(
        elementAssociation.getElement2()) == null)
      throw new IllegalStateException();
    this.adjustAncestors(elementAssociation, -1);
    this.removeChildAssociations(elementAssociation.getElement1());
  }

  private void removeChildAssociations(Element element1) {
    Node childNode1 = element1.getFirstChild();
    while (childNode1 != null) {
      if (childNode1.getNodeType() == Node.ELEMENT_NODE) {
        ElementAssociation elementAssociation =
            this.getElement1Association((Element)childNode1);
        if (elementAssociation != null)
          this.removeElementAssociation(elementAssociation);
      }
      childNode1 = childNode1.getNextSibling();
    }
  }

  private void adjustAncestors(
      ElementAssociation elementAssociation, int addedCount) {
    Node parent1 = elementAssociation.getElement1().getParentNode();
    while (parent1 != null && parent1.getNodeType() == Node.ELEMENT_NODE) {
      elementAssociation =
          this.getElementAssociation((Element)parent1);
      if (elementAssociation != null)
        elementAssociation.addDescendantCount(addedCount);
      parent1 = parent1.getParentNode();
    }
  }

  /////////////////////////////////////////////
  // BuildingDocumentAssociation implementation
  /////////////////////////////////////////////

  public void computeElementCharacterizations() throws CancelException {
    this.computeElementCharacterizations(this.document1.getDocumentElement());
    this.computeElementCharacterizations(this.document2.getDocumentElement());
  }

  public void computeElementCharacterizations(Element element)
      throws CancelException {
    Node childNode = element.getFirstChild();
    while (childNode != null) {
      if (this.isCanceled)
        throw new CancelException();
      if (childNode.getNodeType() == Node.ELEMENT_NODE)
        this.computeElementCharacterizations((Element)childNode);
      childNode = childNode.getNextSibling();
    }
    this.fromElementToElementCharacterization.put(
        element, new ElementCharacterizationImpl(
        element, (ElementCharacterizationAccess)this));
  }

  private static final class AssociationProject implements Comparable {
    public Element element1;
    public Element element2;
    private int descendantCount;

    AssociationProject(Element element1, Element element2, int descendantCount)
    {
      this.element1 = element1;
      this.element2 = element2;
      this.descendantCount = descendantCount;
    }

    // Decreasing order of descendant element count.
    public int compareTo(Object o) {
      return ((AssociationProject)o).descendantCount
          - this.descendantCount;
    }
  }

  public final int associateEquallyCharacterizedElements()
      throws CancelException {
    int newAssociationCount = associateEquallyCharacterizedElements(
        this.document1.getDocumentElement(),
        this.document2.getDocumentElement());
    if (DEBUG)
      if (newAssociationCount != 0)
        System.out.println(
            "associateEquallyCharacterizedElements creates "
            + newAssociationCount + " new associations");
    return newAssociationCount;
  }

  private final int associateEquallyCharacterizedElements(
      Element rootElement1, Element rootElement2) throws CancelException {
    if (DEBUG)
      System.out.println(
          "associateEquallyCharacterizedElements("
          + debugElement(rootElement1) + ", "
          + debugElement(rootElement2) + ")");
    List elementList1 = this.sortedElementList(rootElement1);
    int elementCount1 = elementList1.size();
    if (elementCount1 == 0)
      return 0;
    List elementList2 = this.sortedElementList(rootElement2);
    int elementCount2 = elementList2.size();
    if (elementCount2 == 0)
      return 0;
    int newAssociationCount = 0;
    List associationProjectList = new ArrayList();
    int index1 = 0;
    int index2 = 0;
    while (index1 < elementCount1 && index2 < elementCount2) {
      if (this.isCanceled)
        throw new CancelException();
      Element element1 = (Element) elementList1.get(index1);
      Element element2 = (Element) elementList2.get(index2);
      ElementCharacterization elementCharacterization1 =
          this.getElementCharacterization(element1);
      ElementCharacterization elementCharacterization2 =
          this.getElementCharacterization(element2);
      int hashCode1 = elementCharacterization1.getFullHashCode();
      int hashCode2 = elementCharacterization2.getFullHashCode();
      if (hashCode1 < hashCode2)
        index1++;
      else if (hashCode1 > hashCode2)
        index2++;
      else {
        int nextIndex1 = index1 + 1;
        while (nextIndex1 < elementCount1
               && this.getElementCharacterization(
            (Element) elementList1.get(nextIndex1)).
               getFullHashCode() == hashCode1)
          nextIndex1++;
        int nextIndex2 = index2 + 1;
        while (nextIndex2 < elementCount2
               && this.getElementCharacterization(
            (Element) elementList2.get(nextIndex2)).
               getFullHashCode() == hashCode2)
          nextIndex2++;
        if (nextIndex1 == index1 + 1 && nextIndex2 == index2 + 1
            && elementCharacterization1.equals(elementCharacterization2))
          associationProjectList.add(new AssociationProject(
              element1, element2,
              elementCharacterization1.getDescendantElementCount()));
        index1 = nextIndex1;
        index2 = nextIndex2;
      }
    }
    Collections.sort(associationProjectList);
    for (int k = 0; k < associationProjectList.size(); k++) {
      AssociationProject associationProject =
          (AssociationProject) associationProjectList.get(k);
      Element element1 = associationProject.element1;
      Element element2 = associationProject.element2;
      ElementAssociation elementAssociation =
          this.addElementAssociation(element1, element2);
      if (elementAssociation != null) {
        newAssociationCount++;
        newAssociationCount += this.associateEquallyCharacterizedElements(
            element1, element2);
      }
    }
    associationProjectList.clear();
    return newAssociationCount;
  }

  private List sortedElementList(Element rootElement) {
    if (!hasChildElement(rootElement))
      return Collections.EMPTY_LIST;
    List elementList = new ArrayList();
    addFreeDescendantElements(elementList, rootElement);
    Collections.sort(elementList, new Comparator() {
      public int compare(Object o1, Object o2) {
        int ec1 = getElementCharacterization((Element)o1).getFullHashCode();
        int ec2 = getElementCharacterization((Element)o2).getFullHashCode();
        // To not compute (ec1 - ec2). This would overload.
        return ec1 < ec2 ? -1 : ec1 > ec2 ? 1 : 0;
      }
    });
    return elementList;
  }

  private void addFreeDescendantElements(List elementList, Element parent) {
    Node childNode = parent.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE
          && this.getElementAssociation((Element)childNode) == null) {
        elementList.add((Element)childNode);
        addFreeDescendantElements(elementList, (Element)childNode);
      }
      childNode = childNode.getNextSibling();
    }
  }

  public final int associateAncestors() throws CancelException {
    int newAssociationCount = this.associateAncestors(
        document1.getDocumentElement(), document2.getDocumentElement());
    if (DEBUG)
      if (newAssociationCount != 0)
        System.out.println(
            "associateAncestors creates " + newAssociationCount
            + " new associations");
    return newAssociationCount;
  }

  /**
   * Associates descendants of rootElement1 and rootElement2 whose descendants
   *   are associated.
   */
  private final int associateAncestors(
      Element rootElement1, Element rootElement2) throws CancelException {
    if (DEBUG)
      System.out.println("associateAncestors("
                         + debugElement(rootElement1) + ", "
                         + debugElement(rootElement2) + ")");
    int newAssociationCount = 0;
    Node node1 = rootElement1.getFirstChild();
    while (node1 != null) {
      if (this.isCanceled)
        throw new CancelException();
      if (node1.getNodeType() == Node.ELEMENT_NODE) {
        ElementAssociation elementAssociation =
          this.getElementAssociation((Element)node1);
        if (elementAssociation != null)
          newAssociationCount += this.associateAncestors(
              (Element)node1, elementAssociation.getElement2());
        else {
          newAssociationCount += this.associateAncestors(
              (Element)node1, rootElement2);
          elementAssociation = this.associateParents(
              (Element)node1, rootElement2);
          if (elementAssociation != null) {
            newAssociationCount++;
            newAssociationCount += this.associateEquallyCharacterizedElements(
                elementAssociation.getElement1(),
                elementAssociation.getElement2());
          }
        }
      }
      node1 = node1.getNextSibling();
    }
    return newAssociationCount;
  }

  /**
   * Try to associate <code>element1</code> to an element, said "e2".
   * e2 is choosen in a way that maximizes the descendant element association
   *   count.
   * Adding the new association may have the consequence of breakink some
   *   uncoherent child element associations.
   * Return the element association in case of success, else null.
   */
  private final ElementAssociation associateParents(
      Element element1, Element rootElement2) {
    Map fromElementToElementSimilarity = new HashMap();
    this.addCandidatesForAssociation(
        fromElementToElementSimilarity, element1, rootElement2);
    Element element2 =
        ElementSimilarity.bestElement(fromElementToElementSimilarity.values());
    return element2 != null /*&& this.getElement2Association(element2) == null*/ ?
        this.addElementAssociation(element1, element2) : null;
  }

  private static final class ElementSimilarity {
    private Element element;
    public int similarityValue = 0;

    public static final Element bestElement(Collection elementSimilarityList) {
      Element bestElement = null;
      int bestSimilarityValue = -1;
      Iterator iterator = elementSimilarityList.iterator();
      while (iterator.hasNext()) {
        ElementSimilarity elementSimilarity =
          (ElementSimilarity)iterator.next();
        if (elementSimilarity.similarityValue > bestSimilarityValue) {
          bestElement = elementSimilarity.element;
          bestSimilarityValue = elementSimilarity.similarityValue;
        }
      }
      return bestElement;
    }

    public ElementSimilarity(Element element) {
      this.element = element;
    }
  }

  /**
   * For each first descendant "d1" of <code>element1</code> that is associated
   *   to an element, said "d2", add all the ancestors of d2 that could
   *   be associated to <code>element1</code> to the map
   *   <code>fromElementToElementSimilarity</code>.
   */
  private final void addCandidatesForAssociation(
    Map fromElementToElementSimilarity,
    Element element1, Element rootElement2) {
    String namespaceURI1 = element1.getNamespaceURI();
    String localName1 = element1.getLocalName();
    Node childNode1 = element1.getFirstChild();
    while (childNode1 != null) {
      if (childNode1.getNodeType() == Node.ELEMENT_NODE) {
        ElementAssociation elementAssociation =
          this.getElementAssociation((Element)childNode1);
        if (elementAssociation != null) {
          Element element2 = elementAssociation.getElement2();
          while (true) {
            {
              Node node2 = element2.getParentNode();
              if (node2.getNodeType() != Node.ELEMENT_NODE
                  || node2 == rootElement2)
                break;
              element2 = (Element)node2;
            }
            if (this.getElementAssociation(element2) != null)
              throw new IllegalStateException();
            if (!element2.getLocalName().equals(localName1))
              continue;
            {
              String namespaceURI2 = element2.getNamespaceURI();
              if (namespaceURI1 != null ?
                  !namespaceURI1.equals(namespaceURI2) : namespaceURI2 != null)
                continue;
            }
            {
              ElementSimilarity elementSimilarity =
                  (ElementSimilarity) fromElementToElementSimilarity.
                  get(element2);
              if (elementSimilarity == null) {
                elementSimilarity = new ElementSimilarity(element2);
                fromElementToElementSimilarity.put(element2, elementSimilarity);
              }
              elementSimilarity.similarityValue +=
                  elementAssociation.getDescendantCount();
              break;
            }
          }
        }
        else
          this.addCandidatesForAssociation(
              fromElementToElementSimilarity,
              (Element)childNode1, rootElement2);
      }
      childNode1 = childNode1.getNextSibling();
    }
  }

  /////////////////
  // Static methods
  /////////////////

  private static final boolean isDescendant(Node node, final Node ancestor) {
    while (true) {
      node = node.getParentNode();
      if (node == ancestor)
        return true;
      if (node == null)
        return false;
    }
  }

  private static final boolean hasChildElement(Element element) {
    Node childNode = element.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE)
        return true;
      childNode = childNode.getNextSibling();
    }
    return false;
  }
}
