/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.global;

import java.util.ResourceBundle;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Frame;
import javax.swing.JOptionPane;

import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.apache.xerces.xni.XNIException;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.doctype.DTDInfo;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.parse.Dereferencer;
import org.xmloperator.framework.document.parse.ToolEntityResolver;
import org.xmloperator.framework.document.transform.TransformerInfo;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.Dialog;
import org.xmloperator.framework.view.dialog.DialogSelection;
import org.xmloperator.framework.view.global.GlobalViewAccess;
import org.xmloperator.framework.view.global.DocumentTypeInfoView;
import org.xmloperator.framework.view.global.TransformerInfoView;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.util.io.MIME2Java;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.dom.RelaxNGSimplifier;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.FileDialogs;
import org.xmloperator.util.view.ExceptionDialogs;
import org.xmloperator.option.impl.DocumentTypeInfoImpl;
import org.xmloperator.option.impl.TransformerInfoImpl;
import org.xmloperator.option.OptionsImpl;
import org.xmloperator.document.parse.ToolParser;
import org.xmloperator.document.parse.DereferencerImpl;
import org.xmloperator.document.DTDXMLToDTDTranslator;
import org.xmloperator.document.util.DocumentWriter;
import org.xmloperator.document.util.DocumentTypeInfoUtils;
import org.xmloperator.document.id.IdStore;
import org.xmloperator.control.io.ToolEntityResolverImpl;
import org.xmloperator.control.io.IOUtils;

public final class GlobalOptionsControlImpl implements GlobalOptionsControl {
  private Options options;
  private GlobalViewControl globalViewControl = null;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  public GlobalOptionsControlImpl(Options options,
    GlobalViewControl globalViewControl) {
    this.options = options;
    this.globalViewControl = globalViewControl;
  }

  private DocumentTypeInfo selectResultDocumentTypeInfo() {
    List documentTypeInfoList =
      options.getOptionList(Options.DOCUMENTTYPE);
    Collections.sort(documentTypeInfoList);
    DialogSelection dialog =
        this.globalViewControl.getDialogSelectionDocumentType(true, true);
    dialog.setItemList(documentTypeInfoList);
    dialog.setTitle(Label.SELECT_RESULT_DOCUMENT_TYPE);
    dialog.show();
    return dialog.isOK() ? (DocumentTypeInfo)dialog.getSelectedItem() : null;
  }

  //////////////////////////////////////
  // GlobalOptionsControl implementation
  //////////////////////////////////////

  public void buildDTDFromDTDXML(String dTDXMLSystemId, String dTDSystemId,
    boolean arePatternsTranslatedToParameterEntityDeclarations) {
    if (dTDXMLSystemId == null || dTDXMLSystemId.length() == 0
        || dTDSystemId == null || dTDSystemId.length() == 0)
      return;
    String dTDPathname =
        FileUtils.fromSystemIdToFilePathName(dTDSystemId);
    try {
      if (dTDPathname == null)
        throw new IOException("Cannot save " + '"' + dTDSystemId + '"');
      if (!FileDialogs.isFileWritable(dTDPathname))
        return;
      ToolParser parser = new ToolParser();
      parser.setErrorHandler(ToolParser.DEFAULT_ERROR_HANDLER);
      {
        ToolEntityResolver entityResolver =
            new ToolEntityResolverImpl(options, null);
        parser.setXMLEntityResolver(entityResolver);
        parser.parse(entityResolver.resolveEntity(null, dTDXMLSystemId));
      }
      Document document = parser.getDocument();
      if (document == null)
        return;
      DTD xMLDTD = parser.getDTD();
      parser = null;
      OutputStream outputStream = new FileOutputStream(dTDPathname);
      final String encoding = Constants.DEFAULT_ENCODING;
      PrintWriter out = new PrintWriter(new OutputStreamWriter(outputStream,
          MIME2Java.convert(encoding)));
      out.println("<?xml version=\"1.0\" encoding=\"" + encoding + "\" ?>");
      int result = DTDXMLToDTDTranslator.printDTD(out, document,
          new IdStore(document, xMLDTD),
          arePatternsTranslatedToParameterEntityDeclarations);
      out.close();
      String title = getString(Label.BUILDING_DTD);
      if (result != 0)
        ExceptionDialogs.displayWarningMessage(title,
            DTDXMLToDTDTranslator.resultToString(result));
      else
        ExceptionDialogs.displayPlainMessage(title,
            getString(Label.DTD_IS_BUILT));
    }
    catch (XNIException exception) {
      ExceptionDialogs.displayXNIException(exception);
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException("I/O exception", exception);
    }
  }

  /*
  public void buildDTDXMLFromDTD(DTDInfo dTDInfo, String nsUriReference,
    boolean useNamesForIds) {
    String dTDSystemId = dTDInfo.getUserRelativeDTDSystemId();
    String dTDXMLSystemId = dTDInfo.getDTDXMLSystemId();
    if (dTDSystemId == null || dTDSystemId.length() == 0
        || dTDXMLSystemId == null || dTDXMLSystemId.length() == 0)
      return;
    String dTDXMLPathname =
        FileUtils.fromSystemIdToFilePathName(dTDXMLSystemId);
    try {
      if (dTDXMLPathname == null)
        throw new IOException("Cannot save " + '"' + dTDXMLSystemId + '"');
      if (!FileDialogs.isFileWritable(dTDXMLPathname))
        return;
      DTD dTD = ToolParser.parseDTD(dTDSystemId,
                                    new ToolEntityResolverImpl(options, null));
      if (dTD.getElementNameList().isEmpty())
        throw new IOException(dTDSystemId + " is empty or non-existent");
      if (dTD.isThereAPrefixAttachedToDifferentNamespaces())
        throw new IOException(dTDSystemId + " cannot be translated to DTDXML");
      final String encoding = Constants.DEFAULT_ENCODING;
      OutputStream outputStream = new FileOutputStream(dTDXMLPathname);
      PrintWriter out = new PrintWriter(new OutputStreamWriter(outputStream,
          MIME2Java.convert(encoding)));
      DTDToDTDXMLTranslator.printDTDXML(
          out, encoding, dTD,
          dTDInfo.getDTDPublicId(), dTDInfo.getHTTPDTDSystemId(),
          nsUriReference, useNamesForIds);
      out.close();
      IOUtils.addResourceFirst(options, dTDXMLSystemId);
      ExceptionDialogs.displayPlainMessage(getString(Label.BUILDING_DTDXML),
          getString(Label.DTDXML_IS_BUILT));
    }
    catch (XNIException exception) {
      ExceptionDialogs.displayXNIException(exception);
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException("I/O exception", exception);
    }
  }
  */

  public void buildRNGSimpleSyntaxFromFullSyntax(
    String rNGFullSyntaxSystemId, String rNGSimpleSyntaxSystemId,
    int simplificationType) {
    if (rNGFullSyntaxSystemId == null || rNGFullSyntaxSystemId.length() == 0
        || rNGSimpleSyntaxSystemId == null
        || rNGSimpleSyntaxSystemId.length() == 0)
      return;
    String rNGPathname =
        FileUtils.fromSystemIdToFilePathName(rNGSimpleSyntaxSystemId);
    try {
      if (rNGPathname == null)
        throw new IOException("Cannot save "
                              + '"' + rNGSimpleSyntaxSystemId + '"');
      if (!FileDialogs.isFileWritable(rNGPathname))
        return;
      Dereferencer dereferencer =
          new DereferencerImpl(new ToolEntityResolverImpl(options, null));
      Element patternElement = dereferencer.constructElementUsingURIReference(
          null, rNGFullSyntaxSystemId);
      if (patternElement == null)
        return;
      Document document = patternElement.getOwnerDocument();
      String baseURI = FileUtils.expandedSystemId(rNGFullSyntaxSystemId);
      RelaxNGSimplifier.simplify(
          document, baseURI, dereferencer, simplificationType);
      OutputStream outputStream = new FileOutputStream(rNGPathname);
      final String encoding = Constants.DEFAULT_ENCODING;
      PrintWriter out = new PrintWriter(new OutputStreamWriter(outputStream,
          MIME2Java.convert(encoding)));
      DocumentWriter documentWriter = new DocumentWriter(out);
      documentWriter.setLineFeedAtStartTagEnd(
          options.getThreeStateOptionValue(Options.ADDLINEFEEDATSTARTTAGEND)
          == ThreeStateOption.TRUE);
      documentWriter.printDocument(document, null, encoding, true);
      out.close();
      ExceptionDialogs.displayPlainMessage(
          getString(Label.BUILDING_RNG_SIMPLE_SYNTAX),
          getString(Label.RNG_SIMPLE_SYNTAX_IS_BUILT));
    }
    catch (RelaxNGSimplifier.RelaxNGSimplificationException exception) {
      ExceptionDialogs.displayException("RELAX NG simplification exception",
        exception);
    }
    catch (XNIException exception) {
      ExceptionDialogs.displayXNIException(exception);
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException("I/O exception", exception);
    }
  }

  public DocumentTypeInfo newDocumentType() {
    return newDocumentTypeInternal(new DocumentTypeInfoImpl(), false);
  }

  public DocumentTypeInfo newDocumentType(String documentTypeName,
    DocumentType doctype) {
    if (documentTypeName == null)
      documentTypeName = "";
    DocumentTypeInfo documentTypeInfo =
      new DocumentTypeInfoImpl(documentTypeName, "");
    if (doctype != null) {
      DTDInfo dTDInfo = documentTypeInfo.getDTDInfo();
      dTDInfo.setDTDPublicId(doctype.getPublicId());
      dTDInfo.setDocumentRelativeDTDSystemId(doctype.getSystemId());
    }
    return newDocumentTypeInternal(documentTypeInfo,
      documentTypeName.length() != 0);
  }

  public DocumentTypeInfo newDocumentType(
    DocumentTypeInfo documentTypeInfoSource) {
    return newDocumentTypeInternal(documentTypeInfoSource != null ?
      new DocumentTypeInfoImpl(documentTypeInfoSource) :
      new DocumentTypeInfoImpl(), false);
  }

  private DocumentTypeInfo newDocumentTypeInternal(
    DocumentTypeInfo documentTypeInfo, boolean isNameReadOnly) {
    DocumentTypeInfoView documentTypeInfoView =
        globalViewControl.createDocumentTypeInfoView();
    Frame frame = documentTypeInfoView.getFrame();
    isNameReadOnly &= documentTypeInfo.getDocumentTypeName().length() != 0;
    Dialog dialog =
        this.globalViewControl.getDialogDocumentTypeInfo(
        documentTypeInfo, options.getOptionList(Options.TRANSFORMER),
        isNameReadOnly, false);
    dialog.show();
    if (!dialog.isOK())
      return null;
    List documentTypeInfoList =
        options.getOptionList(Options.DOCUMENTTYPE);
    boolean isAdditionalKeyToBeChanged =
        documentTypeInfoList.contains(documentTypeInfo);
    if (isAdditionalKeyToBeChanged)
      while (documentTypeInfoList.contains(documentTypeInfo))
        documentTypeInfo.setDocumentTypeAdditionalKey(
            documentTypeInfo.getDocumentTypeAdditionalKey() + "x");
      documentTypeInfoList.add(documentTypeInfo);
      Collections.sort(documentTypeInfoList);
      documentTypeInfoView.update();
      documentTypeInfoView.select(
          documentTypeInfo.getDocumentTypeName(),
          documentTypeInfo.getDocumentTypeAdditionalKey());
      if (isAdditionalKeyToBeChanged)
        JOptionPane.showMessageDialog(
            frame,
            getString(Label.ADDITIONAL_KEY_HAS_BEEN_CHANGED),
            getString(Label.DOCUMENT_TYPE_ALREADY_DEFINED),
            JOptionPane.OK_OPTION);
      return documentTypeInfo;
  }

  public void editDocumentTypeInfo(
    String documentTypeName, String documentTypeAdditionalKey) {
    List documentTypeInfoList =
        options.getOptionList(Options.DOCUMENTTYPE);
    DocumentTypeInfo documentTypeInfo =
        DocumentTypeInfoUtils.getDocumentTypeInfoFromList(documentTypeInfoList,
        documentTypeName, documentTypeAdditionalKey);
    if (documentTypeInfo == null)
      return;
    DocumentTypeInfoView documentTypeInfoView =
        globalViewControl.createDocumentTypeInfoView();
    Frame frame = documentTypeInfoView.getFrame();
    Dialog dialog =
        this.globalViewControl.getDialogDocumentTypeInfo(
        documentTypeInfo,
        TransformerInfoImpl.createTransformerInfoSubList(
        options.getOptionList(Options.TRANSFORMER),
        documentTypeInfo.getDocumentTypeName()), true, true);
    dialog.show();
    if (!dialog.isOK())
      return;
    documentTypeInfoView.update();
    documentTypeInfoView.select(
        documentTypeInfo.getDocumentTypeName(),
        documentTypeInfo.getDocumentTypeAdditionalKey());
  }

  public void removeDocumentType(
    String documentTypeName, String documentTypeAdditionalKey) {
    options.getOptionList(Options.DOCUMENTTYPE).remove(
        new DocumentTypeInfoImpl(documentTypeName, documentTypeAdditionalKey));
    globalViewControl.createDocumentTypeInfoView().update();
  }

  public TransformerInfo newTransformer() {
    return newTransformer(null);
  }

  public TransformerInfo newTransformer(
    TransformerInfo transformerInfoSource) {
    List transformerInfoList =
        options.getOptionList(Options.TRANSFORMER);
    TransformerInfo transformerInfo =
        transformerInfoSource != null ?
        new TransformerInfoImpl(transformerInfoSource) :
        new TransformerInfoImpl();
    TransformerInfoView transformerInfoView =
        globalViewControl.createTransformerInfoView();
    Frame frame = transformerInfoView.getFrame();
    Dialog dialog =
        this.globalViewControl.getDialogTransformerInfo(transformerInfo, false);
    dialog.show();
    if (!dialog.isOK())
      return null;
    boolean isTransformerNameToBeChanged =
        transformerInfoList.contains(transformerInfo);
    if (isTransformerNameToBeChanged)
      while (transformerInfoList.contains(transformerInfo))
        transformerInfo.setTransformerName(
            transformerInfo.getTransformerName() + "x");
    transformerInfoList.add(transformerInfo);
    Collections.sort(transformerInfoList);
    transformerInfoView.update();
    transformerInfoView.select(
        transformerInfo.getTransformerName());
    if (isTransformerNameToBeChanged)
      JOptionPane.showMessageDialog(
          frame,
          getString(Label.TRANSFORMER_NAME_HAS_BEEN_CHANGED),
          getString(Label.TRANSFORMER_ALREADY_DEFINED),
          JOptionPane.OK_OPTION);
    return transformerInfo;
  }

  public void editTransformerInfo(String transformerName) {
    List transformerInfoList =
        options.getOptionList(Options.TRANSFORMER);
    TransformerInfo transformerInfo =
        TransformerInfoImpl.getTransformerInfo(
        transformerInfoList, transformerName);
    if (transformerInfo == null)
      return;
    TransformerInfoView transformerInfoView =
        globalViewControl.createTransformerInfoView();
    Frame frame = transformerInfoView.getFrame();
    Dialog dialog =
        this.globalViewControl.getDialogTransformerInfo(transformerInfo, true);
    dialog.show();
    if (!dialog.isOK())
      return;
    transformerInfoView.update();
    transformerInfoView.select(transformerInfo.getTransformerName());
  }

  public void removeTransformer(String transformerName) {
    options.getOptionList(Options.TRANSFORMER).remove(
        new TransformerInfoImpl(transformerName, "", "", null, ""));
    globalViewControl.createTransformerInfoView().update();
  }
}
