/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.instance;

import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Clipboard;

import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import org.w3c.dom.Text;
import org.w3c.dom.Comment;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.namespace.UndefinedNamespaceException;
import org.xmloperator.framework.document.namespace.NamespacePrefixPair;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.id.IdInfo;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.id.IdManager;
import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.schema.
  NotConformantToContentModelException;
import org.xmloperator.framework.document.schema.UnsatisfiedPatternException;
import org.xmloperator.framework.document.manage.NodeManager;
import org.xmloperator.framework.document.manage.AttributeManager;
import org.xmloperator.framework.document.manage.ElementDefinitionMapper;
import org.xmloperator.framework.document.manage.DocumentUtil;
import org.xmloperator.framework.document.util.PresseFragment;
import org.xmloperator.framework.document.util.VariableNodeList;
import org.xmloperator.framework.document.util.NodeInterval;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.node.TextSelection;
import org.xmloperator.framework.view.node.NodeDetailView;
import org.xmloperator.framework.view.global.GlobalViewAccess;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ToolViewAccess;
import org.xmloperator.framework.view.instance.ViewAccess;
import org.xmloperator.framework.view.dialog.DialogSelection;
import org.xmloperator.framework.view.dialog.DialogListSelection;
import org.xmloperator.framework.operation.Operation;
import org.xmloperator.framework.control.operators.OperatorException;
import org.xmloperator.framework.control.operators.Operator;
import org.xmloperator.framework.control.operators.OperatorInfo;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.NamespaceUtils;
import org.xmloperator.util.dom.FragmentUtils;
import org.xmloperator.util.dom.ComparableNode;
import org.xmloperator.util.dom.ElementIndexer;
import org.xmloperator.util.dom.ElementNamesUtils;
import org.xmloperator.util.view.ExceptionDialogs;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.NameSelection;
import org.xmloperator.option.impl.OperatorInfoImpl;
import org.xmloperator.document.parse.StringParser;
import org.xmloperator.document.manage.AbstractDocumentUtil;
import org.xmloperator.document.manage.DTDDocumentUtil;
import org.xmloperator.document.util.NodeIntervalUtils;
import org.xmloperator.document.util.DocumentWriter;
import org.xmloperator.operation.core.TransitionOperation;
import org.xmloperator.operation.core.NodeSetValueOperation;
import org.xmloperator.operation.core.AttrSetValueOperation;
import org.xmloperator.operation.core.NodesInsertOperation;
import org.xmloperator.operation.core.NodesRemoveOperation;
import org.xmloperator.operation.core.NodesReplaceOperation;
import org.xmloperator.operation.core.NodeIntervalPackOperation;
import org.xmloperator.operation.core.ElementUnpackOperation;
import org.xmloperator.operation.core.NodeIntervalMoveOperation;
import org.xmloperator.operation.core.NodeIntervalReorderOperation;
import org.xmloperator.operation.core.ElementNameReplaceOperation;
import org.xmloperator.operation.core.TextSplitOperation;
import org.xmloperator.operation.core.NextTextJoinOperation;
import org.xmloperator.operation.OperationStack;
import org.xmloperator.operation.OperationLoader;
import org.xmloperator.operation.OperationSaver;

public final class EditControlImpl implements EditControl {
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private FullyFledgedDocument fullyFledgedDocument;
  private Options options;
  private OperationStack operationStack;
  private ViewAccess viewAccess;
  private DocumentViewControl documentViewControl;
  private GlobalViewAccess globalViewAccess;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  public EditControlImpl(final FullyFledgedDocument fullyFledgedDocument,
    final Options options, final OperationStack operationStack,
    final ViewAccess viewAccess,
    final DocumentViewControl documentViewControl,
    final GlobalViewAccess globalViewAccess) {
    if (fullyFledgedDocument == null || options == null
      || operationStack == null || viewAccess == null
      || documentViewControl == null || globalViewAccess == null)
      throw new IllegalArgumentException();
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.operationStack = operationStack;
    this.viewAccess = viewAccess;
    this.documentViewControl = documentViewControl;
    this.globalViewAccess = globalViewAccess;
  }

  private NodeManager getNodeManager() {
    return this.fullyFledgedDocument.getDocumentManager().getNodeManager();
  }

  private AttributeManager getAttributeManager() {
    return this.fullyFledgedDocument.getDocumentManager().getAttributeManager();
  }

  /////////////////////////////
  // EditControl implementation
  /////////////////////////////

  public final void setReadOnly(final boolean isReadOnly) {
    if (isReadOnly == fullyFledgedDocument.isReadOnly())
      return;
    fullyFledgedDocument.setReadOnly(isReadOnly);
    documentViewControl.displaySelectedNode();
  }

  public final void setLazy(final boolean isLazy) {
    if (isLazy == fullyFledgedDocument.isLazy())
      return;
    this.setLazyInternal(isLazy);
    if (!isLazy) {
      Node[] nodes = this.getSelectedNodes();
      for (int k = 0; k < nodes.length; k++) {
        Node node = nodes[k];
        if (node.getNodeType() == Node.ELEMENT_NODE)
          try {
          addChildsAndSiblingsIfNecessary(false, (Element)node);
        }
        catch (UndefinedNamespaceException exception) {
          displayUndefinedNamespaceException(exception);
        }
        catch (IdException exception) {
          displayIdException(exception);
        }
      }
    }
    documentViewControl.displaySelectedNode();
  }

  public final void setXMLVersion(String xMLVersion) {
    clearMessage();
    if (fullyFledgedDocument.getDocument() != null && !isDocumentReadOnly())
      fullyFledgedDocument.setXMLVersion(xMLVersion);
  }

  public final void setEncoding(final String encoding) {
    clearMessage();
    if (fullyFledgedDocument.getDocument() != null && !isDocumentReadOnly())
      fullyFledgedDocument.setEncoding(encoding);
  }

  public final void setStandalone(final boolean isStandalone) {
    clearMessage();
    if (fullyFledgedDocument.getDocument() != null && !isDocumentReadOnly())
      fullyFledgedDocument.setStandalone(isStandalone);
  }

  public final void insertNewNode(final Node relativeNode,
    int insertionType, String elementName) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    Node firstNode;
    Node lastNode;
    {
      Node[] nodes = relativeNode == null ? getSelectedNodes() : null;
      if (relativeNode == null && nodes == null)
        return;
      firstNode = relativeNode != null ? relativeNode : nodes[0];
      lastNode = relativeNode != null ?
                 relativeNode : nodes[nodes.length - 1];
    }
    int initialInsertionType = insertionType;
    if (insertionType == INSERTION_CHILD) {
      if (firstNode != lastNode)
        return;
      if (firstNode.hasChildNodes()) {
        // requalification
        insertionType = INSERTION_NEXT;
        firstNode = firstNode.getLastChild();
        lastNode = firstNode;
      }
    }
    Node parentNode = firstNode.getParentNode();
    if (parentNode == null)
      return;
    Node modelNode = null;
    Node nextSibling = null;
    switch (insertionType) {
      case INSERTION_PREVIOUS: {
        modelNode = firstNode;
        nextSibling = firstNode;
        break;
      }
      case INSERTION_NEXT: {
        modelNode = lastNode;
        nextSibling = lastNode.getNextSibling();
        break;
      }
      case INSERTION_CHILD: {
        parentNode = firstNode;
        break;
      }
      default:
        return;
    }
    try {
      Node newNode = null;
      switch (parentNode.getNodeType()) {
        case Node.DOCUMENT_NODE:
          if (elementName == null)
            newNode = createComment(parentNode);
          break;
        case Node.ELEMENT_NODE: {
          if (elementName != null) {
            elementName = NameSelection.completeName(
                this.globalViewAccess,
                (Element)parentNode, elementName);
            if (elementName == null)
              return;
            newNode = createElement((Element)parentNode, elementName);
            break;
          }
          ElementDefinitionMapper elementDefinitionMapper =
              fullyFledgedDocument.getElementDefinitionMapper();
          DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
          if (elementDefinitionMapper == null || documentUtil == null)
            return;
          Collection elementNameSet = null;
          boolean isLazy = fullyFledgedDocument.isLazy();
          if (insertionType == INSERTION_CHILD) {
            if (!documentUtil.canElementHaveChilds((Element)parentNode))
              break;
            if (documentUtil.canElementHaveText((Element)parentNode)) {
              newNode = createTextNode(parentNode);
              break;
            }
            if (!elementDefinitionMapper.
                isElementContentValid((Element)parentNode)) {
              this.addChildsWhileNecessary(false, (Element)parentNode);
              return;
            }
            elementNameSet =
                documentUtil.elementNamesForAppend(isLazy, (Element)parentNode);
          }
          else {
            int nodeType = modelNode.getNodeType();
            String nodeName = modelNode.getNodeName();
            if (nodeType != Node.ELEMENT_NODE) {
              newNode = createNode(parentNode, nodeType, nodeName);
              break;
            }
            if (isLazy) {
              newNode = createElement((Element)parentNode, nodeName);
              break;
            }
            if (!elementDefinitionMapper.
                isElementContentValid((Element)parentNode)) {
              this.addChildsWhileNecessary(false, (Element)parentNode);
              return;
            }
            elementNameSet = documentUtil.elementNamesForInsert(isLazy,
                (Element)modelNode, insertionType == INSERTION_NEXT);
          }
          if (elementNameSet != null && !elementNameSet.isEmpty()) {
            elementName = selectElementName(
                this.viewResource.getString(insertionType == INSERTION_CHILD ?
                Label.CHILD_ELEMENT_NAME : Label.SIBLING_ELEMENT_NAME),
                (Element)parentNode, elementNameSet,
                modelNode != null ? modelNode.getNodeName() : null, true);
            if (elementName == null)
              return;
          }
          newNode = elementName != null ?
                    (Node)createElement((Element)parentNode, elementName) :
                    (Node)createComment(parentNode);
          break;
        }
      }
      if (newNode == null) {
        displayCannotInsertMessage(initialInsertionType);
        return;
      }
      {
        boolean isNewElement = newNode.getNodeType() == Node.ELEMENT_NODE;
        if (isNewElement && modelNode != null
            && modelNode.getNodeType() == Node.ELEMENT_NODE
            && modelNode.getNodeName().equals(newNode.getNodeName()))
          createSameAttributesThan((Element)modelNode, (Element)newNode);
        boolean isLazy = fullyFledgedDocument.isLazy();
        this.operationStack.executeAndAdd(new NodesInsertOperation(
            this.getNodeManager(), parentNode, nextSibling, newNode));
        if (isNewElement && newNode.getParentNode() == parentNode) {
          this.addRequiredAttributes((Element)newNode);
          if (!isLazy)
            addChildsAndSiblingsIfNecessary(true, (Element)newNode);
        }
      }
    }
    catch (DOMException exception) {
      displayDOMException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  private static final void createSameAttributesThan(Element modelElement,
    Element newElement) throws UndefinedNamespaceException {
    Document document = modelElement.getOwnerDocument();
    NamedNodeMap modelAttributes = modelElement.getAttributes();
    for (int k = 0; k < modelAttributes.getLength(); k++) {
      Attr modelAttr = (Attr)modelAttributes.item(k);
      String attrName = modelAttr.getName();
      if (newElement.getAttributeNode(attrName) == null) {
        String attrNamespaceURI =
          NamespaceUtils.attrQNameToNamespace(modelElement, attrName);
        Attr newAttr = document.createAttributeNS(attrNamespaceURI, attrName);
        newElement.setAttributeNodeNS(newAttr);
      }
    }
  }

  public final void deleteNodeInterval(Node[] nodes) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (nodes == null) {
      nodes = getSelectedNodes();
      if (nodes == null)
        return;
    }
    try {
      if (!areNodesRemovable(nodes)) {
        displayResourceMessage(Label.OPERATION_INTERDITE);
        return;
      }
      mustNotBeReferencedOutside(nodes, nodes);
      this.operationStack.executeAndAdd(new NodesRemoveOperation(
          this.getNodeManager(), nodes));
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void storeSelectedNodesForMove() {
    clearMessage();
    if (isDocumentReadOnly())
      return;
    Node[] nodes = getSelectedNodes();
    if (nodes == null)
      return;
    try {
      if (!areNodesRemovable(nodes)) {
        displayResourceMessage(Label.OPERATION_INTERDITE);
        return;
      }
      NodeInterval nodeIntervalForMove =
          fullyFledgedDocument.getNodeIntervalForMove();
      if (nodeIntervalForMove == null)
        return;
      nodeIntervalForMove.set(nodes[0], nodes[nodes.length - 1]);
      documentViewControl.displaySelectedNode();
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
  }

  public final void clearNodeIntervalForMove() {
    clearMessage();
    NodeInterval nodeIntervalForMove =
        fullyFledgedDocument.getNodeIntervalForMove();
    if (nodeIntervalForMove == null)
      return;
    nodeIntervalForMove.clear();
    documentViewControl.displaySelectedNode();
  }

  public final void moveNodeInterval(Node[] nodes, int moveType) {
    clearMessage();
    if (isDocumentReadOnly())
      return;
    if (nodes == null) {
      nodes = getSelectedNodes();
      if (nodes == null)
        return;
    }
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    if (documentUtil == null)
      return;
    NodeInterval nodeIntervalForMove =
        fullyFledgedDocument.getNodeIntervalForMove();
    boolean isMoveInProgress =
        nodeIntervalForMove != null && nodeIntervalForMove.isDefined();
    try {
      if (!isMoveInProgress && !areNodesMovable(nodes, moveType)) {
        displayResourceMessage(Label.OPERATION_INTERDITE);
        return;
      }
      Node firstNode = nodes[0];
      Node lastNode = nodes[nodes.length - 1];
      Node parentNode = firstNode.getParentNode();
      if (parentNode == null)
        return;
      Node previousSibling = firstNode.getPreviousSibling();
      Node nextSibling = lastNode.getNextSibling();
      Node destinationParentNode;
      Node destinationNextSibling;
      switch (moveType) {
        case MOVE_PREVIOUS: {
          if (!isMoveInProgress && previousSibling == null)
            return;
          destinationParentNode = parentNode;
          destinationNextSibling =
              isMoveInProgress ? firstNode : previousSibling;
          break;
        }
      case MOVE_NEXT: {
        if (!isMoveInProgress && nextSibling == null)
          return;
        destinationParentNode = parentNode;
        destinationNextSibling =
            isMoveInProgress ? nextSibling : nextSibling.getNextSibling();
        break;
      }
    case MOVE_CHILD: {
      if (isMoveInProgress) {
        if (firstNode != lastNode
            || firstNode.getNodeType() != Node.ELEMENT_NODE
            || !documentUtil.canElementHaveChilds((Element)firstNode)) {
          displayResourceMessage(Label.OPERATION_INTERDITE);
          return;
        }
        destinationParentNode = firstNode;
      }
      else {
        if (previousSibling == null
            || previousSibling.getNodeType() != Node.ELEMENT_NODE
            || !documentUtil.canElementHaveChilds((Element)previousSibling)) {
          displayResourceMessage(Label.OPERATION_INTERDITE);
          return;
        }
        destinationParentNode = previousSibling;
      }
      destinationNextSibling = null;
      break;
    }
  case MOVE_PARENT: {
    if (isMoveInProgress)
      return;
    destinationParentNode = parentNode.getParentNode();
    destinationNextSibling = parentNode.getNextSibling();
    if (destinationParentNode == null)
      return;
    if (destinationParentNode.getNodeType() == Node.DOCUMENT_NODE)
      for (int k = 0; k < nodes.length; k++) {
    int nodeType = nodes[k].getNodeType();
    if (nodeType != Node.COMMENT_NODE
        && nodeType != Node.PROCESSING_INSTRUCTION_NODE){
      displayResourceMessage(Label.OPERATION_INTERDITE);
      return;
    }
      }
      break;
  }
default:
  return;
      }
      if (isMoveInProgress) {
        firstNode = nodeIntervalForMove.getFirstNode();
        lastNode = nodeIntervalForMove.getLastNode();
        if (!fullyFledgedDocument.isLazy()
            && destinationParentNode.getNodeType() == Node.ELEMENT_NODE
            && !documentUtil.areElementNamesInsertable(
            ElementNamesUtils.elementNames(
            NodeIntervalUtils.nodesFromInterval(firstNode, lastNode)),
            (Element)destinationParentNode, destinationNextSibling)) {
          displayResourceMessage(Label.OPERATION_INTERDITE);
          return;
        }
        nodeIntervalForMove.clear();
      }
      this.operationStack.executeAndAdd(
          new NodeIntervalMoveOperation(
          this.getNodeManager(), firstNode, lastNode,
          destinationParentNode, destinationNextSibling));
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void reorderNodeInterval(Node[] nodes,
    String attributeNames, final boolean isReverse) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (nodes == null) {
      nodes = getSelectedNodes();
      if (nodes == null)
        return;
    }
    if (nodes.length <= 1)
      return;
    if (!ComparableNode.areNodesElementsWithTheSameName(nodes)) {
      displayResourceMessage(Label.THERE_ARE_NOT_ELEMENTS_WITH_THE_SAME_NAME);
      return;
    }
    if (nodes.length <= 1)
      return;
    if (attributeNames == null || attributeNames.length() == 0) {
      NamedNodeMap attrs = nodes[0].getAttributes();
      if (attrs == null)
        return;
      StringBuffer sb = new StringBuffer();
      for (int k = 0; k < attrs.getLength(); k++) {
        if (k > 0)
          sb.append(' ');
        sb.append(attrs.item(k).getNodeName());
      }
      attributeNames = sb.toString();
      if (attributeNames.length() == 0)
        return;
    }
    ComparableNode.reorderElements(nodes, attributeNames, isReverse);
    try {
      this.operationStack.executeAndAdd(new NodeIntervalReorderOperation(
          this.getNodeManager(), nodes));
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void packNodeInterval(Node[] nodes, String elementName) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (nodes == null) {
      nodes = getSelectedNodes();
      if (nodes == null)
        return;
    }
    Node firstNode = nodes[0];
    Node lastNode = nodes[nodes.length - 1];
    Node parentNode = firstNode.getParentNode();
    if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE) {
      displayResourceMessage(Label.OPERATION_INTERDITE);
      return;
    }
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    if (documentUtil == null)
      return;
    if (elementName == null) {
      Collection elementNameSet =
          documentUtil.elementNamesForPack(fullyFledgedDocument.isLazy(),
          nodes);
      if (elementNameSet != null && !elementNameSet.isEmpty()) {
        elementName =
            selectElementName(viewResource.getString(Label.PACK_ELEMENT_NAME),
            (Element)parentNode, elementNameSet, null, true);
        if (elementName == null)
          return;
      }
    }
    if (elementName == null) {
      displayResourceMessage(Label.OPERATION_INTERDITE);
      return;
    }
    try {
      if (firstNode == lastNode && firstNode.getNodeType() == Node.TEXT_NODE) {
        TextSelection textSelection =
            viewAccess.getToolView().getNodeDetailView().getTextSelection();
        int p0 = textSelection != null ? textSelection.getCaretPosition() : -1;
        int p1 = textSelection != null ? textSelection.getMarkPosition() : -1;
        int length = firstNode.getNodeValue().length();
        if (p0 != p1) {
          if (p0 > p1) {
            int p = p0;
            p0 = p1;
            p1 = p;
          }
          if (p1 > 0 && p1 < length) {
            this.operationStack.executeAndAdd(new TextSplitOperation(
                this.getNodeManager(), (Text)firstNode, p1));
            this.operationStack.executeAndAdd(new TransitionOperation());
          }
          if (p0 > 0) {
            this.operationStack.executeAndAdd(
                new TextSplitOperation(
                this.getNodeManager(), (Text)firstNode, p0));
            this.operationStack.executeAndAdd(new TransitionOperation());
            firstNode = firstNode.getNextSibling();
          }
          lastNode = firstNode;
        }
      }
      {
        Element newElement = createElement((Element)parentNode, elementName);
        this.operationStack.executeAndAdd(
            new NodeIntervalPackOperation(
            this.getNodeManager(), firstNode, lastNode, newElement));
        if (newElement.getParentNode() == parentNode)
          this.addRequiredAttributes(newElement);
      }
    }
    catch (DOMException exception) {
      displayDOMException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void unpackElement(Element element) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (element == null) {
      Node selectedNode = getSelectedNode();
      if (selectedNode == null
          || selectedNode.getNodeType() != Node.ELEMENT_NODE)
        return;
      element = (Element)selectedNode;
    }
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    if (documentUtil == null)
      return;
    boolean isLazy = fullyFledgedDocument.isLazy();
    Node parentNode = element.getParentNode();
    try{
      if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE
          || (!isLazy && documentUtil != null
          && !documentUtil.isElementUnpackable(element))) {
        displayResourceMessage(Label.OPERATION_INTERDITE);
        return;
      }
      String id = documentUtil.getId(element);
      if (id != null && fullyFledgedDocument.getIdManager().isIdReferenced(id))
        throw new IdException(IdException.CANNOT_REMOVE_REFERENCED_ID, id);
      this.operationStack.executeAndAdd(
          element.hasChildNodes() ?
          (Operation)new ElementUnpackOperation(
          this.getNodeManager(), element) :
          (Operation)new NodesRemoveOperation(
          this.getNodeManager(), element));
    }
    catch (DOMException exception) {
      displayDOMException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void cutNodes(
      final Node[] nodes, final boolean copyToSystemClipoard) {
    cutOrCopyNodes(nodes, true, copyToSystemClipoard);
  }

  public final void copyNodes(
      final Node[] nodes, final boolean copyToSystemClipoard) {
    cutOrCopyNodes(nodes, false, copyToSystemClipoard);
  }

  private void cutOrCopyNodes(
      Node[] nodes, final boolean cut, final boolean copyToSystemClipoard) {
    clearMessage();
    if (cut && (isDocumentReadOnly() || isMoveInProgress()))
      return;
    if (nodes == null)
      nodes = getSelectedNodes();
    if (nodes == null)
      return;
    try {
      if (cut && !areNodesRemovable(nodes)) {
        displayResourceMessage(Label.OPERATION_INTERDITE);
        return;
      }
      fullyFledgedDocument.getPresseFragment().setByCloningNodes(nodes);
      if (cut)
        deleteNodeInterval(nodes);
      if (copyToSystemClipoard)
        this.copyDocumentFragmentToSystemClipboard();
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void pasteDocumentFragment(
      final Node relativeNode,
      int insertionType,
      final boolean copyFromSystemClipboard) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    Node[] nodes = relativeNode == null ? getSelectedNodes() : null;
    if (relativeNode == null && nodes == null)
      return;
    Node firstNode = relativeNode != null ?
                     relativeNode : nodes[0];
    Node lastNode = relativeNode != null ?
                    relativeNode : nodes[nodes.length - 1];
    int initialInsertionType = insertionType;
    if (insertionType == INSERTION_CHILD) {
      if (firstNode != lastNode)
        return;
      if (firstNode.hasChildNodes()) {
        // requalification
        insertionType = INSERTION_NEXT;
        firstNode = firstNode.getLastChild();
        lastNode = firstNode;
      }
    }
    Node parentNode = firstNode.getParentNode();
    if (parentNode == null)
      return;
    if (copyFromSystemClipboard)
      this.copySystemClipboardToDocumentFragment();
    PresseFragment presseFragment = fullyFledgedDocument.getPresseFragment();
    if (!presseFragment.hasNodes()) {
      displayResourceMessage(Label.CLIPBOARD_IS_EMPTY);
      return;
    }
    if (parentNode.getNodeType() == Node.DOCUMENT_NODE
        && !presseFragment.isPastableAsDocumentChild()) {
      displayResourceMessage(Label.COLLAGE_INTERDIT);
      return;
    }
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    if (documentUtil == null)
      return;
    Node nextSibling;
    switch (insertionType) {
      case INSERTION_PREVIOUS: {
        nextSibling = firstNode;
        break;
      }
      case INSERTION_NEXT: {
        nextSibling = lastNode.getNextSibling();
        break;
      }
      case INSERTION_CHILD: {
        if ((relativeNode == null && nodes.length != 1)
            || firstNode.getNodeType() != Node.ELEMENT_NODE)
        return;
        if (!documentUtil.canElementHaveChilds((Element)firstNode)) {
          displayResourceMessage(Label.NE_PEUT_AVOIR_DE_DESCENDANT);
          return;
        }
        parentNode = firstNode;
        nextSibling = null;
        break;
      }
      default:
        return;
    }
    DocumentFragment fragment =
        (DocumentFragment)presseFragment.getFragment().cloneNode(true);
    try {
      if (!fullyFledgedDocument.isLazy()
          && parentNode.getNodeType() == Node.ELEMENT_NODE
          && !documentUtil.areElementNamesInsertable(
          ElementNamesUtils.elementNames(fragment),
          (Element)parentNode, nextSibling)) {
        displayCannotInsertMessage(initialInsertionType);
        return;
      }
      Node[] newNodes = FragmentUtils.removeNodesFromFragment(fragment);
      this.operationStack.executeAndAdd(new NodesInsertOperation(
          this.getNodeManager(), parentNode, nextSibling, newNodes));
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public void replaceNodesByDocumentFragment(
      Node[] nodes, boolean copyFromSystemClipboard) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (nodes == null) {
      nodes = getSelectedNodes();
      if (nodes == null)
        return;
    }
    try {
      if (!areNodesRemovable(nodes)) {
        displayResourceMessage(Label.OPERATION_INTERDITE);
        return;
      }
      mustNotBeReferencedOutside(nodes, nodes);
      if (copyFromSystemClipboard)
        this.copySystemClipboardToDocumentFragment();
      PresseFragment presseFragment = fullyFledgedDocument.getPresseFragment();
      if (!presseFragment.hasNodes()) {
        displayResourceMessage(Label.CLIPBOARD_IS_EMPTY);
        return;
      }
      Node parentNode = nodes[0].getParentNode();
      if (parentNode == null || (parentNode.getNodeType() == Node.DOCUMENT_NODE
          && !presseFragment.isPastableAsDocumentChild())) {
        displayResourceMessage(Label.OPERATION_INTERDITE);
        return;
      }
      DocumentFragment fragment =
          (DocumentFragment)presseFragment.getFragment().cloneNode(true);
      Node[] newNodes = FragmentUtils.removeNodesFromFragment(fragment);
      this.operationStack.executeAndAdd(
          new NodesReplaceOperation(
          this.getNodeManager(), nodes, newNodes));
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void clearDocumentFragment() {
    PresseFragment presseFragment = fullyFledgedDocument.getPresseFragment();
    if (presseFragment != null)
      presseFragment.clear();
  }

  public void copyDocumentFragmentToSystemClipboard() {
    PresseFragment presseFragment = fullyFledgedDocument.getPresseFragment();
    if (presseFragment == null)
      return;
    Clipboard clipboard = viewAccess.getToolView().getSystemClipboard();
    if (clipboard == null)
      return;
    String text = DocumentWriter.toString(presseFragment.getFragment());
    clipboard.setContents(new StringSelection(text), new ClipboardOwner() {
      public void lostOwnership(Clipboard clipboard, Transferable contents) {
      }
    });
  }

  public void copySystemClipboardToDocumentFragment() {
    PresseFragment presseFragment = fullyFledgedDocument.getPresseFragment();
    if (presseFragment == null)
      return;
    presseFragment.clear();
    Clipboard clipboard = viewAccess.getToolView().getSystemClipboard();
    if (clipboard == null)
      return;
    Transferable content = clipboard.getContents(this);
    if (content == null)
      return;
    try {
      String text = (String)(content.getTransferData(DataFlavor.stringFlavor));
      text = "<?xml version=\"1.0\" standalone=\"yes\"?><d>"
           + text + "</d>";
      Element rootElement = StringParser.constructElement(text);
      if (rootElement == null)
        return;
      presseFragment.setByImportingFragment(rootElement);
    }
    catch (UnsupportedFlavorException exception) {
    }
    catch (IOException exception) {
    }
  }

  public final void setNodeValue(Node node, final String newValue) {
    if (node == null || newValue == null)
      throw new IllegalArgumentException();
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    try {
      this.operationStack.executeAndAdd(
          node.getNodeType() == Node.ATTRIBUTE_NODE ?
          (Operation)new AttrSetValueOperation(
          this.getAttributeManager(),
          ((Attr)node).getOwnerElement(), ((Attr)node).getName(), newValue) :
          (Operation)new NodeSetValueOperation(
          this.getNodeManager(), node, newValue));
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void addNewAttribute(Element element, String attributeName) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (element == null) {
      Node selectedNode = getSelectedNode();
      if (selectedNode == null
          || selectedNode.getNodeType() != Node.ELEMENT_NODE)
        return;
      element = (Element)selectedNode;
    }
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    AttributesContent attributesContent =
        documentUtil.getAttributesContent(element);
    if (attributeName == null)
      attributeName = selectUnusedAttributeName(element, attributesContent);
    if (attributeName == null)
      return;
    attributeName = NameSelection.completeName(
        this.globalViewAccess, element, attributeName);
    if (attributeName == null)
      return;
    if (element.getAttributeNode(attributeName) != null) {
      displayResourceMessage(Label.ALREADY_EXISTS);
      return;
    }
    try {
      this.operationStack.executeAndAdd(new AttrSetValueOperation(
          this.getAttributeManager(), element, attributeName, ""));
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  private String selectUnusedAttributeName(Element element,
    AttributesContent attributesContent) {
    final String symbolAnyPrefix = "***";
    final String symbolAnyLocalName = "*****";
    final String symbolAnyName = symbolAnyPrefix + ":" + symbolAnyLocalName;
    List list = new ArrayList();
    boolean isAnyNameEnabled = false;
    if (attributesContent != null)
      for (int k = 0; k < attributesContent.getPossibleNameCount(); k++) {
        String possibleName = attributesContent.getPossibleName(k);
        boolean isGenericName =
          possibleName.length() == 0 || possibleName.endsWith(":");
        if (isGenericName || element.getAttributeNode(possibleName) == null) {
          if (isGenericName)
            if (possibleName.equals(":")) {
              possibleName = symbolAnyName;
              isAnyNameEnabled = true;
            }
            else
              possibleName += symbolAnyLocalName;
          list.add(possibleName);
        }
      }
    if (attributesContent == null
      || (this.fullyFledgedDocument.isLazy() && !isAnyNameEnabled))
      list.add(symbolAnyName);
    if (element.getAttributeNode("xmlns") == null)
      list.add("xmlns");
    list.add("xmlns:" + symbolAnyLocalName);
    DialogListSelection dialog = this.globalViewAccess.getDialogSelectionName();
    dialog.setTitle(Label.NEW_ATTRIBUTE);
    dialog.setItemList(list);
    dialog.show();
    if (!dialog.isOK())
      return null;
    String selectedName = (String)dialog.getSelectedItem();
    if (selectedName.indexOf('(') >= 0)
      return null;
    if (selectedName.endsWith(symbolAnyLocalName))
      if (selectedName.equals(symbolAnyName))
        selectedName = ":";
      else
        selectedName = selectedName.substring(0,
          selectedName.length() - symbolAnyLocalName.length());
    return selectedName;
  }

  public final void setAttributeValue(Element element,
    String attributeName, final String newAttributeValue) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (element == null) {
      Node selectedNode = getSelectedNode();
      if (selectedNode == null
          || selectedNode.getNodeType() != Node.ELEMENT_NODE)
        return;
      element = (Element)selectedNode;
    }
    if (attributeName == null) {
      attributeName = getSelectedAttributeName();
      if (attributeName == null)
        return;
    }
    if (newAttributeValue == null && !fullyFledgedDocument.isLazy()) {
      AttributesContent attributesContent =
          fullyFledgedDocument.getElementDefinitionMapper().
          getAttributesContent(element);
      if (attributesContent != null
          && attributesContent.isRequired(attributeName)) {
        displayResourceMessage(Label.THIS_ATTRIBUTE_IS_REQUIRED);
        return;
      }
    }
    try {
      this.operationStack.executeAndAdd(new AttrSetValueOperation(
          this.getAttributeManager(), element, attributeName,
          newAttributeValue));
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void addNamespace(NamespacePrefixPair namespacePrefixPair) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (namespacePrefixPair == null) {
      List namespacePrefixPairList = options.getOptionList(Options.NAMESPACE);
      if (namespacePrefixPairList.size() == 0)
        return;
      DialogListSelection dialog =
          this.globalViewAccess.getDialogSelectionName();
      dialog.setTitle(Label.ADD_NAMESPACE);
      dialog.setItemList(namespacePrefixPairList);
      dialog.show();
      if (!dialog.isOK())
        return;
      namespacePrefixPair = (NamespacePrefixPair)dialog.getSelectedItem();
    }
    Element element = fullyFledgedDocument.getDocument().getDocumentElement();
    String attributeName = "xmlns:" + namespacePrefixPair.getPrefix();
    if (element.getAttributeNode(attributeName) != null)
      return;
    try {
      this.operationStack.executeAndAdd(new AttrSetValueOperation(
          this.getAttributeManager(), element, attributeName,
          namespacePrefixPair.getNamespaceURI()));
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void replaceNode(Node node, final int nodeType, String nodeName)
    {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (node == null) {
      node = getSelectedNode();
      if (node == null)
        return;
    }
    Node parentNode = node.getParentNode();
    if (parentNode == null)
      return;
    if (nodeType == Node.ELEMENT_NODE
        && parentNode.getNodeType() != Node.ELEMENT_NODE)
      return;
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    if (documentUtil == null)
      return;
    if (nodeName == null) {
      switch (nodeType) {
        case Node.ELEMENT_NODE:
          Collection elementNameSet = documentUtil.elementNamesForReplace(
              fullyFledgedDocument.isLazy(), node);
          if (elementNameSet == null || elementNameSet.isEmpty()) {
            displayResourceMessage(Label.UN_ELEMENT_NE_PEUT_PAS_VENIR_LA);
            return;
          }
          /*
          {
            String currentName = node.getNodeName();
            if (!elementNameSet.contains(currentName))
              elementNameSet.add(currentName);
          }
          */
          nodeName = selectElementName(
              viewResource.getString(Label.REPLACEMENT_ELEMENT_NAME),
              (Element)parentNode, elementNameSet, node.getNodeName(), false);
          if (nodeName == null)
            return;
          break;
        case Node.ENTITY_REFERENCE_NODE: {
          String[] entityNames =
              DTDDocumentUtil.entityReferenceNames(fullyFledgedDocument.getDTD());
          if (entityNames != null && entityNames.length != 0)
            if (entityNames.length == 1)
              nodeName = entityNames[0];
            else {
              nodeName = NameSelection.selectName(
                  this.globalViewAccess,
                  viewResource.getString(Label.ENTITY_NAME), entityNames);
              if (nodeName == null)
                return;
            }
          if (nodeName == null) {
            displayResourceMessage(Label.IL_N_EST_PAS_DEFINI_D_ENTITE);
            return;
          }
          break;
        }
        case Node.PROCESSING_INSTRUCTION_NODE:
          nodeName = Constants.TOOL;
          break;
      }
    }
    if (nodeType == node.getNodeType()
        && (nodeName == null || nodeName.equals(node.getNodeName())))
      return;
    boolean isElementNameReplaced = nodeType == Node.ELEMENT_NODE
                                  && node.getNodeType() == Node.ELEMENT_NODE;
    try {
      if (isElementNameReplaced) {
        nodeName = NameSelection.completeName(
            this.globalViewAccess, (Element)parentNode, nodeName);
        if (nodeName == null)
          return;
        isElementNameReplaced =
            fullyFledgedDocument.isLazy()
            || documentUtil.isElementNameReplacable((Element)node, nodeName);
      }
      if (node.getNodeType() == Node.ELEMENT_NODE) {
        if (nodeType != Node.ELEMENT_NODE
        && !areNodesRemovable(new Node[] {node})) {
          displayResourceMessage(Label.OPERATION_INTERDITE);
          return;
        }
        String id = documentUtil.getId((Element)node);
        if (id != null
            && fullyFledgedDocument.getIdManager().isIdReferenced(id))
          throw new IdException(IdException.CANNOT_REMOVE_REFERENCED_ID, id);
      }
      Node newNode = createNode(parentNode, nodeType, nodeName);
      if (newNode == null)
        return;
      boolean isLazy = fullyFledgedDocument.isLazy();
      this.operationStack.executeAndAdd(
          isElementNameReplaced ?
          (Operation)new ElementNameReplaceOperation(
          this.getNodeManager(), (Element)node, (Element)newNode) :
          (Operation)new NodesReplaceOperation(
          this.getNodeManager(), node, newNode));
      if (newNode.getNodeType() == Node.ELEMENT_NODE
          && newNode.getParentNode() == parentNode) {
        this.addRequiredAttributes((Element)newNode);
        if (!ElementIndexer.hasSignificantChilds((Element)newNode)
            && !isLazy)
          addChildsAndSiblingsIfNecessary(true, (Element)newNode);
      }
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (DOMException exception) {
      displayDOMException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void joinText(Text text, final boolean withNextSibling) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (text == null) {
      Node selectedNode = getSelectedNode();
      if (selectedNode == null || selectedNode.getNodeType() != Node.TEXT_NODE)
        return;
      text = (Text)selectedNode;
    }
    Node firstNode, secondNode;
    if (withNextSibling) {
      firstNode = text;
      secondNode = text.getNextSibling();
    }
    else {
      firstNode = text.getPreviousSibling();
      secondNode = text;
    }
    if (firstNode == null || secondNode ==null
        || firstNode.getNodeType() != Node.TEXT_NODE
        || secondNode.getNodeType() != Node.TEXT_NODE)
      return;
    try {
      this.operationStack.executeAndAdd(new NextTextJoinOperation(
          this.getNodeManager(), (Text)firstNode));
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void splitText(Text text, int offset) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (text == null) {
      Node selectedNode = getSelectedNode();
      if (selectedNode == null
          || selectedNode.getNodeType() != Node.TEXT_NODE)
        return;
      text = (Text)selectedNode;
      TextSelection textSelection  =
          viewAccess.getToolView().getNodeDetailView().getTextSelection();
      offset = textSelection != null ? textSelection.getCaretPosition() : -1;
    }
    String textValue = text.getNodeValue();
    if (offset < 0 || offset > textValue.length())
      return;
    Node parentNode = text.getParentNode();
    try {
      this.operationStack.executeAndAdd(
          offset == textValue.length() ?
          (Operation)new NodesInsertOperation(
          this.getNodeManager(), parentNode, text.getNextSibling(),
          createTextNode(parentNode)) :
          (Operation)new TextSplitOperation(
          this.getNodeManager(), text, offset));
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void operateElement(Element element, final String operatorName) {
    org.xmloperator.control.operators.DocumentImpl operateDocument = null;
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (element == null) {
      Node selectedNode = getSelectedNode();
      if (selectedNode == null
          || selectedNode.getNodeType() != Node.ELEMENT_NODE)
        return;
      element = (Element)selectedNode;
    }
    List operatorInfoList = OperatorInfoImpl.createOperatorInfoSubList(
        options.getOptionList(Options.OPERATOR),
        fullyFledgedDocument.getDocument().getDocumentElement().getNodeName());
    if (operatorInfoList.isEmpty()) {
      displayResourceMessage(Label.NO_OPERATOR_DEFINED);
      return;
    }
    Collections.sort(operatorInfoList);
    OperatorInfo operatorInfo =
        operatorName != null ?
        OperatorInfoImpl.getOperatorInfo(operatorInfoList, operatorName) : null;
    if (operatorInfo == null) {
      DialogSelection dialog =
          this.globalViewAccess.getDialogSelectionOperator();
      dialog.setItems(operatorInfoList.toArray());
      dialog.show();
      if (!dialog.isOK())
        return;
      operatorInfo = (OperatorInfo)dialog.getSelectedItem();
    }
    String operatorClassName = operatorInfo.getOperatorClassName();
    if (operatorClassName == null || operatorClassName.length() == 0) {
      displayResourceMessage(Label.NO_OPERATOR_CLASS_DEFINED);
      return;
    }
    Operator operator_ = null;
    try {
      operator_ = (Operator)Class.forName(operatorClassName).newInstance();
    }
    catch (Exception exception) {
      ExceptionDialogs.displayException(exception);
      return;
    }
    try {
      operateDocument =
          new org.xmloperator.control.operators.DocumentImpl(
          fullyFledgedDocument.getIdManager(),
          this.getNodeManager(),
          this.getAttributeManager(),
          (ToolViewAccess)viewAccess, operationStack,
          fullyFledgedDocument.getDocument());
      org.xmloperator.framework.control.operators.Element operateElement =
          operateDocument.getElement(element);
      operator_.operateElement(operateElement);
      documentViewControl.controlSelectedNodeSchemaConformance();
    }
    catch (OperatorException exception) {
      if (operateDocument.useTransitionOperation())
        try {
        operationStack.undo(true);
      }
      catch (Exception secondException) {
        ExceptionControler.printStackTrace(secondException);
      }
      org.xmloperator.control.operators.ElementImpl returnedElement =
          (org.xmloperator.control.operators.ElementImpl)exception.getElement();
      displayException(
          "Operator exception", exception,
          returnedElement != null ? returnedElement.getDomElement() : null);
    }
    catch (IdException exception) {
      if (operateDocument.useTransitionOperation())
        try {
        operationStack.undo(true);
      }
      catch (Exception secondException) {
        ExceptionControler.printStackTrace(secondException);
      }
      displayIdException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      if (operateDocument.useTransitionOperation())
        try {
        operationStack.undo(true);
      }
      catch (Exception secondException) {
        ExceptionControler.printStackTrace(secondException);
      }
      displayUndefinedNamespaceException(exception);
    }
  }

  public final void operateElement(final Element element,
    final int operatorIndex) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    Document document = fullyFledgedDocument.getDocument();
    if (document == null)
      return;
    Element documentElement = document.getDocumentElement();
    if (documentElement == null)
      return;
    List operatorInfoList = OperatorInfoImpl.createOperatorInfoSubList(
        options.getOptionList(Options.OPERATOR), documentElement.getNodeName());
    if (operatorIndex < 0 || operatorIndex >= operatorInfoList.size())
      return;
    operateElement(
        element,
        ((OperatorInfo)operatorInfoList.get(operatorIndex)).getOperatorName());
  }

  public final void addChildsIfNecessary(Element element) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    if (element == null) {
      Node selectedNode = getSelectedNode();
      if (selectedNode == null
          || selectedNode.getNodeType() != Node.ELEMENT_NODE)
        return;
      element = (Element)selectedNode;
    }
    try {
      addChildsWhileNecessary(false, element);
    }
    catch (DOMException exception) {
      displayDOMException(exception);
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
  }

  public final void undoOperation() {
    clearMessage();
    if (operationStack.getPossibleUndoCount() == 0)
      return;
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    boolean isLazy = fullyFledgedDocument.isLazy();
    try {
      operationStack.undo(!isLazy);
      if (!isLazy) {
        fullyFledgedDocument.setLazy(false);
        viewAccess.getToolView().getToolStatusBar().displayLazy(false);
      }
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void undoAllOperations() {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    boolean isLazy = fullyFledgedDocument.isLazy();
    try {
      operationStack.undoAllOperations();
      if (!isLazy) {
        fullyFledgedDocument.setLazy(false);
        viewAccess.getToolView().getToolStatusBar().displayLazy(false);
      }
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void redoOperation() {
    clearMessage();
    if (operationStack.getPossibleRedoCount() == 0)
      return;
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    boolean isLazy = fullyFledgedDocument.isLazy();
    try {
      operationStack.redo(!isLazy);
      if (!isLazy) {
        fullyFledgedDocument.setLazy(false);
        viewAccess.getToolView().getToolStatusBar().displayLazy(false);
      }
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void redoAllOperations() {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    boolean isLazy = fullyFledgedDocument.isLazy();
    try {
      operationStack.redoAllOperations();
      if (!isLazy) {
        fullyFledgedDocument.setLazy(false);
        viewAccess.getToolView().getToolStatusBar().displayLazy(false);
      }
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void clearUndoOperations() {
    clearMessage();
    operationStack.clearUndoOperations();
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void clearRedoOperations() {
    clearMessage();
    operationStack.clearRedoOperations();
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void loadOperationStack(final String pathName) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    OperationLoader.loadAndExecuteOperations(
        fullyFledgedDocument.getDocument(),
        this.getNodeManager(), this.getAttributeManager(),
        pathName, operationStack);
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  public final void saveOperationStack(final String pathName) {
    clearMessage();
    if (isDocumentReadOnly() || isMoveInProgress())
      return;
    operationStack.clearRedoOperations();
    try {
      operationStack.undoAllOperations();
      try {
        OperationSaver.saveOperationStack(operationStack, pathName);
      }
      catch (IOException exception) {
        ExceptionDialogs.displayException(exception);
      }
    }
    catch (UndefinedNamespaceException exception) {
      displayUndefinedNamespaceException(exception);
    }
    catch (IdException exception) {
      displayIdException(exception);
    }
    documentViewControl.controlSelectedNodeSchemaConformance();
  }

  /////////////////////////////////////
  // Methods for authorizing operations
  /////////////////////////////////////

  private boolean isNodeTypeInstanciable(final int nodeType) {
    String optionId = null;
    switch (nodeType) {
      case Node.COMMENT_NODE:
        optionId = Options.ALLOWSCOMMENT;
        break;
      case Node.PROCESSING_INSTRUCTION_NODE:
        optionId = Options.ALLOWSPROCESSINGINSTRUCTION;
        break;
      case Node.CDATA_SECTION_NODE:
        optionId = Options.ALLOWSCDATASECTION;
        break;
      case Node.ENTITY_REFERENCE_NODE:
        optionId = Options.ALLOWSENTITYREFERENCE;
        break;
    }
    return optionId == null
      || options.getThreeStateOptionValue(optionId) == ThreeStateOption.TRUE;
  }

  private boolean areNodesRemovable(final Node[] nodes)
    throws UndefinedNamespaceException {
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    return AbstractDocumentUtil.areNodesMovable(nodes)
      && (fullyFledgedDocument.isLazy() || documentUtil == null
      || documentUtil.areNodesRemovable(nodes));
  }

  private boolean areNodesMovable(final Node[] nodes, final int moveType)
    throws UndefinedNamespaceException {
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    return AbstractDocumentUtil.areNodesMovable(nodes)
      && (fullyFledgedDocument.isLazy() || documentUtil == null
      || documentUtil.areNodesMovable(nodes, moveType));
  }

  /////////////////////////////
  // Methods for creating nodes
  /////////////////////////////

  private Element createElement(final Element contextElement,
    final String elementName) throws UndefinedNamespaceException {
    return (Element)createNode(contextElement, Node.ELEMENT_NODE, elementName);
  }

  private Text createTextNode(final Node contextNode) {
    try {
      return (Text)createNode(contextNode, Node.TEXT_NODE, null);
    }
    catch (UndefinedNamespaceException exception) {
      return null;
    }
  }

  private Comment createComment(final Node contextNode) {
    try {
      return (Comment)createNode(contextNode, Node.COMMENT_NODE, null);
    }
    catch (UndefinedNamespaceException exception) {
      return null;
    }
  }

  private Node createNode(final Node contextNode,
    int nodeType, String nodeName) throws UndefinedNamespaceException {
    if (!isNodeTypeInstanciable(nodeType))
      return null;
    Document document = contextNode.getOwnerDocument();
    if (document == null)
      if (contextNode.getNodeType() == Node.DOCUMENT_NODE)
        document = (Document)contextNode;
      else
        throw new IllegalArgumentException();
    switch (nodeType) {
      case Node.ELEMENT_NODE: {
        if (nodeName == null || contextNode.getNodeType() != Node.ELEMENT_NODE)
          throw new IllegalArgumentException();
        String prefix = NamespaceUtils.prefixFromQName(nodeName);
        String namespaceURI = prefix.length() != 0 ?
          NamespaceUtils.prefixToNamespace((Element)contextNode, prefix) :
          NamespaceUtils.defaultNamespace((Element)contextNode);
        return document.createElementNS(namespaceURI, nodeName);
      }
      case Node.TEXT_NODE:
        return document.createTextNode("");
      case Node.CDATA_SECTION_NODE:
        return document.createCDATASection("");
      case Node.ENTITY_REFERENCE_NODE:
        if (nodeName == null)
          throw new IllegalArgumentException();
        return document.createEntityReference(nodeName);
      case Node.PROCESSING_INSTRUCTION_NODE:
        if (nodeName == null)
          throw new IllegalArgumentException();
        return document.createProcessingInstruction(nodeName, "");
      case Node.COMMENT_NODE:
        return document.createComment("");
      default:
        throw new IllegalArgumentException();
    }
  }

  ///////////////////////////////////////////////////////////////////////
  // Methods for completing automatically the document against its schema
  ///////////////////////////////////////////////////////////////////////

  private void addRequiredAttributes(final Node fragment)
    throws IdException, UndefinedNamespaceException {
    if (fragment.getNodeType() == Node.ELEMENT_NODE)
      addRequiredAttributes((Element)fragment);
    {
      Node childNode = fragment.getFirstChild();
      while (childNode != null) {
        addRequiredAttributes(childNode);
        childNode = childNode.getNextSibling();
      }
    }
  }

  private void addRequiredAttributes(final Element element)
    throws IdException, UndefinedNamespaceException {
    AttributesContent attributesContent =
      fullyFledgedDocument.getElementDefinitionMapper().
      getAttributesContent(element);
    if (attributesContent == null)
      return;
    String idAttrName = attributesContent.getIdAttributeName();
    IdManagerRead idManager = fullyFledgedDocument.getIdManager();
    for (int k = 0; k < attributesContent.getPossibleNameCount(); k++) {
      String attrName = attributesContent.getPossibleName(k);
      if (element.getAttributeNode(attrName) == null
        && attributesContent.isRequired(attrName)) {
        String attrValue = attrName.equals(idAttrName) ?
          idManager.getNewVerifiedId() : "";
        this.operationStack.executeAndAdd(new TransitionOperation());
        this.operationStack.executeAndAdd(new AttrSetValueOperation(
          this.getAttributeManager(), element, attrName, attrValue));
      }
    }
  }

  // Return true if an operation sequence is started
  private boolean addChildsAndSiblingsIfNecessary(
    boolean isOperationSequenceStarted, final Element element)
    throws IdException, UndefinedNamespaceException {
    Node parentNode = element.getParentNode();
    if (parentNode.getNodeType() == Node.ELEMENT_NODE)
      isOperationSequenceStarted = addChildsWhileNecessary(
        isOperationSequenceStarted, (Element)parentNode);
    isOperationSequenceStarted = addChildsWhileNecessary(
      isOperationSequenceStarted, element);
    this.setLazyInternal(false);
    return isOperationSequenceStarted;
  }

  // Return true if an operation sequence is started
  private boolean addChildsWhileNecessary(boolean isOperationSequenceStarted,
    final Element parentElement)
    throws IdException, UndefinedNamespaceException {
    {
      DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
      if (documentUtil != null
        && documentUtil.shouldElementHaveText(parentElement)
        && !parentElement.hasChildNodes()) {
        if (isOperationSequenceStarted)
          this.operationStack.executeAndAdd(new TransitionOperation());
        this.operationStack.executeAndAdd(new NodesInsertOperation(
            this.getNodeManager(), parentElement, null,
            this.createTextNode(parentElement)));
        return true;
      }
    }
    this.setLazyInternal(false);
    {
      List newElementList = new ArrayList();
      isOperationSequenceStarted =
        addChildElementOrAttributeWhileNecessary(isOperationSequenceStarted,
        parentElement, newElementList);
      for (int k = 0; k < newElementList.size(); k++)
        addChildsWhileNecessary(true, (Element)newElementList.get(k));
    }
    return isOperationSequenceStarted;
  }

  // Return true if an operation sequence is started
  private boolean addChildElementOrAttributeWhileNecessary(
    boolean isOperationSequenceStarted,
    final Element parentElement, final List newElementList)
    throws IdException, UndefinedNamespaceException {
    ElementDefinitionMapper elementDefinitionMapper =
      fullyFledgedDocument.getElementDefinitionMapper();
    if (elementDefinitionMapper == null)
      return isOperationSequenceStarted;
    while (true)
      try {
        elementDefinitionMapper.getElementContent(parentElement);
        break;
      }
      catch (UnsatisfiedPatternException patternException) {
        {
          ToolView toolView = viewAccess.getToolView();
          if (toolView != null)
            toolView.setSelectedNode(parentElement, true);
        }
        Node nextSibling = null;
        {
          int contentElementSpecIndex =
              patternException.contentElementSpecIndex;
          if (contentElementSpecIndex >= 0) {
            try {
              nextSibling = ElementIndexer.getSignificantChild(
                parentElement, contentElementSpecIndex);
            }
            catch (
              ElementIndexer.NodeInsideEntityException entityException) {
              break;
            }
          }
        }
        String elementOrAttributeName;
        {
          DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
          if (documentUtil == null)
            break;
          Collection elementAndAttributeNameSet =
            documentUtil.elementAndAttributeNamesForRequiredInsert(
            parentElement, patternException.getRequiredPatterns());
          if (elementAndAttributeNameSet.isEmpty())
            break;
          String[] elementAndAttributeNames =
            new String[elementAndAttributeNameSet.size()];
          elementAndAttributeNameSet.toArray(elementAndAttributeNames);
          elementOrAttributeName = elementAndAttributeNames.length == 1 ?
              elementAndAttributeNames[0] :
              NameSelection.selectName(
              this.globalViewAccess,
              viewResource.getString(Label.CHILD_ELEMENT_OR_ATTRIBUTE_NAME),
              elementAndAttributeNames);
        }
        if (elementOrAttributeName == null)
          break;
        boolean isAttribute = elementOrAttributeName.startsWith("@");
        if (isAttribute)
          elementOrAttributeName = elementOrAttributeName.substring(1);
        elementOrAttributeName = NameSelection.completeName(
            this.globalViewAccess, parentElement, elementOrAttributeName);
        if (elementOrAttributeName == null)
          break;
        {
          Element newElement = null;
          Operation operation;
          if (isAttribute) {
            String value = this.fixedValue(
              elementDefinitionMapper.getAttributesContent(parentElement),
              elementOrAttributeName);
            operation = new AttrSetValueOperation(
              this.getAttributeManager(), parentElement, elementOrAttributeName,
              value);
          }
          else {
            newElement = createElement(parentElement, elementOrAttributeName);
            operation = new NodesInsertOperation(
                this.getNodeManager(), parentElement, nextSibling, newElement);
          }
          if (isOperationSequenceStarted)
            this.operationStack.executeAndAdd(new TransitionOperation());
          this.operationStack.executeAndAdd(operation);
          isOperationSequenceStarted = true;
          if (newElement != null && newElement.getParentNode() == parentElement)
            newElementList.add(newElement);
        }
      }
      catch (NotConformantToContentModelException exception) {
        break;
      }
    return isOperationSequenceStarted;
  }

  private static final String fixedValue(AttributesContent attributesContent,
    String attrName) {
    if (attributesContent != null
      && attributesContent.getType(attrName).
      equals(AttributesContent.TYPE_ENUMERATION)) {
      String[] enumeration =
        attributesContent.getEnumeration(attrName);
      if (enumeration.length == 1)
        return enumeration[0];
    }
    return "";
  }

  ////////////////////////////
  // Methods about referencing
  ////////////////////////////

  private void mustNotBeReferencedOutside(final Node[] nodes,
    final Node[] contextNodes) throws IdException {
    for (int k = 0; k < nodes.length; k++) {
      Node node = nodes[k];
      if (node.getNodeType() == Node.ELEMENT_NODE)
        mustNotBeReferencedOutside((Element)node, contextNodes);
    }
  }

  private void mustNotBeReferencedOutside(final Element element,
    final Node[] contextNodes) throws IdException {
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    if (documentUtil == null)
      return;
    String id = documentUtil.getId(element);
    if (id != null && fullyFledgedDocument.getIdManager().isIdReferenced(id)
      && areReferencersOutside(id, contextNodes))
      throw new IdException(IdException.CANNOT_REMOVE_REFERENCED_ID, id);
    Node childNode = element.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE)
        mustNotBeReferencedOutside((Element)childNode, contextNodes);
      childNode = childNode.getNextSibling();
    }
  }

  private boolean areReferencersOutside(final String id,
    final Node[] contextNodes) {
    IdInfo idInfo = fullyFledgedDocument.getIdManager().getIdInfo(id);
    for (int k = 0; k < idInfo.getReferencerCount(); k++) {
      Element referencer = idInfo.getReferencer(k).getOwnerElement();
      if (!isNodeInsideContext(referencer, contextNodes))
        return true;
    }
    return false;
  }

  private boolean isNodeInsideContext(final Node node,
    final Node[] contextNodes) {
    Node parentNode = node.getParentNode();
    for (int k = 0; k < contextNodes.length; k++)
      if (node == contextNodes[k]
        || (parentNode != null
        && isNodeInsideContext(parentNode, contextNodes)))
        return true;
    return false;
  }

  //////////////
  // GUI methods
  //////////////

  private String selectElementName(final String title,
    final Element contextElement, final Collection elementNameSet,
    String preferredName, boolean isAutomaticForOne) {
    String[] elementNames = new String[elementNameSet.size()];
    elementNameSet.toArray(elementNames);
    String elementName = elementNames.length == 1
        && (isAutomaticForOne || elementNames[0].endsWith(":")) ?
        elementNames[0] :
        NameSelection.selectName(
        this.globalViewAccess, title, elementNames, preferredName);
    return elementName != null && elementName.indexOf('(') < 0 ?
        NameSelection.completeName(
        this.globalViewAccess, contextElement, elementName) :
        null;
  }

  private Node getSelectedNode() {
    return viewAccess.getToolView().getDocumentTreeView().getSelectedNode();
  }

  private Node[] getSelectedNodes() {
    return viewAccess.getToolView().getDocumentTreeView().getSelectedNodes();
  }

  private String getSelectedAttributeName() {
    Attr attr =
      viewAccess.getToolView().getNodeDetailView().getCurrentAttribute();
    return attr != null ? attr.getName() : null;
  }

  private static final void displayDOMException(final DOMException exception) {
    ExceptionControler.printStackTrace(exception,
      "DOMException code=" + exception.code);
  }

  private void displayIdException(final IdException exception) {
    Attr attr = exception.getAttribute();
    displayException("ID exception", exception,
      attr != null ? attr.getOwnerElement() : null);
  }

  private void displayUndefinedNamespaceException(
    final UndefinedNamespaceException exception) {
    displayException("Namespace exception", exception, exception.getElement());
  }

  private void displayException(final String title,
    final Exception exception, final Element element) {
    if (element != null && isElementAttachedToRoot(element)) {
      ToolView toolView = viewAccess.getToolView();
      if (toolView != null)
        toolView.displayNode(element);
    }
    ExceptionDialogs.displayException(title, exception);
  }

  private static final boolean isElementAttachedToRoot(final Element element) {
    Element ancestorElement = element;
    Element documentElement = element.getOwnerDocument().getDocumentElement();
    while (true) {
      if (ancestorElement == documentElement)
        return true;
      ancestorElement = (Element)ancestorElement.getParentNode();
      if (ancestorElement == null)
        return false;
    }
  }

  private void setLazyInternal(boolean isLazy) {
    if (fullyFledgedDocument.isLazy() != isLazy) {
      fullyFledgedDocument.setLazy(isLazy);
      {
        ToolView toolView = viewAccess.getToolView();
        if (toolView != null)
          toolView.getToolStatusBar().displayLazy(isLazy);
      }
    }
  }

  private void displayMessage(final String message) {
    ToolView toolView = viewAccess.getToolView();
    if (toolView != null)
      toolView.getToolStatusBar().displayMessage(message);
  }

  private void clearMessage() {
    displayMessage(" ");
  }

  private void displayResourceMessage(final String key) {
    displayMessage(getString(key));
  }

  private void displayCannotInsertMessage(final int insertionType) {
    switch (insertionType) {
      case INSERTION_PREVIOUS:
        displayResourceMessage(Label.NE_PEUT_AVOIR_DE_PRECEDANT);
        break;
      case INSERTION_NEXT:
        displayResourceMessage(Label.NE_PEUT_AVOIR_DE_SUIVANT);
        break;
      case INSERTION_CHILD:
        displayResourceMessage(Label.NE_PEUT_AVOIR_DE_DESCENDANT);
        break;
    }
  }

  private boolean isDocumentReadOnly() {
    if (!fullyFledgedDocument.isReadOnly())
      return false;
    displayResourceMessage(Label.DOCUMENT_EN_LECTURE_SEULE);
    return true;
  }

  private boolean isMoveInProgress() {
    NodeInterval nodeIntervalForMove =
      fullyFledgedDocument.getNodeIntervalForMove();
    if (nodeIntervalForMove == null || !nodeIntervalForMove.isDefined())
      return false;
    displayResourceMessage(Label.MOVE_IN_PROGRESS);
    return true;
  }
}
