/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.io;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.apache.xerces.dom.DocumentImpl;

import org.xmloperator.framework.document.util.ResourceContext;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.Options;
import org.xmloperator.util.io.MIME2Java;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.NodePathBuilder;
import org.xmloperator.option.impl.ResourceContextImpl;
import org.xmloperator.document.util.DocumentWriter;

public abstract class IOUtils {

  public static final ResourceContext addResourceFirst(Options options,
    String documentSystemId) {
    ResourceContext resourceContext =
      new ResourceContextImpl(documentSystemId);
    List resourceList = options.getOptionList(Options.RESOURCE);
    int index = resourceList.indexOf(resourceContext);
    if (index >= 0) {
      resourceContext = (ResourceContext)resourceList.get(index);
      resourceList.remove(index);
    }
    resourceList.add(0, resourceContext);
    int maxResources = options.getIntOptionValue(Options.MAXRESOURCES);
    if (maxResources > 0)
      while (resourceList.size() > maxResources)
        resourceList.remove(resourceList.size() - 1);
    return resourceContext;
  }

  public static final void saveSelectedNodePath(Node selectedNode,
    ResourceContext resourceContext) {
    StringBuffer elementPath = new StringBuffer();
    int nodeIndex = -1;
    if (selectedNode.getNodeType() != Node.DOCUMENT_NODE) {
      if (selectedNode.getNodeType() != Node.ELEMENT_NODE) {
        nodeIndex = NodePathBuilder.getNodeIndex(selectedNode);
        selectedNode = selectedNode.getParentNode();
      }
      while (selectedNode.getNodeType() == Node.ELEMENT_NODE) {
        if (elementPath.length() != 0)
          elementPath.insert(0, ' ');
        elementPath.insert(0,
          NodePathBuilder.getElementIndex((Element)selectedNode));
        selectedNode = selectedNode.getParentNode();
      }
    }
    resourceContext.setSelectedElementPath(elementPath.toString());
    resourceContext.setSelectedNodeIndex(nodeIndex);
  }

  public static final void createAnXMLFile(String pathName) throws IOException {
    final String encoding = Constants.DEFAULT_ENCODING;
    PrintWriter out = new PrintWriter(new OutputStreamWriter(
      new FileOutputStream(pathName), MIME2Java.convert(encoding)));
    try {
      out.println("<?xml version=\"1.0\" encoding=\"" + encoding + "\"?>");
    }
    finally {
      out.close();
    }
  }

  public static final void createAndSaveAMinimalDocument(
    String documentTypeName, String namespaceId,
    String dTDPublicId, String dTDSystemId,
    String documentPathName, Options options) throws IOException {
    if (documentTypeName == null || documentTypeName.length() == 0
        || documentPathName == null || documentPathName.length() == 0)
      throw new IllegalArgumentException();
    // Create a document with the document element
    Document document = new DocumentImpl();
    Element documentElement = document.createElement(documentTypeName);
    int separatorPosition = documentTypeName.indexOf(':');
    if (namespaceId != null && namespaceId.length() != 0)
      documentElement.setAttributeNS(
          "http://www.w3.org/2000/xmlns/", separatorPosition >= 0 ?
          "xmlns:" + documentTypeName.substring(0, separatorPosition) : "xmlns",
          namespaceId);
    else if (separatorPosition >= 0)
      throw new IllegalArgumentException();
    document.appendChild(documentElement);
    // Write out this document
    String encoding = options != null ?
                      options.getStringOptionValue(Options.DEFAULTENCODING) :
                      Constants.DEFAULT_ENCODING;
    if (MIME2Java.convert(encoding) == null)
      encoding = Constants.DEFAULT_ENCODING;
    PrintWriter out =
      new PrintWriter(new OutputStreamWriter(
      new FileOutputStream(documentPathName), MIME2Java.convert(encoding)));
    new DocumentWriter(out).printDocument(document, null,
      dTDPublicId, dTDSystemId, null, encoding, false, false);
    out.close();
  }
}
