/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.task;

import java.util.Enumeration;
import java.util.Hashtable;
import java.io.IOException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.ErrorHandler;
import org.xml.sax.XMLReader;
import org.apache.xerces.parsers.SAXParser;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.control.task.TransformBatch;

public final class BatchHandler implements ContentHandler {

  public static class BatchException extends SAXException {
    private long eventCount = 0;

    // eventCount is the total count of startElement and endElement events
    //   occuring before the exception was raised.
    public BatchException(long eventCount, int state, int subState) {
      super("Batch parsing error (state=" + state
            + ", sub-state=" + subState + ")");
      this.eventCount = eventCount;
    }

    // return the total count of startElement and endElement events
    //   occuring before the exception was raised.
    public long getEventCount() {
      return eventCount;
    }
  }

  public static final void process(
      TransformBatch transformBatch, String systemId)
      throws IOException, SAXException {
    XMLReader xmlReader = new SAXParser();
    xmlReader.setContentHandler(new BatchHandler(transformBatch));
    xmlReader.setErrorHandler(
        new ErrorHandler() {
      public void warning(SAXParseException exception) throws SAXException {
      }
      public void error(SAXParseException exception) throws SAXException {
        throw exception;
      }
      public void fatalError(SAXParseException exception)
          throws SAXException {
        throw exception;
      }
    });
    xmlReader.parse(systemId);
  }

  private String[] fromHashtableToNameValuePairs(Hashtable table) {
    int count = table.size();
    String[] nameValuePairs = new String[2 * count];
    Enumeration names = table.keys();
    int index = 0;
    while (names.hasMoreElements()) {
      String name = (String)names.nextElement();
      nameValuePairs[index++] = name;
      nameValuePairs[index++] = (String)table.get(name);
    }
    return nameValuePairs;
  }

  private static final int STATE_EXPECT_BATCH = 0;
  private static final int STATE_EXPECT_BATCH_DOCUMENTATION = 1;
  private static final int STATE_EXPECT_END_BATCH_DOCUMENTATION = 2;
  private static final int STATE_EXPECT_BATCH_PARAM = 3;
  private static final int STATE_EXPECT_END_BATCH_PARAM = 4;
  private static final int STATE_EXPECT_TRANSFORM = 5;
  private static final int STATE_EXPECT_TRANSFORM_SOURCE = 6;
  private static final int STATE_INSIDE_TRANSFORM_SOURCE = 7;
  private static final int STATE_EXPECT_TRANSFORM_DESTINATION = 8;
  private static final int STATE_INSIDE_TRANSFORM_DESTINATION = 9;
  private static final int STATE_EXPECT_TRANSFORM_PARAM = 10;
  private static final int STATE_EXPECT_END_TRANSFORM_PARAM = 11;
  private static final int STATE_EXPECT_END_TRANSFORM_DOCUMENTATION = 12;
  private static final int STATE_EXPECT_END_TRANSFORM = 13;
  private static final int STATE_END = 14;

  private static final int SUBSTATE_EXPECT_VALUE = 0;
  private static final int SUBSTATE_EXPECT_END_VALUE = 1;

  private TransformBatch transformBatch;
  private long eventCount = 0;
  private int state = STATE_EXPECT_BATCH;
  private int subState = SUBSTATE_EXPECT_VALUE;
  private final Hashtable batchParameters = new Hashtable();
  private String transformerName = null;
  private final StringBuffer sourceDocumentURI = new StringBuffer();
  private final StringBuffer destinationDocumentPathname = new StringBuffer();
  private final Hashtable transformerParameters = new Hashtable();

  private BatchHandler(TransformBatch transformBatch) {
    this.transformBatch = transformBatch;
  }

  private void error() throws SAXException {
    throw new BatchException(eventCount, state, subState);
  }

  private void completeURI(StringBuffer uRI, String localName, Attributes atts)
      throws SAXException {
    String uRIPart = null;
    if (localName.equals("value"))
      uRIPart = atts.getValue("value");
    else if (localName.equals("paramValue")) {
      String paramName = atts.getValue("paramName");
      if (paramName == null || paramName.length() == 0)
        error();
      uRIPart = (String)batchParameters.get(paramName);
    }
    else
      error();
    if (uRIPart == null)
      error();
    uRI.append(uRIPart);
    subState = SUBSTATE_EXPECT_END_VALUE;
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void setDocumentLocator (Locator locator) {
  }

  public void startDocument () throws SAXException {
  }

  public void endDocument () throws SAXException {
  }

  public void startPrefixMapping (String prefix, String uri)
    throws SAXException {
  }

  public void endPrefixMapping (String prefix) throws SAXException {
  }

  public void startElement (String namespaceURI, String localName,
  String rawName, Attributes atts) throws SAXException {
    eventCount++;
    switch (state) {
      case STATE_EXPECT_BATCH:
        if (!localName.equals("batch"))
          error();
        state = STATE_EXPECT_BATCH_DOCUMENTATION;
        break;
      case STATE_EXPECT_BATCH_DOCUMENTATION:
        if (localName.equals("documentation")) {
          state = STATE_EXPECT_END_BATCH_DOCUMENTATION;
          break;
        }
        state = STATE_EXPECT_BATCH_PARAM;
      case STATE_EXPECT_BATCH_PARAM:
        if (localName.equals("param")) {
          String paramName = atts.getValue("name");
          String paramValue = atts.getValue("value");
          if (paramName == null || paramName.length() == 0
              || paramValue == null || paramValue.length() == 0
              || batchParameters.get(paramName) != null)
            error();
          batchParameters.put(paramName, paramValue);
          state = STATE_EXPECT_END_BATCH_PARAM;
          break;
        }
        state = STATE_EXPECT_TRANSFORM;
      case STATE_EXPECT_TRANSFORM:
        if (localName.equals("transform")) {
          transformerName = atts.getValue("name");
          if (transformerName == null) {
            String transformerName_paramName = atts.getValue("name_paramName");
            if (transformerName_paramName == null
                || transformerName_paramName.length() == 0)
              error();
            transformerName =
                (String)batchParameters.get(transformerName_paramName);
            if (transformerName == null)
              error();
          }
          if (transformerName.length() == 0)
            error();
          transformerParameters.clear();
          state = STATE_EXPECT_TRANSFORM_SOURCE;
          break;
        }
        error();
      case STATE_EXPECT_TRANSFORM_SOURCE:
        if (!localName.equals("sourceDocumentURI"))
          error();
        sourceDocumentURI.setLength(0);
        state = STATE_INSIDE_TRANSFORM_SOURCE;
        subState = SUBSTATE_EXPECT_VALUE;
        break;
      case STATE_INSIDE_TRANSFORM_SOURCE:
        if (subState != SUBSTATE_EXPECT_VALUE)
          error();
        completeURI(sourceDocumentURI, localName, atts);
        break;
      case STATE_EXPECT_TRANSFORM_DESTINATION:
        if (!localName.equals("destinationDocumentPathname"))
          error();
        destinationDocumentPathname.setLength(0);
        state = STATE_INSIDE_TRANSFORM_DESTINATION;
        subState = SUBSTATE_EXPECT_VALUE;
        break;
      case STATE_INSIDE_TRANSFORM_DESTINATION:
        if (subState != SUBSTATE_EXPECT_VALUE)
          error();
        completeURI(destinationDocumentPathname, localName, atts);
        break;
      case STATE_EXPECT_TRANSFORM_PARAM:
        if (localName.equals("param")) {
          String name = atts.getValue("name");
          if (name == null || name.length() == 0)
            error();
          String value = atts.getValue("value");
          if (value == null) {
            String value_paramName = atts.getValue("value_paramName");
            if (value_paramName == null || value_paramName.length() == 0)
              error();
            value = (String)batchParameters.get(value_paramName);
            if (value == null)
              error();
          }
          transformerParameters.put(name, value);
          state = STATE_EXPECT_END_TRANSFORM_PARAM;
          break;
        }
        if (localName.equals("documentation")) {
          state = STATE_EXPECT_END_TRANSFORM_DOCUMENTATION;
          break;
        }
        error();
      default:
        error();
    }
  }

  public void endElement (String namespaceURI, String localName,
   String rawName)	throws SAXException {
    eventCount++;
    switch (state) {
      case STATE_EXPECT_BATCH_DOCUMENTATION:
      case STATE_EXPECT_BATCH_PARAM:
      case STATE_EXPECT_TRANSFORM:
        state = STATE_END;
        break;
      case STATE_EXPECT_END_BATCH_DOCUMENTATION:
        state = STATE_EXPECT_BATCH_PARAM;
        break;
      case STATE_EXPECT_END_BATCH_PARAM:
        state = STATE_EXPECT_BATCH_PARAM;
        break;
      case STATE_INSIDE_TRANSFORM_SOURCE:
        switch (subState) {
          case SUBSTATE_EXPECT_VALUE:
            state = STATE_EXPECT_TRANSFORM_DESTINATION;
            break;
          case SUBSTATE_EXPECT_END_VALUE:
            subState = SUBSTATE_EXPECT_VALUE;
            break;
          default:
            error();
        }
        break;
      case STATE_INSIDE_TRANSFORM_DESTINATION:
        switch (subState) {
          case SUBSTATE_EXPECT_VALUE:
            state = STATE_EXPECT_TRANSFORM_PARAM;
            break;
          case SUBSTATE_EXPECT_END_VALUE:
            subState = SUBSTATE_EXPECT_VALUE;
            break;
          default:
            error();
        }
        break;
      case STATE_EXPECT_END_TRANSFORM_PARAM:
        state = STATE_EXPECT_TRANSFORM_PARAM;
        break;
      case STATE_EXPECT_END_TRANSFORM_DOCUMENTATION:
        state = STATE_EXPECT_END_TRANSFORM;
        break;
      case STATE_EXPECT_TRANSFORM_PARAM:
        state = STATE_EXPECT_END_TRANSFORM;
      case STATE_EXPECT_END_TRANSFORM:
        if (sourceDocumentURI.length() == 0
            || destinationDocumentPathname.length() == 0)
          error();
        if (!transformBatch.transform(transformerName,
            sourceDocumentURI.toString(),
            destinationDocumentPathname.toString(),
            fromHashtableToNameValuePairs(transformerParameters)))
          error();
        state = STATE_EXPECT_TRANSFORM;
        break;
      default:
        error();
    }
  }

  public void characters (char ch[], int start, int length)
    throws SAXException {
  }

  public void ignorableWhitespace (char ch[], int start, int length)
    throws SAXException {
  }

  public void processingInstruction (String target, String data)
    throws SAXException {
  }

  public void skippedEntity (String name)	throws SAXException {
  }
}
