/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.task;

import java.io.IOException;
import java.util.ResourceBundle;
import javax.swing.JOptionPane;

import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.status.ToolStatusBar;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ViewAccess;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.task.Task;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.DomWriter;
import org.xmloperator.util.view.ExceptionDialogs;
import org.xmloperator.operation.OperationStack;

public final class DocumentSaveTask implements Task {
  private FullyFledgedDocument fullyFledgedDocument;
  private boolean haveElementsToBeIndented;
  private ViewAccess viewAccess;
  private OperationStack operationStack;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  public DocumentSaveTask(
      FullyFledgedDocument fullyFledgedDocument,
      boolean haveElementsToBeIndented,
      ViewAccess viewAccess, OperationStack operationStack) {
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.haveElementsToBeIndented = haveElementsToBeIndented;
    this.viewAccess = viewAccess;
    this.operationStack = operationStack;
  }

  public boolean saveDocument() {
    try {
      displayMessage(getString(Label.ENREGISTREMENT_EN_COURS));
      int returnCode =
          fullyFledgedDocument.saveDocument(this.haveElementsToBeIndented);
      displayMessage(returnCode != 0 ?
        DomWriter.fromReportCodeToReplacedStringList(returnCode)
        + getString(Label.HAVE_BEEN_REPLACED_FOR_WRITING_OUT)
        : " ");
      if (operationStack != null)
        operationStack.clearUndoOperations();
      viewAccess.getToolView().getToolStatusBar().
        displayEditStatus(ToolStatusBar.EDIT_READY);
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException(exception);
      return false;
    }
    return true;
  }

  private void displayMessage(String message) {
    ToolView toolView = viewAccess.getToolView();
    if (toolView != null)
      toolView.getToolStatusBar().displayMessage(message);
  }

  //////////////////////
  // Task implementation
  //////////////////////

  public boolean run() {
    return saveDocument();
  }
}
