/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.task;

import java.util.ResourceBundle;
import org.w3c.dom.Node;

import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.control.task.CancelableTask;

public final class ExpandNodesTask implements CancelableTask {
  private final ToolView toolView;
  private final Node[] nodes;
  private boolean isCanceled = false;

  public ExpandNodesTask(ToolView toolView, Node[] nodes) {
    if (toolView == null || nodes == null)
      throw new IllegalArgumentException();
    this.toolView = toolView;
    this.nodes = nodes;
  }

  public void expandNode(Node node) {
    this.toolView.getDocumentTreeView().expandNode(node);
    Node childNode = node.getFirstChild();
    while (childNode != null) {
      if (this.isCanceled)
        return;
      this.expandNode(childNode);
      childNode = childNode.getNextSibling();
    }
  }

  private void displayMessage(String message) {
    this.toolView.getToolStatusBar().displayMessage(message);
  }

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  ////////////////////////////////
  // CancelableTask implementation
  ////////////////////////////////

  public boolean run() {
    this.isCanceled = false;
    this.displayMessage(getString(Label.EXPANDING_NODES));
    for (int k = 0; k < this.nodes.length; k++)
      this.expandNode(nodes[k]);
    this.displayMessage(" ");
    return !this.isCanceled;
  }

  public void cancel() {
    this.displayMessage(getString(Label.CANCELED));
    this.isCanceled = true;
  }
}
