/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document;

import java.util.List;
import java.util.StringTokenizer;
import java.io.PrintWriter;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.id.IdBrowser;
import org.xmloperator.util.dom.ElementBrowser;
import org.xmloperator.document.id.IdBrowserImpl;

public class DTDXMLToDTDTranslator {
  public static final int IS_CHILDREN_ELEMENT_MISSING = 1;
  public static final int IS_MIXED_ELEMENT_MISSING = 2;
  public static final int IS_ELEMENT_DECLARATION_MISSING = 4;
  public static final int IS_CHILDREN_PATTERN_MISSING = 8;
  public static final int IS_ATT_DEFS_PATTERN_MISSING = 16;
  public static final int IS_NAMESPACE_ELEMENT_MISSING = 32;
  public static final int IS_PREFIX_ELEMENT_MISSING = 64;
  public static final int IS_SOME_ENUMERATION_EMPTY = 128;

  private static final String[] ATT_DEF_ELEMENT_NAMES =
    new String[] {"attDef", "attDefsPatternRef"};
  private PrintWriter out;
  private IdBrowser idBrowser;
  private boolean arePatternsTranslatedToParameterEntityDeclarations;
  private boolean isChildrenElementMissing = false;
  private boolean isMixedElementMissing = false;
  private boolean isElementDeclarationMissing = false;
  private boolean isChildrenPatternMissing = false;
  private boolean isAttDefsPatternMissing = false;
  private boolean isNamespaceElementMissing = false;
  private boolean isPrefixElementMissing = false;
  private boolean isSomeEnumerationEmpty = false;

  // return a sum of IS_*
  public static final int printDTD(PrintWriter out, Document document,
    IdBrowser idBrowser,
    boolean arePatternsTranslatedToParameterEntityDeclarations) {
    if (out == null || document == null)
      throw new IllegalArgumentException();
    if (idBrowser == null)
      idBrowser = new IdBrowserImpl(document);
    Element dtdElement = document.getDocumentElement();
    if (dtdElement == null)
      throw new RuntimeException("Document element is missing");
    DTDXMLToDTDTranslator translator = new DTDXMLToDTDTranslator(out, idBrowser,
      arePatternsTranslatedToParameterEntityDeclarations);
    translator.printDTD(dtdElement);
    return translator.getResult();
  }

  public static final String resultToString(int result) {
    StringBuffer sb = new StringBuffer();
    if ((result & IS_CHILDREN_ELEMENT_MISSING) != 0)
      sb.append("\nChildren elements are missing");
    if ((result & IS_MIXED_ELEMENT_MISSING) != 0)
      sb.append("\nMixed elements are missing");
    if ((result & IS_ELEMENT_DECLARATION_MISSING) != 0)
      sb.append("\nElement declarations are missing");
    if ((result & IS_CHILDREN_PATTERN_MISSING) != 0)
      sb.append("\nChildren patterns are missing");
    if ((result & IS_ATT_DEFS_PATTERN_MISSING) != 0)
      sb.append("\nAttribute definitions patterns are missing");
    if ((result & IS_NAMESPACE_ELEMENT_MISSING) != 0)
      sb.append("\nNamespace elements are missing");
    if ((result & IS_PREFIX_ELEMENT_MISSING) != 0)
      sb.append("\nPrefix elements are missing");
    if ((result & IS_SOME_ENUMERATION_EMPTY) != 0)
      sb.append("\nSome enumerations are empty");
    if (sb.length() != 0)
      sb.insert(0, "DTDXML concerns :");
    return sb.toString();
  }

  private DTDXMLToDTDTranslator(PrintWriter out, IdBrowser idBrowser,
    boolean arePatternsTranslatedToParameterEntityDeclarations) {
    this.out = out;
    this.idBrowser = idBrowser;
    this.arePatternsTranslatedToParameterEntityDeclarations =
      arePatternsTranslatedToParameterEntityDeclarations;
  }

  private int getResult() {
    int result = 0;
    if (isMixedElementMissing)
      result |= IS_MIXED_ELEMENT_MISSING;
    if (isChildrenElementMissing)
      result |= IS_CHILDREN_ELEMENT_MISSING;
    if (isElementDeclarationMissing)
      result |= IS_ELEMENT_DECLARATION_MISSING;
    if (isChildrenPatternMissing)
      result |= IS_CHILDREN_PATTERN_MISSING;
    if (isAttDefsPatternMissing)
      result |= IS_ATT_DEFS_PATTERN_MISSING;
    if (isNamespaceElementMissing)
      result |= IS_NAMESPACE_ELEMENT_MISSING;
    if (isPrefixElementMissing)
      result |= IS_PREFIX_ELEMENT_MISSING;
    if (isSomeEnumerationEmpty)
      result |= IS_SOME_ENUMERATION_EMPTY;
    return result;
  }

  private void printDTD(Element dtdElement) {
    out.println("<!--");
    {
      String name = dtdElement.getAttribute("dTDName");
      if (name.length() != 0)
        out.println("  " + name + " " + dtdElement.getAttribute("dTDVersion"));
    }
    {
      String description = dtdElement.getAttribute("dTDDescription");
      if (description.length() != 0)
        out.println("  " + description);
    }
    {
      String lastUpdate = dtdElement.getAttribute("lastUpdate");
      if (lastUpdate.length() != 0)
        out.println("  Last update : " + lastUpdate);
    }
    {
      String publicId = dtdElement.getAttribute("dTDPublicId");
      if (publicId.length() != 0)
        out.println("  PUBLIC \"" + publicId + '"');
    }
    {
      String systemId = dtdElement.getAttribute("dTDSystemId");
      if (systemId.length() != 0)
        out.println("  SYSTEM \"" + systemId + '"');
    }
    {
      String nsURIReference = dtdElement.getAttribute("nsURIReference");
      if (nsURIReference.length() != 0)
        out.println("  Namespace : " + nsURIReference);
    }
    {
      String rootElementDeclId = dtdElement.getAttribute("rootElementDecl");
      if (rootElementDeclId.length() != 0) {
        Element elementDeclElement = idBrowser.getElementById(rootElementDeclId);
        if (elementDeclElement != null)
          out.println("  Root element : " + qualifiedName(elementDeclElement));
        else
          isElementDeclarationMissing = true;
      }
    }
    out.println("-->");
    {
      Element notationElement =
        ElementBrowser.getFirstChildElement(dtdElement, "notationDecl");
      while (notationElement != null) {
        out.println(notationToString(notationElement));
        notationElement =
          ElementBrowser.getNextSiblingElement(notationElement, "notationDecl");
      }
    }
    {
      final String[] entityDefElementNames =
        new String[] {"internalEntityDecl", "externalEntityDecl"};
      Element entityDefElement =
        ElementBrowser.getFirstChildElement(dtdElement, entityDefElementNames);
      while (entityDefElement != null) {
        out.println(entityDefToString(entityDefElement));
        entityDefElement = ElementBrowser.getNextSiblingElement(
          entityDefElement, entityDefElementNames);
      }
    }
    if (arePatternsTranslatedToParameterEntityDeclarations) {
      {
        Element attDefsPatternElement =
          ElementBrowser.getFirstChildElement(dtdElement, "attDefsPattern");
        while (attDefsPatternElement != null) {
          out.println();
          printAttDefsPattern(attDefsPatternElement);
          printIdentityConstraints(attDefsPatternElement, true);
          attDefsPatternElement = ElementBrowser.getNextSiblingElement(
            attDefsPatternElement, "attDefsPattern");
        }
      }
      {
        Element childrenPatternElement =
          ElementBrowser.getFirstChildElement(dtdElement, "childrenPattern");
        while (childrenPatternElement != null) {
          out.println();
          printChildrenPattern(childrenPatternElement);
          childrenPatternElement = ElementBrowser.getNextSiblingElement(
            childrenPatternElement, "childrenPattern");
        }
      }
    }
    {
      Element elementDeclElement =
        ElementBrowser.getFirstChildElement(dtdElement, "elementDecl");
      while (elementDeclElement != null) {
        out.println();
        out.println(elementDeclToString(elementDeclElement));
        printAttrListDecl(elementDeclElement);
        printIdentityConstraints(elementDeclElement,
          !arePatternsTranslatedToParameterEntityDeclarations);
        printElementDeclNotes(elementDeclElement);
        elementDeclElement = ElementBrowser.getNextSiblingElement(
          elementDeclElement, "elementDecl");
      }
    }
  }

  private static final String notationToString(Element notationElement) {
    StringBuffer s = new StringBuffer();
    s.append("<!NOTATION ");
    s.append(notationElement.getAttribute("name"));
    s.append(' ');
    s.append(externalDefinitionToString(notationElement));
    s.append('>');
    return s.toString();
  }

  private static final String externalDefinitionToString(Element externalDefElement) {
    String publicId = externalDefElement.getAttribute("publicId");
    String systemId = externalDefElement.getAttribute("systemId");
    StringBuffer s = new StringBuffer();
    if (publicId.length() != 0) {
      s.append("PUBLIC");
      s.append(' ');
      s.append('"');
      s.append(publicId);
      s.append('"');
    }
    else
      s.append("SYSTEM");
    if (systemId.length() != 0) {
      s.append(' ');
      s.append('"');
      s.append(systemId);
      s.append('"');
    }
    return s.toString();
  }

  private static final String entityDefToString(Element entityDefElement) {
    StringBuffer s = new StringBuffer();
    s.append("<!ENTITY ");
    s.append(entityDefElement.getAttribute("name"));
    s.append(' ');
    if (entityDefElement.getNodeName().equals("internalEntityDecl")) {
      s.append('"');
      s.append(exportValue(entityDefElement.getAttribute("value")));
      s.append('"');
    }
    else {
      s.append(externalDefinitionToString(entityDefElement));
      String notationName = entityDefElement.getAttribute("notationName");
      if (notationName.length() != 0) {
        s.append(' ');
        s.append("NDATA");
        s.append(' ');
        s.append(notationName);
      }
    }
    s.append('>');
    return s.toString();
  }

  private static final String exportValue(String value) {
    StringBuffer s = new StringBuffer();
    for (int k = 0; k < value.length(); k++) {
      char ch = value.charAt(k);
      if (ch == '"')
        s.append("&#34;");
      else if (ch == '&')
        s.append("&#38;");
      else
        s.append(ch);
    }
    return s.toString();
  }

  private void printAttDefsPattern(Element attDefsPatternElement) {
    out.println(beginParameterEntityDecl(attDefsPatternElement));
    printAttributeDefinitions(attDefsPatternElement);
    out.println("\'>");
  }

  private void printChildrenPattern(Element childrenPatternElement) {
    out.println(beginParameterEntityDecl(childrenPatternElement));
    out.print("  ");
    out.println(childrenToString(childrenPatternElement));
    out.println("\'>");
  }

  private static final String beginParameterEntityDecl(Element patternElement) {
    StringBuffer s = new StringBuffer();
    s.append("<!ENTITY % ");
    s.append(patternElement.getAttribute("id"));
    s.append(" \'");
    return s.toString();
  }

  private String elementDeclToString(Element elementDeclElement) {
    StringBuffer s = new StringBuffer();
    s.append("<!ELEMENT ");
    s.append(qualifiedName(elementDeclElement));
    s.append(' ');
    String contentType =
      elementDeclElement.getAttribute("contentType").toUpperCase();
    if (contentType.equals("EMPTY") || contentType.equals("ANY"))
      s.append(contentType);
    else if (contentType.equals("CHILDREN")) {
      Element childrenElement =
        ElementBrowser.getFirstChildElement(elementDeclElement, "children");
      if (childrenElement != null)
        s.append(childrenToString(childrenElement));
      else {
        Element childrenPatternRefElement = ElementBrowser.getFirstChildElement(
          elementDeclElement, "childrenPatternRef");
        if (childrenPatternRefElement != null)
          s.append(childrenPatternRefToString(childrenPatternRefElement));
        else
          isChildrenElementMissing = true;
      }
    }
    else if (contentType.equals("MIXED")) {
      Element mixedElement =
        ElementBrowser.getFirstChildElement(elementDeclElement, "mixed");
      if (mixedElement != null)
        s.append(mixedToString(mixedElement));
      else
        isMixedElementMissing = true;
    }
    s.append('>');
    return s.toString();
  }

  private String qualifiedName(Element element) {
    StringBuffer s = new StringBuffer();
    String prefixId = element.getAttribute("prefix");
    if (prefixId.length() != 0) {
      Element prefixElement = idBrowser.getElementById(prefixId);
      if (prefixElement != null)
        s.append(prefixElement.getAttribute("prefix"));
      else
        isPrefixElementMissing = true;
      s.append(':');
    }
    s.append(element.getAttribute("localPartName"));
    return s.toString();
  }

  private String childrenToString(Element childrenElement) {
    StringBuffer s = new StringBuffer();
    s.append('(');
    s.append(childrenToString(childrenElement,
      childrenElement.getAttribute("binaryOperator").equals("seq") ?
      ',' : '|'));
    s.append(')');
    char unaryOperator =
      unaryOperatorToChar(childrenElement.getAttribute("unaryOperator"));
    if (unaryOperator != '.')
      s.append(unaryOperator);
    return s.toString();
  }

  private static final char unaryOperatorToChar(String unaryOperator) {
    if (unaryOperator.equals("zeroOrOne"))
      return '?';
    if (unaryOperator.equals("zeroOrMore"))
      return '*';
    if (unaryOperator.equals("oneOrMore"))
      return '+';
    return '.';
  }

  private String childrenToString(Element childrenElement,
    char binaryOperator) {
    StringBuffer s = new StringBuffer();
    Element specElement = ElementBrowser.getFirstChildElement(childrenElement);
    if (specElement == null)
      return "";
    while (true) {
      String elementName = specElement.getNodeName();
      if (elementName.equals("elementDeclRef"))
        s.append(elementDeclRefToString(specElement));
      else if (elementName.equals("children"))
        s.append(childrenToString(specElement));
      else if (elementName.equals("childrenPatternRef"))
        s.append(childrenPatternRefToString(specElement));
      specElement = ElementBrowser.getNextSiblingElement(specElement);
      if (specElement == null)
        break;
      s.append(binaryOperator);
    }
    return s.toString();
  }

  private String childrenPatternRefToString(Element childrenPatternRefElement) {
    String childrenPatternId =
      childrenPatternRefElement.getAttribute("childrenPattern");
    if (childrenPatternId.length() == 0) {
      isChildrenPatternMissing = true;
      return "";
    }
    Element childrenPatternElement =
      idBrowser.getElementById(childrenPatternId);
    if (childrenPatternElement == null) {
      isChildrenPatternMissing = true;
      return "";
    }
    return arePatternsTranslatedToParameterEntityDeclarations ?
      entityReferenceToString(childrenPatternId) :
      childrenToString(childrenPatternElement);
  }

  private String mixedToString(Element mixedElement) {
    return mixedElement.hasChildNodes() ?
      "(#PCDATA|" + internalMixedToString(mixedElement) + ")*" : "(#PCDATA)";
  }

  private String internalMixedToString(Element mixedElement) {
    StringBuffer s = new StringBuffer();
    Element specElement = ElementBrowser.getFirstChildElement(mixedElement);
    if (specElement == null)
      return "";
    while (true) {
      String elementName = specElement.getNodeName();
      if (elementName.equals("elementDeclRef"))
        s.append(elementDeclRefToString(specElement));
      specElement = ElementBrowser.getNextSiblingElement(specElement);
      if (specElement == null)
        break;
      s.append('|');
    }
    return s.toString();
  }

  private String elementDeclRefToString(Element elementDeclRefElement) {
    String elementDeclId = elementDeclRefElement.getAttribute("elementDecl");
    if (elementDeclId.length() == 0) {
      isElementDeclarationMissing = true;
      return "";
    }
    Element elementDeclElement = idBrowser.getElementById(elementDeclId);
    if (elementDeclElement == null) {
      isElementDeclarationMissing = true;
      return "";
    }
    StringBuffer s = new StringBuffer();
    s.append(qualifiedName(elementDeclElement));
    char unaryOperator =
      unaryOperatorToChar(elementDeclRefElement.getAttribute("unaryOperator"));
    if (unaryOperator != '.')
      s.append(unaryOperator);
    return s.toString();
  }

  private void printAttrListDecl(Element elementDeclElement) {
    out.print("<!ATTLIST ");
    out.println(qualifiedName(elementDeclElement));
    printAttributeDefinitions(elementDeclElement);
    out.println('>');
  }

  private void printAttributeDefinitions(Element attDefParentElement) {
    Element attDefElement = ElementBrowser.getFirstChildElement(
      attDefParentElement, ATT_DEF_ELEMENT_NAMES);
    while (attDefElement != null) {
      String attDefElementName = attDefElement.getNodeName();
      if (attDefElementName.equals("namespaceRef")) {
        out.print("  xmlns");
        printNamespaceReference(attDefElement, false);
      }
      else if (attDefElementName.equals("namespacePrefix")) {
        String prefixId = attDefElement.getAttribute("prefix");
        if (prefixId.length() != 0) {
          Element prefixElement = idBrowser.getElementById(prefixId);
          if (prefixElement != null) {
            out.print("  xmlns:");
            out.print(prefixElement.getAttribute("prefix"));
            printNamespaceReference(prefixElement, true);
          }
          else
            isPrefixElementMissing = true;
        }
        else
          isPrefixElementMissing = true;
      }
      else if (attDefElementName.equals("attDef")) {
        out.print("  ");
        out.println(attributeDefinitionToString(attDefElement));
      }
      else {
        String attDefsPatternId = attDefElement.getAttribute("attDefsPattern");
        if (attDefsPatternId.length() != 0) {
          Element attDefsPatternElement =
            idBrowser.getElementById(attDefsPatternId);
          if (attDefsPatternElement != null)
            if (arePatternsTranslatedToParameterEntityDeclarations) {
              out.print("  ");
              out.println(entityReferenceToString(attDefsPatternId));
            }
            else
              printAttributeDefinitions(attDefsPatternElement);
          else
            isAttDefsPatternMissing = true;
        }
        else
          isAttDefsPatternMissing = true;
      }
      attDefElement = ElementBrowser.getNextSiblingElement(attDefElement,
        ATT_DEF_ELEMENT_NAMES);
    }
  }

  private void printNamespaceReference(Element namespaceReferencingElement,
    boolean isURIReferenceRequired) {
    out.print(" CDATA #FIXED \"");
    String namespaceId = namespaceReferencingElement.getAttribute("namespace");
    if (namespaceId.length() != 0) {
      Element namespaceElement = idBrowser.getElementById(namespaceId);
      if (namespaceElement != null) {
        String uRIReference = namespaceElement.getAttribute("uRIReference");
        if (uRIReference.length() != 0)
          out.print(uRIReference);
        else
          isNamespaceElementMissing |= isURIReferenceRequired;
      }
      else
        isNamespaceElementMissing = true;
    }
    else
      isNamespaceElementMissing |= isURIReferenceRequired;
    out.println('"');
  }

  private String attributeDefinitionToString(Element attDefElement) {
    StringBuffer s = new StringBuffer();
    s.append(qualifiedName(attDefElement));
    String attrType = attDefElement.getAttribute("type").toUpperCase();
    boolean isNotation = attrType.equals("NOTATION");
    boolean isEnumeration = attrType.equals("ENUMERATION");
    if (!isEnumeration) {
      s.append(' ');
      s.append(attrType);
    }
    if (isNotation | isEnumeration) {
      s.append(' ');
      StringTokenizer tokens =
        new StringTokenizer(attDefElement.getAttribute("enumeration"));
      s.append('(');
      if (tokens.hasMoreTokens()) {
        s.append(tokens.nextToken());
        while (tokens.hasMoreTokens()) {
          s.append('|');
          s.append(tokens.nextToken());
        }
      }
      else
        isSomeEnumerationEmpty = true;
      s.append(')');
    }
    String attrDefaultType =
      attDefElement.getAttribute("defaultType").toUpperCase();
    boolean isFixedValue = attrDefaultType.equals("FIXED");
    boolean isDefaultValue = attrDefaultType.equals("DEFAULT");
    if (!isDefaultValue) {
      s.append(" #");
      s.append(attrDefaultType);
    }
    if (isFixedValue | isDefaultValue) {
      s.append(" \"");
      s.append(attDefElement.getAttribute("defaultValue"));
      s.append('"');
    }
    return s.toString();
  }

  private void printIdentityConstraints(Element attDefParentElement,
    boolean deeply) {
    Element attDefElement = ElementBrowser.getFirstChildElement(
      attDefParentElement, ATT_DEF_ELEMENT_NAMES);
    while (attDefElement != null) {
      if (attDefElement.getNodeName().equals("attDef")) {
        String identityConstraint =
          attDefElement.getAttribute("identityConstraint");
        if (identityConstraint.length() != 0)
          out.println("<!-- " + qualifiedName(attDefElement)
            + " identity constraint : " + identityConstraint + " -->");
      }
      else if (deeply) {
        String attDefsPatternId = attDefElement.getAttribute("attDefsPattern");
        if (attDefsPatternId.length() != 0) {
          Element attDefsPatternElement =
            idBrowser.getElementById(attDefsPatternId);
          if (attDefsPatternElement != null)
            printIdentityConstraints(attDefsPatternElement, true);
          else
            isAttDefsPatternMissing = true;
        }
        else
          isAttDefsPatternMissing = true;
      }
      attDefElement = ElementBrowser.getNextSiblingElement(attDefElement,
        ATT_DEF_ELEMENT_NAMES);
    }
  }

  private void printElementDeclNotes(Element elementDeclElement) {
    Element noteElement = ElementBrowser.getFirstChildElement(
      elementDeclElement, "elementDeclNote");
    while (noteElement != null) {
      String noteType = noteElement.getAttribute("type");
      Node childNode = noteElement.getFirstChild();
      if (childNode != null && childNode.getNodeType() == Node.TEXT_NODE)
        out.println("<!-- " + noteType + " : "
          + childNode.getNodeValue() + " -->");
      noteElement = ElementBrowser.getNextSiblingElement(noteElement,
        "elementDeclNote");
    }
  }

  private static final String entityReferenceToString(String entityName) {
    return "%" + entityName + ";";
  }
}
