/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.dtd;

import org.xmloperator.framework.document.dtd.AttributeDefinition;
import org.xmloperator.framework.document.dtd.AttributeDefinitionList;
import org.xmloperator.framework.document.schema.AttributesContent;

public final class AttributesContentImpl implements AttributesContent {
  private AttributeDefinitionList attributeDefinitionList;

  // Constructs an AttributesContent for an element
  //   given by its attributeDefinitionList (may be null).
  public AttributesContentImpl(AttributeDefinitionList attributeDefinitionList)
    {
    this.attributeDefinitionList = attributeDefinitionList;
  }

  ///////////////////////////////////
  // AttributesContent implementation
  ///////////////////////////////////

  public int getPossibleNameCount() {
    return attributeDefinitionList != null ? attributeDefinitionList.size() : 0;
  }

  public String getPossibleName(int index) {
    return attributeDefinitionList.get(index).getAttrName();
  }

  public String getType(String attrName) {
    if (attributeDefinitionList == null)
      return TYPE_CDATA;
    AttributeDefinition attributeDefinition =
      attributeDefinitionList.get(attrName);
    if (attributeDefinition == null)
      return TYPE_CDATA;
    String attrType = attributeDefinition.getAttrType();
    if (attrType.equals(AttributeDefinition.ID))
      return TYPE_ID;
    if (attrType.equals(AttributeDefinition.IDREF))
      return TYPE_IDREF;
    if (attrType.equals(AttributeDefinition.IDREFS))
      return TYPE_IDREFS;
    if (attributeDefinition.getAttrDefaultType().
      equals(AttributeDefinition.FIXED)
      || attrType.equals(AttributeDefinition.ENUMERATION))
      return TYPE_ENUMERATION;
    return TYPE_CDATA;
  }

  public String[] getEnumeration(String attrName) {
    if (attributeDefinitionList == null)
      return null;
    AttributeDefinition attributeDefinition =
      attributeDefinitionList.get(attrName);
    if (attributeDefinition == null)
      return null;
    if (attributeDefinition.getAttrDefaultType().
      equals(AttributeDefinition.FIXED))
      return new String[] {attributeDefinition.getAttrDefaultValue()};
    return attributeDefinition.getEnumeration();
  }

  public boolean isRequired(String attrName) {
    if (attributeDefinitionList == null)
      return false;
    AttributeDefinition attributeDefinition =
      attributeDefinitionList.get(attrName);
    if (attributeDefinition == null)
      return false;
    return attributeDefinition.getAttrDefaultType().
      equals(AttributeDefinition.REQUIRED);
  }

  public String getIdAttributeName() {
    return attributeDefinitionList != null ?
      attributeDefinitionList.getIdAttrName() : null;
  }
}
