/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.dtd;

import java.util.Iterator;
import java.util.Collection;
import java.util.Set;
import java.util.List;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.StringTokenizer;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.dtd.SpecificationNode;
import org.xmloperator.framework.document.dtd.ElementSpecification;
import org.xmloperator.framework.document.dtd.AttributeDefinition;
import org.xmloperator.framework.document.dtd.AttributeDefinitionList;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.document.dtd.SpecNodeUtils;
import org.xmloperator.document.dtd.ElementContentImpl;

public abstract class DTDUtils {

  /////////////////////////////////////
  // Methods about attribute definition
  /////////////////////////////////////

  public static final String getAttributeType(DTD dTD,
    String elementName, String attributeName) {
    if (dTD == null)
      return null;
    AttributeDefinitionList attributeDefinitionList =
      dTD.getAttributeDefinitionList(elementName);
    if (attributeDefinitionList == null)
      return null;
    AttributeDefinition attributeDefinition =
      attributeDefinitionList.get(attributeName);
    return attributeDefinition != null ?
      attributeDefinition.getAttrType() : null;
  }

  public static final List referencesFromElement(DTD dTD, Element element) {
    AttributeDefinitionList attributeDefinitionList =
      dTD.getAttributeDefinitionList(element.getNodeName());
    if (attributeDefinitionList == null)
      return null;
    List references = new ArrayList();
    for (int k = 0; k < attributeDefinitionList.size(); k++) {
      AttributeDefinition attributeDefinition = attributeDefinitionList.get(k);
      if (attributeDefinition.getAttrType().
        indexOf(AttributeDefinition.IDREF) == 0) {
        StringTokenizer st = new StringTokenizer(
          element.getAttribute(attributeDefinition.getAttrName()));
        while (st.hasMoreTokens()) {
          String token = st.nextToken();
          if (!references.contains(token))
            references.add(token);
        }
      }
    }
    return references.size() != 0 ? references : null;
  }

  //////////////////////////////////////
  // Methods about element specification
  //////////////////////////////////////

  public static final Collection childElementNameSet(DTD dTD,
    String parentElementName) {
    if (dTD == null)
      return null;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElementName);
    if (parentElementSpec == null)
      return null;
    String parentElementSpecType = parentElementSpec.getSpecType();
    if (parentElementSpecType == ElementSpecification.ANY)
      return dTD.getElementNameList();
    if (parentElementSpecType == ElementSpecification.EMPTY)
      return null;
    Set elementNameSet = new HashSet();
    SpecNodeUtils.addElementNames(elementNameSet,
      parentElementSpec.getSpecNode());
    return elementNameSet;
  }

  public static final boolean canHaveChilds(DTD dTD, String parentElementName) {
    if (dTD == null)
      return true;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElementName);
    return parentElementSpec == null
      || parentElementSpec.getSpecType() != ElementSpecification.EMPTY;
  }

  public static final boolean canHaveChildText(DTD dTD,
    String parentElementName) {
    if (dTD == null)
      return true;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElementName);
    if (parentElementSpec == null)
      return true;
    String parentElementSpecType = parentElementSpec.getSpecType();
    return parentElementSpecType == ElementSpecification.MIXED
      || parentElementSpecType == ElementSpecification.ANY;
  }

  public static final boolean shouldHaveChildText(DTD dTD,
    String parentElementName) {
    if (dTD == null)
      return false;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElementName);
    if (parentElementSpec == null)
      return false;
    String parentElementSpecType = parentElementSpec.getSpecType();
    if (parentElementSpecType != ElementSpecification.MIXED)
      return false;
    SpecificationNode specNode = parentElementSpec.getSpecNode();
    return specNode == null || !SpecNodeUtils.hasElementNames(specNode);
  }

  /*
  public static final boolean canHaveChildElements(DTD dTD,
    String parentElementName) {
    if (dTD == null)
      return true;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElementName);
    if (parentElementSpec == null)
      return true;
    String parentElementSpecType = parentElementSpec.getSpecType();
    if (parentElementSpecType == ElementSpecification.CHILDREN
      || parentElementSpecType == ElementSpecification.ANY)
      return true;
    if (parentElementSpecType == ElementSpecification.EMPTY)
      return false;
    SpecificationNode specNode = parentElementSpec.getSpecNode();
    return specNode != null && SpecNodeUtils.hasElementNames(specNode);
  }
  */

  public static final SpecificationNode getSpecificationNode(DTD dTD,
    String parentElementName) {
    if (dTD == null)
      return null;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElementName);
    return parentElementSpec != null ? parentElementSpec.getSpecNode() : null;
  }

  /////////////////////////////////
  // Methods about children content
  /////////////////////////////////

  public static final boolean canElementNameBeParentOf(DTD dTD,
    String parentElementName, String[] childElementNames) {
    if (dTD == null)
      return true;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElementName);
    if (parentElementSpec == null)
      return true; // No specification
    String parentElementSpecType = parentElementSpec.getSpecType();
    if (parentElementSpecType == ElementSpecification.ANY)
      return true;
    if (parentElementSpecType == ElementSpecification.EMPTY)
      return childElementNames.length == 0;
    SpecificationNode specNode = parentElementSpec.getSpecNode();
    return ElementContentImpl.isContentValid(specNode, childElementNames);
  }

  /////////////////////////////////////////////////////////
  // Methods for determining element names for append child
  /////////////////////////////////////////////////////////

  // The parent element is considered as having no child.
  public static final Collection elementNamesForAppendChild(DTD dTD,
    String parentElementName) {
    if (dTD == null)
      return null;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElementName);
    if (parentElementSpec == null)
      return null;
    String specType = parentElementSpec.getSpecType();
    if (specType.equals(ElementSpecification.EMPTY))
      return null;
    if (specType.equals(ElementSpecification.ANY))
      return anyElementNames(dTD);
    SpecificationNode specNode = parentElementSpec.getSpecNode();
    Set elementNameSet = new HashSet();
    SpecNodeUtils.addRequiredElementNames(elementNameSet, specNode);
    if (elementNameSet.size() == 0)
      SpecNodeUtils.addElementNames(elementNameSet, specNode);
    return elementNameSet;
  }

  public static final Collection anyElementNames(DTD dTD) {
    return dTD != null ? dTD.getElementNameList() : null;
  }
}
