/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.dtd.impl;

import org.xmloperator.framework.document.dtd.AttributeDefinition;

public final class AttributeDefinitionImpl
  implements AttributeDefinition, Comparable {
  private String attrName;
  private String attrType;
  private String[] enumeration;
  private String attrDefaultType;
  private String attrDefaultValue;

  public AttributeDefinitionImpl(String attrName) {
    this(attrName, CDATA, null, IMPLIED, null);
  }

  public AttributeDefinitionImpl(String attrName, String attrType,
    String[] enumeration, String attrDefaultType, String attrDefaultValue) {
    if (attrName == null || attrType == null)
      throw new IllegalArgumentException();
    this.attrName = attrName;
    this.attrType = attrType != null ? attrType : CDATA;
    this.enumeration = enumeration;
    this.attrDefaultType = attrDefaultType != null ? attrDefaultType : IMPLIED;
    this.attrDefaultValue = attrDefaultValue;
  }

  //////////////////
  // Object overload
  //////////////////

  public boolean equals(Object object) {
    return (object != null) && object instanceof AttributeDefinitionImpl
      && attrName.equals(((AttributeDefinitionImpl)object).attrName);
  }

  ////////////////////////////
  // Comparable implementation
  ////////////////////////////

  public int compareTo(Object object) {
    if (object == null || !(object instanceof AttributeDefinitionImpl))
      throw new IllegalArgumentException();
    return attrName.compareTo(((AttributeDefinitionImpl)object).attrName);
  }

  /////////////////////////////////////
  // AttributeDefinition implementation
  /////////////////////////////////////

  public String getAttrName() {
    return attrName;
  }

  public String getAttrType() {
    return attrType;
  }

  public String[] getEnumeration() {
    return enumeration;
  }

  public String getAttrDefaultType() {
    return attrDefaultType;
  }

  public String getAttrDefaultValue() {
    return attrDefaultValue;
  }
}
