/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.dtd.parse;

import java.util.List;
import java.util.ArrayList;

import org.xmloperator.framework.document.dtd.SpecificationNode;
import org.xmloperator.framework.document.dtd.ElementSpecification;
import org.xmloperator.document.dtd.impl.SpecificationNodeImpl;
import org.xmloperator.document.dtd.impl.ElementSpecificationImpl;

public class ContentModelParser {
  private char[] spec;
  private int length;
  private int index = 0;

  /**
   * Parse a valid content model containing no space :
   * contentspec ::=    'EMPTY' | 'ANY' | mixed | children
   * mixed       ::=    '(#PCDATA' ('|' name)* ')*' | '(#PCDATA)'
   * children    ::=    (choice | seq) ('?' | '*' | '+')?
   * choice      ::=    '(' cp ( '|' cp )+ ')'
   * seq         ::=    '(' cp ( ',' cp )* ')'
   * cp          ::=    (name | choice | seq) ('?' | '*' | '+')?
   *
   * Result is undefined if the content model contains spaces or is not valid.
   */
  public static final ElementSpecification parseContentModel(
    String contentModel) {
    if (contentModel == null)
      throw new IllegalArgumentException();
    return contentModel.equals(ElementSpecification.EMPTY)
      || contentModel.equals(ElementSpecification.ANY) ?
      new ElementSpecificationImpl(contentModel) :
      new ElementSpecificationImpl(contentModel.startsWith("(#") ?
      ElementSpecification.MIXED : ElementSpecification.CHILDREN,
      new ContentModelParser(contentModel).parseSpec());
  }

  private ContentModelParser(String contentModel) {
    this.spec = contentModel.toCharArray();
    this.length = spec.length;
  }

  // spec ::=  ('#PCDATA' | name | ('(' spec ( (',' | '|') spec )* ')'))
  //           ('?' | '*' | '+')?
  private SpecificationNode parseSpec() {
    if (index >= length)
      return null;
    SpecificationNode specNode;
    if (spec[index] == '(') {
      index++;
      List childSpecNodeList = new ArrayList();
      char separator = '|';
      while (true) {
        SpecificationNode childSpecNode = parseSpec();
        if (childSpecNode == null)
          return null;
        childSpecNodeList.add(childSpecNode);
        if (index >= length)
          return null;
        char ch = spec[index];
        index++;
        if (ch == ')')
          break;
        separator = ch;
      }
      SpecificationNode[] childSpecNodes =
        new SpecificationNode[childSpecNodeList.size()];
      childSpecNodeList.toArray(childSpecNodes);
      specNode = new SpecificationNodeImpl(separator == ',', childSpecNodes);
    }
    else {
      String name = parseName();
      specNode = name.equals("#PCDATA") ?
        new SpecificationNodeImpl() : new SpecificationNodeImpl(name);
    }
    if (index < length) {
      char occurrence = spec[index];
      if (occurrence == '?' || occurrence == '*' || occurrence == '+') {
        index++;
        specNode.setOccurrence(occurrence);
      }
    }
    return specNode;
  }

  private String parseName() {
    int start = index;
    while (index < length) {
      char separator = spec[index];
      if (separator == '?' || separator == '*' || separator == '+'
        || separator == ',' || separator == '|' || separator == ')')
        break;
      index++;
    }
    return new String(spec, start, index - start);
  }
}
