/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.id;

import java.util.List;
import java.util.ArrayList;

import org.w3c.dom.Attr;

import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.document.id.IdInfo;
import org.xmloperator.util.dom.DomRefIds;

public final class IdInfoImpl implements IdInfo {
  private Attr idAttribute;
  private List referencerSet = null;
  private ObserverView observerView = null;

  public IdInfoImpl(Attr idAttribute) {
    this.idAttribute = idAttribute;
  }

  public void setObserverView(ObserverView observerView) {
    if (this.observerView != null && observerView != this.observerView)
      this.observerView.dispose();
    this.observerView = observerView;
  }

  ////////////////////////
  // IdInfo implementation
  ////////////////////////

  public Attr getIdAttribute() {
    return idAttribute;
  }

  public int getReferencerCount() {
    return referencerSet != null ? referencerSet.size() : 0;
  }

  public Attr getReferencer(int index) {
    return (Attr)referencerSet.get(index);
  }

  public void addReferencer(Attr referencer) {
    if (referencerSet == null)
      referencerSet = new ArrayList();
    else if (referencerSet.contains(referencer))
      return;
    referencerSet.add(referencer);
    if (observerView != null)
      observerView.update(this, null);
  }

  public void removeReferencer(Attr referencer) {
    referencerSet.remove(referencer);
    if (observerView != null)
      observerView.update(this, null);
  }

  public void changeId(String oldId, String newId) {
    if (referencerSet != null)
      for (int k = 0; k < referencerSet.size(); k++) {
        Attr referencer = (Attr)referencerSet.get(k);
        DomRefIds refIds = new DomRefIds(referencer.getValue());
        if (refIds.replace(oldId, newId))
          referencer.setValue(refIds.toString());
      }
    if (observerView != null)
      observerView.update(this, null);
  }

  public void removeId(String oldId) {
    if (referencerSet != null)
      for (int k = 0; k < referencerSet.size(); k++) {
        Attr referencer = (Attr)referencerSet.get(k);
        DomRefIds refIds = new DomRefIds(referencer.getValue());
        if (refIds.remove(oldId))
          referencer.setValue(refIds.toString());
      }
    if (observerView != null)
      observerView.update(this, null);
  }
}
