/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.id;

import java.util.Random;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import org.xmloperator.util.dom.DomRefIds;
import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.id.IdInfo;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.id.IdManager;
import org.xmloperator.document.id.IdInfoImpl;

public final class IdManagerImpl implements IdManager {
  private static final String hexa = "0123456789abcdef";
  private Options options;
  private Random random = new Random();
  private final Hashtable fromIdToIdInfo = new Hashtable();
  private boolean isEnabled = true;

  public IdManagerImpl(Options options) {
    this.options = options;
  }

  ///////////////////////////////
  // IdManagerRead implementation
  ///////////////////////////////

  public boolean isEnabled() {
    return isEnabled;
  }

  public boolean isEmpty() {
    return fromIdToIdInfo.isEmpty();
  }

  public String getNewVerifiedId() {
    int idLength = options.getIntOptionValue(Options.IDLENGTH);
    if (idLength < 2 || idLength > 15)
      idLength = 4;
    String idPrefix = options.getStringOptionValue(Options.IDPREFIX);
    while (true) {
      StringBuffer s = new StringBuffer();
      long alea = random.nextLong();
      for (int k = 0; k < idLength; k++) {
        s.insert(0, hexa.charAt((int)(alea & 15)));
        alea >>= 4;
      }
      s.insert(0, idPrefix);
      String id = s.toString();
      if (fromIdToIdInfo.get(id) == null)
        return id;
      if (idLength < 15)
        idLength++;
    }
  }

  public IdInfo getIdInfo(String id) {
    return (IdInfo)fromIdToIdInfo.get(id);
  }

  public Attr getAttributeById(String id) {
    IdInfo idInfo = (IdInfo)fromIdToIdInfo.get(id);
    return idInfo != null ? idInfo.getIdAttribute() : null;
  }

  public Element getElementById(String id) {
    Attr attribute = getAttributeById(id);
    return attribute != null ? attribute.getOwnerElement() : null;
  }

  public boolean isIdDefined(String id) {
    return fromIdToIdInfo.get(id) != null;
  }

  public boolean isIdReferenced(String id) {
    IdInfo idInfo = getIdInfo(id);
    return idInfo != null ? idInfo.getReferencerCount() != 0 : false;
  }

  ///////////////////////////
  // IdManager implementation
  ///////////////////////////

  public void clearAndSetEnabled(boolean isEnabled) {
    Enumeration idInfos = fromIdToIdInfo.elements();
    while (idInfos.hasMoreElements())
      ((IdInfoImpl)idInfos.nextElement()).setObserverView(null);
    fromIdToIdInfo.clear();
    this.isEnabled = isEnabled;
  }

  public void setObserverView(ObserverView observerView, Attr idAttribute) {
    String id = idAttribute.getValue();
    if (id.length() == 0)
      return;
    IdInfoImpl idInfo = (IdInfoImpl)fromIdToIdInfo.get(id);
    if (idInfo != null)
      idInfo.setObserverView(observerView);
  }

  public void addId(Attr idAttribute, String id) throws IdException {
    if (id.length() == 0)
      return;
    IdInfo idInfo = (IdInfo)fromIdToIdInfo.get(id);
    if (idInfo == null) {
      idInfo = new IdInfoImpl(idAttribute);
      fromIdToIdInfo.put(id, idInfo);
    }
    else if (idAttribute != idInfo.getIdAttribute())
      throw new IdException(IdException.CANNOT_DUPLICATE_ID, id, idAttribute);
  }

  public void changeId(Attr idAttribute, String oldId, String newId)
    throws IdException {
    boolean isOldIdNull = oldId.length() == 0;
    boolean isNewIdNull = newId == null || newId.length() == 0;
    if ((isOldIdNull && isNewIdNull) || oldId.equals(newId))
      return;
    if (!isNewIdNull && getElementById(newId) != null)
      throw new IdException(IdException.CANNOT_DUPLICATE_ID, newId);
    if (isOldIdNull)
      addId(idAttribute, newId);
    else if (isNewIdNull)
      removeId(idAttribute, oldId);
    else {
      IdInfo idInfo = (IdInfo)fromIdToIdInfo.get(oldId);
      if (idInfo == null)
        throw new IdException(IdException.CANNOT_FIND_ID, oldId, idAttribute);
      fromIdToIdInfo.remove(oldId);
      idInfo.changeId(oldId, newId);
      fromIdToIdInfo.put(newId, idInfo);
    }
  }

  public void removeId(Attr idAttribute, String oldId) {
    IdInfoImpl idInfo = (IdInfoImpl)fromIdToIdInfo.get(oldId);
    if (idInfo == null || idInfo.getIdAttribute() != idAttribute)
      return;
    idInfo.setObserverView(null);
    fromIdToIdInfo.remove(oldId);
    idInfo.removeId(oldId);
  }

  public void addIdRef(Attr idRefAttribute) throws IdException {
    String id = idRefAttribute.getValue();
    if (id.length() != 0)
      addIdRef(idRefAttribute, id);
  }

  public void removeIdRef(Attr idRefAttribute) {
    removeIdRef(idRefAttribute, idRefAttribute.getValue());
  }

  public void addIdRefs(Attr idRefsAttribute) throws IdException {
    StringTokenizer st = new StringTokenizer(idRefsAttribute.getValue());
    while (st.hasMoreTokens()) {
      String id = (String)st.nextToken();
      addIdRef(idRefsAttribute, id);
    }
  }

  public void removeIdRefs(Attr idRefsAttribute) {
    StringTokenizer st = new StringTokenizer(idRefsAttribute.getValue());
    while (st.hasMoreTokens())
      removeIdRef(idRefsAttribute, (String)st.nextToken());
  }

  private void addIdRef(Attr idRefAttribute, String id) throws IdException {
    IdInfo idInfo = (IdInfo)fromIdToIdInfo.get(id);
    if (idInfo == null)
      throw new IdException(IdException.CANNOT_FIND_ID, id, idRefAttribute);
    idInfo.addReferencer(idRefAttribute);
  }

  private void removeIdRef(Attr idRefAttribute, String id) {
    IdInfo idInfo = (IdInfo)fromIdToIdInfo.get(id);
    if (idInfo != null)
      idInfo.removeReferencer(idRefAttribute);
  }
}
