/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.manage;

import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.manage.ElementDefinitionMapper;
import org.xmloperator.framework.document.manage.DocumentUtil;

public abstract class AbstractDocumentUtil implements DocumentUtil {
  protected ElementDefinitionMapper elementDefinitionMapper;

  public AbstractDocumentUtil(ElementDefinitionMapper elementDefinitionMapper) {
    if (elementDefinitionMapper == null)
      throw new IllegalArgumentException();
    this.elementDefinitionMapper = elementDefinitionMapper;
  }

  public static final boolean areNodesMovable(final Node[] nodes) {
    Node parentNode = nodes[0].getParentNode();
    if (parentNode == null)
      return false;
    for (int k = 0; k < nodes.length; k++) {
      int nodeType = nodes[k].getNodeType();
      if (nodeType == Node.DOCUMENT_NODE
        || nodeType == Node.DOCUMENT_TYPE_NODE
        || (nodeType == Node.ELEMENT_NODE
        && parentNode.getNodeType() == Node.DOCUMENT_NODE))
        return false;
    }
    return true;
  }

  protected static final boolean hasTextChild(final Element element) {
    Node childNode = element.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.TEXT_NODE)
        return true;
      childNode = childNode.getNextSibling();
    }
    return false;
  }

  //////////////////////////////////////
  // DocumentUtil partial implementation
  //////////////////////////////////////

  public AttributesContent getAttributesContent(Element element) {
    return elementDefinitionMapper.getAttributesContent(element);
  }

  public String getId(Element element) {
    AttributesContent attributesContent = this.getAttributesContent(element);
    if (attributesContent == null)
      return null;
    String idAttrName = attributesContent.getIdAttributeName();
    String id = idAttrName != null ? element.getAttribute(idAttrName) : "";
    return id.length() != 0 ? id : null;
  }

  public String[] referencesFromElement(Element element) {
    AttributesContent attributesContent = this.getAttributesContent(element);
    if (attributesContent == null)
      return null;
    List referenceList = new ArrayList();
    for (int k = 0; k < attributesContent.getPossibleNameCount(); k++) {
      String attrName = attributesContent.getPossibleName(k);
      String attrValue = element.getAttribute(attrName);
      if (attrValue.length() != 0) {
        String attrType = attributesContent.getType(attrName);
        if (attrType.equals(AttributesContent.TYPE_IDREF))
          referenceList.add(attrValue);
        else if (attrType.equals(AttributesContent.TYPE_IDREFS)) {
          StringTokenizer st = new StringTokenizer(attrValue);
          while (st.hasMoreTokens()) {
            String token = st.nextToken();
            if (!referenceList.contains(token))
              referenceList.add(token);
          }
        }
      }
    }
    if (referenceList.isEmpty())
      return null;
    String[] references = new String[referenceList.size()];
    referenceList.toArray(references);
    return references;
  }
}
