/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.parse;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.io.InputStream;
import java.util.List;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.StringTokenizer;

import org.apache.xerces.xni.parser.XMLConfigurationException;
import org.apache.xerces.xni.parser.XMLInputSource;
import org.apache.xerces.xni.parser.XMLEntityResolver;
import org.apache.xerces.xni.XNIException;
import org.apache.xerces.xni.XMLString;
import org.apache.xerces.xni.QName;
import org.apache.xerces.xni.XMLAttributes;
import org.apache.xerces.xni.Augmentations;
import org.apache.xerces.xni.XMLResourceIdentifier;
import org.apache.xerces.xni.XMLLocator;
import org.apache.xerces.xni.XMLDTDContentModelHandler;
import org.apache.xerces.parsers.DOMParser;

import org.xmloperator.framework.document.dtd.SpecificationNode;
import org.xmloperator.framework.document.dtd.ElementSpecification;
import org.xmloperator.framework.document.dtd.AttributeDefinition;
import org.xmloperator.framework.document.dtd.AttributeDefinitionList;
import org.xmloperator.framework.document.dtd.ExternalDefinition;
import org.xmloperator.framework.document.dtd.EntityDefinition;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.parse.DocumentParsingListener;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.document.dtd.impl.AttributeDefinitionImpl;
import org.xmloperator.document.dtd.impl.AttributeDefinitionListImpl;
import org.xmloperator.document.dtd.impl.SpecificationNodeImpl;
import org.xmloperator.document.dtd.impl.ElementSpecificationImpl;
import org.xmloperator.document.dtd.impl.ExternalDefinitionImpl;
import org.xmloperator.document.dtd.impl.EntityDefinitionImpl;
import org.xmloperator.document.dtd.impl.DTDImpl;
import org.xmloperator.document.dtd.parse.ContentModelParser;
import org.xmloperator.document.dtd.parse.ElementSpecificationBuilder;
import org.xmloperator.document.parse.DefaultErrorHandler;

public class ToolParser extends DOMParser {
  public static final DefaultErrorHandler DEFAULT_ERROR_HANDLER =
    new DefaultErrorHandler();

  protected DocumentParsingListener documentParsingListener = null;
  protected String xMLVersion = null;
  protected String encoding = null;
  protected String standalone = null;
  protected boolean isThereDoctypeDecl = false;
  protected DTDImpl dTD = new DTDImpl();
  protected boolean isParsingDTD = false;
  protected static final boolean PARSE_CONTENT_MODEL = false;
  protected String currentElementName = null;
  protected ElementSpecificationBuilder elementSpecificationBuilder =
    new ElementSpecificationBuilder();

  public static final DTD parseDTD(String dTDSystemId,
    XMLEntityResolver entityResolver) throws XNIException, IOException {
    if (dTDSystemId == null)
      throw new IllegalArgumentException();
    ToolParser parser = new ToolParser();
    parser.setErrorHandler(DEFAULT_ERROR_HANDLER);
    parser.setXMLEntityResolver(entityResolver);
    XMLInputSource inputSource = new XMLInputSource(null, null, null,
        new StringReader("<?xml version=\"1.0\"?><!DOCTYPE D SYSTEM \""
        + dTDSystemId + "\"><D/>"), null);
    parser.parse(inputSource);
    return parser.getDTD();
  }

  public ToolParser() {
  }

  public void setXMLEntityResolver(XMLEntityResolver entityResolver) {
    try {
      fConfiguration.setProperty(ENTITY_RESOLVER, entityResolver);
    }
    catch (XMLConfigurationException exception) {
    }
  }

  public void setDocumentParsingListener(
    DocumentParsingListener documentParsingListener) {
    this.documentParsingListener = documentParsingListener;
  }

  public String getXMLVersion() {
    return xMLVersion;
  }

  public String getEncoding() {
    return encoding;
  }

  public String getStandalone() {
    return standalone;
  }

  public boolean isThereDoctypeDecl() {
    return isThereDoctypeDecl;
  }

  public DTD getDTD() {
    return dTD;
  }

  ////////////////////////
  // DomParser overloading
  ////////////////////////

  public void parse(XMLInputSource inputSource)
      throws XNIException, IOException {
    dTD.addNamespacePrefix("xml", "http://www.w3.org/XML/1998/namespace");
    super.parse(inputSource);
    dTD.addPredefinedEntities();
  }

  public void comment(XMLString text, Augmentations augs) throws XNIException {
    if (isParsingDTD)
      return;
    super.comment(text, augs);
  }

  public void startDTD(XMLLocator locator, Augmentations augs)
    throws XNIException {
    isParsingDTD = true;
    if (documentParsingListener != null)
      documentParsingListener.notifyStartDTD();
    super.startDTD(locator, augs);
  }

  public void endDTD(Augmentations augs) throws XNIException {
    isParsingDTD = false;
    if (documentParsingListener != null)
      documentParsingListener.notifyEndDTD();
    super.endDTD(augs);
  }

  public void startElement(QName element, XMLAttributes attributes,
    Augmentations augs) throws XNIException {
    if (documentParsingListener != null)
      documentParsingListener.notifyStartElement(element.rawname);
    super.startElement(element, attributes, augs);
  }

  public void endElement(QName element, Augmentations augs)
    throws XNIException {
    if (documentParsingListener != null)
      documentParsingListener.notifyEndElement();
    super.endElement(element, augs);
  }

  public void startContentModel(String elementName, Augmentations augmentations)
    throws XNIException {
    if (!PARSE_CONTENT_MODEL)
      currentElementName = elementName;
    super.startContentModel(elementName,augmentations);
  }

  public void any(Augmentations augmentations) throws XNIException {
    if (!PARSE_CONTENT_MODEL)
      elementSpecificationBuilder.any();
    super.any(augmentations);
  }

  public void empty(Augmentations augmentations) throws XNIException {
    if (!PARSE_CONTENT_MODEL)
      elementSpecificationBuilder.empty();
    super.empty(augmentations);
  }

  public void startGroup(Augmentations augmentations) throws XNIException {
    if (!PARSE_CONTENT_MODEL)
      elementSpecificationBuilder.startGroup();
    super.startGroup(augmentations);
  }

  public void pcdata(Augmentations augmentations) throws XNIException {
    if (!PARSE_CONTENT_MODEL)
      elementSpecificationBuilder.pcdata();
    super.pcdata(augmentations);
  }

  public void element(String elementName, Augmentations augmentations)
    throws XNIException {
    if (!PARSE_CONTENT_MODEL)
      elementSpecificationBuilder.element(elementName);
    super.element(elementName, augmentations);
  }

  public void separator(short separator, Augmentations augmentations)
    throws XNIException {
    if (!PARSE_CONTENT_MODEL) {
      char separatorChar;
      switch (separator) {
        case XMLDTDContentModelHandler.SEPARATOR_CHOICE:
          separatorChar = '|';
          break;
        case XMLDTDContentModelHandler.SEPARATOR_SEQUENCE:
          separatorChar = ',';
          break;
        default:
          throw new IllegalArgumentException();
      }
      elementSpecificationBuilder.separator(separatorChar);
    }
    super.separator(separator, augmentations);
  }

  public void occurrence(short occurrence, Augmentations augmentations)
    throws XNIException {
    if (!PARSE_CONTENT_MODEL) {
      char occurrenceChar;
      switch (occurrence) {
        case XMLDTDContentModelHandler.OCCURS_ONE_OR_MORE:
          occurrenceChar = '+';
          break;
        case XMLDTDContentModelHandler.OCCURS_ZERO_OR_MORE:
          occurrenceChar = '*';
          break;
        case XMLDTDContentModelHandler.OCCURS_ZERO_OR_ONE:
          occurrenceChar = '?';
          break;
        default:
          throw new IllegalArgumentException();
      }
      elementSpecificationBuilder.occurrence(occurrenceChar);
    }
    super.occurrence(occurrence, augmentations);
  }

  public void endGroup(Augmentations augmentations) throws XNIException {
    if (!PARSE_CONTENT_MODEL)
      elementSpecificationBuilder.endGroup();
    super.endGroup(augmentations);
  }

  public void endContentModel(Augmentations augmentations) throws XNIException {
    if (!PARSE_CONTENT_MODEL) {
      ElementSpecification elementSpecification =
        elementSpecificationBuilder.result();
      dTD.fromElementNameToElementSpecification().
        put(currentElementName, elementSpecification);
      currentElementName = null;
      elementSpecificationBuilder.empty();
    }
    super.endContentModel(augmentations);
  }

  public void elementDecl(String name, String contentModel, Augmentations augs)
    throws XNIException {
    int posSeparator = name.indexOf(':');
    if (posSeparator > 0)
      dTD.addNamespacePrefix(name.substring(0, posSeparator));
    dTD.getElementNameList().add(name);
    if (PARSE_CONTENT_MODEL) {
      ElementSpecification elementSpecification =
        ContentModelParser.parseContentModel(contentModel);
      dTD.fromElementNameToElementSpecification().
        put(name, elementSpecification);
    }
    super.elementDecl(name, contentModel, augs);
  }

  public void attributeDecl(String elementName, String attributeName,
    String type, String[] enumeration, String defaultType,
    XMLString defaultValue, XMLString nonNormalizedDefaultValue,
    Augmentations augs) throws XNIException {
    int attrSeparatorPos = attributeName.indexOf(':');
    String attributePrefix = attrSeparatorPos > 0 ?
      attributeName.substring(0, attrSeparatorPos) : null;
    String attrDefaultValue =
      defaultValue != null ? defaultValue.toString() : "";
    if (attributeName.equals("xmlns")) {
      if (attrDefaultValue.length() != 0)
        dTD.addNamespace(attrDefaultValue);
    }
    else if (attributePrefix != null) {
      if (attributePrefix.equals("xmlns")) {
        String attributeLocalPartName =
          attributeName.substring(attrSeparatorPos + 1);
        if (attrDefaultValue.length() != 0)
          dTD.addNamespacePrefix(attributeLocalPartName, attrDefaultValue);
      }
      else
        dTD.addNamespacePrefix(attributePrefix);
    }
    AttributeDefinitionListImpl attributeDefList =
      (AttributeDefinitionListImpl)dTD.fromElementNameToAttrDefList().
      get(elementName);
    if (attributeDefList == null)
      attributeDefList = new AttributeDefinitionListImpl();
    attributeDefList.add(
      new AttributeDefinitionImpl(attributeName, type,
      enumeration, defaultType != null ? defaultType : "", attrDefaultValue));
    dTD.fromElementNameToAttrDefList().put(elementName, attributeDefList);
    super.attributeDecl(elementName, attributeName, type, enumeration,
      defaultType, defaultValue, nonNormalizedDefaultValue, augs);
  }

  public void internalEntityDecl(String name, XMLString text,
    XMLString nonNormalizedText, Augmentations augs) throws XNIException {
    if (name.charAt(0) != '%') {
      dTD.getEntityNameList().add(name);
      dTD.fromEntityNameToEntityDefinition().put(name,
        new EntityDefinitionImpl(text.toString()));
    }
    super.internalEntityDecl(name, text, nonNormalizedText, augs);
  }

  public void externalEntityDecl(String name, XMLResourceIdentifier identifier,
    Augmentations augs) throws XNIException {
    dTD.getEntityNameList().add(name);
    String publicId = identifier.getPublicId();
    String systemId = identifier.getExpandedSystemId();
    dTD.fromEntityNameToEntityDefinition().put(name,
      new EntityDefinitionImpl(publicId, systemId));
    super.externalEntityDecl(name, identifier, augs);
  }

  public void unparsedEntityDecl(String name, XMLResourceIdentifier identifier,
    String notation, Augmentations augs) throws XNIException {
    dTD.getEntityNameList().add(name);
    String publicId = identifier.getPublicId();
    String systemId = identifier.getExpandedSystemId();
    dTD.fromEntityNameToEntityDefinition().put(name,
      new EntityDefinitionImpl(publicId, systemId, notation));
    super.unparsedEntityDecl(name, identifier, notation, augs);
  }

  public void notationDecl(String name, XMLResourceIdentifier identifier,
  	Augmentations augs) throws XNIException {
    dTD.getNotationNameList().add(name);
    String publicId = identifier.getPublicId();
    String systemId = identifier.getExpandedSystemId();
    dTD.fromNotationNameToNotationDefinition().put(name,
      new ExternalDefinitionImpl(publicId, systemId));
    super.notationDecl(name, identifier, augs);
  }

  public void xmlDecl(String version, String encoding, String standalone,
    Augmentations augs) throws XNIException {
    this.xMLVersion = version;
    this.encoding = encoding;
    this.standalone = standalone;
    super.xmlDecl(version, encoding, standalone, augs);
  }

  public void doctypeDecl(String rootElement,
    String publicId, String systemId, Augmentations augs) throws XNIException {
    this.isThereDoctypeDecl = true;
    super.doctypeDecl(rootElement, publicId, systemId, augs);
  }
}
