/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.relaxng.parse;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import org.xmloperator.framework.document.relaxng.NameClass;
import org.xmloperator.framework.document.relaxng.AnyName;
import org.xmloperator.document.relaxng.impl.AnyNameImpl;
import org.xmloperator.document.relaxng.parse.RelaxNGParseException;
import org.xmloperator.document.relaxng.parse.NameClassHandler;
import org.xmloperator.document.relaxng.parse.ExceptNameClassBuilder;
  // Reciprocal import

// anyNameClass     ::= <anyName> [exceptNameClass] </anyName>
public class AnyNameBuilder implements NameClassHandler {
  private static final int STATE_EXPECT_ANYNAME = 0;
  private static final int STATE_EXPECT_EXCEPT = 1;
  private static final int STATE_INSIDE_EXCEPT = 2;
  private static final int STATE_EXPECT_END_ANYNAME = 3;
  private static final int STATE_END = 4;

  private int state = STATE_EXPECT_ANYNAME; // STATE_*
  private ExceptNameClassBuilder exceptNameClassBuilder = null;

  public static final boolean isStart(String elementName) {
    return elementName.equals("anyName");
  }

  public AnyNameBuilder() {
  }

  private void error() throws SAXException {
    throw new RelaxNGParseException("anyName", state);
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void startElement(String name, Attributes atts) throws SAXException {
    switch (state) {
      case STATE_EXPECT_ANYNAME:
        if (!name.equals("anyName"))
          error();
        state = STATE_EXPECT_EXCEPT;
        break;
      case STATE_EXPECT_EXCEPT:
        exceptNameClassBuilder = new ExceptNameClassBuilder();
        state = STATE_INSIDE_EXCEPT;
      case STATE_INSIDE_EXCEPT:
        exceptNameClassBuilder.startElement(name, atts);
        break;
      default:
        error();
    }
  }

  public boolean endElement() throws SAXException {
    switch (state) {
      case STATE_INSIDE_EXCEPT:
        if (exceptNameClassBuilder.endElement())
          state = STATE_EXPECT_END_ANYNAME;
        return false;
      case STATE_EXPECT_EXCEPT:
      case STATE_EXPECT_END_ANYNAME:
        state = STATE_END;
        return true;
      default:
        error();
        return true;
    }
  }

  public void characters(char ch[], int start, int length) {
    switch (state) {
      case STATE_INSIDE_EXCEPT:
        exceptNameClassBuilder.characters(ch, start, length);
        break;
    }
  }

  //////////////////////////////////
  // NameClassHandler implementation
  //////////////////////////////////

  public NameClass result() {
    if (state != STATE_END)
      return null;
    AnyNameImpl anyName = new AnyNameImpl();
    if (exceptNameClassBuilder != null)
      anyName.setExcept((NameClass)exceptNameClassBuilder.result());
    return anyName;
  }
}
