/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.relaxng.parse;

import java.util.Map;
import java.util.List;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import org.xmloperator.framework.document.relaxng.NonEmptyPattern;
import org.xmloperator.framework.document.relaxng.GroupPattern;
import org.xmloperator.document.relaxng.impl.NonEmptyPatternImpl;
import org.xmloperator.document.relaxng.impl.GroupPatternImpl;
import org.xmloperator.document.relaxng.parse.RelaxNGParseException;
import org.xmloperator.document.relaxng.parse.NonEmptyPatternHandler;
import org.xmloperator.document.relaxng.parse.NonEmptyPatternBuilder;

// nonEmptyPattern  ::= [...]
//   | <choice> pattern nonEmptyPattern </choice>
//   | <group> nonEmptyPattern nonEmptyPattern </group>
//   | <interleave> nonEmptyPattern nonEmptyPattern </interleave>
// pattern  ::= <empty/> | nonEmptyPattern
public class GroupPatternBuilder implements NonEmptyPatternHandler {
  private static final int STATE_EXPECT_GROUP = 0;
  private static final int STATE_EXPECT_PATTERN = 1;
  private static final int STATE_INSIDE_PATTERN = 2;
  private static final int STATE_EXPECT_END_PATTERN = 3;
  private static final int STATE_END = 4;

  private List nonEmptyPatternBuilderList;
  private int state = STATE_EXPECT_GROUP; // STATE_*
  private int groupType = GroupPattern.GROUP_CHOICE_EXCLUDING_EMPTY_PATTERN;
  private String groupName = null;
  private ContentHandler nonEmptyPatternBuilder = null;
  private boolean isGroupPatternBuilder = false;
  private int childCount = 0;

  public static final boolean isStart(String elementName) {
    return elementName.equals("choice")
      || elementName.equals("group")
      || elementName.equals("interleave");
  }

  public GroupPatternBuilder(List nonEmptyPatternBuilderList) {
    this.nonEmptyPatternBuilderList = nonEmptyPatternBuilderList;
  }

  private void error() throws SAXException {
    throw new RelaxNGParseException("(choice|group|interleave) pattern", state);
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void startElement(String name, Attributes atts) throws SAXException {
    switch (state) {
      case STATE_EXPECT_GROUP:
        if (name.equals("choice"))
          groupType = GroupPattern.GROUP_CHOICE_EXCLUDING_EMPTY_PATTERN;
        else if (name.equals("group"))
          groupType = GroupPattern.GROUP_GROUP;
        else if (name.equals("interleave"))
          groupType = GroupPattern.GROUP_INTERLEAVE;
        else
          error();
        groupName = name;
        state = STATE_EXPECT_PATTERN;
        break;
      case STATE_EXPECT_PATTERN:
        childCount++;
        if (name.equals("empty")) {
          if (groupType != GroupPattern.GROUP_CHOICE_EXCLUDING_EMPTY_PATTERN)
            error();
          groupType = GroupPattern.GROUP_CHOICE_INCLUDING_EMPTY_PATTERN;
          state = STATE_EXPECT_END_PATTERN;
          break;
        }
        isGroupPatternBuilder = name.equals(groupName);
        nonEmptyPatternBuilder = isGroupPatternBuilder ?
          (ContentHandler)new GroupPatternBuilder(nonEmptyPatternBuilderList) :
          (ContentHandler)new NonEmptyPatternBuilder();
        state = STATE_INSIDE_PATTERN;
      case STATE_INSIDE_PATTERN:
        nonEmptyPatternBuilder.startElement(name, atts);
        break;
      default:
        error();
    }
  }

  public boolean endElement() throws SAXException {
    switch (state) {
      case STATE_INSIDE_PATTERN:
        if (nonEmptyPatternBuilder.endElement()) {
          if (isGroupPatternBuilder) {
            if (((GroupPatternBuilder)nonEmptyPatternBuilder).groupType
                == GroupPattern.GROUP_CHOICE_INCLUDING_EMPTY_PATTERN)
              groupType = GroupPattern.GROUP_CHOICE_INCLUDING_EMPTY_PATTERN;
          }
          else
            nonEmptyPatternBuilderList.add(nonEmptyPatternBuilder);
          nonEmptyPatternBuilder = null;
          state = STATE_EXPECT_PATTERN;
        }
        return false;
      case STATE_EXPECT_END_PATTERN:
        state = STATE_EXPECT_PATTERN;
        return false;
      case STATE_EXPECT_PATTERN:
        if (childCount < 2)
          error();
        state = STATE_END;
        return true;
      default:
        error();
        return true;
    }
  }

  public void characters(char ch[], int start, int length) {
    switch (state) {
      case STATE_INSIDE_PATTERN:
        nonEmptyPatternBuilder.characters(ch, start, length);
        break;
    }
  }

  ////////////////////////////////////////
  // NonEmptyPatternHandler implementation
  ////////////////////////////////////////

  public NonEmptyPatternImpl result(Map fromNameToElementDefinition) {
    if (state != STATE_END)
      return null;
    int nonEmptyPatternCount = nonEmptyPatternBuilderList.size();
    NonEmptyPattern[] nonEmptyPatterns =
      new NonEmptyPattern[nonEmptyPatternCount];
    for (int k = 0; k < nonEmptyPatternCount; k++)
      nonEmptyPatterns[k] =
        ((NonEmptyPatternBuilder)nonEmptyPatternBuilderList.get(k)).
        result(fromNameToElementDefinition);
    return new GroupPatternImpl(groupType, nonEmptyPatterns);
  }
}
