/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.util;

import java.io.PrintWriter;
import java.io.StringWriter;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;

import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.document.util.DomWriterReport;
import org.xmloperator.util.dom.DomWriter;
import org.xmloperator.document.dtd.DTDWriter;

public class DocumentWriter {
  protected static final String indentString = "  ";
  protected PrintWriter out;
  protected boolean isShortcutUsedForEmptyElement = true;
  protected boolean addLineFeedAtStartTagEnd = false;
  protected boolean haveEntityReferencesToBeExpanded = false;
  protected boolean haveElementsToBeIndented = false;
  protected int indentationLevel = 0;
  protected boolean isNewLine = true;

  public static final String toString(final Node fragment) {
    StringWriter stringWriter = new StringWriter();
    DocumentWriter documentWriter =
        new DocumentWriter(new PrintWriter(stringWriter));
    documentWriter.setEntityReferencesToBeExpanded(true);
    documentWriter.printFragment(fragment);
    return stringWriter.toString();
  }

  public DocumentWriter(final PrintWriter out) {
    this.out = out;
  }

  public final void setShortcutUsedForEmptyElement(
      boolean isShortcutUsedForEmptyElement) {
    this.isShortcutUsedForEmptyElement = isShortcutUsedForEmptyElement;
  }

  public final void setLineFeedAtStartTagEnd(boolean addLineFeedAtStartTagEnd) {
    this.addLineFeedAtStartTagEnd = addLineFeedAtStartTagEnd;
    if (addLineFeedAtStartTagEnd)
      this.haveElementsToBeIndented = false;
  }

  public final void setEntityReferencesToBeExpanded(
      boolean haveEntityReferencesToBeExpanded) {
    this.haveEntityReferencesToBeExpanded = haveEntityReferencesToBeExpanded;
  }

  public final void setElementsToBeIndented(
      boolean haveElementsToBeIndented) {
    this.haveElementsToBeIndented = haveElementsToBeIndented;
    if (haveElementsToBeIndented)
      this.addLineFeedAtStartTagEnd = false;
  }

  // Return a sum of DomWriterReport.REPLACED_*
  public final int printDocument(
      final Document document, final String xMLVersion,
      final String encodingName, final boolean isStandalone) {
    return printDocument(
        document, null, null, null, xMLVersion, encodingName, isStandalone,
        false);
  }

  // Return a sum of DomWriterReport.REPLACED_*
  public final int printDocument(
      final Document document, final DTD dTD,
      final String dTDPublicId, final String dTDSystemId,
      final String xMLVersion, final String encodingName,
      final boolean isStandalone, boolean isDoctypeIncluded) {
    if (document == null)
      throw new IllegalArgumentException();
    printHeading(xMLVersion, encodingName, isStandalone);
    if (dTDSystemId != null && dTDSystemId.length() != 0)
      isDoctypeIncluded = true;
    else if (dTD == null || dTD.getElementNameList().isEmpty())
      isDoctypeIncluded = false;
    Element documentElement = document.getDocumentElement();
    if (isDoctypeIncluded && documentElement != null)
      printDocumentType(documentElement.getNodeName(),
        dTD, dTDPublicId, dTDSystemId);
    return printFragment(document);
  }

  // Return a sum of DomWriterReport.REPLACED_*
  public final int printFragment(final Node fragment) {
    this.indentationLevel = 0;
    this.isNewLine = true;
    int returnCode = printChildList(fragment);
    this.out.flush();
    return returnCode;
  }

  protected final void printHeading(
      String xMLVersion, final String encodingName, final boolean isStandalone)
  {
    if (xMLVersion == null)
      xMLVersion = "1.0";
    this.out.print("<?xml version=\"");
    this.out.print(xMLVersion);
    this.out.print("\"");
    if (encodingName != null && encodingName.length() != 0) {
      this.out.print(" encoding=\"");
      this.out.print(encodingName);
      this.out.print('"');
    }
    if (isStandalone)
      this.out.print(" standalone=\"yes\"");
    this.out.println("?>");
  }

  protected final void printDocumentType(final String documentTypeName,
    final DTD dTD, final String dTDPublicId, final String dTDSystemId) {
    this.out.print("<!DOCTYPE ");
    this.out.print(documentTypeName);
    this.out.print(' ');
    if (dTDSystemId == null || dTDSystemId.length() == 0) {
      this.out.println('[');
      DTDWriter.printDTD(out, dTD);
      this.out.print(']');
    }
    else {
      if (dTDPublicId != null && dTDPublicId.length() != 0) {
        this.out.print("PUBLIC \"");
        this.out.print(dTDPublicId);
        this.out.print("\" \"");
      }
      else
        this.out.print("SYSTEM \"");
      this.out.print(dTDSystemId);
      this.out.print('"');
    }
    this.out.println('>');
  }

  // Return a sum of DomWriterReport.REPLACED_*
  protected final int printNode(final Node node) {
    switch (node.getNodeType()) {
    case Node.ELEMENT_NODE:
      return this.printElementNode((Element)node);
    case Node.ATTRIBUTE_NODE:
      return this.printAttributeNode((Attr)node);
    case Node.TEXT_NODE:
      return this.printTextNode(node);
    case Node.CDATA_SECTION_NODE:
      return this.printCdataSectionNode(node);
    case Node.COMMENT_NODE:
      return this.printCommentNode(node);
    case Node.ENTITY_REFERENCE_NODE:
      if (haveEntityReferencesToBeExpanded)
        this.printChildList(node);
      else
        this.printEntityReferenceNode(node);
      return 0;
    case Node.PROCESSING_INSTRUCTION_NODE:
      return printProcessingInstructionNode(node);
    default:
      return 0;
    }
  }

  // Return a sum of DomWriterReport.REPLACED_*
  protected final int printElementNode(final Element element) {
    int returnCode = 0;
    String elementName = element.getNodeName();
    if (this.haveElementsToBeIndented)
      this.indent();
    this.out.print('<');
    this.out.print(elementName);
    NamedNodeMap attrs = element.getAttributes();
    for (int k = 0; k < attrs.getLength(); k++)
      returnCode |= printAttributeNode((Attr)attrs.item(k));
    if (addLineFeedAtStartTagEnd)
      this.out.println();
    if (isShortcutUsedForEmptyElement && !hasChildNodes(element))
      this.out.print("/>");
    else {
      this.out.print('>');
      this.indentationLevel++;
      returnCode |= printChildList(element);
      this.indentationLevel--;
      if (this.haveElementsToBeIndented && hasChildElement(element))
        this.indent();
      this.out.print('<');
      this.out.print('/');
      this.out.print(elementName);
      this.out.print('>');
    }
    return returnCode;
  }

  protected final void indent() {
    if (!this.isNewLine)
      this.out.println();
    for (int k = 0; k < this.indentationLevel; k++)
      this.out.print(indentString);
    this.isNewLine = false;
  }

  // Return a sum of DomWriterReport.REPLACED_*
  protected final int printAttributeNode(final Attr attr) {
    return DomWriter.printAttribute(
        out, attr.getNodeName(), attr.getNodeValue());
  }

  // Return a sum of DomWriterReport.REPLACED_*
  private int printTextNode(final Node node) {
    return DomWriter.printText(out, node.getNodeValue());
  }

  // Return a sum of DomWriterReport.REPLACED_*
  private final int printCdataSectionNode(final Node node) {
    String value = node.getNodeValue();
    return value.length() != 0 ?
      DomWriter.printCdataSection(out, value) : 0;
  }

  // Return a sum of DomWriterReport.REPLACED_*
  private final int printCommentNode(final Node node) {
    String value = node.getNodeValue();
    if (value.length() == 0)
      return 0;
    if (this.haveElementsToBeIndented)
      this.indent();
    return DomWriter.printComment(out, value);
  }

  private final void printEntityReferenceNode(final Node node) {
    this.out.print('&');
    this.out.print(node.getNodeName());
    this.out.print(';');
  }

  // Return a sum of DomWriterReport.REPLACED_*
  private final int printProcessingInstructionNode(final Node node) {
    if (this.haveElementsToBeIndented)
      this.indent();
    return DomWriter.printProcessingInstruction(
        out, node.getNodeName(), node.getNodeValue());
  }

  // Return a sum of DomWriterReport.REPLACED_*
  private final int printChildList(final Node parentNode) {
    int returnCode = 0;
    Node childNode = parentNode.getFirstChild();
    while (childNode != null) {
      returnCode |= printNode(childNode);
      childNode = childNode.getNextSibling();
    }
    return returnCode;
  }

  protected static final boolean hasChildNodes(final Node parentNode) {
    Node childNode = parentNode.getFirstChild();
    while (childNode != null) {
      switch (childNode.getNodeType()) {
        case Node.CDATA_SECTION_NODE:
        case Node.TEXT_NODE:
        case Node.COMMENT_NODE: {
          if (childNode.getNodeValue().length() != 0)
            return true;
          break;
        }
        default:
          return true;
      }
      childNode = childNode.getNextSibling();
    }
    return false;
  }

  protected static final boolean hasChildElement(final Element parentElement) {
    Node childNode = parentElement.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE)
        return true;
      childNode = childNode.getNextSibling();
    }
    return false;
  }
}
