/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.document.compare;

import java.util.Collection;

import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.compare.ElementCharacterizationAccess;
import org.xmloperator.framework.document.compare.ElementAssociation;

/**
 * A <code>DocumentAssociation</code> associates 2 documents.
 * It contains a set of coherent <code>ElementAssociation</code>s.
 * By coherence one means that being the associations (e1, e2) and (ep1, ep2),
 *   ep1 being an ancestor of e1 implies ep2 being an ancestor of e2.
 * By this rule, (ep1, ep2) may be considered as ancestor of (e1, e2).
 * Any element of the first or the second document is the object of zero or one
 *   element association.
 */
public interface DocumentAssociation {

  /**
   * Return an access to the element characterization function.
   */
  public ElementCharacterizationAccess getElementCharacterizationAccess();

  /**
   * Return the first document.
   * Is not null.
   */
  public Document getDocument1();

  /**
   * Return the second document.
   * Is not null.
   */
  public Document getDocument2();

  /**
   * Return the document associated to <code>document</code>.
   * Return null if <code>document</code> is neither the first nor the second
   *   document.
   */
  public Document getAssociatedDocument(Document document);

  /**
   * Return the set of the element associations.
   */
  public Collection elementAssociations();

  /**
   * Return the element association where <code>element</code> is one or the
   *   other element.
   * Return null if there is no such an element association.
   */
  public ElementAssociation getElementAssociation(Element element);
}
