/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.operation;

import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.namespace.UndefinedNamespaceException;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.operation.OperationException;

/**
 * An operation is a reversible change on a document.
 */
public interface Operation {

  /**
   * Operation name. Not null.
   */
  public abstract String getOperationName();

  /**
   * Name of reverse operation. Not null.
   */
  public abstract String getReverseOperationName();

  /**
   * Return true if the operation has no effect on the document.
   */
  public abstract boolean isNull();

  /**
   * Return true if the operation has been executed.
   */
  public abstract boolean isExecuted();

  /**
   * Execute the operation.
   * Return an element which represents the operation before its execution
   *   or null if the operationDocument == null.
   * pre-condition : isExecuted == false.
   * post-condition : isExecuted == true.
   * If a "clean" exception is raised then the operation is not executed.
   * For other type of exception, the operation may be not executed,
   *   partially executed or completed.
   * Clean exceptions : OperationException, UndefinedNamespaceException.
   * Unclean exceptions : IdException, DOMException, ...
   */
  public abstract Element execute(Document operationDocument)
    throws IdException, UndefinedNamespaceException;

  /**
   * Return true if some other operation of the same class
   * could replace this operation by hiding its impact on the document.
   * Reverse condition, by permutting the two operations, must be also true.
   */
  public abstract boolean isHidable();

  /**
   * Return true if this operation could replace the operation argument
   * by hiding its impact on the document.
   * Reverse condition, by permutting the two operations, must be also true.
   * By deduction, isHidable must be also true.
   */
  public abstract boolean canHide(Operation operation);

  /**
   * Return true if this operation has replaced the operation argument.
   * Pre-condition : isExecuted == true
   * Pre-condition : operation.isExecuted == true
   * May raise an OperationException.
   */
  public abstract boolean hidesIfCan(Operation operation);

  /**
   * Create a reverse operation for this operation.
   * pre-condition : isExecuted == true.
   * post-condition : retour.isExecuted == false
   * post-condition : returned operation != this
   * post-condition : this is unchanged
   * May raise an OperationException.
   * Any operation is reversible.
   */
  public abstract Operation createReverseOperation();
}
