/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation;

import java.awt.Point;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Container;
import java.awt.Frame;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import javax.swing.tree.TreeNode;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.JScrollPane;
import javax.swing.BoxLayout;
import javax.swing.border.EmptyBorder;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.util.view.ViewUtils;

public class DialogOperation extends JDialog {
  public static final int REJETER = 0;
  public static final int TOUT_REJETER = 1;
  public static final int EXECUTER = 2;
  public static final int TOUT_EXECUTER = 3;
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected int consigne = EXECUTER;
  protected TreeNode rootTreeNode;
  protected DefaultTreeModel treeModel;
  protected JTree arbreOperation;

  public static int showDialog(Frame parent, String title,
    Element operationElement) {
    return showDialog(parent, title, operationElement, null, new Point());
  }

  public static int showDialog(Frame parent, String title,
    Element operationElement, Point delta) {
    return showDialog(parent, title, operationElement, null, delta);
  }

  public static int showDialog(Frame parent, String title,
    Element operationElement, String message) {
    return showDialog(parent, title, operationElement, message, new Point());
  }

  public static int showDialog(Frame parent, String title,
    Element operationElement, String message, Point delta) {
    DialogOperation dialogue =
      new DialogOperation(parent, operationElement, message);
    dialogue.setTitle(title);
    ViewUtils.moveComponentToCentre(dialogue, delta);
    dialogue.show();
    Point newDelta = ViewUtils.fromCentreToComponent(dialogue);
    delta.move(newDelta.x, newDelta.y);
    return dialogue.getConsigne();
  }

  protected DialogOperation(Frame parent, Element operationElement,
    String message) {
    // construction d'ensemble
    super(parent, true);
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    this.setResizable(true);
    Container voletDialogue = getContentPane();
    JPanel voletContenu = new JPanel();
    voletContenu.setLayout(new BoxLayout(voletContenu, BoxLayout.Y_AXIS));
    voletContenu.setBorder(new EmptyBorder(4, 4, 4, 4));
    voletDialogue.add(voletContenu);
    // Volet des boutons
    JPanel voletBoutons = new JPanel(new FlowLayout());
    JButton boutonRejeter =
      new JButton(viewResource.getString(Label.REJETER));
    boutonRejeter.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.REJETER).charAt(0));
    boutonRejeter.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          boutonRejeter_actionPerformed();
        }
      });
    voletBoutons.add(boutonRejeter);
    JButton boutonToutRejeter =
      new JButton(viewResource.getString(Label.TOUT_REJETER));
    boutonToutRejeter.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.TOUT_REJETER).charAt(0));
    boutonToutRejeter.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          boutonToutRejeter_actionPerformed();
        }
      });
    voletBoutons.add(boutonToutRejeter);
    if (message == null) {
      JButton boutonExecuter =
        new JButton(viewResource.getString(Label.EXECUTER));
      boutonExecuter.setMnemonic(viewResource.getString(Label.MNEMO
        + Label.EXECUTER).charAt(0));
      boutonExecuter.addActionListener(
        new ActionListener() {
          public void actionPerformed(ActionEvent event) {
            boutonExecuter_actionPerformed();
          }
        });
      voletBoutons.add(boutonExecuter);
      JButton boutonToutExecuter =
        new JButton(viewResource.getString(Label.TOUT_EXECUTER));
      boutonToutExecuter.setMnemonic(viewResource.getString(Label.MNEMO
        + Label.TOUT_EXECUTER).charAt(0));
      boutonToutExecuter.addActionListener(
        new ActionListener() {
          public void actionPerformed(ActionEvent event) {
            boutonToutExecuter_actionPerformed();
          }
        });
      voletBoutons.add(boutonToutExecuter);
    }
    voletDialogue.add(voletBoutons, BorderLayout.SOUTH);
    // Volet d'opration
    JPanel voletOperation = new JPanel(new BorderLayout());
    voletOperation.setPreferredSize(new Dimension(400, 160));
    this.rootTreeNode = createTreeNodeFromElement(operationElement);
    this.treeModel = new DefaultTreeModel(rootTreeNode, false);
    this.arbreOperation = new JTree(treeModel);
    arbreOperation.setRowHeight(16);
    arbreOperation.setFont(Constants.FONT_NORMAL);
    JScrollPane scrollPanel = new JScrollPane(arbreOperation);
    voletOperation.add(scrollPanel);
    voletContenu.add(voletOperation);
    // Volet de message
    if (message != null) {
      JPanel voletMessage = new JPanel(new BorderLayout());
      JLabel champMessage = new JLabel(message);
      voletMessage.add(champMessage);
      voletContenu.add(voletMessage);
    }
    // Mise aux dimensions
    pack();
  }

  protected DefaultMutableTreeNode createTreeNodeFromElement(Element element) {
    DefaultMutableTreeNode elementTreeNode =
      new DefaultMutableTreeNode("<" + element.getNodeName() + ">");
    NamedNodeMap attrs = element.getAttributes();
    for (int k = 0; k < attrs.getLength(); k++) {
      Node attr = attrs.item(k);
      elementTreeNode.add(
        new DefaultMutableTreeNode(attr.getNodeName() + "=\""
        + attr.getNodeValue() + '"'));
    }
    Node childNode = element.getFirstChild();
    while (childNode != null) {
      DefaultMutableTreeNode childNodeTreeNode =
        childNode.getNodeType() == Node.ELEMENT_NODE ?
        createTreeNodeFromElement((Element)childNode) :
        createTreeNodeFromNotElementNode(childNode);
      if (childNodeTreeNode != null)
        elementTreeNode.add(childNodeTreeNode);
      childNode = childNode.getNextSibling();
    }
    return elementTreeNode;
  }

  protected DefaultMutableTreeNode createTreeNodeFromNotElementNode(Node node) {
    String string;
    switch (node.getNodeType()) {
      case Node.COMMENT_NODE:
        string = "comment=\"" + node.getNodeValue() + '"';
        break;
      case Node.TEXT_NODE:
        string = "data=\"" + node.getNodeValue() + '"';
        break;
      default:
        return null;
    }
    return new DefaultMutableTreeNode(string);
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  protected void boutonRejeter_actionPerformed() {
    consigne = REJETER;
    dispose();
  }

  protected void boutonToutRejeter_actionPerformed() {
    consigne = TOUT_REJETER;
    dispose();
  }

  protected void boutonExecuter_actionPerformed() {
    consigne = EXECUTER;
    dispose();
  }

  protected void boutonToutExecuter_actionPerformed() {
    consigne = TOUT_EXECUTER;
    dispose();
  }

  public int getConsigne() {
    return consigne;
  }
}

