/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation.core;

import org.w3c.dom.DOMException;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.namespace.UndefinedNamespaceException;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.manage.AttributeManager;
import org.xmloperator.framework.operation.OperationException;
import org.xmloperator.framework.operation.Operation;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.NodePathBuilder;
import org.xmloperator.operation.core.AbstractOperation;
import org.xmloperator.operation.core.OperationNameUtils;

public final class AttrSetValueOperation extends AbstractOperation {
  private final AttributeManager attributeManager;
  private final Element element;
  private final String attrName;
  private final String newValue;
  private String oldValue = null;

  // newValue == null for removing the attribute node.
  public AttrSetValueOperation(
    AttributeManager attributeManager, Element element, String attrName,
    String newValue) {
    if (attributeManager == null || element == null || attrName == null)
      throw new IllegalArgumentException();
    this.attributeManager = attributeManager;
    this.element = element;
    this.attrName = attrName;
    this.newValue = newValue;
  }

  public boolean isNull() {
    return (this.newValue == null) != (element.getAttributeNode(attrName) == null) ?
      false : this.newValue == null ? true :
      this.newValue.equals(element.getAttribute(attrName));
  }

  public Element execute(Document operationDocument)
    throws OperationException, IdException, UndefinedNamespaceException {
    if (this.isExecuted)
      throw new OperationException(OperationException.ILLEGAL_STATE);
    this.operationName = "Set value of \"" + attrName + "\" in "
      + OperationNameUtils.getNodeName(element);
    Element operationElement = this.createOperationElement(operationDocument);
    {
      Attr attr = this.element.getAttributeNode(attrName);
      oldValue = attr != null ? attr.getValue() : null;
        // pour la future cration de l'opration inverse
    }
    this.attributeManager.setAttribute(
        this.element, this.attrName, this.newValue);
    this.isExecuted = true;
    return operationElement;
  }

  private Element createOperationElement(Document operationDocument) {
    if (operationDocument == null)
      return null;
    try {
      Element operationElement =
          operationDocument.createElement("attrSetValueOperation");
      operationElement.setAttribute(
          "elementPath", NodePathBuilder.buildElementPathWithNames(element));
      operationElement.setAttribute("attrName", attrName);
      if (newValue == null)
        operationElement.setAttribute("isNull", "true");
      else
        operationElement.appendChild(
            operationDocument.createTextNode(newValue));
      return operationElement;
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
      return null;
    }
  }

  public boolean isHidable() {
    return true;
  }

  public boolean canHide(Operation operation) {
    return operation instanceof AttrSetValueOperation
      && this.element == ((AttrSetValueOperation)operation).element
      && this.attrName.equals(((AttrSetValueOperation)operation).attrName);
  }

  protected void hides(Operation operation) {
    this.oldValue = ((AttrSetValueOperation)operation).oldValue;
  }

  public Operation createReverseOperation() throws OperationException {
    if (!this.isExecuted)
      throw new OperationException(OperationException.ILLEGAL_STATE);
    AbstractOperation reverseOperation =
      new AttrSetValueOperation(
        this.attributeManager,
        this.element, this.attrName, this.oldValue);
    reverseOperation.reverseOperationName = this.operationName;
    return reverseOperation;
  }
}
