/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation.core;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.manage.NodeManager;
import org.xmloperator.framework.operation.OperationException;
import org.xmloperator.framework.operation.Operation;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.NodePathBuilder;
import org.xmloperator.operation.core.AbstractOperation;
import org.xmloperator.operation.core.OperationNameUtils;
import org.xmloperator.operation.core.MetaDocument;
import org.xmloperator.operation.core.NodeIntervalPackOperation; // import rciproque

public final class ElementUnpackOperation extends AbstractOperation {
  private final NodeManager nodeManager;
  private final Element element;
  private Node firstChildNode = null;
  private Node lastChildNode = null;

  public ElementUnpackOperation(NodeManager nodeManager, Element element) {
    if (element == null)
      throw new IllegalArgumentException();
    this.nodeManager = nodeManager;
    this.element = element;
  }

  public Element execute(Document operationDocument) {
    if (this.isExecuted)
      throw new OperationException(OperationException.ILLEGAL_STATE);
    Node parentNode = this.element.getParentNode();
    if (parentNode == null)
      throw new OperationException(OperationException.NODE_HAS_NO_PARENT);
    if (!this.element.hasChildNodes())
      throw new OperationException(OperationException.ELEMENT_HAS_NO_CHILD);
    this.operationName =
        "Unpack " + OperationNameUtils.getNodeName(this.element);
    Element operationElement = this.createOperationElement(operationDocument);
    this.firstChildNode = element.getFirstChild();
      // pour la future cration de l'opration inverse
    this.lastChildNode = element.getLastChild();
      // pour la future cration de l'opration inverse
    this.nodeManager.unPackElement(element);
    this.isExecuted = true;
    return operationElement;
  }

  private Element createOperationElement(Document operationDocument) {
    if (operationDocument == null)
      return null;
    try {
      Element operationElement =
          operationDocument.createElement("elementUnpackOperation");
      operationElement.setAttribute(
          "elementPath",
          NodePathBuilder.buildElementPathWithNames(this.element));
      return operationElement;
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
      return null;
    }
  }

  public Operation createReverseOperation() {
    if (!this.isExecuted)
      throw new OperationException(OperationException.ILLEGAL_STATE);
    AbstractOperation reverseOperation =
      new NodeIntervalPackOperation(
      this.nodeManager, this.firstChildNode, this.lastChildNode, this.element);
    reverseOperation.reverseOperationName = this.operationName;
    return reverseOperation;
  }
}
