/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation.core;

import java.util.StringTokenizer;
import java.util.Vector;

import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;

import org.xmloperator.util.dom.ElementBrowser;
import org.xmloperator.framework.operation.OperationException;

public abstract class MetaDocument {
  protected static final String ELEMENT_NAME_EXPECTED = "Element name expected";
  protected static final String ATTRIBUTE_NAME_EXPECTED =
    "Attribute name expected";
  protected static final String ATTRIBUTE_OR_NODE_EXPECTED =
    "<attribute> or <node> expected";
  protected static final String ENTITY_REFERENCE_NAME_EXPECTED =
    "Entity reference name expected";
  protected static final String PROCESSING_INSTRUCTION_TARGET_EXPECTED =
    "Processing instruction target expected";
  protected static final String NODE_TYPE_EXPECTED = "Node type expected";
  protected static final String NODE_ELEMENT_EXPECTED = "<node> expected";

  ///////////////////////////////////////////////////
  // Construire le mta-document  partir du document
  ///////////////////////////////////////////////////

  public static final Element createNodeElement(Document metaDocument,
    Node node) {
    Element nodeElement = metaDocument.createElement("node");
    nodeElement.appendChild(createInternalNodeElement(metaDocument, node));
    return nodeElement;
  }

  private static final Element createInternalNodeElement(
    Document metaDocument, Node node) {
    switch (node.getNodeType()) {
    case Node.ELEMENT_NODE: {
        Element nodeElement =
          metaDocument.createElement("element");
        nodeElement.setAttribute("name", node.getNodeName());
        NamedNodeMap attrs = ((Element)node).getAttributes();
        int attrCount = (attrs != null) ? attrs.getLength() : 0;
        for (int k = 0; k < attrCount; k++) {
          Node attr = attrs.item(k);
          Element attrElement = metaDocument.createElement("attribute");
          attrElement.setAttribute("name", attr.getNodeName());
          attrElement.setAttribute("value", attr.getNodeValue());
          nodeElement.appendChild(attrElement);
        }
        Node childNode = node.getFirstChild();
        while (childNode != null) {
          Element childNodeElement =
            createNodeElement(metaDocument, childNode);
          nodeElement.appendChild(childNodeElement);
          childNode = childNode.getNextSibling();
        }
        return nodeElement;
      }
    case Node.TEXT_NODE: {
        Element nodeElement = metaDocument.createElement("text");
        nodeElement.appendChild(
          metaDocument.createTextNode(node.getNodeValue()));
        return nodeElement;
      }
    case Node.CDATA_SECTION_NODE: {
        Element nodeElement = metaDocument.createElement("cdataSection");
        nodeElement.appendChild(
          metaDocument.createTextNode(node.getNodeValue()));
        return nodeElement;
      }
    case Node.ENTITY_REFERENCE_NODE: {
        Element nodeElement =
          metaDocument.createElement("entityReference");
        nodeElement.setAttribute("name", node.getNodeName());
        return nodeElement;
      }
    case Node.PROCESSING_INSTRUCTION_NODE: {
        Element nodeElement =
          metaDocument.createElement("processingInstruction");
        nodeElement.setAttribute("target", node.getNodeName());
        nodeElement.appendChild(
          metaDocument.createTextNode(node.getNodeValue()));
        return nodeElement;
      }
    case Node.COMMENT_NODE: {
        Element nodeElement = metaDocument.createElement("comment");
        nodeElement.appendChild(
          metaDocument.createTextNode(node.getNodeValue()));
        return nodeElement;
      }
    default:
      throw new IllegalArgumentException();
    }
  }

  ///////////////////////////////////////////////////
  // Construire le document  partir du mta-document
  ///////////////////////////////////////////////////

  public static final Node[] createNodes(Document document,
    Element nodeElementParent) {
    Vector nodeVector = new Vector();
    Node nodeElementChild = nodeElementParent.getFirstChild();
    while (nodeElementChild != null) {
      if (nodeElementChild.getNodeType() == Node.ELEMENT_NODE) {
        if (!nodeElementChild.getNodeName().equals("node"))
          throw new OperationException(NODE_ELEMENT_EXPECTED);
        Node node = createNode(document, (Element)nodeElementChild);
        if (node != null)
          nodeVector.add(node);
      }
      nodeElementChild = nodeElementChild.getNextSibling();
    }
    Node[] nodes = new Node[nodeVector.size()];
    nodeVector.toArray(nodes);
    return nodes;
  }

  public static final Node createNode(Document document, Element nodeElement) {
    Element internalNodeElement =
      ElementBrowser.getFirstChildElement(nodeElement);
    return internalNodeElement != null ?
      createNodeInternal(document, internalNodeElement) : null;
  }

  private static final Node createNodeInternal(Document document,
    Element nodeElement) {
    String nodeElementName = nodeElement.getNodeName();
    Node firstNodeElementChild = nodeElement.getFirstChild();
    if (nodeElementName.equals("element")) {
      String elementName = nodeElement.getAttribute("name");
      if (elementName.length() == 0)
        throw new OperationException(ELEMENT_NAME_EXPECTED);
      Element element = document.createElement(elementName);
      Node nodeElementChild = firstNodeElementChild;
      while (nodeElementChild != null) {
        if (nodeElementChild.getNodeType() == Node.ELEMENT_NODE) {
          String nodeElementChildName = nodeElementChild.getNodeName();
          if (nodeElementChildName.equals("attribute")) {
            String attrName = ((Element)nodeElementChild).getAttribute("name");
            if (attrName.length() == 0)
              throw new OperationException(ATTRIBUTE_NAME_EXPECTED);
            String attrValue =
              ((Element)nodeElementChild).getAttribute("value");
            element.setAttribute(attrName, attrValue);
          }
          else if (nodeElementChildName.equals("node")) {
            Node childNode = createNode(document, (Element)nodeElementChild);
            element.appendChild(childNode);
          }
          else
            throw new OperationException(ATTRIBUTE_OR_NODE_EXPECTED);
        }
        nodeElementChild = nodeElementChild.getNextSibling();
      }
      return element;
    }
    else if (nodeElementName.equals("text")) {
      String value = firstNodeElementChild != null ?
        firstNodeElementChild.getNodeValue() : "";
      Node node = document.createTextNode(value);
      return node;
    }
    else if (nodeElementName.equals("cdataSection")) {
      String value = firstNodeElementChild != null ?
        firstNodeElementChild.getNodeValue() : "";
      Node node = document.createCDATASection(value);
      return node;
    }
    else if (nodeElementName.equals("entityReference")) {
      String entityReferenceName = nodeElement.getAttribute("name");
      if (entityReferenceName.length() == 0)
        throw new OperationException(ENTITY_REFERENCE_NAME_EXPECTED);
      Node node = document.createEntityReference(entityReferenceName);
      return node;
    }
    else if (nodeElementName.equals("processingInstruction")) {
      String target = nodeElement.getAttribute("target");
      if (target.length() == 0)
        throw new OperationException(PROCESSING_INSTRUCTION_TARGET_EXPECTED);
      String value = firstNodeElementChild != null ?
        firstNodeElementChild.getNodeValue() : "";
      Node node = document.createProcessingInstruction(target, value);
      return node;
    }
    else if (nodeElementName.equals("comment")) {
      String value = firstNodeElementChild != null ?
        firstNodeElementChild.getNodeValue() : "";
      Node node = document.createComment(value);
      return node;
    }
    else
      throw new OperationException(NODE_TYPE_EXPECTED);
  }
}
