/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JComponent;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTabbedPane;
import javax.swing.BoxLayout;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.xmloperator.framework.util.Proxy;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.IntOption;
import org.xmloperator.framework.option.StringOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.instance.OptionsControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.DialogModal;
import org.xmloperator.option.impl.ProxyImpl;
import org.xmloperator.option.OptionsImpl;

public final class DialogOptions extends DialogModal {
  // Action possibles suite  modification
  public static final int ACTION_A_FAIRE_NEANT = 0;
  public static final int ACTION_A_FAIRE_RAFFRAICHISSEMENT_ARBRE_DOCUMENT = 1;
  public static final int ACTION_A_FAIRE_RECONSTRUCTION_FENETRE_OUTIL = 2;
  public static final int ACTION_A_FAIRE_RECONSTRUCTION_COMPLETE = 3;
  // Etats d'options
  private final String DEFAULT = viewResource.getString(Label.DEFAULT);
  private final String[] FALSE_TRUE = {
    viewResource.getString(Label.FALSE),
    viewResource.getString(Label.TRUE)};
  private final String[] NO_YES = {
    viewResource.getString(Label.NO),
    viewResource.getString(Label.YES)};
  private final String[] NORMALE_GRANDE = {
    viewResource.getString(Label.NORMALE),
    viewResource.getString(Label.GRANDE)};
  private final String[] PROPORTIONAL_MONOSPACE = {
    viewResource.getString(Label.PROPORTIONAL),
    viewResource.getString(Label.MONOSPACED)};
  private final String[] CACHE_AFFICHE = {
    viewResource.getString(Label.CACHE),
    viewResource.getString(Label.AFFICHE)};
  private final String[] ABSOLUTE_RELATIVE = {
    viewResource.getString(Label.ABSOLUTE),
    viewResource.getString(Label.RELATIVE)};
  private final String[] MULTIPLES_OF_10 =
    {"10", "20", "30", "40", "50", "60"};
  private final String[] POWERS_OF_10 =
    {"10", "100", "1000", "10000", "100000", "1000000"};
  // Thema definition data structure
  private static final int THEMADEF_LABEL = 0;
  private static final int THEMADEF_ADVANCED = 1;
  // Thema definition
  private static final int THEMA_GENERAL = 0;
  private static final int THEMA_PATH = 1;
  private static final int THEMA_LOADING = 2;
  private static final int THEMA_PARSING = 3;
  private static final int THEMA_SAVING = 4;
  private static final int THEMA_XSLT = 5;
  private static final int THEMA_OPERATIONS = 6;
  private static final int THEMA_IDENTIFICATION = 7;
  private static final int THEMA_NODETYPES = 8;
  private static final int THEMA_DISPLAY = 9;
  private static final int THEMA_STATUSBAR = 10;
  private static final Object[][] THEMA_INFO = {
    {Label.GENERAL, Boolean.FALSE},
    {Label.PATHS, Boolean.TRUE},
    {Label.CHARGEMENT, Boolean.TRUE},
    {Label.PARSING, Boolean.TRUE},
    {Label.ENREGISTREMENT, Boolean.TRUE},
    {Label.XSLT, Boolean.TRUE},
    {Label.OPERATIONS, Boolean.TRUE},
    {Label.IDENTIFICATION, Boolean.TRUE},
    {Label.NODE_TYPE, Boolean.TRUE},
    {Label.AFFICHAGE, Boolean.FALSE},
    {Label.STATUS_BAR, Boolean.TRUE}};
  // Option description data structure
  private static final int OPTIONDEF_THEMA = 0;
  private static final int OPTIONDEF_ID = 1;
  private static final int OPTIONDEF_STATES = 2;
  private static final int OPTIONDEF_FIRSTVALUE = 3;
  private static final int OPTIONDEF_LABEL = 4;
  private static final int OPTIONDEF_ACTION = 5;
  // Option description
  private final Object[][] optionDefinitions = {
    {new Integer(THEMA_GENERAL),     Options.ADVANCEDUSE,
      NO_YES,     null, Label.ADVANCED_USE,
      new Integer(ACTION_A_FAIRE_RECONSTRUCTION_FENETRE_OUTIL)},
    {new Integer(THEMA_PATH),     Options.XMLPATH,
      null,       null, Label.CHEMIN_DE_RECHERCHE_XML,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PATH),     Options.XSLPATH,
      null,       null, Label.CHEMIN_DE_RECHERCHE_XSL,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PATH),     Options.DTDPATH,
      null,       null, Label.CHEMIN_DE_RECHERCHE_DTD,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PATH),     Options.RNGPATH,
      null,       null, Label.CHEMIN_DE_RECHERCHE_RNG,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PATH),     Options.OPTIONSPATH,
      null,       null, Label.CHEMIN_DE_RECHERCHE_OPTIONS,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PATH),     Options.OPERATIONSPATH,
      null,       null, Label.CHEMIN_DE_RECHERCHE_OPERATIONS,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_LOADING),     Options.FILEEXTENSIONS,
      null,       null, Label.FILE_EXTENSIONS,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_LOADING),     Options.PROXY,
      null,       null, "Proxy http",
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_LOADING),     Options.OPENERRORS,
      NO_YES,     null, Label.OPEN_ERRORS_WINDOW_ON_ERRORS,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_LOADING),     Options.MAXRESOURCES,
      MULTIPLES_OF_10, null, Label.RESOURCES_MAX_COUNT,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_LOADING),     Options.DTDAUTOMATICLOADING,
      NO_YES,     null, Label.DTD_AUTOMATIC_LOADING,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_LOADING),     Options.RNGAUTOMATICLOADING,
      NO_YES,     null, Label.RNG_AUTOMATIC_LOADING,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PARSING),        Options.PARSINGVALIDATION,
      FALSE_TRUE, null, "xml.org/sax/features/validation",
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PARSING),        Options.PARSINGVALIDATIONDYNAMIC,
      FALSE_TRUE, null, "apache.org/xml/features/validation/dynamic",
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PARSING),        Options.PARSINGVALIDATIONSCHEMA,
      FALSE_TRUE, null, "apache.org/xml/features/validation/schema",
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PARSING),        Options.PARSINGNONVALIDATINGLOADDTDGRAMMAR,
      FALSE_TRUE, null, "apache.org/xml/features/nonvalidating/load-dtd-grammar",
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PARSING),        Options.PARSINGNONVALIDATINGLOADEXTERNALDTD,
      FALSE_TRUE, null, "apache.org/xml/features/nonvalidating/load-external-dtd",
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_PARSING),        Options.PARSINGDOMINCLUDEIGNORABLEWHITESPACE,
      FALSE_TRUE, null, "apache.org/xml/features/dom/include-ignorable-whitespace",
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_SAVING), Options.DEFAULTENCODING,
      null,       null, Label.DEFAULT_ENCODING,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_SAVING), Options.FILENAMEAUTOMATICINCREMENTING,
      NO_YES,     null, Label.FILE_NAME_AUTOMATIC_INCREMENTING,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_SAVING), Options.DOCTYPEIFDTD,
      NO_YES,     null, Label.DOCTYPE_IF_DTD,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_SAVING), Options.ADDLINEFEEDATSTARTTAGEND,
      NO_YES,     null, Label.ADD_LINEFEED_AT_STARTTAGEND,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_XSLT),           Options.XSLTTRACEPATHNAME,
      null,       null, Label.TRACE_PATH_NAME,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_XSLT),           Options.TRACEELEMENTS,
      NO_YES,     null, Label.TRACE_ELEMENTS,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_XSLT),           Options.TRACEGENERATION,
      NO_YES,     null, Label.TRACE_GENERATION,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_XSLT),           Options.TRACESELECTION,
      NO_YES,     null, Label.TRACE_SELECTION,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_XSLT),           Options.TRACETEMPLATES,
      NO_YES,     null, Label.TRACE_TEMPLATES,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_XSLT),           Options.QUIETCONFLICTWARNINGS,
      NO_YES,     null, Label.QUIET_CONFLICT_WARNINGS,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_OPERATIONS),     Options.OPERATIONSSTORED,
      NO_YES,     null, Label.SAUVEGARDE_DES_OPERATIONS,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_OPERATIONS),     Options.CLEARUNDOAFTERSAVE,
      NO_YES,     null, Label.EFFACEMENT_DE_DEFAIRE_APRES_ENREGISTREMENT,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_IDENTIFICATION), Options.IDLENGTH,
      interval(2, 7), new Integer(2), Label.LONGUEUR_IDENTIFIANT,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_IDENTIFICATION), Options.IDPREFIX,
      null,       null, Label.ID_PREFIX,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_NODETYPES),     Options.ALLOWSCOMMENT,
      NO_YES,     null, Label.ALLOWS_COMMENTS,
      new Integer(ACTION_A_FAIRE_RAFFRAICHISSEMENT_ARBRE_DOCUMENT)},
    {new Integer(THEMA_NODETYPES),     Options.ALLOWSPROCESSINGINSTRUCTION,
      NO_YES,     null, Label.ALLOWS_PROCESSING_INSTRUCTIONS,
      new Integer(ACTION_A_FAIRE_RAFFRAICHISSEMENT_ARBRE_DOCUMENT)},
    {new Integer(THEMA_NODETYPES),     Options.ALLOWSCDATASECTION,
      NO_YES,     null, Label.ALLOWS_CDATA_SECTIONS,
      new Integer(ACTION_A_FAIRE_RAFFRAICHISSEMENT_ARBRE_DOCUMENT)},
    {new Integer(THEMA_NODETYPES),     Options.ALLOWSENTITYREFERENCE,
      NO_YES,     null, Label.ALLOWS_ENTITY_REFERENCES,
      new Integer(ACTION_A_FAIRE_RAFFRAICHISSEMENT_ARBRE_DOCUMENT)},
    {new Integer(THEMA_DISPLAY),      Options.FONT,
      NORMALE_GRANDE, null, Label.FONTE,
      new Integer(ACTION_A_FAIRE_RECONSTRUCTION_FENETRE_OUTIL)},
    {new Integer(THEMA_DISPLAY),      Options.MONOSPACED,
      PROPORTIONAL_MONOSPACE, null, Label.FONTE,
      new Integer(ACTION_A_FAIRE_RECONSTRUCTION_FENETRE_OUTIL)},
    {new Integer(THEMA_DISPLAY),      Options.AUTOMATICLABELLING,
      NO_YES,     null, Label.AUTOMATIC_ELEMENT_LABELLING,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_DISPLAY),      Options.MESSAGEPERSISTENCE,
      interval(0, 5), null, Label.MESSAGE_PERSISTENCE_TIME_SECOND,
      new Integer(ACTION_A_FAIRE_NEANT)},
    {new Integer(THEMA_DISPLAY),      Options.LOCALEINDEX,
      localesDisplay(), new Integer(1), Label.LANGUE,
      new Integer(ACTION_A_FAIRE_RECONSTRUCTION_COMPLETE)},
    {new Integer(THEMA_DISPLAY),      Options.LOOKANDFEELINDEX,
      Constants.lookAndFeelClassNames, null, "Look and feel",
      new Integer(ACTION_A_FAIRE_RECONSTRUCTION_COMPLETE)},
    {new Integer(THEMA_STATUSBAR),   Options.STATUSSHOWN,
      CACHE_AFFICHE, null, Label.STATUS,
      new Integer(ACTION_A_FAIRE_RECONSTRUCTION_FENETRE_OUTIL)},
    {new Integer(THEMA_STATUSBAR),   Options.LAZYSHOWN,
      CACHE_AFFICHE, null, "Lazy",
      new Integer(ACTION_A_FAIRE_RECONSTRUCTION_FENETRE_OUTIL)}};
  // Arguments
  private Options options;
  private boolean isAdvancedOptions;
  // Composants
  private JTabbedPane voletAOnglet;
  private final OptionComponent[] optionComponents =
    new OptionComponent[optionDefinitions.length];
  // Rsultats
  private int actionAFaire = ACTION_A_FAIRE_NEANT;

  private static final String[] interval(int min, int max) {
    String[] interval = new String[max - min + 1];
    for (int k = 0; k < interval.length; k++)
      interval[k] = Integer.toString(min + k);
    return interval;
  }

  private final String[] localesDisplay() {
    String[] localesDisplay = new String[Constants.locales.length];
    localesDisplay[0] = DEFAULT;
    for (int k = 1; k < Constants.locales.length; k++)
      localesDisplay[k] = Constants.locales[k].getDisplayLanguage();
    return localesDisplay;
  }

  public DialogOptions(Frame parent, Options options,
    boolean isAdvancedOptions, int selectedThemaIndex,
    boolean isLookAndFeelModifiable) {
    super(parent);
    this.options = options;
    this.isAdvancedOptions = isAdvancedOptions;
    setTitle(viewResource.getString(Label.OPTIONS));
    voletAOnglet = new JTabbedPane();
    contentPanel.add(voletAOnglet);
    JComponent voletsContenuTheme[] = new JComponent[THEMA_INFO.length];
    for (int themaIndex = 0; themaIndex < THEMA_INFO.length; themaIndex++) {
      JPanel voletTheme = new JPanel(new BorderLayout());
      voletTheme.setPreferredSize(new Dimension(520, 260));
      JComponent voletContenuTheme = new JPanel();
      voletContenuTheme.setLayout(
        new BoxLayout(voletContenuTheme, BoxLayout.Y_AXIS));
      voletsContenuTheme[themaIndex] = voletContenuTheme;
      voletTheme.add(voletContenuTheme, BorderLayout.NORTH);
      Object[] themaInfo = THEMA_INFO[themaIndex];
      if (isAdvancedOptions
        || !((Boolean)themaInfo[THEMADEF_ADVANCED]).booleanValue())
        voletAOnglet.addTab(
          viewResource.getString((String)themaInfo[THEMADEF_LABEL]),
          voletTheme);
    }
    for (int optionDefIndex = 0; optionDefIndex < optionDefinitions.length;
      optionDefIndex++) {
      Object[] optionDefinition = optionDefinitions[optionDefIndex];
      String optionId = (String)optionDefinition[OPTIONDEF_ID];
      int optionIndex = OptionsImpl.getOptionIndex(optionId);
      if (!isAdvancedOptions && OptionsImpl.isAdvancedOption(optionIndex))
        continue;
      int optionTypeId = OptionsImpl.getOptionTypeId(optionIndex);
      OptionComponent optionComponent;
      switch (optionTypeId) {
        case OptionsImpl.OPTIONTYPE_THREESTATE:
          optionComponent = new ThreeStateOptionComponent(optionId);
          break;
        case OptionsImpl.OPTIONTYPE_INTEGER:
          Object firstValueObject = optionDefinition[OPTIONDEF_FIRSTVALUE];
          int firstValue = firstValueObject != null ?
          ((Integer)firstValueObject).intValue() : 0;
          optionComponent = new IntOptionComponent(optionId, firstValue);
          break;
        case OptionsImpl.OPTIONTYPE_STRING:
          optionComponent = new StringOptionComponent(optionId);
          break;
        case OptionsImpl.OPTIONTYPE_PROXY:
          optionComponent = new ProxyOptionComponent(optionId);
          break;
        default:
          throw new RuntimeException("Option type not supported");
      }
      optionComponents[optionDefIndex] = optionComponent;
      int themaIndex = ((Integer)optionDefinition[OPTIONDEF_THEMA]).intValue();
      String label = (String)optionDefinition[OPTIONDEF_LABEL];
      String[] states = (String[])optionDefinition[OPTIONDEF_STATES];
      boolean isEnabled = !optionId.equals(Options.LOOKANDFEELINDEX)
        || isLookAndFeelModifiable;
      voletsContenuTheme[themaIndex].add(
        optionComponent.createComponent(label, states, isEnabled));
    }
    // Slection de volet
    select(selectedThemaIndex);
    // Mise au dimensions
    pack();
  }

  public int getSelectedThemaIndex() {
    String selectedTitle =
      voletAOnglet.getTitleAt(voletAOnglet.getSelectedIndex());
    for (int themaIndex = 0; themaIndex < THEMA_INFO.length; themaIndex++)
      if (viewResource.getString(
        (String)THEMA_INFO[themaIndex][THEMADEF_LABEL]).equals(selectedTitle))
        return themaIndex;
    return 0;
  }

  private void select(int themaIndex) {
    String selectedTitle =
      viewResource.getString((String)THEMA_INFO[themaIndex][THEMADEF_LABEL]);
    for (int k = 0; k < voletAOnglet.getTabCount(); k++)
      if (voletAOnglet.getTitleAt(k).equals(selectedTitle)) {
        voletAOnglet.setSelectedIndex(k);
        return;
      }
    voletAOnglet.setSelectedIndex(0);
  }

  protected void buttonOK_actionPerformed(ActionEvent event) {
    try {
      for (int optionDefIndex = 0; optionDefIndex < optionDefinitions.length;
        optionDefIndex++) {
        Object[] optionDefinition = optionDefinitions[optionDefIndex];
        String optionId = (String)optionDefinition[OPTIONDEF_ID];
        int optionIndex = OptionsImpl.getOptionIndex(optionId);
        if (!isAdvancedOptions && OptionsImpl.isAdvancedOption(optionIndex))
          continue;
        if (optionComponents[optionDefIndex].setOptionValue()) {
          int actionAFaireEnCasDeChangement =
            ((Integer)optionDefinition[OPTIONDEF_ACTION]).intValue();
          if (actionAFaireEnCasDeChangement > actionAFaire)
            actionAFaire = actionAFaireEnCasDeChangement;
        }
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    super.buttonOK_actionPerformed(event);
  }

  public int getActionAFaire() {
    return actionAFaire;
  }

  private abstract class OptionComponent {
    protected String optionId;

    public OptionComponent(String optionId) {
      this.optionId = optionId;
    }

    public abstract JComponent createComponent(String labelKey, String[] states,
      boolean isEnabled);
      // "states" n'est pas toujours utilis

    public abstract boolean setOptionValue();
      // rend true si la valeur change

    protected JComponent createComponent(String labelKey) {
      JPanel panel = new JPanel();
      panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
      panel.setBorder(new EmptyBorder(1, 4, 1, 4));
      String label;
      try {
        label = viewResource.getString(labelKey);
      }
      catch (Exception exception) {
        label = labelKey;
      }
      panel.add(new JLabel(label + " : "));
      return panel;
    }
  }

  private class StringOptionComponent extends OptionComponent {
    protected JTextField textField = null;
    protected JCheckBox defaultCheckBox = null;

    public StringOptionComponent(String optionId) {
      super(optionId);
    }

    public JComponent createComponent(String labelKey, String[] states,
      boolean isEnabled) {
      StringOption option = (StringOption)options.getOption(optionId);
      String value = option != null ? option.getStringValue() : null;
      if (value != null && value.length() == 0)
        value = null;
      String defaultValue =
        options.createStringOption(optionId, "").getStringDefaultValue();
      JComponent component = createComponent(labelKey);
      textField = new JTextField();
      textField.setText(value != null ? value : defaultValue);
      textField.setEnabled(isEnabled && value != null);
      component.add(textField);
      defaultCheckBox = new JCheckBox(DEFAULT, value == null);
      defaultCheckBox.setEnabled(isEnabled);
      defaultCheckBox.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent event) {
            defaultCheckBox_actionPerformed();
          }
        });
      component.add(defaultCheckBox);
      return component;
    }

    protected void defaultCheckBox_actionPerformed() {
      boolean isDefault = defaultCheckBox.getModel().isSelected();
      String defaultValue =
        options.createStringOption(optionId, "").getStringDefaultValue();
      String value = options.getStringOptionValue(optionId);
      textField.setText(isDefault ? defaultValue : value);
      textField.setEnabled(!isDefault);
    }

    public boolean setOptionValue() {
      boolean isDefault = defaultCheckBox.getModel().isSelected();
      String newValue = isDefault ? "" : textField.getText();
      boolean isChanged =
        !newValue.equals(options.getStringOptionValue(optionId));
      if (isChanged)
        options.setStringOptionValue(optionId, newValue);
      return isChanged;
    }
  }

  private class ThreeStateOptionComponent extends OptionComponent {
    protected JComboBox comboBox = null;

    public ThreeStateOptionComponent(String optionId) {
      super(optionId);
    }

    public JComponent createComponent(String labelKey, String[] states,
      boolean isEnabled) {
      if (states == null || states.length != 2)
        throw new IllegalArgumentException();
      int defaultValue =
        options.createThreeStateOption(optionId, ThreeStateOption.UNDEFINED).
        getThreeStateDefaultValue();
      String[] statesIncludingDefault = new String[3];
      statesIncludingDefault[0] = (defaultValue != ThreeStateOption.UNDEFINED) ?
        DEFAULT + " (" + states[defaultValue] + ")" : DEFAULT;
      statesIncludingDefault[1] = states[0];
      statesIncludingDefault[2] = states[1];
      int value = options.getThreeStateOptionValue(optionId);
      comboBox = new JComboBox(statesIncludingDefault);
      comboBox.setSelectedIndex(value + 1);
      comboBox.setEnabled(isEnabled);
      JComponent component = createComponent(labelKey);
      component.add(comboBox);
      return component;
    }

    public boolean setOptionValue() {
      int newIndex = comboBox.getSelectedIndex();
      int newValue = newIndex > 0 ? newIndex - 1 : -1;
      boolean isChanged =
        newValue != options.getThreeStateOptionValue(optionId);
      if (isChanged)
        options.setThreeStateOptionValue(optionId, newValue);
      return isChanged;
    }
  }

  private class IntOptionComponent extends OptionComponent {
    protected int firstValue = 0;
    protected JComboBox comboBox = null;
    protected String[] enumeratedValues = null;

    public IntOptionComponent(String optionId) {
      this(optionId, 0);
    }

    public IntOptionComponent(String optionId, int firstValue) {
      super(optionId);
      if (firstValue < 0)
        throw new IllegalArgumentException();
      this.firstValue = firstValue;
    }

    public JComponent createComponent(String labelKey, String[] states,
      boolean isEnabled) {
      int defaultValue =
        options.createIntOption(optionId, IntOption.UNDEFINED).
        getIntDefaultValue();
      int value = options.getIntOptionValue(optionId);
      if (states == MULTIPLES_OF_10 || states == POWERS_OF_10) {
        enumeratedValues = states;
        defaultValue = indexWithin(defaultValue, enumeratedValues);
        value = indexWithin(value, enumeratedValues);
      }
      String[] statesIncludingDefault = states;
      if (states.length == 0 || !states[0].equals(DEFAULT)) {
        statesIncludingDefault = new String[states.length + 1];
        statesIncludingDefault[0] = (defaultValue != IntOption.UNDEFINED) ?
          DEFAULT + " (" + states[defaultValue - firstValue] + ")" : DEFAULT;
        System.arraycopy(states, 0, statesIncludingDefault, 1, states.length);
      }
      comboBox = new JComboBox(statesIncludingDefault);
      comboBox.setSelectedIndex(
        value >= firstValue
        && value < firstValue + statesIncludingDefault.length - 1 ?
        1 + value - firstValue : 0);
      comboBox.setEnabled(isEnabled);
      JComponent component = createComponent(labelKey);
      component.add(comboBox);
      return component;
    }

    private int indexWithin(int value, String[] enumeratedValues) {
      for (int k = 0; k < enumeratedValues.length; k++)
        if (value == new Integer(enumeratedValues[k]).intValue())
          return k;
      return -1;
    }

    public boolean setOptionValue() {
      int newIndex = comboBox.getSelectedIndex();
      int newValue = newIndex > 0 ? firstValue + newIndex - 1 : -1;
      if (newValue >= 0 && enumeratedValues != null)
        newValue = new Integer(enumeratedValues[newValue]).intValue();
      boolean isChanged = newValue != options.getIntOptionValue(optionId);
      if (isChanged)
        options.setIntOptionValue(optionId, newValue);
      return isChanged;
    }
  }

  private class ProxyOptionComponent extends OptionComponent {
    protected JTextField[] textFieldAddress = new JTextField[4];
    protected JTextField textFieldPort = null;

    public ProxyOptionComponent(String optionId) {
      super(optionId);
    }

    public JComponent createComponent(String labelKey, String[] states,
      boolean isEnabled) {
      Proxy proxy = (Proxy)options.getOption(optionId);
      JComponent component = createComponent(labelKey);
      for (int k = 0; k < 4; k++) {
        JTextField textField = new JTextField(4);
        textField.setHorizontalAlignment(JTextField.RIGHT);
        if (proxy != null) {
          int ak = proxy.getAddress(k);
          if (ak >= 0 && ak <= 255)
            textField.setText(new Integer(ak).toString());
        }
        textField.setEnabled(isEnabled);
        textFieldAddress[k] = textField;
        if (k != 0)
          component.add(new JLabel("."));
        component.add(textField);
      }
      textFieldPort = new JTextField(5);
      textFieldPort.setHorizontalAlignment(JTextField.RIGHT);
      int port = proxy != null ? proxy.getPort() : -1;
      if (port >= 0 && port <= 65535)
        textFieldPort.setText(new Integer(port).toString());
      textFieldPort.setEnabled(isEnabled);
      component.add(new JLabel(" Port : "));
      component.add(textFieldPort);
      return component;
    }

    public boolean setOptionValue() {
      ProxyImpl newProxy = new ProxyImpl();
      for (int k = 0; k < 4; k++)
        newProxy.setAddress(k, intValue(textFieldAddress[k]));
      newProxy.setPort(intValue(textFieldPort));
      boolean isChanged = !newProxy.equals(options.getOption(optionId));
      if (isChanged)
        options.setOption(optionId, newProxy);
      return isChanged;
    }

    private int intValue(JTextField textField) {
      try {
        return new Integer(textField.getText()).intValue();
      }
      catch (Exception exception) {
        return -1;
      }
    }
  }
}
