/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option.impl;

import java.awt.event.ActionListener;

import org.xmloperator.framework.option.part.OptionPartType;
import org.xmloperator.framework.option.part.BooleanOptionPart;
import org.xmloperator.framework.option.part.StringOptionPart;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.Option;
import org.xmloperator.framework.option.MenuItemData;
import org.xmloperator.option.part.BooleanOptionPartType;
import org.xmloperator.option.part.BooleanOptionPartImpl;
import org.xmloperator.option.part.StringOptionPartType;
import org.xmloperator.option.part.StringOptionPartImpl;
import org.xmloperator.option.core.OptionTypeImpl;
import org.xmloperator.option.core.OptionImpl;

public final class MenuItemDataImpl extends OptionImpl implements MenuItemData {
  private static final int LABEL = 0;
  private static final int COMMAND = 1;
  private static final int TOCLOSE = 2;
  public static final OptionType optionType =
    new OptionTypeImpl(
      new String[] {
        "label",
        "command",
        "toClose"},
      new OptionPartType[] {
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new BooleanOptionPartType(false)}) {
      public Option createOption() {
        return new MenuItemDataImpl();
      }
    };
  private ActionListener actionListener = null;

  public MenuItemDataImpl() {
    super(optionType);
  }

  public MenuItemDataImpl(String label, ActionListener actionListener,
    String command) {
    this(label, actionListener, command, false);
  }

  public MenuItemDataImpl(String label, String command,
    boolean hasToolInstanceToBeClosed) {
    this(label, null, command, hasToolInstanceToBeClosed);
  }

  private MenuItemDataImpl(String label, ActionListener actionListener,
    String command, boolean hasToolInstanceToBeClosed) {
    this();
    if (label == null || (actionListener == null && command == null))
      throw new IllegalArgumentException();
    setLabel(label);
    this.actionListener = actionListener;
    if (command != null)
      setCommand(command);
    setToolInstanceToBeClosed(hasToolInstanceToBeClosed);
  }

  //////////////////////////////
  // Implementation MenuItemData
  //////////////////////////////

  public final String getLabel() {
    return ((StringOptionPartImpl)getOptionPart(LABEL)).getStringValue();
  }

  public final void setLabel(String label) {
    if (label == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(LABEL)).setStringValue(label);
  }

  public final ActionListener getActionListener() {
    return actionListener;
  }

  public final void setActionListener(ActionListener actionListener) {
    if (actionListener == null)
      throw new IllegalArgumentException();
    this.actionListener = actionListener;
  }

  public final String getCommand() {
    return ((StringOptionPartImpl)getOptionPart(COMMAND)).getStringValue();
  }

  public final void setCommand(String command) {
    if (command == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(COMMAND)).setStringValue(command);
  }

  public final boolean hasToolInstanceToBeClosed() {
    return ((BooleanOptionPartImpl)getOptionPart(TOCLOSE)).getBooleanValue();
  }

  public final void setToolInstanceToBeClosed(
    boolean hasToolInstanceToBeClosed) {
    ((BooleanOptionPartImpl)getOptionPart(TOCLOSE)).
      setBooleanValue(hasToolInstanceToBeClosed);
  }
}
