/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option.impl;

import java.util.List;

import org.xmloperator.framework.document.util.Parameter;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.part.OptionPartType;
import org.xmloperator.framework.option.part.StringOptionPart;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.Option;
import org.xmloperator.option.part.StringOptionPartType;
import org.xmloperator.option.part.StringOptionPartImpl;
import org.xmloperator.option.core.OptionTypeImpl;
import org.xmloperator.option.core.OptionImpl;

public class ParameterImpl extends OptionImpl implements Parameter {
  private static final int NAME = 0;
  private static final int VALUE = 1;
  public static final OptionType optionType =
    new OptionTypeImpl(
      new String[] {
        "name",
        "value"},
      new OptionPartType[] {
        new StringOptionPartType(""),
        new StringOptionPartType("")}) {
      public Option createOption() {
        return new ParameterImpl();
      }
    };

  public static final void initializeListIfEmpty(List list) {
    if (!list.isEmpty())
      return;
    list.add(new ParameterImpl("stylesheet", Constants.TOOL + ".css"));
    list.add(new ParameterImpl("homeURL", "index.html"));
    list.add(new ParameterImpl("homeLabel", "Home"));
    list.add(new ParameterImpl("copyright", Constants.COPYRIGHT));
  }

  private ParameterImpl() {
    super(optionType);
  }

  public ParameterImpl(String name) {
    this(name, "");
  }

  public ParameterImpl(String name, String value) {
    this();
    if (name == null || name.length() == 0 || value == null)
      throw new IllegalArgumentException();
    setName(name);
    setValue(value);
  }

  private void setName(String name) {
    ((StringOptionPartImpl)getOptionPart(NAME)).setStringValue(name);
  }

  ///////////////////////////
  // Parameter Implementation
  ///////////////////////////

  public String getName() {
    return ((StringOptionPartImpl)getOptionPart(NAME)).getStringValue();
  }

  public String getValue() {
    return ((StringOptionPartImpl)getOptionPart(VALUE)).getStringValue();
  }

  public void setValue(String value) {
    if (value == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(VALUE)).setStringValue(value);
  }
}
