/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option.impl;

import org.xmloperator.framework.option.part.OptionPartType;
import org.xmloperator.framework.option.part.IntOptionPart;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.Option;
import org.xmloperator.framework.util.Proxy;
import org.xmloperator.option.part.IntOptionPartType;
import org.xmloperator.option.part.IntOptionPartImpl;
import org.xmloperator.option.core.OptionTypeImpl;
import org.xmloperator.option.core.OptionImpl;

public final class ProxyImpl extends OptionImpl implements Proxy {
  private static final int ADDRESS = 0;
  private static final int PORT = 4;
  public static final OptionType optionType =
    new OptionTypeImpl(
      new String[] {
        "a1",
        "a2",
        "a3",
        "a4",
        "port"},
      new OptionPartType[] {
        new IntOptionPartType(-1),
        new IntOptionPartType(-1),
        new IntOptionPartType(-1),
        new IntOptionPartType(-1),
        new IntOptionPartType(-1)}) {
      public Option createOption() {
        return new ProxyImpl();
      }
    };

  public ProxyImpl() {
    super(optionType);
  }

  //////////////////
  // Object overload
  //////////////////

  public boolean equals(Object object) {
    if (object == null || !(object instanceof ProxyImpl))
      return false;
    for (int k = 0; k < 4; k++)
      if (getAddress(k) != ((ProxyImpl)object).getAddress(k))
        return false;
    return getPort() == ((ProxyImpl)object).getPort();
  }

  public String toString() {
    return getAddress();
  }

  ///////////////////////
  // Proxy implementation
  ///////////////////////

  public final boolean isValid() {
    for (int k = 0; k < 4; k++) {
      int ak = getAddress(k);
      if (ak < 0 || ak > 255)
        return false;
    }
    int port = getPort();
    return (port >= 0 && port <= 65535);
  }

  public final int getAddress(int index) {
    if (index < 0 || index > 3)
      throw new IllegalArgumentException();
    return ((IntOptionPartImpl)getOptionPart(ADDRESS + index)).getIntValue();
  }

  public final void setAddress(int index, int value) {
    if (index < 0 || index > 3)
      throw new IllegalArgumentException();
    ((IntOptionPartImpl)getOptionPart(ADDRESS + index)).setIntValue(value);
  }

  public final String getAddress() {
    StringBuffer sb = new StringBuffer();
    for (int k = 0; k < 4; k++) {
      if (k != 0)
        sb.append('.');
      sb.append(getAddress(k));
    }
    return sb.toString();
  }

  public final int getPort() {
    return ((IntOptionPartImpl)getOptionPart(PORT)).getIntValue();
  }

  public final void setPort(int port) {
    ((IntOptionPartImpl)getOptionPart(PORT)).setIntValue(port);
  }
}
