/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option.impl;

import java.util.List;
import java.util.ArrayList;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.part.OptionPartType;
import org.xmloperator.framework.option.part.BooleanOptionPart;
import org.xmloperator.framework.option.part.StringOptionPart;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.Option;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.transform.
    TransformerInfoXMLResultPart;
import org.xmloperator.framework.document.transform.TransformerInfo;
import org.xmloperator.option.part.BooleanOptionPartType;
import org.xmloperator.option.part.BooleanOptionPartImpl;
import org.xmloperator.option.part.StringOptionPartType;
import org.xmloperator.option.part.StringOptionPartImpl;
import org.xmloperator.option.core.OptionTypeImpl;
import org.xmloperator.option.core.OptionImpl;

public final class TransformerInfoImpl extends OptionImpl
  implements TransformerInfo, TransformerInfoXMLResultPart, Comparable {
  private static final int NAME = 0;
  private static final int TYPENAME = 1;
  private static final int CLASSNAME = 2;
  private static final int STYLESHEETSYSTEMID = 3;
  private static final int PARAMNAMES = 4;
  private static final int ISRESULTXML = 5;
  private static final int DTDPUBLICID = 6;
  private static final int DTDSYSTEMID = 7;
  private static final int ENCODINGNAME = 8;
  private static final int ISSTANDALONEDOCUMENT = 9;
  private static final int ISEMPTYELEMENTSHORTCUTUSED = 10;
  public static final OptionType optionType =
    new OptionTypeImpl(
      new String[] {
        "name",
        "typeName",
        "className",
        "styleSheetSystemId",
        "paramNames",
        "isResultXML",
        "dTDPublicId",
        "dTDSystemId",
        "encodingName",
        "isStandaloneDocument",
        "isEmptyElementShortcutUsed"},
      new OptionPartType[] {
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new BooleanOptionPartType(true),
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType("UTF-8"),
        new BooleanOptionPartType(false),
        new BooleanOptionPartType(false)}) {
      public Option createOption() {
        return new TransformerInfoImpl();
      }
    };
    private String lastDestinationPathname = ""; // Not persistent

  public static final void initializeListIfEmpty(Options options, List list) {
    if (!list.isEmpty())
      return;
    final String HTML_DEFAULT_ENCODING = "ISO-8859-1";
    final String xSLPath = options.getStringOptionValue(Options.XSLPATH);
    final String toolParamNames = "stylesheet homeURL homeLabel copyright";
    list.add(new TransformerInfoImpl("SchemaToHtml", "schema",
      "org.xmloperator.transform.SchemaToHtmlTransformer", null, toolParamNames,
      HTML_DEFAULT_ENCODING, false));
    list.add(new TransformerInfoImpl("SchemaToHtmlByXSLT", "schema",
      Constants.XSLT_TRANSFORMER, xSLPath + "schemaToHtml.xsl", toolParamNames,
      HTML_DEFAULT_ENCODING, false));
    /* Deprecated by JavaSkeleton
    list.add(new TransformerInfoImpl("DependToHtml", "depend",
      "org.xmloperator.transform.DependToHtmlTransformer", null, toolParamNames,
      HTML_DEFAULT_ENCODING, false));
    list.add(new TransformerInfoImpl("DependToHtmlByXSLT", "depend",
      Constants.XSLT_TRANSFORMER, xSLPath + "dependToHtml.xsl", toolParamNames,
      HTML_DEFAULT_ENCODING, false));
     */
    list.add(new TransformerInfoImpl("RelaxNGToHtml", "",
      Constants.XSLT_TRANSFORMER, xSLPath + "relaxngToHtml.xsl", "title",
      HTML_DEFAULT_ENCODING, false));
    list.add(new TransformerInfoImpl("CSSXMLToCSS", "css",
      Constants.XSLT_TRANSFORMER, xSLPath + "CSSXMLToCSS.xsl", ""));
    list.add(new TransformerInfoImpl("JavaSkeletonToHtml", "javaSkeleton",
      Constants.XSLT_TRANSFORMER, xSLPath + "skeletonToHtml.xsl", "title",
      HTML_DEFAULT_ENCODING, false));
  }

  public static final List createTransformerInfoSubList(
    List transformerInfoList, String documentTypeName) {
    List subList = new ArrayList();
    for (int k = 0; k < transformerInfoList.size(); k++) {
      TransformerInfo transformerInfo =
        (TransformerInfo)transformerInfoList.get(k);
      String transformerTypeName = transformerInfo.getDocumentTypeName();
      if (documentTypeName == null || documentTypeName.length() == 0
          || transformerTypeName == null || transformerTypeName.length() == 0
          || documentTypeName.equals(transformerTypeName))
        subList.add(transformerInfo);
    }
    return subList;
  }

  public static final TransformerInfo getTransformerInfo(
    List transformerInfoList, String transformerName) {
    for (int k = 0; k < transformerInfoList.size(); k++) {
      TransformerInfo transformerInfo =
        (TransformerInfo)transformerInfoList.get(k);
      if (transformerName.equals(transformerInfo.getTransformerName()))
        return transformerInfo;
    }
    return null;
  }

  public TransformerInfoImpl() {
    super(optionType);
  }

  // For a plain text output
  public TransformerInfoImpl(
    String transformerName, String documentTypeName,
    String transformerClassName, String stylesheetSystemId, String paramNames) {
    this();
    if (transformerName == null || transformerClassName == null
        || paramNames == null)
      throw new IllegalArgumentException();
    setTransformerName(transformerName);
    setDocumentTypeName(documentTypeName);
    setTransformerClassName(transformerClassName);
    setStylesheetSystemId(stylesheetSystemId);
    setParamNames(paramNames);
    setResultXML(false);
  }

  // For an XML output
  public TransformerInfoImpl(
    String transformerName, String documentTypeName,
    String transformerClassName, String stylesheetSystemId, String paramNames,
    String dTDPublicId, String dTDSystemId,
    String encodingName, boolean isStandaloneDocument,
    boolean isShortcutUsedForEmptyElement) {
    this(transformerName, documentTypeName, transformerClassName,
         stylesheetSystemId, paramNames);
    setResultXML(true);
    setResultDTDPublicId(dTDPublicId);
    setResultDTDSystemId(dTDSystemId);
    setEncodingName(encodingName);
    setStandaloneDocument(isStandaloneDocument);
    setShortcutUsedForEmptyElement(isShortcutUsedForEmptyElement);
  }

  // For an XML output without DOCTYPE
  public TransformerInfoImpl(
      String transformerName, String documentTypeName,
      String transformerClassName, String stylesheetSystemId, String paramNames,
      String encodingName, boolean isShortcutUsedForEmptyElement) {
    this(transformerName, documentTypeName, transformerClassName,
         stylesheetSystemId, paramNames, null, null, encodingName, true,
         isShortcutUsedForEmptyElement);
  }

  public TransformerInfoImpl(
    TransformerInfo transformerInfo) {
    this();
    setTransformerName(transformerInfo.getTransformerName());
    setDocumentTypeName(transformerInfo.getDocumentTypeName());
    setTransformerClassName(transformerInfo.getTransformerClassName());
    setStylesheetSystemId(transformerInfo.getStylesheetSystemId());
    setParamNames(transformerInfo.getParamNames());
    setResultXML(transformerInfo.isResultXML());
    TransformerInfoXMLResultPart xMLResultPart =
        transformerInfo.getXMLResultPart();
    setResultDTDPublicId(xMLResultPart.getResultDTDPublicId());
    setResultDTDSystemId(xMLResultPart.getResultDTDSystemId());
    setEncodingName(xMLResultPart.getEncodingName());
    setStandaloneDocument(xMLResultPart.isStandaloneDocument());
    setShortcutUsedForEmptyElement(
        xMLResultPart.isShortcutUsedForEmptyElement());
  }

  //////////////////
  // Object overload
  //////////////////

  public boolean equals(Object object) {
    return (object != null)
      && object instanceof TransformerInfoImpl
      && getTransformerName().equals(
      ((TransformerInfoImpl)object).getTransformerName());
  }

  public String toString() {
    return getTransformerName();
  }

  ////////////////////////////
  // Comparable implementation
  ////////////////////////////

  public int compareTo(Object object) {
    if (object == null || !(object instanceof TransformerInfoImpl))
      throw new IllegalArgumentException();
    return getTransformerName().compareTo(
      ((TransformerInfoImpl)object).getTransformerName());
  }

  /////////////////////////////////
  // TransformerInfo implementation
  /////////////////////////////////

  public final boolean isValid() {
    return getTransformerName().length() != 0;
  }

  public final String getTransformerName() {
    return ((StringOptionPartImpl)getOptionPart(NAME)).getStringValue();
  }

  public final void setTransformerName(String transformerName) {
    if (transformerName == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(NAME)).setStringValue(transformerName);
  }

  public String getDocumentTypeName() {
    return ((StringOptionPartImpl)getOptionPart(TYPENAME)).getStringValue();
  }

  public void setDocumentTypeName(String documentTypeName) {
    ((StringOptionPartImpl)getOptionPart(TYPENAME)).
      setStringValue(documentTypeName);
  }

  public final String getTransformerClassName() {
    return ((StringOptionPartImpl)getOptionPart(CLASSNAME)).getStringValue();
  }

  public final void setTransformerClassName(String transformerClassName) {
    if (transformerClassName == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(CLASSNAME)).
      setStringValue(transformerClassName);
  }

  public final String getStylesheetSystemId() {
    return ((StringOptionPartImpl)getOptionPart(STYLESHEETSYSTEMID)).
      getStringValue();
  }

  public final void setStylesheetSystemId(String stylesheetSystemId) {
    ((StringOptionPartImpl)getOptionPart(STYLESHEETSYSTEMID)).
      setStringValue(stylesheetSystemId != null ? stylesheetSystemId : "");
  }

  public String getParamNames() {
    return ((StringOptionPartImpl)getOptionPart(PARAMNAMES)).getStringValue();
  }

  public void setParamNames(String paramNames) {
    ((StringOptionPartImpl)getOptionPart(PARAMNAMES)).
      setStringValue(paramNames != null ? paramNames : "");
  }

  public boolean isResultXML() {
    return ((BooleanOptionPartImpl)getOptionPart(ISRESULTXML)).
        getBooleanValue();
  }

  public void setResultXML(boolean isResultXML) {
    ((BooleanOptionPartImpl)getOptionPart(ISRESULTXML)).
      setBooleanValue(isResultXML);
  }

  public TransformerInfoXMLResultPart getXMLResultPart() {
    return (TransformerInfoXMLResultPart)this;
  }

  public final String getLastDestinationPathname() {
    return this.lastDestinationPathname;
  }

  public final void setLastDestinationPathname(String lastDestinationPathname) {
    this.lastDestinationPathname = lastDestinationPathname;
  }

  //////////////////////////////////////////////
  // TransformerInfoXMLResultPart implementation
  //////////////////////////////////////////////

  public final String getResultDTDPublicId() {
    return ((StringOptionPartImpl)getOptionPart(DTDPUBLICID)).getStringValue();
  }

  public final void setResultDTDPublicId(String dTDPublicId) {
    ((StringOptionPartImpl)getOptionPart(DTDPUBLICID)).
      setStringValue(dTDPublicId != null ? dTDPublicId : "");
  }

  public final String getResultDTDSystemId() {
    return ((StringOptionPartImpl)getOptionPart(DTDSYSTEMID)).getStringValue();
  }

  public final void setResultDTDSystemId(String dTDSystemId) {
    ((StringOptionPartImpl)getOptionPart(DTDSYSTEMID)).
      setStringValue(dTDSystemId != null ? dTDSystemId : "");
  }

  public final String getEncodingName() {
    return ((StringOptionPartImpl)getOptionPart(ENCODINGNAME)).getStringValue();
  }

  public final void setEncodingName(String encodingName) {
    ((StringOptionPartImpl)getOptionPart(ENCODINGNAME)).
      setStringValue(encodingName != null ? encodingName : "UTF-8");
  }

  public final boolean isStandaloneDocument() {
    return ((BooleanOptionPartImpl)getOptionPart(ISSTANDALONEDOCUMENT))
      .getBooleanValue();
  }

  public final void setStandaloneDocument(boolean isStandaloneDocument) {
    ((BooleanOptionPartImpl)getOptionPart(ISSTANDALONEDOCUMENT)).
      setBooleanValue(isStandaloneDocument);
  }

  public final boolean isShortcutUsedForEmptyElement() {
    return ((BooleanOptionPartImpl)getOptionPart(ISEMPTYELEMENTSHORTCUTUSED)).
      getBooleanValue();
  }

  public final void setShortcutUsedForEmptyElement(
    boolean isShortcutUsedForEmptyElement) {
    ((BooleanOptionPartImpl)getOptionPart(ISEMPTYELEMENTSHORTCUTUSED)).
      setBooleanValue(isShortcutUsedForEmptyElement);
  }
}
