/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.transform;

import java.util.StringTokenizer;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.w3c.dom.Document;

import org.xmloperator.util.dom.ElementBrowser;
import org.xmloperator.framework.document.id.IdBrowser;
import org.xmloperator.document.id.IdBrowserImpl;

public abstract class PackageTransformer {
  protected String packageElementName = "package";
  protected String nameAttributeName = "name";
  protected String idAttributeName = "id";
  protected Element bodyElement;
  protected Document document;
  protected IdBrowser idBrowser;

  public PackageTransformer(Element bodyElement, IdBrowser idBrowser) {
    this.bodyElement = bodyElement;
    this.document = bodyElement.getOwnerDocument();
    this.idBrowser = idBrowser != null ?
      idBrowser : new IdBrowserImpl(document);
  }

  public void addTitle(String productName, String title) {
    Element hElement = document.createElement("h1");
    hElement.appendChild(packElement("i",
      document.createTextNode(productName)));
    hElement.appendChild(
      document.createTextNode(" - " + title));
    bodyElement.appendChild(hElement);
  }

  public void addParagraph(Node fragment) {
    bodyElement.appendChild(packElement("p", fragment));
  }

  public Node createPackageStack(Element packageElement)
    throws TransformerException {
    Node fragment = document.createDocumentFragment();
    Node firstChild = null;
    while (true) {
      firstChild =
        fragment.insertBefore(document.createTextNode(" / "), firstChild);
      packageElement = (Element)packageElement.getParentNode();
      if (!packageElement.getNodeName().equals(packageElementName))
        break;
      firstChild =
        fragment.insertBefore(createRef(packageElement), firstChild);
    }
    return fragment;
  }

  public void addIndex(Element parentElement, String elementName, int minCount)
    throws TransformerException {
    addIndex(parentElement, elementName, minCount, null);
  }

  public void addIndex(Element parentElement, String elementName, int minCount,
    String title) throws TransformerException {
    Element indexElement = createIndex(parentElement, elementName);
    if (ElementBrowser.getChildElementCount(indexElement) >= minCount) {
      if (title != null)
        bodyElement.appendChild(packElement("p", packElement("b",
          document.createTextNode(title))));
      bodyElement.appendChild(indexElement);
    }
  }

  public Element createIndex(Element parentElement, String elementName)
    throws TransformerException {
    Element ulElement = document.createElement("ul");
    Element childElement =
      ElementBrowser.getFirstChildElement(parentElement, elementName);
    while (childElement != null) {
      Element liElement = document.createElement("li");
      liElement.appendChild(createRef(childElement));
      ulElement.appendChild(liElement);
      childElement =
        ElementBrowser.getNextSiblingElement(childElement, elementName);
    }
    return ulElement;
  }

  public Node createTokensFragment(String idRefs) throws TransformerException {
    return createTokensFragment(idRefs, true);
  }

  public Node createTokensFragment(String idRefs, boolean asLinks)
    throws TransformerException {
    Node fragment = document.createDocumentFragment();
    StringTokenizer tokenizer = new StringTokenizer(idRefs);
    int count = 0;
    while (tokenizer.hasMoreTokens()) {
      String idRef = tokenizer.nextToken();
      if (count != 0)
        fragment.appendChild(document.createTextNode(", "));
      fragment.appendChild(createRef(idRef, asLinks));
      count++;
    }
    return fragment;
  }

  public Element packElement(String elementName, Node fragment) {
    Element packElement = document.createElement(elementName);
    packElement.appendChild(fragment);
    return packElement;
  }

  public Element createAnchor(Element element) throws TransformerException {
    String id = element.getAttribute(idAttributeName);
    return createAnchor(id);
  }

  public Element createAnchor(String id) throws TransformerException {
    Element aElement = document.createElement("a");
    aElement.setAttribute("id", id);
    aElement.setAttribute("name", id);
    return aElement;
  }

  public Element packAnchor(Element element, Node fragment)
    throws TransformerException {
    Element anchorElement = createAnchor(element);
    anchorElement.appendChild(fragment);
    return anchorElement;
  }

  public Element packAnchor(String id, Node fragment)
    throws TransformerException {
    Element anchorElement = createAnchor(id);
    anchorElement.appendChild(fragment);
    return anchorElement;
  }

  public Node createRef(String idRef) throws TransformerException {
    return createRef(idRef, true);
  }

  public Node createRef(String idRef, boolean asLink)
    throws TransformerException {
    Element referencedElement = idBrowser.getElementById(idRef);
    if (referencedElement == null)
      throw new TransformerException(idRef.length() != 0 ?
        "ID " + idRef + " not found" : "ID missing");
    return createRef(referencedElement, asLink);
  }

  public Node createRef(Element element) {
    return createRef(element, true);
  }

  public Node createRef(Element element, boolean asLink) {
    String name = element.getAttribute(nameAttributeName);
    return asLink ?
      createRef(element.getAttribute(idAttributeName), name) :
      packElement("i", document.createTextNode(name));
  }

  public Node createRef(String id, String name) {
    Text nameText = document.createTextNode(name);
    if (id.length() == 0)
      return nameText;
    Element aElement = document.createElement("a");
    aElement.setAttribute("href", "#" + id);
    aElement.appendChild(nameText);
    return aElement;
  }
}

