/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.io;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.InetAddress;
import java.net.URL;
import java.net.HttpURLConnection;

public class HttpImpl extends HttpURLConnection {
  protected String proxy;
  protected int proxy_port;
  protected Socket socket = null;
  protected OutputStream outputStream = null;
  protected InputStream inputStream = null;
  protected boolean areHeadersRead = false;

  public HttpImpl(URL url) {
    this(url, null, 0);
  }

  public HttpImpl(URL url, String proxy, int proxy_port) {
    super(url);
    this.proxy = proxy;
    this.proxy_port = proxy_port;
  }

  public void connect() throws IOException {
    if (connected)
      return;
    InetAddress address = InetAddress.getByName(
      proxy != null ? proxy : url.getHost());
    int port;
    if (proxy != null)
      port = proxy_port;
    else {
      port = getURL().getPort();
      if (port < 0)
        port = 80; // default port number for HTTP
    }
    socket = new Socket(address, port);
    outputStream = socket.getOutputStream();
    inputStream = socket.getInputStream();
    send(getRequestMethod() + " " + url.toString() + " HTTP/1.1\r\n");
    send("\r\n");
    connected  = true;
  }

  public void disconnect() {
    if (socket != null) {
      try {
        socket.close();
      }
      catch (IOException exception) {
      }
      socket = null;
    }
    connected = false;
  }

  public boolean usingProxy() {
    return proxy != null;
  }

  public OutputStream getOutputStream() {
    return outputStream;
  }

  public InputStream getInputStream() {
    return inputStream;
  }

  private void send(String string) throws IOException {
    for (int k = 0; k < string.length(); k++)
      outputStream.write((byte)string.charAt(k));
    outputStream.flush();
  }

  private String receive() throws IOException {
    StringBuffer result = new StringBuffer();
    while (true) {
      int c = inputStream.read();
      if (c < 0 || c == '\n')
        return result.toString();
      if (c != '\r')
        result.append((char)c);
    }
  }

  public void getHeaders() throws IOException {
    if (areHeadersRead)
      return;
    connect();
    while (true) {
      String header = receive();
      if (header.length() == 0)
        break;
    }
    areHeadersRead = true;
  }
}
