/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.dialog;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.xmloperator.framework.document.doctype.DTDInfo;
import org.xmloperator.framework.view.resource.Label;

public final class DTDInfoPanel extends JPanel {
  private static final Border border = new EmptyBorder(2, 4, 2, 4);
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private DTDInfo dTDInfo = null;
  private final JTextField fieldDTDPublicId = new JTextField(30);
  private final JTextField fieldHTTPDTDSystemId = new JTextField(30);
  private final JTextField fieldDocumentRelativeDTDSystemId =
    new JTextField(30);
  private final JTextField fieldUserRelativeDTDSystemId = new JTextField(30);
  private final JTextField fieldDTDXMLSystemId = new JTextField(30);

  public DTDInfoPanel() {
    this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    // DTD Public Id
    JPanel panelDTDPublicId = new JPanel(new BorderLayout());
    panelDTDPublicId.setBorder(border);
    panelDTDPublicId.add(new JLabel("DTD Public Id : "), BorderLayout.WEST);
    panelDTDPublicId.add(fieldDTDPublicId);
    this.add(panelDTDPublicId);
    // HTTP DTD System Id
    JPanel panelHTTPDTDSystemId = new JPanel(new BorderLayout());
    panelHTTPDTDSystemId.setBorder(border);
    panelHTTPDTDSystemId.add(
        new JLabel("HTTP DTD System Id : "), BorderLayout.WEST);
    panelHTTPDTDSystemId.add(fieldHTTPDTDSystemId);
    this.add(panelHTTPDTDSystemId);
    // Document relative DTD System Id
    JPanel panelDocumentRelativeDTDSystemId = new JPanel(new BorderLayout());
    panelDocumentRelativeDTDSystemId.setBorder(border);
    panelDocumentRelativeDTDSystemId.add(
      new JLabel("Document relative DTD System Id : "), BorderLayout.WEST);
    panelDocumentRelativeDTDSystemId.add(fieldDocumentRelativeDTDSystemId);
    this.add(panelDocumentRelativeDTDSystemId);
    // User relative DTD System Id
    JPanel panelUserRelativeDTDSystemId = new JPanel(new BorderLayout());
    panelUserRelativeDTDSystemId.setBorder(border);
    panelUserRelativeDTDSystemId.add(
      new JLabel("User relative DTD System Id : "), BorderLayout.WEST);
    panelUserRelativeDTDSystemId.add(fieldUserRelativeDTDSystemId);
    this.add(panelUserRelativeDTDSystemId);
    // XML DTD System Id
    JPanel panelDTDXMLSystemId = new JPanel(new BorderLayout());
    panelDTDXMLSystemId.setBorder(border);
    panelDTDXMLSystemId.add(
        new JLabel("DTDXML System Id : "), BorderLayout.WEST);
    panelDTDXMLSystemId.add(fieldDTDXMLSystemId);
    this.add(panelDTDXMLSystemId);
  }

  public void setDTDInfo(DTDInfo dTDInfo) {
    this.dTDInfo = dTDInfo;
    this.fieldDTDPublicId.setText(dTDInfo.getDTDPublicId());
    this.fieldHTTPDTDSystemId.setText(dTDInfo.getHTTPDTDSystemId());
    this.fieldDocumentRelativeDTDSystemId.setText(
        dTDInfo.getDocumentRelativeDTDSystemId());
    this.fieldUserRelativeDTDSystemId.setText(
        dTDInfo.getUserRelativeDTDSystemId());
    this.fieldDTDXMLSystemId.setText(dTDInfo.getDTDXMLSystemId());
  }

  public void setOKListener(ActionListener okListener) {
    this.fieldDTDPublicId.addActionListener(okListener);
    this.fieldHTTPDTDSystemId.addActionListener(okListener);
    this.fieldDocumentRelativeDTDSystemId.addActionListener(okListener);
    this.fieldUserRelativeDTDSystemId.addActionListener(okListener);
    this.fieldDTDXMLSystemId.addActionListener(okListener);
  }

  public void ok() {
    this.dTDInfo.setDTDPublicId(this.fieldDTDPublicId.getText());
    this.dTDInfo.setHTTPDTDSystemId(this.fieldHTTPDTDSystemId.getText());
    this.dTDInfo.setDocumentRelativeDTDSystemId(
      this.fieldDocumentRelativeDTDSystemId.getText());
    this.dTDInfo.setUserRelativeDTDSystemId(
        this.fieldUserRelativeDTDSystemId.getText());
    this.dTDInfo.setDTDXMLSystemId(this.fieldDTDXMLSystemId.getText());
  }
}
