/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.dialog;

import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.transform.TransformerInfo;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.DialogModal;
import org.xmloperator.view.dialog.DTDInfoPanel;
import org.xmloperator.view.dialog.RNGInfoPanel;

public final class DialogDocumentTypeInfo extends DialogModal {
  private static final Border border = new EmptyBorder(2, 4, 2, 4);
  private Options options;
  private GlobalOptionsControl globalOptionsControl;
  private DocumentTypeInfo documentTypeInfo = null;
  private final JTextField fieldDocumentTypeName = new JTextField(20);
  private final JTextField fieldDocumentTypeAdditionalKey =
    new JTextField(20);
  private DTDInfoPanel dTDInfoPanel;
  private RNGInfoPanel rNGInfoPanel;
  private final JTextField fieldNamespaceId = new JTextField(20);
  private final JCheckBox fieldShortcut =
    new JCheckBox("Shortcut used for empty elements");
  private final JComboBox comboBoxTransformerName = new JComboBox();

  public DialogDocumentTypeInfo(Frame parent, Options options,
    GlobalOptionsControl globalOptionsControl) {
    super(parent);
    this.setResizable(false);
    this.options = options;
    this.globalOptionsControl = globalOptionsControl;
    setTitle(viewResource.getString(Label.DOCUMENT_TYPE));
    // Document type name
    JPanel voletDocumentTypeName = new JPanel(new BorderLayout());
    voletDocumentTypeName.setBorder(border);
    voletDocumentTypeName.add(
      new JLabel(viewResource.getString(Label.NAME) + " : "),
      BorderLayout.WEST);
    this.fieldDocumentTypeName.addActionListener(oKActionListener);
    voletDocumentTypeName.add(this.fieldDocumentTypeName);
    this.contentPanel.add(voletDocumentTypeName);
    // Document type additional key
    JPanel voletDocumentTypeAdditionalKey = new JPanel(new BorderLayout());
    voletDocumentTypeAdditionalKey.setBorder(border);
    voletDocumentTypeAdditionalKey.add(
      new JLabel(viewResource.getString(Label.ADDITIONAL_KEY) + " : "),
      BorderLayout.WEST);
    this.fieldDocumentTypeAdditionalKey.addActionListener(oKActionListener);
    voletDocumentTypeAdditionalKey.add(this.fieldDocumentTypeAdditionalKey);
    this.contentPanel.add(voletDocumentTypeAdditionalKey);
    // DTD Data Panel
    this.dTDInfoPanel = new DTDInfoPanel();
    this.dTDInfoPanel.setOKListener(oKActionListener);
    this.dTDInfoPanel.setBorder(new TitledBorder("DTD"));
    this.contentPanel.add(this.dTDInfoPanel);
    // RNG Data Panel
    this.rNGInfoPanel = new RNGInfoPanel(globalOptionsControl);
    this.rNGInfoPanel.setOKListener(oKActionListener);
    this.rNGInfoPanel.setBorder(new TitledBorder("RELAX NG"));
    this.contentPanel.add(this.rNGInfoPanel);
    // Namespace identifier
    JPanel panelNamespaceId = new JPanel(new BorderLayout());
    panelNamespaceId.setBorder(border);
    panelNamespaceId.add(
        new JLabel(viewResource.getString(Label.NAMESPACE_IDENTIFIER) + " : "),
        BorderLayout.WEST);
    this.fieldNamespaceId.addActionListener(oKActionListener);
    panelNamespaceId.add(this.fieldNamespaceId);
    this.contentPanel.add(panelNamespaceId);
    // Shortcut for empty elements
    JPanel panelShortcut = new JPanel(new BorderLayout());
    panelShortcut.setBorder(border);
    panelShortcut.add(this.fieldShortcut);
    this.contentPanel.add(panelShortcut);
    // Default transformer name
    JPanel panelTransformerName = new JPanel(new BorderLayout());
    panelTransformerName.setBorder(border);
    panelTransformerName.add(
        new JLabel(viewResource.getString(Label.DEFAULT_TRANSFORMER) + " : "),
        BorderLayout.WEST);
    panelTransformerName.add(this.comboBoxTransformerName);
    JButton buttonNewTransformer =
        new JButton(viewResource.getString(Label.NEW));
    buttonNewTransformer.addActionListener(
        new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        buttonNewTransformer_actionPerformed();
      }
    });
    panelTransformerName.add(buttonNewTransformer, BorderLayout.EAST);
    contentPanel.add(panelTransformerName);
    // Pack
    this.addSeparator();
    this.pack();
  }

  protected void buttonOK_actionPerformed(ActionEvent event) {
    try {
      if (!buttonOK.isEnabled())
        return;
      this.documentTypeInfo.setDocumentTypeName(
          this.fieldDocumentTypeName.getText());
      this.documentTypeInfo.setDocumentTypeAdditionalKey(
        this.fieldDocumentTypeAdditionalKey.getText());
      this.dTDInfoPanel.ok();
      this.rNGInfoPanel.ok();
      this.documentTypeInfo.setNamespaceId(this.fieldNamespaceId.getText());
      this.documentTypeInfo.setShortcutUsedForEmptyElement(
        this.fieldShortcut.isSelected());
      this.documentTypeInfo.setDefaultTransformerName(
        this.comboBoxTransformerName.getSelectedIndex() != 0 ?
        (String)this.comboBoxTransformerName.getSelectedItem() : null);
      this.isOK = true;
      this.dispose();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void buttonNewTransformer_actionPerformed() {
    try {
      TransformerInfo transformerInfo =
        globalOptionsControl.newTransformer();
      if (transformerInfo == null)
        return;
      String transformerName = transformerInfo.getTransformerName();
      this.comboBoxTransformerName.addItem(transformerName);
      this.comboBoxTransformerName.setSelectedItem(transformerName);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  public void setContentFont(Font font) {
  }

  public void setNameReadOnly(boolean isNameReadOnly) {
    this.fieldDocumentTypeName.setEnabled(!isNameReadOnly);
  }

  public void setAddKeyReadOnly(boolean isAddKeyReadOnly) {
    this.fieldDocumentTypeAdditionalKey.setEnabled(!isAddKeyReadOnly);
  }

  public void setTransformerInfoList(List transformerInfoList) {
    if (this.comboBoxTransformerName.getItemCount() != 0)
      this.comboBoxTransformerName.removeAllItems();
    this.comboBoxTransformerName.addItem(
        "-- " + viewResource.getString(Label.UNDEFINED) + " --");
    for (int k = 0; k < transformerInfoList.size(); k++)
      comboBoxTransformerName.addItem(
          ((TransformerInfo)transformerInfoList.get(k)).getTransformerName());
    this.setDefaultTransformerName();
  }

  public void setDocumentTypeInfo(DocumentTypeInfo documentTypeInfo) {
    this.documentTypeInfo = documentTypeInfo;
    this.fieldDocumentTypeName.setText(documentTypeInfo.getDocumentTypeName());
    this.fieldDocumentTypeAdditionalKey.setText(
        documentTypeInfo.getDocumentTypeAdditionalKey());
    this.dTDInfoPanel.setDTDInfo(documentTypeInfo.getDTDInfo());
    this.rNGInfoPanel.setRNGInfo(documentTypeInfo.getRNGInfo());
    this.fieldNamespaceId.setText(documentTypeInfo.getNamespaceId());
    this.fieldShortcut.setSelected(
        documentTypeInfo.isShortcutUsedForEmptyElement());
    this.setDefaultTransformerName();
  }

  private void setDefaultTransformerName() {
    String defaultTransformerName = this.documentTypeInfo != null ?
        documentTypeInfo.getDefaultTransformerName() : null;
    if (defaultTransformerName != null && defaultTransformerName.length() != 0)
      this.comboBoxTransformerName.setSelectedItem(defaultTransformerName);
    else
      this.comboBoxTransformerName.setSelectedIndex(0);
  }
}
