/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.dialog;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.List;
import java.util.Iterator;
import javax.swing.JTextField;
import javax.swing.ComboBoxEditor;
import javax.swing.JComboBox;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.StringOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.util.view.TextFieldComboBoxEditor;

public final class HistoryField {
  private Options options;
  private String optionId;
  private final JTextField textField;
  private final JComboBox comboBox;

  public HistoryField(Options options, String optionId,
    ActionListener oKActionListener) {
    this.options = options;
    this.optionId = optionId;
    this.textField = new JTextField(30) {
        public void processKeyEvent(KeyEvent keyEvent) {
          int keyCode = keyEvent.getKeyCode();
          if (keyCode == KeyEvent.VK_DOWN
            || keyCode == KeyEvent.VK_UP)
            comboBox_showPopup();
          else if (keyCode == KeyEvent.VK_TAB)
            comboBox_hidePopup();
          super.processKeyEvent(keyEvent);
        }
      };
    this.textField.setFont(
      options.getThreeStateOptionValue(Options.FONT) == ThreeStateOption.TRUE ?
      Constants.FONT_LARGE : Constants.FONT_NORMAL);
    this.textField.addActionListener(oKActionListener);
    this.comboBox = new JComboBox();
    this.comboBox.setPreferredSize(new Dimension(300, 24));
    this.comboBox.setEditable(true);
    this.comboBox.setEditor(new TextFieldComboBoxEditor(textField));
    List stringOptionList = options.getOptionList(optionId);
    int optionCount = stringOptionList.size();
    for (int k = 0; k < optionCount; k++) {
      StringOption stringOption = (StringOption)stringOptionList.get(k);
      comboBox.addItem(stringOption.getStringValue());
    }
    if (optionCount != 0)
      this.comboBox.setSelectedIndex(0);
  }

  private void comboBox_showPopup() {
    this.comboBox.showPopup();
  }

  private void comboBox_hidePopup() {
    this.comboBox.hidePopup();
  }

  public Component getComponent() {
    return this.comboBox;
  }

  public void setEnabled(boolean isEnabled) {
    this.textField.setEnabled(isEnabled);
    this.comboBox.setEnabled(isEnabled);
  }

  public String getText() {
    return this.textField.getText();
  }

  public void setText(String text) {
    this.textField.setText(text);
  }

  public void ok() {
    List stringOptionList = this.options.getOptionList(optionId);
    String text = this.getText();
    {
      Iterator iterator = stringOptionList.iterator();
      while (iterator.hasNext()) {
        StringOption stringOption = (StringOption)iterator.next();
        if (stringOption.getStringValue().equals(text))
          iterator.remove();
      }
    }
    stringOptionList.add(0, options.createStringOption(optionId, text));
    int historySize = options.getIntOptionValue(Options.HISTORYSIZE);
    if (historySize > 0)
      while (stringOptionList.size() > historySize)
        stringOptionList.remove(stringOptionList.size() - 1);
  }
}
