/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.dialog;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.xmloperator.framework.document.doctype.RNGInfo;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.RelaxNGSimplifier;

public final class RNGInfoPanel extends JPanel {
  private static final Border border = new EmptyBorder(2, 4, 2, 4);
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private RNGInfo rNGInfo = null;
  private GlobalOptionsControl globalOptionsControl;
  private final JTextField fieldFullRNGSystemId = new JTextField(30);
  private final JTextField fieldSimpleRNGSystemId = new JTextField(30);

  public RNGInfoPanel(GlobalOptionsControl globalOptionsControl) {
    this.globalOptionsControl = globalOptionsControl;
    this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    // RNG full syntax System Id
    JPanel panelFullRNGSystemId = new JPanel(new BorderLayout());
    panelFullRNGSystemId.setBorder(border);
    panelFullRNGSystemId.add(
      new JLabel("RNG full syntax System Id : "), BorderLayout.WEST);
    panelFullRNGSystemId.add(fieldFullRNGSystemId);
    this.add(panelFullRNGSystemId);
    // RNG simple syntax System Id
    JPanel panelSimpleRNGSystemId = new JPanel(new BorderLayout());
    panelSimpleRNGSystemId.setBorder(border);
    panelSimpleRNGSystemId.add(
      new JLabel("RNG simple syntax System Id : "), BorderLayout.WEST);
    panelSimpleRNGSystemId.add(fieldSimpleRNGSystemId);
    JButton buttonBuildSimpleRNG =
      new JButton(viewResource.getString(Label.BUILD));
    buttonBuildSimpleRNG.setMnemonic(
      viewResource.getString(Label.MNEMO + Label.BUILD).charAt(0));
    buttonBuildSimpleRNG.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          buttonBuildSimpleRNG_actionPerformed();
        }
      });
    panelSimpleRNGSystemId.add(buttonBuildSimpleRNG, BorderLayout.EAST);
    this.add(panelSimpleRNGSystemId);
  }

  private void buttonBuildSimpleRNG_actionPerformed() {
    try {
      String fullRNGSystemId = fieldFullRNGSystemId.getText();
      if (fullRNGSystemId.length() == 0)
        return;
      String simpleRNGSystemId = fieldSimpleRNGSystemId.getText();
      if (simpleRNGSystemId.length() == 0) {
        if (!fullRNGSystemId.endsWith(".rng"))
          return;
        simpleRNGSystemId =
            fullRNGSystemId.substring(0, fullRNGSystemId.length() - 4)
            + "_s.rng";
        fieldSimpleRNGSystemId.setText(simpleRNGSystemId);
      }
      globalOptionsControl.buildRNGSimpleSyntaxFromFullSyntax(
          fullRNGSystemId, simpleRNGSystemId, RelaxNGSimplifier.
          SIMPLIFICATION_TYPE_GROUP_WITH_MORE_THAN_TWO_CHILDREN);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  public void setRNGInfo(RNGInfo rNGInfo) {
    this.rNGInfo = rNGInfo;
    this.fieldFullRNGSystemId.setText(rNGInfo.getRNGFullSyntaxSystemId());
    this.fieldSimpleRNGSystemId.setText(rNGInfo.getRNGSimpleSyntaxSystemId());
  }

  public void setOKListener(ActionListener okListener) {
    this.fieldFullRNGSystemId.addActionListener(okListener);
    this.fieldSimpleRNGSystemId.addActionListener(okListener);
  }

  public void ok() {
    this.rNGInfo.setRNGFullSyntaxSystemId(fieldFullRNGSystemId.getText());
    this.rNGInfo.setRNGSimpleSyntaxSystemId(fieldSimpleRNGSystemId.getText());
  }
}
