/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.frame;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JCheckBox;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.TreePath;

import org.w3c.dom.Node;
import org.w3c.dom.DocumentFragment;

import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.util.PresseFragment;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.ToolControl;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.view.model.DomTreeNode;
import org.xmloperator.view.model.DomTreeModel;

public class DocumentFragmentFrame extends JFrame implements ObserverView {
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private FullyFledgedDocument fullyFledgedDocument;
  private Options options;
  private EditControl editControl;
  private ToolControl toolControl;
  private JMenuItem menuEffacer;
  private DomTreeModel domTreeModel;
  private JTree treePanel;
  private JScrollPane scrollPanel;

  public DocumentFragmentFrame(FullyFledgedDocument fullyFledgedDocument,
    Options options, EditControl editControl, ToolControl toolControl,
    Shortcuts shortcuts) {
    setTitle(Constants.TOOL
      + '.' + toolControl.getInstanceIdentifier() + " - "
      + viewResource.getString(Label.FRAGMENT));
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.editControl = editControl;
    this.toolControl = toolControl;
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    // Volet contenu
    JPanel voletContenu = new JPanel(new BorderLayout());
    this.getContentPane().add(voletContenu);
    voletContenu.setPreferredSize(new Dimension(480, 160));
    shortcuts.registerKeyboardAction(voletContenu);
    // Menus
    JMenuBar menuBar = new JMenuBar();
    this.setJMenuBar(menuBar);
    menuEffacer = new JMenuItem(viewResource.getString(Label.EFFACER));
    menuEffacer.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.EFFACER).charAt(0));
    menuEffacer.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          clear_actionPerformed();
        }
      });
    menuBar.add(menuEffacer);
    JMenuItem menuImporter =
      new JMenuItem(viewResource.getString(Label.IMPORT) + "...");
    menuImporter.setMnemonic(
        viewResource.getString(Label.MNEMO + Label.IMPORT).charAt(0));
    menuImporter.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          import_actionPerformed();
        }
      });
    menuBar.add(menuImporter);
    JMenuItem menuToClipboard =
      new JMenuItem(viewResource.getString(Label.TO_CLIPBOARD));
    menuToClipboard.setMnemonic(
        viewResource.getString(Label.MNEMO + Label.TO_CLIPBOARD).charAt(0));
    menuToClipboard.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          fragmentToClipboard_actionPerformed();
        }
      });
    menuBar.add(menuToClipboard);
    JMenuItem menuFromClipboard =
      new JMenuItem(viewResource.getString(Label.FROM_CLIPBOARD));
    menuFromClipboard.setMnemonic(
        viewResource.getString(Label.MNEMO + Label.FROM_CLIPBOARD).charAt(0));
    menuFromClipboard.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          clipboardToFragment_actionPerformed();
        }
      });
    menuBar.add(menuFromClipboard);
    // Tree panel
    domTreeModel = new DomTreeModel(fullyFledgedDocument);
    treePanel = new JTree(domTreeModel);
    treePanel.setRootVisible(false);
    treePanel.setRowHeight(16);
    treePanel.setFont(Constants.FONT_NORMAL);
    scrollPanel = new JScrollPane(treePanel);
    voletContenu.add(scrollPanel);
    this.pack();
    update(null, null);
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  private void clear_actionPerformed() {
    try {
      editControl.clearDocumentFragment();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void import_actionPerformed() {
    try {
      toolControl.importFragment();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void fragmentToClipboard_actionPerformed() {
    try {
      editControl.copyDocumentFragmentToSystemClipboard();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void clipboardToFragment_actionPerformed() {
    try {
      editControl.copySystemClipboardToDocumentFragment();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  //////////////////////////
  // Observer implementation
  //////////////////////////

  public void update(Object observable, Object argument) {
    PresseFragment presseFragment = fullyFledgedDocument.getPresseFragment();
    if (presseFragment == null)
      throw new RuntimeException();
    Node documentFragment = presseFragment.getFragment();
    domTreeModel.setRootNode(documentFragment, true);
    treePanel.expandPath(domTreeModel.getTreePath(documentFragment));
    menuEffacer.setEnabled(documentFragment != null
      && documentFragment.hasChildNodes());
  }
}
