/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.frame;

import java.awt.Point;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.io.IOException;
import java.util.ResourceBundle;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.AbstractListModel;
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.border.EmptyBorder;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.document.parse.ParseError;
import org.xmloperator.framework.document.parse.ParseErrorList;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.observer.TextFileEditor;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.instance.ViewAccess;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.util.io.FileUtils;

public final class ParseErrorListFrame extends JFrame {
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private FullyFledgedDocument fullyFledgedDocument;
  private ViewAccess viewAccess;
  private ParseErrorListModel listModel;
  private JList listPanel;
  private JLabel systemIdLabel;
  private int lastSelectedIndex = -1;
  private String currentEditedFilePathname = "";

  public ParseErrorListFrame(int instanceIdentifier,
    FullyFledgedDocument fullyFledgedDocument, ViewAccess viewAccess,
    Shortcuts shortcuts) {
    super(Constants.TOOL + '.' + instanceIdentifier
      + " - Warning & Errors");
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.viewAccess = viewAccess;
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    // Volet contenu
    JPanel voletContenu = new JPanel(new BorderLayout());
    this.getContentPane().add(voletContenu);
    voletContenu.setPreferredSize(new Dimension(640, 160));
    shortcuts.registerKeyboardAction(voletContenu);
    // Volet liste
    listModel = new ParseErrorListModel(fullyFledgedDocument);
    listPanel = new JList(listModel);
    listPanel.addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent event) {
          list_selectionChanged();
        }
      });
    listPanel.addMouseListener(
      new MouseAdapter() {
        public void mouseClicked(MouseEvent event) {
          if (event.getClickCount() == 2)
            list_activation();
        }
        public void mousePressed(MouseEvent event) {
          if ((event.getModifiers() & InputEvent.BUTTON3_MASK) != 0)
            list_localMenu(event.getX(), event.getY());
        }
      });
    JScrollPane scrollPanel = new JScrollPane(listPanel);
    voletContenu.add(scrollPanel);
    JPanel systemIdPanel = new JPanel(new BorderLayout());
    systemIdPanel.setBorder(new EmptyBorder(0, 4, 0, 0));
    systemIdLabel = new JLabel(" ");
    systemIdPanel.add(systemIdLabel);
    voletContenu.add(systemIdPanel, BorderLayout.NORTH);
    // Mise aux dimensions
    this.pack();
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  private void list_selectionChanged() {
    int index = listPanel.getSelectedIndex();
    if (index == lastSelectedIndex)
      return;
    lastSelectedIndex = index;
    ParseErrorList parseErrorList = fullyFledgedDocument.getParseErrorList();
    if (parseErrorList == null || index < 0 || index >= parseErrorList.size()) {
      setSelectedNode(null);
      return;
    }
    ParseError parseError = parseErrorList.get(index);
    String systemId = parseError.getSystemId();
    systemIdLabel.setText(
      systemId != null && systemId.length() != 0 ? systemId : " ");
    Document document = fullyFledgedDocument.getDocument();
    Element element = document != null ?
      parseError.elementPathToElement(document.getDocumentElement()) : null;
    setSelectedNode(element != null ? (Node)element : document);
  }

  private void list_activation() {
    int index = listPanel.getSelectedIndex();
    ParseErrorList parseErrorList = fullyFledgedDocument.getParseErrorList();
    if (parseErrorList == null || index < 0 || index >= parseErrorList.size())
      return;
    ParseError parseError = parseErrorList.get(index);
    String systemId = parseError.getSystemId();
    if (systemId == null || systemId.indexOf("file://") != 0)
      return;
    TextFileEditor sourceFrame = viewAccess.createSourceEditor();
    String pathname = FileUtils.fromSystemIdToFilePathName(systemId);
    if (!pathname.equals(currentEditedFilePathname))
      try {
        if (sourceFrame.hasBeenModifiedAfterLastSave()
          && JOptionPane.showConfirmDialog(this,
          viewResource.getString(Label.DO_YOU_CONFIRM) + " "
          + viewResource.getString(Label.L_ABANDON_DES_MODIFICATIONS)
          + " ?",
          viewResource.getString(Label.MODIFICATIONS_NON_ENREGISTREES),
          JOptionPane.OK_CANCEL_OPTION) != JOptionPane.OK_OPTION)
          return;
        sourceFrame.load(pathname);
        currentEditedFilePathname = pathname;
      }
      catch (IOException exception) {
        JOptionPane.showMessageDialog(this,
          exception.getMessage(), exception.getClass().getName(),
          JOptionPane.ERROR_MESSAGE);
        return;
      }
    sourceFrame.show();
    sourceFrame.selectLine(parseError.getLineNumber());
  }

  private void list_localMenu(int x, int y) {
    int index = listPanel.locationToIndex(new Point(x, y));
    if (index < 0)
      return;
    if (!listPanel.isSelectedIndex(index))
      listPanel.setSelectedIndex(index);
    JPopupMenu menu = new JPopupMenu();
    JMenuItem menuSource =
      new JMenuItem(viewResource.getString(Label.SOURCE));
    menuSource.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          list_activation();
        }
      });
    menu.add(menuSource);
    menu.show(listPanel, x, y);
  }

  private void setSelectedNode(Node node) {
    viewAccess.getToolView().getDocumentTreeView().setSelectedNode(node, true);
  }

  private static final class ParseErrorListModel extends AbstractListModel {
    private FullyFledgedDocument fullyFledgedDocument;
    private int lastSize = 0;

    public ParseErrorListModel(FullyFledgedDocument fullyFledgedDocument) {
      this.fullyFledgedDocument = fullyFledgedDocument;
    }

    ///////////////////////////
    // ListModel implementation
    ///////////////////////////

    public int getSize() {
      ParseErrorList parseErrorList = fullyFledgedDocument.getParseErrorList();
      return parseErrorList != null ? parseErrorList.size() : 0;
    }

    public Object getElementAt(int index) {
      return fullyFledgedDocument.getParseErrorList().get(index);
    }

    public void processUpdate() {
      int size = getSize();
      if (lastSize != 0)
        fireIntervalRemoved(this, 0, lastSize - 1);
      if (size != 0)
        fireIntervalAdded(this, 0, size - 1);
      lastSize = size;
    }
  }

  public void update() {
    lastSelectedIndex = - 1;
    listModel.processUpdate();
    if (listModel.getSize() != 0)
      listPanel.setSelectedIndex(0);
  }
}
