/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.frame;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.AbstractListModel;
import javax.swing.JPanel;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.document.id.IdInfo;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.observer.ViewObserver;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.instance.ToolViewAccess;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.document.elementKey.ElementKeyUtil;
import org.xmloperator.view.model.DomTreeNode;

public final class ReferencersFrame extends JFrame implements ObserverView {
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private ViewObserver viewObserver = null;
  private int instanceIdentifier;
  private FullyFledgedDocument fullyFledgedDocument;
  private ToolViewAccess toolViewAccess;
  private Element referencedElement;
  private ReferencerListModel listModel;
  private JList listPanel;
  private JScrollPane scrollPanel;
  private String id;

  public ReferencersFrame(int instanceIdentifier,
    FullyFledgedDocument fullyFledgedDocument,
    ToolViewAccess toolViewAccess,
    Options options, Element referencedElement, Shortcuts shortcuts) {
    if (fullyFledgedDocument == null || referencedElement == null)
      throw new IllegalArgumentException();
    this.instanceIdentifier = instanceIdentifier;
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.toolViewAccess = toolViewAccess;
    this.referencedElement = referencedElement;
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    // Volet contenu
    JPanel voletContenu = new JPanel(new BorderLayout());
    this.getContentPane().add(voletContenu);
    voletContenu.setPreferredSize(
      new Dimension(options.getIntOptionValue(Options.NODELISTWIDTH),
        options.getIntOptionValue(Options.NODELISTHEIGHT)));
    shortcuts.registerKeyboardAction(voletContenu);
    // Menus
    JMenuBar menuBar = new JMenuBar();
    this.setJMenuBar(menuBar);
    JMenuItem menuReferencedElement =
      new JMenuItem(viewResource.getString(Label.CIBLE));
    menuReferencedElement.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.CIBLE).charAt(0));
    menuReferencedElement.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          referencedElement_actionPerformed();
        }
      });
    menuBar.add(menuReferencedElement);
    // Volet liste
    listModel = new ReferencerListModel(fullyFledgedDocument);
    listPanel = new JList(listModel);
    listPanel.addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent event) {
          list_selectionChanged();
        }
      });
    JScrollPane scrollPanel = new JScrollPane(listPanel);
    voletContenu.add(scrollPanel);
    // Mise aux dimensions
    this.pack();
    // Initialisation
    update(null, null);
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  private void referencedElement_actionPerformed() {
    gotoElement(referencedElement);
  }

  private void list_selectionChanged() {
    int index = listPanel.getSelectedIndex();
    if (index >= 0)
      gotoElement(listModel.getDomElementAt(index));
  }

  private void gotoElement(Element element) {
    toolViewAccess.getToolView().getDocumentTreeView().
      setSelectedNode(element, true);
  }

  public Element getReferencedElement() {
    return referencedElement;
  }

  public void setViewObserver(ViewObserver viewObserver) {
    this.viewObserver = viewObserver;
  }

  public static final class ReferencerListModel extends AbstractListModel {
    private FullyFledgedDocument fullyFledgedDocument;
    private int lastSize = 0;
    private IdInfo idInfo = null;

    public ReferencerListModel(FullyFledgedDocument fullyFledgedDocument) {
      this.fullyFledgedDocument = fullyFledgedDocument;
    }

    public int getSize() {
      return idInfo != null ? idInfo.getReferencerCount() : 0;
    }

    public Object getElementAt(int index) {
      return DomTreeNode.domTreeNodeFromNode(
          this.getDomElementAt(index),
          this.fullyFledgedDocument.getNodeLinker(),
          this.fullyFledgedDocument.getElementKeyProvider());
    }

    public Element getDomElementAt(int index) {
      return idInfo.getReferencer(index).getOwnerElement();
    }

    public void processUpdate(String id) {
      this.idInfo = id != null ?
        fullyFledgedDocument.getIdManager().getIdInfo(id) : null;
      int size = getSize();
      if (lastSize != 0)
        fireIntervalRemoved(this, 0, lastSize - 1);
      if (size != 0)
        fireIntervalAdded(this, 0, size - 1);
      lastSize = size;
    }
  }

  //////////////////////////
  // Observer implementation
  //////////////////////////

  public void update(Object observable, Object argument) {
    String idAttrName = fullyFledgedDocument.getDocumentUtil().
      getAttributesContent(referencedElement).getIdAttributeName();
    id = idAttrName != null ? referencedElement.getAttribute(idAttrName) : null;
    setTitle(Constants.TOOL + '.' + instanceIdentifier
      + " - " + viewResource.getString(Label.REFERING)
      + ' ' + ElementKeyUtil.nodeToString(referencedElement, null));
    listModel.processUpdate(id);
  }

  //////////////////////////////
  // ObserverView implementation
  //////////////////////////////

  public void dispose() {
    super.dispose();
    if (viewObserver != null)
      viewObserver.notifyDispose(this);
  }
}
