/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.frame;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.util.ResourceBundle;
import java.util.List;
import java.util.Collections;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.JTable;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.document.transform.TransformerInfo;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.global.TransformerInfoView;
import org.xmloperator.framework.control.global.GlobalOptionsControl;

public final class TransformerInfoFrame extends JFrame
  implements TransformerInfoView {
  private static final int COLUMN_NAME = 0;
  private static final int COLUMN_CLASSNAME = 1;
  private static final int COLUMN_STYLESHEETSYSTEMID = 2;
  private static final String[] enteteColonne = {
    Label.TRANSFORMER,
    Label.CLASS_NAME,
    "Style sheet System Id"};
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private GlobalOptionsControl globalOptionsControl;
  private List transformerInfoList;
  private TableModel tableModel;
  private JTable table;
  private JMenuItem menuDuppliquer;
  private JMenuItem menuModifier;
  private JMenuItem menuSupprimer;

  public TransformerInfoFrame(List transformerInfoList,
    GlobalOptionsControl globalOptionsControl) {
    setTitle(Constants.TOOL + " - "
             + viewResource.getString(Label.TRANSFORMERS));
    this.transformerInfoList = transformerInfoList;
    this.globalOptionsControl = globalOptionsControl;
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    Collections.sort(transformerInfoList);
    // Volet contenu
    JPanel voletContenu = new JPanel(new BorderLayout());
    this.getContentPane().add(voletContenu);
    voletContenu.setPreferredSize(new Dimension(480, 200));
    // Menus
    JMenuBar menuBar = new JMenuBar();
    this.setJMenuBar(menuBar);
    JMenuItem menuAjouter =
      new JMenuItem(viewResource.getString(Label.NEW));
    menuAjouter.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.NEW).charAt(0));
    menuAjouter.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          add_actionPerformed();
        }
      });
    menuBar.add(menuAjouter);
    menuDuppliquer =
      new JMenuItem(viewResource.getString(Label.DUPLICATE));
    menuDuppliquer.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.DUPLICATE).charAt(0));
    menuDuppliquer.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          duplicate_actionPerformed();
        }
      });
    menuBar.add(menuDuppliquer);
    menuModifier = new JMenuItem(viewResource.getString(Label.MODIFY));
    menuModifier.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.MODIFY).charAt(0));
    menuModifier.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          modify_actionPerformed();
        }
      });
    menuBar.add(menuModifier);
    menuSupprimer =
      new JMenuItem(viewResource.getString(Label.SUPPRIMER));
    menuSupprimer.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.SUPPRIMER).charAt(0));
    menuSupprimer.addActionListener(
      new ActionListener()  {
        public void actionPerformed(ActionEvent event) {
          remove_actionPerformed();
        }
      });
    menuBar.add(menuSupprimer);
    // Table
    tableModel = new AbstractTableModel() {
      public String getColumnName(int col) {
        return table_getColumnName(col);
      }
      public Class getColumnClass(int col) {
        return String.class;
      }
      public int getRowCount() {
        return table_getRowCount();
      }
      public int getColumnCount() {
        return enteteColonne.length;
      }
      public Object getValueAt(int row, int col) {
        return table_getValueAt(row, col);
      }
    };
    table = new JTable(tableModel);
    table.setShowVerticalLines(true);
    table.setRowSelectionAllowed(true);
    table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    table.getSelectionModel().addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent event) {
          setItemMenuEnabled(true);
        }
      });
    table.addMouseListener(
      new MouseAdapter() {
        public void mouseClicked(MouseEvent event) {
          if (event.getClickCount() == 2)
            modify_actionPerformed();
        }
      });
    JTableHeader tableHeader = table.getTableHeader();
    tableHeader.setResizingAllowed(true);
    tableHeader.setReorderingAllowed(false);
    JScrollPane scrollPane = new JScrollPane(table);
    voletContenu.add(scrollPane);
    // Mise aux dimensions
    this.pack();
    // Initialisation menu
    setItemMenuEnabled(false);
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  private String table_getColumnName(int col) {
    if (col < 0 || col >= enteteColonne.length)
      return "";
    String columnName = enteteColonne[col];
    try {
      return viewResource.getString(columnName);
    }
    catch (Exception exception) {
      return columnName;
    }
  }

  private int table_getRowCount() {
    return transformerInfoList.size();
  }

  private Object table_getValueAt(int row, int column) {
    try {
      if (row < 0 || row >= transformerInfoList.size())
        return "";
      TransformerInfo transformerInfo =
        (TransformerInfo)transformerInfoList.get(row);
      switch (column) {
        case COLUMN_NAME:
          return transformerInfo.getTransformerName();
        case COLUMN_CLASSNAME:
          return transformerInfo.getTransformerClassName();
        case COLUMN_STYLESHEETSYSTEMID:
          return transformerInfo.getStylesheetSystemId();
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    return "";
  }

  private void add_actionPerformed() {
    try {
      globalOptionsControl.newTransformer();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void duplicate_actionPerformed() {
    try {
      TransformerInfo transformerInfoSource =
        getSelectedTransformerInfo();
      if (transformerInfoSource != null)
        globalOptionsControl.newTransformer(transformerInfoSource);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void modify_actionPerformed() {
    try {
      TransformerInfo transformerInfo =
        getSelectedTransformerInfo();
      if (transformerInfo != null)
        globalOptionsControl.editTransformerInfo(
          transformerInfo.getTransformerName());
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void remove_actionPerformed() {
    try {
      TransformerInfo transformerInfo =
        getSelectedTransformerInfo();
      if (transformerInfo != null) {
        if (JOptionPane.showConfirmDialog(
          getFrame(),
          viewResource.getString(Label.DO_YOU_CONFIRM) + " ?",
          viewResource.getString(Label.REMOVING_A_TRANSFORMER),
          JOptionPane.OK_CANCEL_OPTION) != JOptionPane.OK_OPTION)
          return;
        globalOptionsControl.removeTransformer(
          transformerInfo.getTransformerName());
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private TransformerInfo getSelectedTransformerInfo() {
    int index = table.getSelectedRow();
    return index >= 0 && index < transformerInfoList.size() ?
      (TransformerInfo)transformerInfoList.get(index) : null;
  }

  private void setItemMenuEnabled(boolean isEnabled) {
    TransformerInfo transformerInfo =
      isEnabled ? getSelectedTransformerInfo() : null;
    menuDuppliquer.setEnabled(isEnabled);
    menuModifier.setEnabled(isEnabled);
    menuSupprimer.setEnabled(isEnabled);
  }

  /////////////////////////////////////
  // TransformerInfoView implementation
  /////////////////////////////////////

  public void update() {
    table.getSelectionModel().clearSelection();
    table.tableChanged(new TableModelEvent(tableModel));
    setItemMenuEnabled(false);
  }

  public Frame getFrame() {
    return (Frame)this;
  }

  public void select(String transformerName) {
    for (int k = 0; k < transformerInfoList.size(); k++ ) {
      TransformerInfo transformerInfo =
        (TransformerInfo)transformerInfoList.get(k);
      if (transformerName.equals(transformerInfo.getTransformerName())) {
        table.setRowSelectionInterval(k, k);
        break;
      }
    }
  }
}
