/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.datatransfer.Clipboard;
import java.util.ResourceBundle;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.border.EmptyBorder;

import org.w3c.dom.Node;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.compare.DocumentAssociationAccess;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.node.NodeViewer;
import org.xmloperator.framework.view.node.NodeDetailView;
import org.xmloperator.framework.view.status.MessageDisplay;
import org.xmloperator.framework.view.status.ToolStatusBar;
import org.xmloperator.framework.view.global.GlobalViewAccess;
import org.xmloperator.framework.view.instance.IdsDialogAccess;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.framework.control.instance.ToolControl;
import org.xmloperator.framework.control.instance.ViewControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.framework.control.instance.OptionsControl;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.view.model.DomTreeModel;
import org.xmloperator.view.node.PanelNodeDetail;
import org.xmloperator.view.main.menu.MenuFile;
import org.xmloperator.view.main.menu.MenuDocument;
import org.xmloperator.view.main.menu.MenuEdit;
import org.xmloperator.view.main.menu.MenuMove;
import org.xmloperator.view.main.menu.MenuFragment;
import org.xmloperator.view.main.menu.MenuView;
import org.xmloperator.view.main.menu.MenuCompare;
import org.xmloperator.view.main.menu.MenuHelp;
import org.xmloperator.view.main.PanelDocumentTree;
import org.xmloperator.view.main.ToolStatusBarImpl;

public final class ToolFrame extends JFrame implements ToolView {
  private static final Dimension panelDimMin = new Dimension(80, 80);
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private int instanceIdentifier;
  private FullyFledgedDocument fullyFledgedDocument;
  private Options options;
  private GlobalControl globalControl;
  private GlobalViewControl globalViewControl;
  private GlobalOptionsControl globalOptionsControl;
  private ToolControl toolControl;
  private ViewControl viewControl;
  private EditControl editControl;
  private DocumentViewControl documentViewControl;
  private FileControl fileControl;
  private OptionsControl optionsControl;
  private JSplitPane mainSplitPane;
  private DomTreeModel domTreeModel;
  private PanelDocumentTree documentTreePanel;
  private JScrollPane treeScrollPane;
  private PanelNodeDetail detailPanel;
  private boolean isHorizontalSplit;
  private ToolStatusBarImpl toolStatusBar;
  private MenuFile menuFile;
  private MenuDocument menuDocument;
  private MenuEdit menuEdit;
  private MenuMove menuMove;
  private MenuFragment menuFragment;
  private MenuView menuView;
  private MenuCompare menuCompare;
  private MenuHelp menuHelp;

  public ToolFrame(int instanceIdentifier,
    FullyFledgedDocument fullyFledgedDocument,
    DocumentAssociationAccess documentAssociationAccess, Options options,
    GlobalControl globalControl,
    GlobalViewControl globalViewControl,
    GlobalOptionsControl globalOptionsControl,
    ToolControl toolControl,
    ViewControl viewControl,
    EditControl editControl,
    DocumentViewControl documentViewControl,
    FileControl fileControl,
    OptionsControl optionsControl,
    Shortcuts shortcuts) {
    super(Constants.TOOL + '.' + instanceIdentifier);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    this.instanceIdentifier = instanceIdentifier;
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.globalControl = globalControl;
    this.globalViewControl = globalViewControl;
    this.globalOptionsControl = globalOptionsControl;
    this.toolControl = toolControl;
    this.viewControl = viewControl;
    this.editControl = editControl;
    this.documentViewControl = documentViewControl;
    this.fileControl = fileControl;
    this.optionsControl = optionsControl;
    // Volet contenu
    JPanel voletContenu = new JPanel(new BorderLayout());
    this.getContentPane().add(voletContenu);
    // Status bar
    toolStatusBar = new ToolStatusBarImpl(fullyFledgedDocument,
      globalControl, editControl, fileControl, options);
    voletContenu.add(toolStatusBar, BorderLayout.SOUTH);
    // Menus
    JMenuBar menuBar = new JMenuBar();
    this.setJMenuBar(menuBar);
    menuFile = new MenuFile(fullyFledgedDocument, options,
      toolControl, fileControl, globalOptionsControl,
      (GlobalViewAccess)globalViewControl, shortcuts,
      (MessageDisplay)toolStatusBar);
    menuBar.add(menuFile);
    menuDocument = new MenuDocument(fullyFledgedDocument, options,
      globalControl, globalViewControl, fileControl, editControl,
      globalOptionsControl, shortcuts);
    menuBar.add(menuDocument);
    menuEdit = new MenuEdit((ToolView)this, editControl,
      shortcuts);
    menuBar.add(menuEdit);
    menuMove = new MenuMove((ToolView)this, editControl,
      documentViewControl, shortcuts);
    menuBar.add(menuMove);
    menuFragment = new MenuFragment((ToolView)this, editControl,
      toolControl, shortcuts);
    menuBar.add(menuFragment);
    menuView = new MenuView(fullyFledgedDocument, options,
      viewControl, documentViewControl, globalViewControl, shortcuts);
    menuBar.add(menuView);
    menuCompare = new MenuCompare(toolControl, shortcuts);
    menuBar.add(menuCompare);
    menuHelp = new MenuHelp(options, optionsControl, toolControl, shortcuts);
    menuBar.add(menuHelp);
    shortcuts.registerKeyboardAction(voletContenu);
    // Document tree panel
    domTreeModel = new DomTreeModel(fullyFledgedDocument);
    documentTreePanel = new PanelDocumentTree(domTreeModel,
      menuEdit, menuMove, menuFragment, menuCompare, documentViewControl,
      fullyFledgedDocument, documentAssociationAccess, options);
    treeScrollPane = new JScrollPane(documentTreePanel);
    documentTreePanel.setViewport(treeScrollPane.getViewport());
    treeScrollPane.setMinimumSize(panelDimMin);
    // Detail view
    isHorizontalSplit =
      options.getThreeStateOptionValue(Options.SPLIT) == ThreeStateOption.TRUE;
    detailPanel = new PanelNodeDetail(this, menuEdit, fullyFledgedDocument,
      (Options)options, editControl, fileControl, documentViewControl,
      (IdsDialogAccess)viewControl, !isHorizontalSplit);
    detailPanel.setMinimumSize(panelDimMin);
    // Volet principal de dcomposition
    mainSplitPane = new JSplitPane();
    mainSplitPane.setContinuousLayout(true);
    try {
      mainSplitPane.setResizeWeight(0.5); // since 1.3
    }
    catch (NoSuchMethodError error) {
    }
    mainSplitPane.setLeftComponent(treeScrollPane);
    mainSplitPane.setRightComponent(detailPanel);
    Dimension mainDim = new Dimension(
      options.getIntOptionValue(Options.MAINWIDTH),
      options.getIntOptionValue(Options.MAINHEIGHT));
    mainSplitPane.setPreferredSize(mainDim);
    voletContenu.add(mainSplitPane);
    this.pack();
    setLayout();
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      toolControl.deleteInstance();
    else
      super.processWindowEvent(event);
  }

  public void setMenuEnabled(boolean isEnabled) {
    menuFile.setEnabled(isEnabled);
    menuDocument.setEnabled(isEnabled);
    menuEdit.setEnabled(isEnabled);
    menuMove.setEnabled(isEnabled);
    menuFragment.setEnabled(isEnabled);
    menuView.setEnabled(isEnabled);
    menuCompare.setEnabled(isEnabled);
    menuHelp.setEnabled(isEnabled);
    toolStatusBar.setMenuEnabled(isEnabled);
  }

  public MenuHelp getMenuHelp() {
    return menuHelp;
  }

  ////////////////////////////
  // NodeViewer implementation
  ////////////////////////////

  public void displayNode(Node node) {
    setSelectedNode(node, true);
  }

  //////////////////////////
  // ToolView implementation
  //////////////////////////

  public Frame getFrame() {
    return this;
  }

  public Clipboard getSystemClipboard() {
    return this.getToolkit().getSystemClipboard();
  }

  public DocumentTreeView getDocumentTreeView() {
    return documentTreePanel;
  }

  public NodeDetailView getNodeDetailView() {
    return detailPanel;
  }

  public void setSelectedNode(Node node, boolean withRecentring) {
    documentTreePanel.setSelectedNode(node, withRecentring);
    detailPanel.setNode(node);
  }

  public void setSelectedNodeInterval(Node firstNode, Node lastNode,
    boolean withRecentring) {
    documentTreePanel.setSelectedNodeInterval(firstNode, lastNode,
      withRecentring);
    detailPanel.setNode(firstNode == lastNode ? firstNode : null);
  }

  public ToolStatusBar getToolStatusBar() {
    return toolStatusBar;
  }

  public void clear() {
    detailPanel.setNode(null);
    documentTreePanel.setRootNode(null);
    toolStatusBar.clear();
  }

  public boolean isHorizontalSplit() {
    return isHorizontalSplit;
  }

  public Dimension getContentPanelDimension() {
    return new Dimension(
      mainSplitPane.getWidth(),
      mainSplitPane.getHeight());
  }

  public int getDetailPartPerCent() {
    int length =
      isHorizontalSplit ? mainSplitPane.getWidth() : mainSplitPane.getHeight();
    length -= mainSplitPane.getDividerSize();
    int treePartPerCent =
      (201 * mainSplitPane.getDividerLocation()) / length / 2;
    return 100 - treePartPerCent;
  }

  public void setLayout() {
    isHorizontalSplit =
      options.getThreeStateOptionValue(Options.SPLIT) == ThreeStateOption.TRUE;
    detailPanel.setLayout(!isHorizontalSplit);
    mainSplitPane.setOrientation(isHorizontalSplit ?
      JSplitPane.HORIZONTAL_SPLIT : JSplitPane.VERTICAL_SPLIT);
    int detailPartPerCent = options.getIntOptionValue(Options.DETAILPART);
    int treePartPerCent = 100 - detailPartPerCent;
    mainSplitPane.setDividerLocation((double)treePartPerCent * 0.01);
  }
}
