/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main;

import java.awt.Component;
import java.awt.Font;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.compare.ElementCharacterizationAccess;
import org.xmloperator.framework.document.compare.ElementAssociation;
import org.xmloperator.framework.document.compare.DocumentAssociation;
import org.xmloperator.framework.document.compare.DocumentAssociationAccess;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.view.model.DomTreeNode;

public final class ToolTreeCellRenderer extends DefaultTreeCellRenderer {
  private static final String FONT_NAME = "dialog";
  private static final Font[][][] fonts = {
    {{new Font(FONT_NAME, Font.PLAIN, 12),
    new Font(FONT_NAME, Font.ITALIC, 12)},
    {new Font(FONT_NAME, Font.BOLD, 12),
    new Font(FONT_NAME, Font.ITALIC + Font.BOLD, 12)}},
    {{new Font(FONT_NAME, Font.PLAIN, 14),
    new Font(FONT_NAME, Font.ITALIC, 14)},
    {new Font(FONT_NAME, Font.BOLD, 14),
    new Font(FONT_NAME, Font.ITALIC + Font.BOLD, 14)}}};
  private DocumentAssociationAccess documentAssociationAccess;
  private FullyFledgedDocument fullyFledgedDocument;
  private Options options;

  public ToolTreeCellRenderer(
    DocumentAssociationAccess documentAssociationAccess,
    FullyFledgedDocument fullyFledgedDocument, Options options) {
    this.documentAssociationAccess = documentAssociationAccess;
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    // putClientProperty("html", null);
  }

  public Component getTreeCellRendererComponent(JTree tree, Object value,
    boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
    Component rendererComponent =
      super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
      row, hasFocus);
    boolean isItalic = false;
    boolean isBold = false;
    DocumentAssociation documentAssociation =
      documentAssociationAccess.getDocumentAssociation();
    if (documentAssociation != null
      && documentAssociation.getAssociatedDocument(
      fullyFledgedDocument.getDocument()) != null
      && value instanceof DomTreeNode) {
      Node node = ((DomTreeNode)value).getNode();
      if (node != null && node.getNodeType() != Node.ELEMENT_NODE)
        node = node.getParentNode();
      if (node == null || node.getNodeType() != Node.ELEMENT_NODE)
        isItalic = true;
      else {
        ElementAssociation elementAssociation =
          documentAssociation.getElementAssociation((Element)node);
        if (elementAssociation == null)
          isBold = true;
        else
          isItalic = !elementAssociation.areElementsEqual(
              documentAssociation.getElementCharacterizationAccess());
      }
    }
    boolean isLarge =
      options.getThreeStateOptionValue(Options.FONT) == ThreeStateOption.TRUE;
    rendererComponent.setFont(
      fonts[isLarge ? 1 : 0][isBold ? 1 : 0][isItalic ? 1 : 0]);
    return rendererComponent;
  }
}
