/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main;

import javax.swing.event.TreeModelEvent;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.manage.TreeViewDocumentListener;
import org.xmloperator.framework.view.tree.DomTree;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ToolViewAccess;
import org.xmloperator.util.exception.ExceptionControler;

public final class ToolViewActivator implements TreeViewDocumentListener {
  private final ToolViewAccess toolViewAccess;
  private ToolView toolView = null;
  private DocumentTreeView documentTreeView = null;
  private DomTree domTree = null;

  public ToolViewActivator(ToolViewAccess toolViewAccess) {
    this.toolViewAccess = toolViewAccess;
  }

  protected boolean ok() {
    if (this.toolViewAccess == null)
      return false;
    this.toolView = this.toolViewAccess.getToolView();
    if (this.toolView == null)
      return false;
    this.documentTreeView = toolView.getDocumentTreeView();
    if (this.documentTreeView == null)
      return false;
    this.domTree = this.documentTreeView.getDomTree();
    return this.domTree != null;
  }

  private Object beforeNodeRemove(Node node) {
    return this.domTree.createTreeModelEvent(node);
  }

  private Object beforeNodesRemove(Node[] nodes) {
    return this.domTree.createTreeModelEvent(nodes);
  }

  private void afterNodesRemove(Object context) {
    if (!(context instanceof TreeModelEvent))
      throw new IllegalArgumentException();
    this.domTree.fireTreeNodesRemoved((TreeModelEvent)context);
  }

  //////////////////////////////////////////
  // TreeViewDocumentListener implementation
  //////////////////////////////////////////

  public void notifyNodeInsert(Node node) {
    if (this.ok()) {
      this.domTree.nodeInserted(node);
      this.toolView.setSelectedNode(node, false);
    }
  }

  public void notifyNodesInsert(Node[] nodes) {
    if (this.ok()) {
      this.domTree.nodesInserted(nodes);
      this.toolView.setSelectedNodeInterval(
          nodes[0], nodes[nodes.length - 1], false);
    }
  }

  public Object notifyBeforeNodeRemoveMoveOrReplace(Node node) {
    return this.ok() ? this.beforeNodeRemove(node) : null;
  }

  public Object notifyBeforeNodesRemoveMoveOrReplace(Node[] nodes) {
    return this.ok() ? this.beforeNodesRemove(nodes) : null;
  }

  public void notifyAfterNodesRemove(Object context, Node nodeToSelect) {
    if (this.ok()) {
      this.afterNodesRemove(context);
      this.toolView.setSelectedNode(nodeToSelect, false);
    }
  }

  public void notifyAfterNodeMoveOrReplace(Object context, Node node) {
    if (this.ok()) {
      this.afterNodesRemove(context);
      this.domTree.nodeInserted(node);
      this.toolView.setSelectedNode(node, false);
    }
  }

  public void notifyAfterNodesMoveOrReplace(Object context, Node[] nodes) {
    if (this.ok()) {
      this.afterNodesRemove(context);
      this.domTree.nodesInserted(nodes);
      this.toolView.setSelectedNodeInterval(
          nodes[0], nodes[nodes.length - 1], false);
    }
  }

  public void notifyNodeSetValue(Node node) {
    if (this.ok()) {
      this.domTree.nodeChanged(node);
      Node parentNode = node.getParentNode();
      if (parentNode != null && parentNode.getNodeType() == Node.ELEMENT_NODE)
        this.domTree.nodeChanged(parentNode);
      this.toolView.setSelectedNode(node, false);
    }
  }

  public Object notifyBeforeTextJoin(Node secondTextNode) {
    return this.ok() ? this.beforeNodeRemove(secondTextNode) : null;
  }

  public void notifyAfterTextJoin(Object context, Node textNode) {
    if (this.ok()) {
      this.afterNodesRemove(context);
      this.domTree.nodeChanged(textNode);
      this.toolView.setSelectedNode(textNode, false);
    }
  }

  public void notifyTextSplit(Node textNode) {
    if (this.ok()) {
      this.domTree.nodeInserted(textNode.getNextSibling());
      this.domTree.nodeChanged(textNode);
      this.toolView.setSelectedNode(textNode, false);
    }
  }

  public void notifyAttributeSetValue(Element element, String attributeName) {
    if (this.ok()) {
      this.domTree.nodeChanged(element);
      this.toolView.setSelectedNode(element, false);
    }
  }

  public void notifyElementNameReplace(Element element) {
    if (this.ok()) {
      this.domTree.nodeChanged(element);
      this.toolView.setSelectedNode(element, false);
      this.documentTreeView.expandNode(element);
    }
  }

  public Object notifyBeforeNodeIntervalPack(Node[] nodes) {
    return this.ok() ? this.beforeNodesRemove(nodes) : null;
  }

  public void notifyAfterNodeIntervalPack(Object context, Element element) {
    if (this.ok()) {
      this.afterNodesRemove(context);
      this.domTree.nodeInserted(element);
      this.toolView.setSelectedNode(element, false);
      this.documentTreeView.expandNode(element);
    }
  }

  public Object notifyBeforeElementUnPack(Element element) {
    return this.ok() ? this.beforeNodeRemove(element) : null;
  }

  public void notifyAfterElementUnpack(Object context, Node[] children) {
    if (this.ok()) {
      this.afterNodesRemove(context);
      this.domTree.nodesInserted(children);
      this.toolView.setSelectedNodeInterval(
          children[0], children[children.length - 1], false);
    }
  }
}
