/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main.menu;

import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcut;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.task.Action;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.view.main.menu.Menu;

public final class MenuEdit extends Menu {
  private static final String[] operatorShortcutIds = new String[] {
    Shortcuts.SHORTCUT_OPERATOR1,
    Shortcuts.SHORTCUT_OPERATOR2,
    Shortcuts.SHORTCUT_OPERATOR3};
  private EditControl editControl;
  private ToolView toolView;

  public MenuEdit(
      ToolView toolView, EditControl editControl,
      Shortcuts shortcuts) {
    super(Label.EDITION, shortcuts);
    this.toolView = toolView;
    this.editControl = editControl;
    // Oprations
    this.add(this.createMenuItem(
        Label.DEFAIRE, Shortcuts.SHORTCUT_DEFAIRE, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          undo_execute();
        }
      }));
    this.add(this.createMenuItem(
        Label.REFAIRE,Shortcuts.SHORTCUT_REFAIRE, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          redo_execute();
        }
      }));
    this.addSeparator();
    // New, replace, remove
    this.add(this.createMenuNewPrevious());
    this.add(this.createMenuNewNext());
    this.add(this.createMenuNewChild());
    this.add(this.createMenuReplaceElement());
    this.add(this.createMenuRemoveNodes());
    this.addSeparator();
    // Attributes
    this.add(this.createMenuNewAttribute());
    this.add(this.createMenuRemoveAttribute());
    this.addSeparator();
    // Pack / unpack
    this.add(this.createMenuPack());
    this.add(this.createMenuUnpack());
    this.addSeparator();
    // Join / Split
    this.add(this.createMenuJoinNext());
    this.add(this.createMenuSplitText());
    this.addSeparator();
    // Operators
    this.add(this.createMenuOperateElement());
    Action operatorAction =
      new Action() {
        public void execute(ActionEvent event) {
          operateElement_execute(event);
        }
      };
    for (int k = 0; k < operatorShortcutIds.length; k++)
      shortcuts.getShortcut(operatorShortcutIds[k]).
      setActionListener(ExceptionControler.control(operatorAction));
  }

  public JMenuItem createMenuNewPrevious() {
    return this.createMenuItem(
        Label.NOUVEAU_PRECEDENT, Shortcuts.SHORTCUT_NOUVEAU_PRECEDENT,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          newNode_execute(EditControl.INSERTION_PREVIOUS);
        }
      });
  }

  public JMenuItem createMenuNewNext() {
    return this.createMenuItem(
        Label.NOUVEAU_SUIVANT, Shortcuts.SHORTCUT_NOUVEAU_SUIVANT,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          newNode_execute(EditControl.INSERTION_NEXT);
        }
      });
  }

  public JMenuItem createMenuNewChild() {
    return this.createMenuItem(
        Label.NOUVEAU_DESCENDANT, Shortcuts.SHORTCUT_NOUVEAU_DESCENDANT,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          newNode_execute(EditControl.INSERTION_CHILD);
        }
      });
  }

  public JMenuItem createMenuReplaceElement() {
    return this.createMenuItem(
        Label.REPLACE, Shortcuts.SHORTCUT_REPLACE, new Action() {
      public void execute(ActionEvent event) throws Exception {
        replaceElement_execute();
      }
    });
  }

  public JMenuItem createMenuRemoveNodes() {
    return this.createMenuItem(
        Label.SUPPRIMER, Shortcuts.SHORTCUT_SUPPRIMER, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          removeNodes_execute();
        }
      });
  }

  public JMenuItem createMenuNewAttribute() {
    return this.createMenuItem(
        Label.NEW_ATTRIBUTE, Shortcuts.SHORTCUT_NEW_ATTRIBUTE,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          newAttribute_execute();
        }
      });
  }

  public JMenuItem createMenuRemoveAttribute() {
    return this.createMenuItem(
        Label.REMOVE_ATTRIBUTE, Shortcuts.SHORTCUT_REMOVE_ATTRIBUTE,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          removeAttribute_execute();
        }
      });
  }

  public JMenuItem createMenuPack() {
    return this.createMenuItem(
        Label.EMPAQUETER, Shortcuts.SHORTCUT_EMPAQUETER,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          pack_execute();
        }
      });
  }

  public JMenuItem createMenuUnpack() {
    return this.createMenuItem(
        Label.DEPAQUETER, Shortcuts.SHORTCUT_DEPAQUETER,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          unpack_execute();
        }
      });
  }

  public JMenuItem createMenuJoinNext() {
    return this.createMenuItem(
        Label.JOINDRE_SUIVANT, Shortcuts.SHORTCUT_JOINDRE,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          joinText_execute(true);
        }
      });
  }

  public JMenuItem createMenuSplitText() {
    return this.createMenuItem(
        Label.DIVISER, Shortcuts.SHORTCUT_DIVISER,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          splitText_execute();
        }
      });
  }

  public JMenuItem createMenuOperateElement() {
    return this.createMenuItem(
        Label.SPECIAL, null, true, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          operateElement_execute();
        }
      });
  }

  private void undo_execute() {
    editControl.undoOperation();
  }

  private void redo_execute() {
    editControl.redoOperation();
  }

  private void newNode_execute(int insertionType) {
    editControl.insertNewNode(null, insertionType, null);
  }

  private void replaceElement_execute() {
    editControl.replaceNode(null, Node.ELEMENT_NODE, null);
  }

  private void removeNodes_execute() {
    editControl.deleteNodeInterval(null);
  }

  private void newAttribute_execute() {
    editControl.addNewAttribute(null, null);
  }

  private void removeAttribute_execute() {
    editControl.setAttributeValue(null, null, null);
  }

  private void pack_execute() {
    editControl.packNodeInterval(null, null);
  }

  private void unpack_execute() {
    editControl.unpackElement(null);
  }

  private void joinText_execute(boolean isSuivant) {
    editControl.joinText(null, isSuivant);
  }

  private void splitText_execute() {
    editControl.splitText(null, 0);
  }

  private void operateElement_execute() {
    editControl.operateElement(null, null);
  }

  private void operateElement_execute(ActionEvent event) {
    String command = event.getActionCommand();
    if (!command.startsWith("OPERATOR"))
      return;
    int operatorIndex = Integer.parseInt(command.substring(8)) - 1;
    editControl.operateElement(null, operatorIndex);
  }
}
