/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main.menu;

import java.io.File;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.List;
import javax.swing.JMenuItem;
import javax.swing.JMenu;

import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcut;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.view.status.MessageDisplay;
import org.xmloperator.framework.view.dialog.DialogText;
import org.xmloperator.framework.view.dialog.DialogListSelection;
import org.xmloperator.framework.view.global.FileChooser;
import org.xmloperator.framework.view.global.GlobalViewAccess;
import org.xmloperator.framework.control.task.Action;
import org.xmloperator.framework.control.instance.ToolControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.util.dom.RelaxNGSimplifier;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.FileDialogs;
import org.xmloperator.util.view.XMLFileChooser;
import org.xmloperator.view.main.menu.Menu;

public final class MenuFile extends Menu {
  private ToolControl toolControl;
  private FileControl fileControl;
  private GlobalOptionsControl globalOptionsControl;
  private GlobalViewAccess globalViewAccess;
  private FullyFledgedDocument fullyFledgedDocument;
  private Options options;
  private MessageDisplay messageDisplay;

  public MenuFile(
      FullyFledgedDocument fullyFledgedDocument,
      Options options, ToolControl toolControl,
      FileControl fileControl, GlobalOptionsControl globalOptionsControl,
      GlobalViewAccess globalViewAccess,
      Shortcuts shortcuts, MessageDisplay messageDisplay) {
    super(Label.FICHIER, shortcuts);
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.toolControl = toolControl;
    this.fileControl = fileControl;
    this.globalOptionsControl = globalOptionsControl;
    this.globalViewAccess = globalViewAccess;
    this.messageDisplay = messageDisplay;
    boolean isAdvancedUse =
      options.getThreeStateOptionValue(Options.ADVANCEDUSE)
        == ThreeStateOption.TRUE;
    {
      JMenu menuOpenDocument = this.createMenu(Label.OPEN_DOCUMENT);
      menuOpenDocument.add(this.createMenuItem(
          Label.BROWSE, Shortcuts.SHORTCUT_OUVRIR, true, new Action() {
        public void execute(ActionEvent event) throws Exception {
          openLocalDocument_execute();
        }
      }));
      menuOpenDocument.add(this.createMenuItem(
          Label.ENTER, null, true, new Action() {
        public void execute(ActionEvent event) throws Exception {
          openDocument_execute();
        }
      }));
      this.add(menuOpenDocument);
    }
    this.add(this.createMenuItem(
        Label.REOUVRIR_DOCUMENT, Shortcuts.SHORTCUT_REOUVRIR, true,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          reopenDocument_execute();
        }
      }));
    this.add(this.createMenuItem(
        Label.FERMER_DOCUMENT, Shortcuts.SHORTCUT_FERMER,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          closeDocument_execute();
        }
      }));
    this.addSeparator();
    this.add(this.createMenuItem(
        Label.SAVE_DOCUMENT, Shortcuts.SHORTCUT_ENREGISTRER,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          saveDocument_execute();
        }
      }));
    this.add(this.createMenuItem(
        Label.SAVE_AND_RELOAD_DOCUMENT,
        Shortcuts.SHORTCUT_ENREGISTRER_ET_RECHARGER,
      new Action()  {
        public void execute(ActionEvent event) throws Exception {
          saveAndReloadDocument_execute();
        }
      }));
    this.add(this.createMenuItem(
        Label.SAVE_DOCUMENT_AS, null, true, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          saveDocumentAs_execute();
        }
      }));
    this.addSeparator();
    this.add(this.createMenuItem(
        Label.TRANSFORM_AS, null, true, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          transformAs_execute();
        }
      }));
      this.add(this.createMenuItem(
          Label.RETRANSFORM, Shortcuts.SHORTCUT_TRANSFORM, true,
        new Action()  {
          public void execute(ActionEvent event) throws Exception {
            retransform_execute();
          }
        }));
      this.add(this.createMenuItem(
        Label.PROCESS_BATCH, null, true, new Action()  {
          public void execute(ActionEvent event) throws Exception {
            processBatch_execute();
          }
        }));
    this.addSeparator();
    {
      JMenu menuOpenDTD = this.createMenu(Label.OPEN_DTD);
      menuOpenDTD.add(this.createMenuItem(
          Label.BROWSE, null, true, new Action() {
        public void execute(ActionEvent event) throws Exception {
          openLocalSchema_execute(DocumentTypeInfo.SCHEMA_TYPE_DTD);
        }
      }));
      menuOpenDTD.add(this.createMenuItem(
          Label.ENTER, null, true, new Action() {
        public void execute(ActionEvent event) throws Exception {
          openSchema_execute(DocumentTypeInfo.SCHEMA_TYPE_DTD);
        }
      }));
      this.add(menuOpenDTD);
    }
    this.add(this.createMenuItem(
        Label.CLOSE_DTD, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          closeDTD_execute();
        }
      }));
    this.addSeparator();
    {
      JMenu menuOpenRNG = this.createMenu(Label.OPEN_RNG);
      menuOpenRNG.add(createMenuItem(
          Label.BROWSE, null, true, new Action() {
        public void execute(ActionEvent event) throws Exception {
          openLocalSchema_execute(DocumentTypeInfo.SCHEMA_TYPE_RNG);
        }
      }));
      menuOpenRNG.add(this.createMenuItem(
          Label.ENTER, null, true, new Action() {
        public void execute(ActionEvent event) throws Exception {
          openSchema_execute(DocumentTypeInfo.SCHEMA_TYPE_RNG);
        }
      }));
      this.add(menuOpenRNG);
    }
    if (isAdvancedUse) {
      JMenu menuSimplifyRNG = this.createMenu(Label.SIMPLIFY_RNG);
      menuSimplifyRNG.add(this.createMenuItem(Label.SIMPLE_SYNTAX, null, true,
          new Action()  {
        public void execute(ActionEvent event) throws Exception {
          simplifyRNG_execute(
              RelaxNGSimplifier.SIMPLIFICATION_TYPE_COMPLETE);
        }
      }));
      menuSimplifyRNG.add(this.createMenuItem(
          Label.GROUP_WITH_MORE_THAN_TWO_CHILDREN, null, true,
          new Action()  {
        public void execute(ActionEvent event) throws Exception {
          simplifyRNG_execute(
              RelaxNGSimplifier.
              SIMPLIFICATION_TYPE_GROUP_WITH_MORE_THAN_TWO_CHILDREN);
        }
      }));
      menuSimplifyRNG.add(this.createMenuItem(
          Label.REF_RESOLUTION_ONLY, null, true, new Action()  {
        public void execute(ActionEvent event) throws Exception {
          simplifyRNG_execute(
              RelaxNGSimplifier.SIMPLIFICATION_TYPE_REF_RESOLUTION_ONLY);
        }
      }));
      this.add(menuSimplifyRNG);
    }
    this.addSeparator();
    this.add(this.createMenuItem(
        Label.QUITTER, Shortcuts.SHORTCUT_EXIT, new Action() {
      public void execute(ActionEvent event) throws Exception {
        exit_execute();
      }
    }));
  }

  private void openLocalDocument_execute() {
    String pathName =
        this.globalViewAccess.getFileChooser().chooseDocumentPathName(
        false, viewResource.getString(Label.OPEN_DOCUMENT));
    if (pathName != null)
      fileControl.loadDocument(FileUtils.expandedSystemId(pathName));
  }

  private void openDocument_execute() {
    String systemId = this.enterSystemId(Label.OPEN_DOCUMENT);
    if (systemId != null)
      fileControl.loadDocument(systemId);
  }

  private final String enterSystemId(String invite) {
    DialogText dialog = this.globalViewAccess.getDialogIdentifier(invite);
    dialog.setTitle("System Id");
    dialog.setText("http://");
    dialog.show();
    return dialog.isOK() ? dialog.getText() : null;
  }

  private void reopenDocument_execute() {
    List resourceList = options.getOptionList(Options.RESOURCE);
    if (resourceList.isEmpty())
      return;
    DialogListSelection dialogReopen = this.globalViewAccess.getDialogReopen();
    dialogReopen.setItemList(resourceList);
    dialogReopen.setSelectedIndex(0);
    dialogReopen.show();
    if (dialogReopen.isOK())
      fileControl.loadDocument(dialogReopen.getSelectedItem().toString());
  }

  private void closeDocument_execute() {
    fileControl.closeDocument();
  }

  private void saveDocument_execute() {
    fileControl.saveDocument(false);
  }

  private void saveAndReloadDocument_execute() {
    fileControl.saveDocument(true);
  }

  private void saveDocumentAs_execute() {
    if (fullyFledgedDocument.getDocument() == null) {
      displayMessageOpenedDocumentRequired();
      return;
    }
    String documentPathName =
        this.globalViewAccess.getFileChooser().chooseDocumentPathName(
        true, viewResource.getString(Label.SAVE_DOCUMENT));
    if (documentPathName != null)
      fileControl.saveDocument(documentPathName, true);
  }

  private void transformAs_execute() {
    fileControl.transformDocument(false);
  }

  private void retransform_execute() {
    fileControl.transformDocument(true);
  }

  private void processBatch_execute() {
    String documentPathName =
        this.globalViewAccess.getFileChooser().chooseDocumentPathName(
        false, viewResource.getString(Label.PROCESS_BATCH));
    if (documentPathName != null)
      fileControl.processTransformBatch(
          FileUtils.reducedSystemId(documentPathName));
  }

  private void openLocalSchema_execute(int schemaType) {
    if (fullyFledgedDocument.getDocument() == null) {
      displayMessageOpenedDocumentRequired();
      return;
    }
    FileChooser fileChooser = globalViewAccess.getFileChooser();
    String pathName = schemaType == DocumentTypeInfo.SCHEMA_TYPE_DTD ?
        fileChooser.chooseDTDPathName(false) :
        fileChooser.chooseRNGPathName(false);
    if (pathName != null)
      fileControl.loadSchema(
          FileUtils.expandedSystemId(pathName), schemaType);
  }

  private void openSchema_execute(int schemaType) {
    if (fullyFledgedDocument.getDocument() == null) {
      displayMessageOpenedDocumentRequired();
      return;
    }
    String systemId = this.enterSystemId(
        schemaType == DocumentTypeInfo.SCHEMA_TYPE_DTD ?
        Label.OPEN_DTD : Label.OPEN_RNG);
    if (systemId != null)
      fileControl.loadSchema(systemId, schemaType);
  }

  private void closeDTD_execute() {
    fileControl.closeDTD();
  }

  private void simplifyRNG_execute(int simplificationType) {
    if (fullyFledgedDocument.getDocument() == null) {
      displayMessageOpenedDocumentRequired();
      return;
    }
    if (fullyFledgedDocument.getDocumentManager().isDocumentModified()) {
      displayResourceMessage(Label.CHANGES_ARE_NOT_SAVED);
      return;
    }
    String fullSyntaxSystemId = fullyFledgedDocument.getDocumentSystemId();
    String simpleSyntaxPathname =
        this.globalViewAccess.getFileChooser().chooseRNGPathName(true);
    if (simpleSyntaxPathname == null)
      return;
    String simpleSyntaxSystemId =
        FileUtils.reducedSystemId(simpleSyntaxPathname);
    globalOptionsControl.buildRNGSimpleSyntaxFromFullSyntax(
        fullSyntaxSystemId, simpleSyntaxSystemId, simplificationType);
  }

  private void exit_execute() {
    toolControl.deleteInstance();
  }

  private void displayResourceMessage(String key) {
    messageDisplay.displayMessage(viewResource.getString(key));
  }

  private void displayMessageOpenedDocumentRequired() {
    displayResourceMessage(Label.THIS_ACTION_REQUIRES_AN_OPENED_DOCUMENT);
  }
}
