/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main.menu;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.ArrayList;
import javax.swing.JMenuItem;
import javax.swing.JMenu;
import javax.swing.ImageIcon;

import org.w3c.dom.Node;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.MenuItemData;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcut;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.task.Action;
import org.xmloperator.framework.control.instance.OptionsControl;
import org.xmloperator.framework.control.instance.ToolControl;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.AboutBox;
import org.xmloperator.view.main.menu.MenuOptions;
import org.xmloperator.view.main.menu.Menu;

public final class MenuHelp extends Menu {
  private static ImageIcon toolImageIcon =
    new ImageIcon(Constants.TOOL + ".gif");
  private static final String[] ABOUT_TOOL = {
    Constants.TOOL + " " + Constants.VERSION,
    "Copyright (c) 2000 - 2003 The_" + Constants.TOOL + "_project",
    "(http://www." + Constants.TOOL + ".net)",
    "",
    "This product includes software developed by the",
    "Apache Software Foundation (http://www.apache.org/)."};
  private static final String[] commandShortcutIds = new String[] {
    Shortcuts.SHORTCUT_COMMAND1,
    Shortcuts.SHORTCUT_COMMAND2,
    Shortcuts.SHORTCUT_COMMAND3};
  private ToolControl toolControl;
  private List menuItemDataList;
  private int menuItemCount = 0;

  public MenuHelp(
      Options options, OptionsControl optionsControl,
      ToolControl toolControl, Shortcuts shortcuts) {
    super(Label.OPTIONS, shortcuts);
    this.toolControl = toolControl;
    boolean isAdvancedUse =
      options.getThreeStateOptionValue(Options.ADVANCEDUSE)
        == ThreeStateOption.TRUE;
    // Options
    this.add(new MenuOptions(
        Label.OPTIONS_DE_L_OUTIL, optionsControl, isAdvancedUse));
    this.addSeparator();
    this.add(this.createMenuItem(
        Label.A_PROPOS, new Action() {
      public void execute(ActionEvent event) throws Exception {
        aPropos_execute();
      }
    }));
    // Additional static menu items
    menuItemDataList =
      new ArrayList(options.getOptionList(Options.MENUITEM));
    if (menuItemDataList.size() != 0) {
      addSeparator();
      for (int k = 0; k < menuItemDataList.size(); k++) {
        MenuItemData menuItemData = (MenuItemData)menuItemDataList.get(k);
        addMenuItem(menuItemData.getLabel(),
          new ActionListener() {
            public void actionPerformed(ActionEvent event) {
              menuItem_actionPerformed(event);
            }
          }, "COMMAND" + Integer.toString(k + 1));
      }
    }
  }

  public void addMenuItem(String label, ActionListener actionListener,
    String command) {
    JMenuItem menuItem = new JMenuItem(label);
    menuItem.setActionCommand(command);
    menuItem.addActionListener(actionListener);
    if (menuItemCount < commandShortcutIds.length) {
      String shortcutId = commandShortcutIds[menuItemCount];
      Shortcut shortcut = shortcuts.getShortcut(shortcutId);
      menuItem.setAccelerator(shortcut.getKeyStroke());
      shortcut.setActionListener(actionListener);
    }
    menuItemCount++;
    add(menuItem);
    validate();
  }

  public void removeMenuItem() {
    if (menuItemCount == menuItemDataList.size())
      return;
    remove(--menuItemCount);
    validate();
  }

  private void aPropos_execute() {
    AboutBox aboutBox =
        new AboutBox(null) {
      protected ImageIcon imageIcon() {
        return toolImageIcon;
      }

      protected String[] propos() {
        return ABOUT_TOOL;
      }
    };
    ViewUtils.moveComponentToCentre(aboutBox);
    aboutBox.show();
  }

  private void menuItem_actionPerformed(ActionEvent event) {
    try {
      String command = event.getActionCommand();
      if (!command.startsWith("COMMAND"))
        return;
      int menuItemIndex = Integer.parseInt(command.substring(7)) - 1;
      MenuItemData menuItemData =
        (MenuItemData)menuItemDataList.get(menuItemIndex);
      toolControl.execCommand(menuItemData.getCommand(),
        menuItemData.hasToolInstanceToBeClosed());
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }
}
