/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node;

import java.awt.Point;
import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.util.ResourceBundle;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Set;
import java.util.HashSet;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.JTableHeader;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.framework.view.node.AttributeListener;
import org.xmloperator.framework.view.node.AttributeSelectionListener;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.instance.IdsDialogAccess;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.view.node.attribute.AttributeTableCellEditor;
import org.xmloperator.view.main.menu.MenuEdit;

public final class PanelAttributes extends JPanel implements AttributeListener {
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private FullyFledgedDocument fullyFledgedDocument;
  private EditControl editControl;
  private DocumentViewControl documentViewControl;
  private IdsDialogAccess idsDialogAccess;
  private Options options;
  private AttributeSelectionListener attributeSelectionListener;
  private Frame frame;
  private MenuEdit menuEdit;
  private TableModel tableModel;
  private JTable table;
  private AttributeTableCellEditor cellEditor;
  private Element element = null;
  private Hashtable fromAttributesLocalMenuToReferencedElement = null;
  private boolean isDisplaying = false;
  private boolean isUpdatingField = false;

  public PanelAttributes(Frame frame, MenuEdit menuEdit,
    FullyFledgedDocument fullyFledgedDocument,
    EditControl editControl, DocumentViewControl documentViewControl,
    IdsDialogAccess idsDialogAccess, Options options,
    AttributeSelectionListener attributeSelectionListener) {
    super(new BorderLayout());
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.editControl = editControl;
    this.documentViewControl = documentViewControl;
    this.idsDialogAccess = idsDialogAccess;
    this.options = options;
    this.attributeSelectionListener = attributeSelectionListener;
    this.frame = frame;
    this.menuEdit = menuEdit;
    Border emptyBorder = new EmptyBorder(4, 4, 4, 4);
    // Table d'attributs
    JPanel voletAttributs = new JPanel(new BorderLayout());
    voletAttributs.setBorder(emptyBorder);
    tableModel = new AbstractTableModel() {
      public String getColumnName(int col) {
        switch (col) {
          case AttributeTableCellEditor.ATTRIBUTE_COLUMN_NAME:
            return viewResource.getString(Label.ATTRIBUTE);
          case AttributeTableCellEditor.ATTRIBUTE_COLUMN_VALUE:
            return viewResource.getString(Label.VALUE);
          default: return "";
        }
      }
      public Class getColumnClass(int col) {
        return String.class;
      }
      public int getRowCount() {
        return element != null ? element.getAttributes().getLength() : 0;
      }
      public int getColumnCount() {
        return 2;
      }
      public Object getValueAt(int row, int col) {
        try {
          if (element == null)
            return "";
          NamedNodeMap attrs = element.getAttributes();
          if (row < 0 || row >= attrs.getLength())
            return "";
          Attr attr = (Attr)attrs.item(row);
          switch (col) {
            case AttributeTableCellEditor.ATTRIBUTE_COLUMN_NAME:
              return attr.getName();
            case AttributeTableCellEditor.ATTRIBUTE_COLUMN_VALUE:
              return attr.getValue();
            default: return "";
          }
        }
        catch (Exception exception) {
          ExceptionControler.printStackTrace(exception);
          return "";
        }
      }
      public boolean isCellEditable(int row, int col) {
        return col == AttributeTableCellEditor.ATTRIBUTE_COLUMN_VALUE ?
          areAttributesEditable() : false;
      }
    };
    table = new JTable(tableModel);
    table.setShowVerticalLines(true);
    cellEditor = new AttributeTableCellEditor(frame, table,
      (AttributeListener)this, fullyFledgedDocument, options,
      this.idsDialogAccess);
    table.getColumnModel().
      getColumn(AttributeTableCellEditor.ATTRIBUTE_COLUMN_VALUE).
      setCellEditor(cellEditor);
    table.setRowHeight(24);
    table.setFont(
      options.getThreeStateOptionValue(Options.FONT) == ThreeStateOption.TRUE ?
      Constants.FONT_LARGE : Constants.FONT_NORMAL);
    table.setRowSelectionAllowed(true);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    table.getSelectionModel().addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent event) {
          attributes_selectionChanged();
        }
      });
    table.addMouseListener(
      new MouseAdapter() {
        public void mouseClicked(MouseEvent event) {
          if (event.getClickCount() == 2)
            attributes_activation();
        }
        public void mousePressed(MouseEvent event) {
          if ((event.getModifiers() & InputEvent.BUTTON3_MASK) != 0)
            attributes_localMenu(event.getX(), event.getY());
        }
      });
    table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
    JTableHeader tableHeader = table.getTableHeader();
    tableHeader.setResizingAllowed(true);
    tableHeader.setReorderingAllowed(false);
    TableColumn valueColumn = tableHeader.getColumnModel().
      getColumn(AttributeTableCellEditor.ATTRIBUTE_COLUMN_VALUE);
    valueColumn.setPreferredWidth(3 * valueColumn.getPreferredWidth());
    tableHeader.addMouseListener(
      new MouseAdapter() {
        public void mousePressed(MouseEvent event) {
          if ((event.getModifiers() & InputEvent.BUTTON3_MASK) != 0)
            attributesHeader_localMenu(event.getX(), event.getY());
        }
      });
    voletAttributs.add(new JScrollPane(table));
    this.add(voletAttributs);
  }

  private boolean areAttributesEditable() {
    return !fullyFledgedDocument.isReadOnly();
  }

  private void attributes_selectionChanged() {
    try {
      int selectedRow = table.getSelectedRow();
      cellEditor.setAttributeIndex(selectedRow);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void attributes_activation() {
    try {
      Attr attr = getCurrentAttribute();
      if (attr != null)
        documentViewControl.activateAttributeOfSelectedNode(attr.getName());
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void attributesHeader_localMenu(int x, int y) {
    try {
      if (element == null)
        return;
      JPopupMenu menu = new JPopupMenu();
      menu.add(menuEdit.createMenuNewAttribute());
      menu.show(table.getTableHeader(), x, y);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void attributes_localMenu(int x, int y) {
    try {
      if (element == null)
        return;
      AttributesContent attributesContent = fullyFledgedDocument.
        getElementDefinitionMapper().getAttributesContent(element);
      if (attributesContent == null)
        return;
      NamedNodeMap attrs = element.getAttributes();
      int index = table.rowAtPoint(new Point(x, y));
      if (index < 0 || index >= attrs.getLength())
        return;
      Attr attr = (Attr)attrs.item(index);
      String attrName = attr.getName();
      String attrType = attributesContent.getType(attrName);
      table.setRowSelectionInterval(index, index);
      JPopupMenu menu = new JPopupMenu();
      {
        JMenuItem menuItem =
          new JMenuItem(viewResource.getString(Label.SELECT_ATTRIBUTE));
        menuItem.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent event) {
              selectAttribute_actionPerformed();
            }
          });
        menuItem.setEnabled(attrType.equals(AttributesContent.TYPE_CDATA));
        menu.add(menuItem);
      }
      menu.add(menuEdit.createMenuRemoveAttribute());
      if (attrType.indexOf("IDREF") == 0) {
        menu.addSeparator();
        StringTokenizer st = new StringTokenizer(attr.getValue());
        fromAttributesLocalMenuToReferencedElement = new Hashtable();
        ActionListener actionListener =
          new ActionListener()  {
            public void actionPerformed(ActionEvent event) {
              attributes_localMenu_actionPerformed(event);
            }
          };
        while (st.hasMoreTokens()) {
          String id = st.nextToken();
          Element referencedElement =
            fullyFledgedDocument.getIdManager().getElementById(id);
          if (referencedElement != null) {
            JMenuItem menuItem =
              new JMenuItem(viewResource.getString(Label.GOTO) + ' ' + id);
            fromAttributesLocalMenuToReferencedElement.put(menuItem,
              referencedElement);
            menuItem.addActionListener(actionListener);
            menu.add(menuItem);
          }
        }
      }
      menu.show(table, x, y);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void selectAttribute_actionPerformed() {
    try {
      Attr currentAttribute = cellEditor.getCurrentAttribute();
      attributeSelectionListener.selectAttribute(
          currentAttribute != null ? currentAttribute.getName() : null);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void attributes_localMenu_actionPerformed(ActionEvent event) {
    try {
      if (fromAttributesLocalMenuToReferencedElement == null)
        return;
      Element referencedElement =
        (Element)fromAttributesLocalMenuToReferencedElement.get(
        event.getSource());
      if (referencedElement != null)
        documentViewControl.gotoNode(referencedElement);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  public void display(Element element) {
    if (element == null)
      throw new IllegalArgumentException();
    if (isUpdatingField)
      return;
    isDisplaying = true;
    try {
      table.getSelectionModel().clearSelection();
      table.removeEditor();
      table.tableChanged(new TableModelEvent(tableModel));
      this.element = element;
      cellEditor.setElement(element);
    }
    finally {
      isDisplaying = false;
    }
  }

  public Attr getCurrentAttribute() {
    return cellEditor.getCurrentAttribute();
  }

  public void selectLastSearchNextInstance() {
    if (element == null)
      return;
    String lastSearch = options.getStringOptionValue(Options.LASTSEARCH);
    if (lastSearch.length() == 0)
      return;
    boolean isCaseIgnored =
      options.getThreeStateOptionValue(Options.CASEIGNOREDFORSEARCH)
      == ThreeStateOption.TRUE;
    if (isCaseIgnored)
      lastSearch = lastSearch.toUpperCase();
    NamedNodeMap attrs = element.getAttributes();
    int count = attrs.getLength();
    int startIndex = table.getSelectedRow() + 1;
    for (int k = 0; k < count; k++) {
      int index = (startIndex + k) % count;
      Attr attr = (Attr)attrs.item(index);
      String attrValue = attr.getValue();
      if (isCaseIgnored)
        attrValue = attrValue.toUpperCase();
      if (attrValue.indexOf(lastSearch) >= 0) {
        table.setRowSelectionInterval(index, index);
        cellEditor.setAttributeIndex(index);
        return;
      }
    }
  }

  public void clearMemory() {
    element = null;
    fromAttributesLocalMenuToReferencedElement = null;
    cellEditor.setElement(null);
  }

  //////////////////////////////////////////////////////////
  // Implementation of AttributeListener, used by cellEditor
  //////////////////////////////////////////////////////////

  public void setAttribute(String attributeName, String attributeValue) {
    if (attributeName == null || attributeValue == null)
      throw new IllegalArgumentException();
    if (element == null)
      return;
    isUpdatingField = true;
    try {
      editControl.setAttributeValue(element, attributeName, attributeValue);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    finally {
      isUpdatingField = false;
    }
  }
}
