/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node;

import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.ResourceBundle;
import javax.swing.border.Border;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.CompoundBorder;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.manage.ElementDefinitionMapper;
import org.xmloperator.framework.document.manage.DocumentUtil;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.node.AttributeSelectionListener;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.NameSelection;

public final class PanelElementName extends JPanel {
  private static final int TOOL_TIP_MAX_LENGTH = 120;
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private FullyFledgedDocument fullyFledgedDocument;
  private AttributeSelectionListener attributeSelectionListener;
  private boolean isDisplaying = false;
  private final JPanel elementNamePanel = new JPanel(new BorderLayout());
  private final JLabel elementNameLabel = new JLabel();
  private final CardLayout layoutA = new CardLayout();
  private final JPanel panelA = new JPanel(layoutA);
  private final JPanel contentModelPanel = new JPanel(new BorderLayout());
  private final JLabel contentModelLabel = new JLabel();
  private final JPanel attributePanel = new JPanel(new BorderLayout());
  private final JButton attributeButton = new JButton();
  private Element displayedElement = null;

  public PanelElementName(FullyFledgedDocument fullyFledgedDocument,
    AttributeSelectionListener attributeSelectionListener, Options options) {
    super(new BorderLayout());
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.attributeSelectionListener = attributeSelectionListener;
    Border emptyBorder = new EmptyBorder(4, 4, 4, 4);
    // Element name
    elementNamePanel.setBorder(
        new CompoundBorder(emptyBorder, new BevelBorder(BevelBorder.LOWERED)));
    elementNameLabel.setHorizontalAlignment(JLabel.CENTER);
    elementNamePanel.add(elementNameLabel);
    elementNamePanel.add(elementNameLabel);
    this.add(elementNamePanel, BorderLayout.NORTH);
    // Content model or Attribute
    contentModelPanel.setBorder(
        new CompoundBorder(emptyBorder, new BevelBorder(BevelBorder.LOWERED)));
    contentModelLabel.setHorizontalAlignment(JLabel.CENTER);
    contentModelPanel.add(contentModelLabel);
    attributePanel.setBorder(emptyBorder);
    attributeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        attribute_actionPerformed();
      }
    });
    attributeButton.setToolTipText(
        viewResource.getString(Label.RETURN_TO_ATTRIBUTE_TABLE));
    attributePanel.add(attributeButton);
    panelA.add(contentModelPanel, "ContentModel");
    panelA.add(attributePanel, "Attribute");
    this.add(panelA, BorderLayout.SOUTH);
  }

  public void attribute_actionPerformed() {
    attributeSelectionListener.selectAttribute(null);
  }

  public void display(Element element, String attributeName) {
    if (element == null)
      throw new IllegalArgumentException();
    try {
      isDisplaying = true;
      displayedElement = element;
      elementNameLabel.setText(displayedElement.getNodeName());
      elementNamePanel.setToolTipText(getPositionModelString(displayedElement));
      ElementDefinitionMapper elementDefinitionMapper =
        fullyFledgedDocument.getElementDefinitionMapper();
      layoutA.show(panelA,
                   attributeName != null ? "Attribute" : "ContentModel");
      if (attributeName != null)
        attributeButton.setText(attributeName);
      else {
        contentModelLabel.setText(
            elementDefinitionMapper.getElementDefinition(displayedElement)
            != null ? viewResource.getString(
            elementDefinitionMapper.isElementContentValid(displayedElement) ?
            Label.VALID_CONTENT : Label.NOT_VALID_CONTENT) : " ");
        contentModelPanel.setToolTipText(
            getContentModelString(displayedElement));
      }
    }
    finally {
      isDisplaying = false;
    }
  }

  private String getPositionModelString(Element element) {
    try {
      Node parentNode = element.getParentNode();
      if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
        return null;
      DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
      if (documentUtil == null)
        return null;
      String positionModelString =
        documentUtil.elementFromContentModelToString(element, false);
      if (positionModelString == null)
        return null;
      if (positionModelString.length() > TOOL_TIP_MAX_LENGTH)
        positionModelString =
          documentUtil.elementFromContentModelToString(element, true);
      return truncate(positionModelString);
    }
    catch (Exception exception) {
      return null;
    }
  }

  private String getContentModelString(Element element) {
    try {
      DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
      if (documentUtil == null)
        return null;
      String contentModelString =
        documentUtil.contentModelToString(element, false);
      if (contentModelString == null)
        return null;
      if (contentModelString.length() > TOOL_TIP_MAX_LENGTH)
        contentModelString = documentUtil.contentModelToString(element, true);
      return truncate(contentModelString);
    }
    catch (Exception exception) {
      return null;
    }
  }

  private static final String truncate(String data) {
    return data.length() <= TOOL_TIP_MAX_LENGTH ? data :
      data.substring(0, TOOL_TIP_MAX_LENGTH - 3) + "...";
  }
}
