/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node;

import java.util.ResourceBundle;
import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.BoxLayout;

import org.w3c.dom.Node;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.document.elementKey.ElementKeyUtil;

public final class PanelNodeInterval extends JPanel {
  private Options options;
  private boolean isDisplaying = false;
  private final JTextArea firstNodeTextArea;
  private final JTextArea lastNodeTextArea;

  public PanelNodeInterval(Options options) {
    super(new BorderLayout());
    this.options = options;
    Border emptyBorder = new EmptyBorder(4, 4, 4, 4);
    JPanel nodesPanel = new JPanel();
    nodesPanel.setBorder(emptyBorder);
    nodesPanel.setLayout(new BoxLayout(nodesPanel, BoxLayout.Y_AXIS));
    firstNodeTextArea = createTextArea();
    lastNodeTextArea = createTextArea();
    nodesPanel.add(new JScrollPane(firstNodeTextArea));
    nodesPanel.add(new JScrollPane(lastNodeTextArea));
    this.add(nodesPanel);
  }

  private JTextArea createTextArea() {
    JTextArea textArea = new JTextArea();
    textArea.setLineWrap(true);
    textArea.setWrapStyleWord(true);
    textArea.setFont(
      options.getThreeStateOptionValue(Options.FONT) == ThreeStateOption.TRUE ?
      Constants.FONT_LARGE : Constants.FONT_NORMAL);
    textArea.setEnabled(false);
    return textArea;
  }

  public void display(Node firstNode, Node lastNode) {
    if (firstNode == null || lastNode == null)
      throw new IllegalArgumentException();
    isDisplaying = true;
    try {
      firstNodeTextArea.setText(ElementKeyUtil.nodeToString(firstNode, null));
      lastNodeTextArea.setText(lastNode != firstNode ?
        ElementKeyUtil.nodeToString(lastNode, null) : "");
      revalidate();
    }
    finally {
      isDisplaying = false;
    }
  }
}
