/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node.attribute;

import java.awt.Point;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JViewport;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.DialogSelectionIds;
import org.xmloperator.util.view.DialogModal;
import org.xmloperator.view.model.DomTreeNode;
import org.xmloperator.view.model.DomIdTreeModel;

public final class DialogSelectIdImpl extends DialogModal
    implements DialogSelectionIds {
  private FullyFledgedDocument fullyFledgedDocument;
  private DomIdTreeModel domIdTreeModel;
  private JTree treePanel;
  private JScrollPane scrollPanel;
  private JButton noValueButton;
  private boolean isIdSelected = false;

  public DialogSelectIdImpl(
      Frame parent, FullyFledgedDocument fullyFledgedDocument) {
    super(parent);
    this.fullyFledgedDocument = fullyFledgedDocument;
    JPanel selectPanel = new JPanel(new BorderLayout());
    domIdTreeModel = new DomIdTreeModel(fullyFledgedDocument);
    // Tree panel
    treePanel = new JTree(domIdTreeModel);
    treePanel.setRootVisible(false);
    treePanel.getSelectionModel().
      setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    treePanel.addTreeSelectionListener(
      new TreeSelectionListener() {
        public void valueChanged(TreeSelectionEvent event) {
          tree_selectionChanged();
        }
      });
    scrollPanel = new JScrollPane(treePanel);
    scrollPanel.setPreferredSize(new Dimension(400, 300));
    selectPanel.add(scrollPanel);
    // No value button
    JPanel noValuePanel = new JPanel(new FlowLayout());
    this.noValueButton =
        new JButton(viewResource.getString(Label.NO_VALUE));
    noValueButton.addActionListener(
        new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        noValue_actionPerformed();
      }
    });
    noValuePanel.add(noValueButton);
    selectPanel.add(noValuePanel, BorderLayout.SOUTH);
    contentPanel.add(selectPanel);
    this.pack();
    Element documentElement =
      fullyFledgedDocument.getDocument().getDocumentElement();
    domIdTreeModel.setRootNode(documentElement, false);
    treePanel.expandPath(domIdTreeModel.getTreePath(documentElement));
    buttonOK.setEnabled(false);
  }

  private void noValue_actionPerformed() {
    isIdSelected = false;
    isOK = true;
    dispose();
  }

  private void tree_selectionChanged() {
    DomTreeNode domTreeNode =
      (DomTreeNode)treePanel.getLastSelectedPathComponent();
    isIdSelected = domTreeNode != null ?
      isIdElement((Element)domTreeNode.getNode()) : false;
    buttonOK.setEnabled(
      isIdSelected && !fullyFledgedDocument.isReadOnly());
  }

  private boolean isIdElement(Element element) {
    String id = getId(element);
    return id != null && id.length() != 0;
  }

  private final String getId(Element element) {
    String idAttrName = fullyFledgedDocument.getDocumentUtil().
      getAttributesContent(element).getIdAttributeName();
    return idAttrName != null ? element.getAttribute(idAttrName) : null;
  }

  public void setContentFont(boolean isFontLarge) {
    this.treePanel.setRowHeight(isFontLarge ? 18 : 16);
    this.treePanel.setFont(
        isFontLarge ? Constants.FONT_LARGE : Constants.FONT_NORMAL);
  }

  public void setNoValueEnabled(boolean isNoValueEnabled) {
    this.noValueButton.setEnabled(isNoValueEnabled);
  }

  ////////////////////////////////////
  // DialogSelectionIds implementation
  ////////////////////////////////////

  public String getSelectedIds() {
    if (!isOK)
      return null;
    if (!isIdSelected)
      return "";
    DomTreeNode domTreeNode =
      (DomTreeNode)treePanel.getLastSelectedPathComponent();
    if (domTreeNode == null)
      return null;
    Element element = (Element)domTreeNode.getNode();
    return getId(element);
  }

  public void setSelectedIds(String id, boolean makeVisibleAndSelected) {
    Element element = fullyFledgedDocument.getIdManager().getElementById(id);
    if (element == null)
      return;
    TreePath treePath = domIdTreeModel.getTreePath(element);
    if (makeVisibleAndSelected)
      treePanel.setSelectionPath(treePath);
    treePanel.makeVisible(treePath);
    {
      Rectangle pathBounds = treePanel.getPathBounds(treePath);
      if (pathBounds != null) {
        JViewport viewport = scrollPanel.getViewport();
        int y = pathBounds.y - viewport.getViewRect().height / 2;
        viewport.setViewPosition(new Point(0, y >= 0 ? y : 0));
      }
    }
  }
}
