/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node.attribute;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Event;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.JComponent;
import javax.swing.KeyStroke;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JTree;
import javax.swing.JSplitPane;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.DialogSelectionIds;
import org.xmloperator.util.view.DialogModal;
import org.xmloperator.view.model.DomTreeNode;
import org.xmloperator.view.model.DomIdTreeModel;

public final class DialogSelectIdsImpl extends DialogModal
    implements DialogSelectionIds {
  private FullyFledgedDocument fullyFledgedDocument;
  private DomIdTreeModel domIdTreeModel;
  private JTree treePanel;
  private JScrollPane scrollTreePanel;
  private JButton buttonAdd;
  private JButton buttonRemove;
  private JButton buttonUp;
  private JButton buttonDown;
  private DefaultListModel resultListModel;
  private JList listPanel;
  private JScrollPane scrollListPanel;

  public DialogSelectIdsImpl(
      Frame parent, FullyFledgedDocument fullyFledgedDocument) {
    super(parent);
    this.fullyFledgedDocument = fullyFledgedDocument;
    // Tree panel
    domIdTreeModel = new DomIdTreeModel(fullyFledgedDocument);
    treePanel = new JTree(domIdTreeModel);
    treePanel.setRootVisible(false);
    treePanel.addTreeSelectionListener(
      new TreeSelectionListener() {
        public void valueChanged(TreeSelectionEvent event) {
          tree_selectionChanged();
        }
      });
    scrollTreePanel = new JScrollPane(treePanel);
    scrollTreePanel.setPreferredSize(new Dimension(400, 200));
    // List and buttons panel
    JPanel voletResultat = new JPanel(new BorderLayout());
    // Add, Remove, Up and Down buttons panel
    JPanel voletBoutons = new JPanel(new FlowLayout());
    buttonAdd = new JButton(viewResource.getString(Label.ADD));
    buttonAdd.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.ADD).charAt(0));
    buttonAdd.setEnabled(false);
    buttonAdd.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonAdd_actionPerformed();
        }
      });
    voletBoutons.add(buttonAdd);
    buttonRemove = new JButton(viewResource.getString(Label.SUPPRIMER));
    buttonRemove.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.SUPPRIMER).charAt(0));
    buttonRemove.setEnabled(false);
    buttonRemove.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonRemove_actionPerformed();
        }
      });
    voletBoutons.add(buttonRemove);
    buttonUp = new JButton(viewResource.getString(Label.MONTER));
    buttonUp.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.MONTER).charAt(0));
    buttonUp.setEnabled(false);
    buttonUp.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonUp_actionPerformed();
        }
      });
    voletBoutons.add(buttonUp);
    buttonDown = new JButton(viewResource.getString(Label.DESCENDRE));
    buttonDown.setMnemonic(viewResource.getString(Label.MNEMO
      + Label.DESCENDRE).charAt(0));
    buttonDown.setEnabled(false);
    buttonDown.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonDown_actionPerformed();
        }
      });
    voletBoutons.add(buttonDown);
    voletResultat.add(voletBoutons, BorderLayout.NORTH);
    // volet liste
    resultListModel = new DefaultListModel();
    listPanel = new JList(resultListModel);
    listPanel.addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent event) {
          list_selectionChanged();
        }
      });
    scrollListPanel = new JScrollPane(listPanel);
    scrollListPanel.setPreferredSize(new Dimension(200, 100));
    voletResultat.add(scrollListPanel);
    // construction
    JSplitPane splitPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
      true, scrollTreePanel, voletResultat);
    contentPanel.add(splitPanel);
    pack();
    // Initialisation
    Element documentElement =
      fullyFledgedDocument.getDocument().getDocumentElement();
    domIdTreeModel.setRootNode(documentElement, false);
    treePanel.expandPath(domIdTreeModel.getTreePath(documentElement));
    buttonOK.setEnabled(!fullyFledgedDocument.isReadOnly());
    // Keyboard shortcuts
    treePanel.registerKeyboardAction(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonAdd_actionPerformed();
        }
      },
      KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0),
      JComponent.WHEN_IN_FOCUSED_WINDOW);
    listPanel.registerKeyboardAction(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonRemove_actionPerformed();
        }
      },
      KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0),
      JComponent.WHEN_IN_FOCUSED_WINDOW);
    listPanel.registerKeyboardAction(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonUp_actionPerformed();
        }
      },
      KeyStroke.getKeyStroke(KeyEvent.VK_UP,
      Event.CTRL_MASK + Event.SHIFT_MASK),
      JComponent.WHEN_IN_FOCUSED_WINDOW);
    listPanel.registerKeyboardAction(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonDown_actionPerformed();
        }
      },
      KeyStroke.getKeyStroke(KeyEvent.VK_DOWN,
      Event.CTRL_MASK + Event.SHIFT_MASK),
      JComponent.WHEN_IN_FOCUSED_WINDOW);
  }

  private void buttonAdd_actionPerformed() {
    if (fullyFledgedDocument.isReadOnly() || treePanel.isSelectionEmpty())
      return;
    TreePath[] treePaths = treePanel.getSelectionPaths();
    int addCount = 0;
    for (int k = 0; k < treePaths.length; k++) {
      DomTreeNode domTreeNode =
        (DomTreeNode)treePaths[k].getLastPathComponent();
      if (!resultListModel.contains(domTreeNode)
        && isIdElement((Element)domTreeNode.getNode())) {
        resultListModel.addElement(domTreeNode);
        addCount++;
      }
    }
    int resultListSize = resultListModel.size();
    listPanel.setSelectionInterval(
      resultListSize - addCount, resultListSize - 1);
    buttonAdd.setEnabled(false);
  }

  private void buttonRemove_actionPerformed() {
    if (fullyFledgedDocument.isReadOnly() || listPanel.isSelectionEmpty())
      return;
    int indexMin = listPanel.getMinSelectionIndex();
    int indexMax = listPanel.getMaxSelectionIndex();
    for (int k = indexMax; k >= indexMin; k--)
      if (listPanel.isSelectedIndex(k))
        resultListModel.remove(k);
    tree_selectionChanged(); // pour le cas de la selection identique
    int index = indexMin;
    if (index == resultListModel.size())
      index--;
    if (index >= 0)
      listPanel.setSelectedIndex(index);
    else {
      buttonRemove.setEnabled(false);
      buttonUp.setEnabled(false);
      buttonDown.setEnabled(false);
    }
  }

  private void buttonUp_actionPerformed() {
    if (fullyFledgedDocument.isReadOnly() || listPanel.isSelectionEmpty())
      return;
    int indexMin = listPanel.getMinSelectionIndex();
    int indexMax = listPanel.getMaxSelectionIndex();
    if (indexMin == 0)
      return;
    Object temp = resultListModel.elementAt(indexMin - 1);
    for (int k = indexMin; k <= indexMax; k++)
      resultListModel.setElementAt(resultListModel.elementAt(k), k - 1);
    resultListModel.setElementAt(temp, indexMax);
    listPanel.setSelectionInterval(indexMin - 1, indexMax - 1);
  }

  private void buttonDown_actionPerformed() {
    if (fullyFledgedDocument.isReadOnly() || listPanel.isSelectionEmpty())
      return;
    int indexMin = listPanel.getMinSelectionIndex();
    int indexMax = listPanel.getMaxSelectionIndex();
    if (indexMax == resultListModel.size() - 1)
      return;
    Object temp = resultListModel.elementAt(indexMax + 1);
    for (int k = indexMax; k >= indexMin; k--)
      resultListModel.setElementAt(resultListModel.elementAt(k), k + 1);
    resultListModel.setElementAt(temp, indexMin);
    listPanel.setSelectionInterval(indexMin + 1, indexMax + 1);
  }

  private void tree_selectionChanged() {
    TreePath[] treePaths = treePanel.getSelectionPaths();
    int addCount = 0;
    for (int k = 0; k < treePaths.length; k++) {
      DomTreeNode domTreeNode =
        (DomTreeNode)treePaths[k].getLastPathComponent();
      if (!resultListModel.contains(domTreeNode)
        && isIdElement((Element)domTreeNode.getNode()))
        addCount++;
    }
    buttonAdd.setEnabled(addCount != 0
      && !fullyFledgedDocument.isReadOnly());
  }

  private void list_selectionChanged() {
    boolean isMovable =
      !(listPanel.isSelectionEmpty() || fullyFledgedDocument.isReadOnly());
    int indexMin = listPanel.getMinSelectionIndex();
    int indexMax = listPanel.getMaxSelectionIndex();
    buttonRemove.setEnabled(isMovable);
    buttonUp.setEnabled(isMovable && indexMin > 0);
    buttonDown.setEnabled(
      isMovable && indexMax < resultListModel.size() - 1);
  }

  private boolean isIdElement(Element element) {
    String id = getId(element);
    return id != null && id.length() != 0;
  }

  private final String getId(Element element) {
    String idAttrName = fullyFledgedDocument.getDocumentUtil().
      getAttributesContent(element).getIdAttributeName();
    return idAttrName != null ? element.getAttribute(idAttrName) : null;
  }

  public void setContentFont(boolean isFontLarge) {
    this.treePanel.setRowHeight(isFontLarge ? 18 : 16);
    this.treePanel.setFont(
        isFontLarge ? Constants.FONT_LARGE : Constants.FONT_NORMAL);
    this.listPanel.setFont(
        new Font("dialog", Font.PLAIN, isFontLarge ? 14 : 12));
  }

  ////////////////////////////////////
  // DialogSelectionIds implementation
  ////////////////////////////////////

  public String getSelectedIds() {
    if (!isOK)
      return null;
    StringBuffer s = new StringBuffer();
    for (int k = 0; k < resultListModel.size(); k++) {
      DomTreeNode domTreeNode = (DomTreeNode)resultListModel.elementAt(k);
      Element element = (Element)domTreeNode.getNode();
      String id = getId(element);
      if (k != 0)
        s.append(' ');
      s.append(id);
    }
    return s.toString();
  }

  public void setSelectedIds(String ids, boolean makeVisibleAndSelected) {
    resultListModel.clear();
    StringTokenizer st = new StringTokenizer(ids);
    while (st.hasMoreTokens()) {
      String id = st.nextToken();
      Element element = fullyFledgedDocument.getIdManager().getElementById(id);
      if (element != null) {
        TreePath treePath = domIdTreeModel.getTreePath(element);
        treePanel.makeVisible(treePath);
        if (makeVisibleAndSelected) {
          DomTreeNode domTreeNode = DomTreeNode.domTreeNodeFromNode(
              element,
              this.fullyFledgedDocument.getNodeLinker(),
              this.fullyFledgedDocument.getElementKeyProvider());
          resultListModel.addElement(domTreeNode);
        }
      }
    }
    if (resultListModel.size() != 0)
      listPanel.setSelectedIndex(0);
  }
}
