/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.parameter;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.util.List;
import javax.swing.ListSelectionModel;
import javax.swing.JDialog;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;

import org.xmloperator.framework.document.util.Parameter;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.util.view.DialogModal;
import org.xmloperator.view.parameter.ParameterTableCellEditor;

public final class ParameterListDialog extends DialogModal {
  private List parameterList;
  private TableModel tableModel;
  private JTable table;
  private ParameterTableCellEditor cellEditor;

  public ParameterListDialog(Frame parent, List parameterList, Options options)
  {
    super(parent);
    this.parameterList = parameterList;
    tableModel = new AbstractTableModel() {
      public String getColumnName(int col) {
        switch (col) {
          case ParameterTableCellEditor.PARAMETER_COLUMN_NAME:
            return viewResource.getString(Label.PARAMETER);
          case ParameterTableCellEditor.PARAMETER_COLUMN_VALUE:
            return viewResource.getString(Label.VALUE);
          default:
            return "";
        }
      }
      public Class getColumnClass(int col) {
        return String.class;
      }
      public int getRowCount() {
        return table_getRowCount();
      }
      public int getColumnCount() {
        return 2;
      }
      public Object getValueAt(int row, int col) {
        return table_getValueAt(row, col);
      }
      public boolean isCellEditable(int row, int col) {
        return col == ParameterTableCellEditor.PARAMETER_COLUMN_VALUE;
      }
    };
    table = new JTable(tableModel);
    table.setShowVerticalLines(true);
    table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
    TableColumn valueColumn = table.getColumnModel().
      getColumn(ParameterTableCellEditor.PARAMETER_COLUMN_VALUE);
    valueColumn.setPreferredWidth(3 * valueColumn.getPreferredWidth());
    cellEditor =
      new ParameterTableCellEditor(table, parameterList, options);
    valueColumn.setCellEditor(cellEditor);
    boolean isFontLarge =
      options.getThreeStateOptionValue(Options.FONT) == ThreeStateOption.TRUE;
    table.setRowHeight(isFontLarge ? 22 : 20);
    table.setFont(isFontLarge ? Constants.FONT_LARGE : Constants.FONT_NORMAL);
    table.setRowSelectionAllowed(true);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    table.getSelectionModel().addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent event) {
          selectionChanged();
        }
      });
    JTableHeader tableHeader = table.getTableHeader();
    tableHeader.setResizingAllowed(true);
    tableHeader.setReorderingAllowed(false);
    JScrollPane scrollPane = new JScrollPane(table);
    scrollPane.setPreferredSize(new Dimension(400, 200));
    contentPanel.add(scrollPane);
    pack();
  }

  protected void selectionChanged() {
    int selectedRow = table.getSelectedRow();
    cellEditor.setParameterIndex(selectedRow);
  }

  protected int table_getRowCount() {
    return parameterList.size();
  }

  protected Object table_getValueAt(int row, int col) {
    if (row >= 0 && row < parameterList.size()) {
      Parameter parameter = (Parameter)parameterList.get(row);
      switch (col) {
        case ParameterTableCellEditor.PARAMETER_COLUMN_NAME:
          return parameter.getName();
        case ParameterTableCellEditor.PARAMETER_COLUMN_VALUE:
          return parameter.getValue();
      }
    }
    return "";
  }
}

