/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.parameter;

import java.util.EventObject;
import java.util.List;
import java.awt.Font;
import java.awt.Component;
import java.awt.Frame;
import javax.swing.JTextField;
import javax.swing.JTable;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.CellEditorListener;
import javax.swing.table.TableCellEditor;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.util.Parameter;

public final class ParameterTableCellEditor implements TableCellEditor {
  public static final int PARAMETER_COLUMN_NAME = 0;
  public static final int PARAMETER_COLUMN_VALUE = 1;
  private Frame frame;
  private List parameterList;
  private JTable parameterTable;
  private JTextField valueField;
  private int currentParameterIndex = -1;
  private Parameter currentParameter = null;
  private boolean isSettingEditor = false;

  public ParameterTableCellEditor(
      JTable parameterTable, List parameterList, Options options) {
    this.parameterTable = parameterTable;
    this.parameterList = parameterList;
    Font font = options.getThreeStateOptionValue(Options.FONT)
      == ThreeStateOption.TRUE ? Constants.FONT_LARGE : Constants.FONT_NORMAL;
    valueField = new JTextField();
    valueField.setFont(font);
    valueField.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent event) {
          valueField_update();
        };
        public void removeUpdate(DocumentEvent event) {
          valueField_update();
        };
        public void changedUpdate(DocumentEvent event) {
          valueField_update();
        };
      });
  }

  public void setParameterIndex(int parameterIndex) {
    if (parameterIndex < 0 || parameterIndex >= parameterList.size())
      parameterIndex = -1;
    if (parameterIndex == currentParameterIndex)
      return;
    isSettingEditor = true;
    try {
      if (currentParameterIndex >= 0)
        parameterTable.removeEditor();
      currentParameterIndex = parameterIndex;
      currentParameter = parameterIndex >= 0 ?
        (Parameter)parameterList.get(parameterIndex) : null;
      if (currentParameterIndex >= 0)
        parameterTable.editCellAt(currentParameterIndex,
          PARAMETER_COLUMN_VALUE);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    finally {
      isSettingEditor = false;
    }
  }

  protected void valueField_update() {
    if (currentParameter != null && !isSettingEditor)
      currentParameter.setValue(valueField.getText());
  }

  ////////////////////////////
  // CellEditor implementation
  ////////////////////////////

  public Object getCellEditorValue() {
    return null; // Editing is never validated
  }

  public boolean isCellEditable(EventObject anEvent) {
    return true;
  }

  public boolean shouldSelectCell(EventObject anEvent) {
    return true;
  }

  public boolean stopCellEditing() {
    return true;
  }

  public void cancelCellEditing() {
  }

  public void addCellEditorListener(CellEditorListener listener) {
    // Not used
  }

  public void removeCellEditorListener(CellEditorListener listener) {
    // Not used
  }

  /////////////////////////////////
  // TableCellEditor implementation
  /////////////////////////////////

  public Component getTableCellEditorComponent(JTable table, Object value,
    boolean isSelected, int row, int column) {
    if (row == currentParameterIndex && currentParameter != null)
      valueField.setText(currentParameter.getValue());
    return valueField;
  }
}
