/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.shortcut;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.awt.Event;
import java.awt.event.KeyEvent;
import java.awt.event.ActionListener;
import javax.swing.KeyStroke;
import javax.swing.JComponent;

import org.xmloperator.framework.view.shortcut.Shortcut;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.view.shortcut.ShortcutImpl;

public final class ShortcutsImpl implements Shortcuts {
  private static final Shortcut[] SHORTCUTS = {
    new ShortcutImpl(SHORTCUT_ASSOCIATED_ELEMENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_A, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_RECONSTRUIRE_TEL_QUEL,
                     KeyStroke.getKeyStroke(KeyEvent.VK_B, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_NAVIGATION,
                     KeyStroke.getKeyStroke(KeyEvent.VK_B, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_RECONSTRUIRE_AUTREMENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_B, Event.ALT_MASK)),
    // Ctrl+C is not usable (copy to clipboard).
    new ShortcutImpl(SHORTCUT_COMPARE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_C, Event.CTRL_MASK + Event.SHIFT_MASK)),
    // Alt+C is used by the <C>ompare menu.
    new ShortcutImpl(SHORTCUT_NOUVEAU_DESCENDANT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_D, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_PASSER_DESCENDANT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_D, Event.CTRL_MASK + Event.SHIFT_MASK)),
    // Alt+D is used by the <D>ocument menu.
    new ShortcutImpl(SHORTCUT_EXPANSER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_E, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_EVALUATE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_E, Event.CTRL_MASK + Event.SHIFT_MASK)),
    // Alt+E is used by the <E>dit menu.
    new ShortcutImpl(SHORTCUT_CHERCHER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_F, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_IMPORT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_F, Event.CTRL_MASK + Event.SHIFT_MASK)),
    // Alt+F is used by the <F>ile menu.
    new ShortcutImpl(SHORTCUT_COPY,
                     KeyStroke.getKeyStroke(KeyEvent.VK_G, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_FRAGMENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_G, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_COPY_TO_CLIP,
                     KeyStroke.getKeyStroke(KeyEvent.VK_G, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_CHANGE_ELEMENT_LABELLING,
                     KeyStroke.getKeyStroke(KeyEvent.VK_H, Event.CTRL_MASK)),
    // Ctrl+Shift+H is free.
    // Alt+H is free.
    // Ctrl+I is free.
    // Ctrl+Shift+I is free.
    // Alt+I is used by the V<i>ew menu.
    new ShortcutImpl(SHORTCUT_JOINDRE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_J, Event.CTRL_MASK)),
    // Ctrl+Shift+J is free.
    // Alt+J is free.
    new ShortcutImpl(SHORTCUT_DIVISER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_K, Event.CTRL_MASK)),
    // Ctrl+Shift+K is free.
    // Alt+K is free.
    // Ctrl+M is not usable (enter).
    new ShortcutImpl(SHORTCUT_TO_BE_MOVED,
                     KeyStroke.getKeyStroke(KeyEvent.VK_M, Event.CTRL_MASK + Event.SHIFT_MASK)),
    // Alt+M is used by the Frag<m>ent menu.
    new ShortcutImpl(SHORTCUT_NOUVEAU_SUIVANT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_CONTENT_MODEL,
                     KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_SELECTIONNER_SUIVANT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_NEW_DOCUMENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_OPERATIONS,
                     KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK + Event.SHIFT_MASK)),
    // Ctrl-Shift-O should not be used (dialogs).
    new ShortcutImpl(SHORTCUT_OUVRIR,
                     KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_NOUVEAU_PRECEDENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_P, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_EMPAQUETER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_P, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_SELECTIONNER_PRECEDENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_P, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_STATISTIQUE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_Q, Event.CTRL_MASK)),
    // Ctrl+Shift+Q is free.
    new ShortcutImpl(SHORTCUT_EXIT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_Q, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_REPLACE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_R, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_REPLACE_BY_FRAGMENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_R, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_REOUVRIR,
                     KeyStroke.getKeyStroke(KeyEvent.VK_R, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_ENREGISTRER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_ENREGISTRER_ET_RECHARGER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK + Event.SHIFT_MASK)),
    // Alt+S is used by Option<s>.
    new ShortcutImpl(SHORTCUT_TO_CLIP,
                     KeyStroke.getKeyStroke(KeyEvent.VK_T, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_PASSER_PARENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_T, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_FROM_CLIP,
                     KeyStroke.getKeyStroke(KeyEvent.VK_T, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_NEW_ATTRIBUTE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_U, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_DOCUMENT_TYPE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_U, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_DOCUMENT_TYPES,
                     KeyStroke.getKeyStroke(KeyEvent.VK_U, Event.ALT_MASK)),
    // Ctrl+V is not usable (paste).
    new ShortcutImpl(SHORTCUT_TRANSFORM,
                     KeyStroke.getKeyStroke(KeyEvent.VK_V, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_PASTE_CLIP_BEFORE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_V, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_PASTE_AFTER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_W, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_PASTE_BEFORE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_W, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_PASTE_CLIP_AFTER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_W, Event.ALT_MASK)),
    // Ctrl+X is not usable (cut).
    new ShortcutImpl(SHORTCUT_CUT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_X, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_FERMER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_X, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_PASTE_UNDER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_Y, Event.CTRL_MASK)),
    // Ctrl+Shift+Y is free
    new ShortcutImpl(SHORTCUT_PASTE_CLIP_UNDER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_Y, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_DEFAIRE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_Z, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_REFAIRE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_Z, Event.CTRL_MASK + Event.SHIFT_MASK)),
    // Alt+Z is free.
    new ShortcutImpl(SHORTCUT_SUPPRIMER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_DEPAQUETER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_REMOVE_ATTRIBUTE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_PASSER_PRECEDENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_UP, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_MONTER,
                     KeyStroke.getKeyStroke(KeyEvent.VK_UP, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_PASSER_SUIVANT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, Event.CTRL_MASK + Event.SHIFT_MASK)),
    new ShortcutImpl(SHORTCUT_DESCENDRE,
                     KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_SELECTIONNER_DESCENDANT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_SELECTIONNER_PARENT,
                     KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_COMMAND1,
                     KeyStroke.getKeyStroke(KeyEvent.VK_F1, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_OPERATOR1,
                     KeyStroke.getKeyStroke(KeyEvent.VK_F1, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_COMMAND2,
                     KeyStroke.getKeyStroke(KeyEvent.VK_F2, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_OPERATOR2,
                     KeyStroke.getKeyStroke(KeyEvent.VK_F2, Event.ALT_MASK)),
    new ShortcutImpl(SHORTCUT_COMMAND3,
                     KeyStroke.getKeyStroke(KeyEvent.VK_F3, Event.CTRL_MASK)),
    new ShortcutImpl(SHORTCUT_OPERATOR3,
                     KeyStroke.getKeyStroke(KeyEvent.VK_F3, Event.ALT_MASK))};

  private final Map fromIdToShortcut = new HashMap();

  public ShortcutsImpl() {
    for (int k = 0; k < SHORTCUTS.length; k++) {
      Shortcut shortcut = SHORTCUTS[k];
      String id = shortcut.getId();
      if (fromIdToShortcut.containsKey(id))
        throw new RuntimeException("Shortkey duplicate key: " + id);
      fromIdToShortcut.put(id, new ShortcutImpl(shortcut));
    }
  }

  ///////////////////////////
  // Shortcuts implementation
  ///////////////////////////

  public final Shortcut getShortcut(String shortcutId) {
    return (Shortcut)fromIdToShortcut.get(shortcutId);
  }

  public final void registerKeyboardAction(JComponent component) {
    Iterator iterator = fromIdToShortcut.values().iterator();
    while (iterator.hasNext()) {
      Shortcut shortcut = (Shortcut)iterator.next();
      ActionListener actionListener = shortcut.getActionListener();
      if (actionListener != null)
        component.registerKeyboardAction(actionListener, shortcut.getId(),
          shortcut.getKeyStroke(), JComponent.WHEN_IN_FOCUSED_WINDOW);
    }
  }

  public final void clearActionListeners() {
    Iterator iterator = fromIdToShortcut.values().iterator();
    while (iterator.hasNext())
      ((Shortcut)iterator.next()).setActionListener(null);
  }
}
